Package: dendroTools
Type: Package
Title: Linear and Nonlinear Methods for Analyzing Daily and Monthly
        Dendroclimatological Data
Version: 1.2.13
Author: Jernej Jevsenak [aut, cre]
Maintainer: Jernej Jevsenak <jernej.jevsenak@gmail.com>
Description: Provides novel dendroclimatological methods, primarily used by the
    Tree-ring research community. There are four core functions. The first one is 
    daily_response(), which finds the optimal sequence of days that are related 
    to one or more tree-ring proxy records. Similar function is daily_response_seascorr(), 
    which implements partial correlations in the analysis of daily response functions.
    For the enthusiast of monthly data, there is monthly_response() function.
    The last core function is compare_methods(), which effectively compares several 
    linear and nonlinear regression algorithms on the task of climate reconstruction.   
License: GPL-3
URL: https://github.com/jernejjevsenak/dendroTools
BugReports: https://github.com/jernejjevsenak/dendroTools/issues
Encoding: UTF-8
LazyData: true
Suggests: testthat, rmarkdown
RoxygenNote: 7.1.2
Imports: ggplot2(>= 2.2.0), brnn(>= 0.6), reshape2(>= 1.4.2), scales(>=
        0.4.1), stats, oce(>= 1.2-0), MLmetrics(>= 1.1.1), dplyr(>=
        0.7.0), knitr(>= 1.19), magrittr(>= 1.5), plotly(>= 4.7.1),
        randomForest(>= 4.6-14), Cubist(>= 0.2.2), lubridate (>=
        1.7.4), psych (>= 1.8.3.3), boot(>= 1.3-22), viridis (>=
        0.5.1), dplR (>= 1.7.2)
Depends: R(>= 3.4)
NeedsCompilation: no
Repository: CRAN
VignetteBuilder: knitr
Packaged: 2024-04-26 05:46:50 UTC; JernejJ
Date/Publication: 2024-04-26 10:10:04 UTC
Built: R 4.2.3; ; 2024-04-26 11:26:02 UTC; unix

Package: dendroTools
Type: Package
Title: Linear and Nonlinear Methods for Analyzing Daily and Monthly
        Dendroclimatological Data
Version: 1.2.16
Authors@R: 
    person(
      given  = "Jernej",
      family = "Jevsenak",
      role   = c("aut", "cre"),
      email  = "jernej.jevsenak@gmail.com"
    )
Description: Provides novel dendroclimatological methods, primarily used by the
    Tree-ring research community. There are four core functions. The first one is 
    daily_response(), which finds the optimal sequence of days that are related 
    to one or more tree-ring proxy records. Similar function is daily_response_seascorr(), 
    which implements partial correlations in the analysis of daily response functions.
    For the enthusiast of monthly data, there is monthly_response() function.
    The last core function is compare_methods(), which effectively compares several 
    linear and nonlinear regression algorithms on the task of climate reconstruction.   
License: GPL-3
URL: https://github.com/jernejjevsenak/dendroTools
BugReports: https://github.com/jernejjevsenak/dendroTools/issues
Encoding: UTF-8
LazyData: true
Suggests: testthat, rmarkdown
RoxygenNote: 7.3.2
Imports: ggplot2 (>= 2.2.0), brnn (>= 0.6), reshape2 (>= 1.4.2), scales
        (>= 0.4.1), stats, oce (>= 1.2-0), MLmetrics (>= 1.1.1), dplyr
        (>= 0.7.0), knitr (>= 1.19), magrittr (>= 1.5), plotly (>=
        4.7.1), randomForest (>= 4.6-14), Cubist (>= 0.2.2), lubridate
        (>= 1.7.4), psych (>= 1.8.3.3), boot (>= 1.3-22), viridis (>=
        0.5.1), dplR (>= 1.7.2)
Depends: R (>= 3.4)
NeedsCompilation: no
Repository: CRAN
VignetteBuilder: knitr
Packaged: 2026-05-20 17:57:41 UTC; JernejJ
Author: Jernej Jevsenak [aut, cre]
Maintainer: Jernej Jevsenak <jernej.jevsenak@gmail.com>
Date/Publication: 2026-05-20 18:30:02 UTC
Built: R 4.5.2; ; 2026-05-25 02:10:33 UTC; unix

export(daily_response)
export(plot_extreme)
export(plot_heatmap)
export(critical_r)
export(count_ones)
export(years_to_rownames)
export(calculate_metrics)
export(compare_methods)
export(smooth_matrix)
export(round_df)
export(boot_f_cor)
export(boot_f_pcor)
export(boot_f_lm)
export(boot_f_brnn)
export(round2)
export(glimpse_daily_data)
export(data_transform)
export(monthly_response)
export(daily_response_seascorr)
export(monthly_response_seascorr)

S3method(summary, "dmrs")
S3method(plot, "dmrs")

import(ggplot2)
import(viridis)
importFrom("brnn", "brnn", "predict.brnn")
importFrom("reshape2", "melt", "dcast")
importFrom("scales", "pretty_breaks")
importFrom("stats", "cor", "lm", "qt", "quantile", "predict", "sd", "as.formula", "princomp", "model.matrix", "median", "step", "cor.test", "loess.smooth", "residuals")
importFrom("utils", "capture.output", "setTxtProgressBar", "txtProgressBar")
importFrom("graphics", "plot")
importFrom("MLmetrics", "RRSE", "RMSE")
importFrom("Cubist", "cubist", "cubistControl")
importFrom("dplyr", "select", "desc", "summarise", "filter", "group_by")
importFrom("oce", "rescale")
importFrom("knitr", "kable")
importFrom("magrittr", "%>%")
importFrom("plotly", "plot_ly", "add_surface", "layout", "subplot", "plotly_build", "colorbar", "add_trace")
importFrom("randomForest", "randomForest")
importFrom("lubridate", "yday", "year", "ymd", "month", "ydm", "mdy", "myd", "dmy", "dym")
importFrom("psych", "partial.r", "corr.p")
importFrom("boot", "boot", "boot.ci")
importFrom("dplR", "detrend")

## ----echo = FALSE, warning = FALSE--------------------------------------------
library(knitr)
dt <- data.frame(Element = c("$mean_std", "$std_ranks", "$edge_results", "$holdout_results", "$bias_cal", "$bias_val", "$transfer_functions", "$transfer_functions_together", "$parameter_values", "$PCA_output", "$reconstructions", "$reconstructions_together", "$normal_QQ_cal", "$normal_QQ_holdout", "$normal_QQ_edge", "$residuals_vs_fitted_cal", "$residuals_vs_fitted_holdout", "$residuals_vs_fitted_edge"), 
                 Element_description = c("data frame with calculated metrics for the selected regression methods. For each regression method and each calculated metric, mean and standard deviation are given", "data frame with ranks of calculated metrics: mean rank and  share of rank_1 are given", "data frame with calculated performance metrics for the central-edge test. The central part of the data represents the calibration data, while the edge data, i.e. extreme values, represent the validation data. Different regression models are calibrated using the central data and validated for the edge (extreme) data. This test is particularly important to assess the performance of models for the prediction of the extreme data. The share of the edge (extreme) data is defined with the edge_share argument", "calculated metrics for the holdout data", "ggplot object of mean bias for calibration data", "ggplot object of mean bias for validation data", "ggplot or plotly object with transfer functions of different methods, facet is used to separate methods", "ggplot or plotly object with transfer functions of methods plotted together", "a data frame with specifications of parameters used for different regression methods", "princomp object: the result output of the PCA analysis", "ggplot object: reconstructed dependent variable based on the dataset_complete argument, facet is used to split plots by methods", "ggplot object: reconstructed dependent variable based on the dataset_complete argument, all reconstructions are on the same plot", "normal q-q plot for calibration data", "normal q-q plot for holdout data", "normal q-q plot for edge data", "residuals vs fitted values plot for calibration data", "residuals vs fitted values plot for holdout data", "residuals vs fitted values plot for edge data"))
kable(dt, "html")


## ----results = 'hide', warning=FALSE------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load the data
data(dataset_MVA)

# Basic example
basic_example <- compare_methods(formula = T_Apr ~ MVA, dataset = dataset_MVA, k = 10, repeats = 1, optimize = TRUE, MT_committees_vector = c(1), RF_maxnodes_vector = c(5), RF_nodesize_vector = c(10))

## -----------------------------------------------------------------------------
# The data frame with mean and standard deviation of performance metrics for the calibration and the validation data
kable(basic_example$mean_std)

## -----------------------------------------------------------------------------
# The data frame with non-parametric estimation of different methods: average rank and share of rank one
kable(basic_example$rank)

## ----fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("Histogram for the validation data for the basic_example")----
# See the histogram of mean bias for the validation data
basic_example$bias_val

## ----fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("Histogram for the calibration data for the basic_example")----
# See the histogram of mean bias for the calibration data
basic_example$bias_cal

## ----fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The transfer functions of different methods, facet is used to separate plots by method.")----
# See the transfer functions, separated by facets. This is a ggplot object and could be easily customized. 
library(ggplot2)
basic_example$transfer_functions +   
  xlab(expression(paste('MVA [',mm^2,']'))) +
  ylab("April Mean Temperature [°C]")

## ----fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The transfer functions of different methods, all functions are on the same plot, therefore it is easy to see the differences among different methods.")----
# See the transfer functions, plotted together. This is a ggplot object and could be easily customized. 
basic_example$transfer_functions_together +   
  xlab(expression(paste('MVA [',mm^2,']'))) +
  ylab("April Mean Temperature [°C]")

## -----------------------------------------------------------------------------
# The data frame of optimized tuning parameters for different methods
kable(basic_example$parameter_values)

## ----rb, fig.align='center', warning=FALSE, fig.width=8, fig.height=10, fig.cap=paste("Residual diagnostic plots for calibration data: Normal Q-Q plot")----
# For calibration data, there are residual diagnostic plots available. Similar plots are available also for holdout and edge data. 

basic_example$normal_QQ_cal

## ----r, fig.align='center', warning=FALSE, fig.width=8, fig.height=10, fig.cap=paste("Residual diagnostic plots for calibration data: residuals vs fitted plot")----
# For calibration data, there are residual diagnostic plots available. Similar plots are available also for holdout and edge data. 

basic_example$residuals_vs_fitted_cal

## ----results = 'hide', warning=FALSE------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load data
data(dataset_MVA_individual)

# Example PCA
example_PCA <- compare_methods(formula = T_Apr ~ ., dataset = dataset_MVA_individual, k = 5, repeats = 1, optimize = TRUE, methods = c("MLR", "MT", "BRNN"), PCA_transformation = TRUE, components_selection = "manual", N_components = 2, seed_factor = 5, MT_committees_vector = c(1), RF_maxnodes_vector = c(5), RF_nodesize_vector = c(10))

## -----------------------------------------------------------------------------
# Get the summary statistics for the PCA
summary(example_PCA$PCA_output)

## -----------------------------------------------------------------------------
# The mean and standard deviation data frame 
kable(example_PCA$mean_std)

## ----results = 'hide'---------------------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load data
data(example_dataset_1)

# Example multiproxy
example_multiproxy <- compare_methods(formula = MVA ~ T_APR + T_aug_sep, dataset = example_dataset_1, k = 10, repeats = 1, optimize = FALSE, methods = c("MT", "BRNN", "RF"))

## -----------------------------------------------------------------------------
# The mean and standard deviation data frame 
kable(example_multiproxy$mean_std)

## ----results = 'hide'---------------------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load the data
data(dataset_TRW)
data(dataset_TRW_complete)

# Example reconstruction
example_reconstruction <- compare_methods(formula = T_Jun_Jul ~ TRW, dataset = dataset_TRW, k = 3, optimize = FALSE, methods = c("MLR", "BRNN", "MT", "RF"), dataset_complete = dataset_TRW_complete)

## ----fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The reconstructed June-July temperatures based on the dataset_complete argument, facet is used to split plots by methods.")----
example_reconstruction$reconstructions

## ----fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The reconstructed June-July temperatures based on the dataset_complete argument, all reconstructions are on the same plot. The RF model reconstructed temperatures with much lower variance than the MLR model.")----
example_reconstruction$reconstructions_together

## -----------------------------------------------------------------------------
# The central-edge test
kable(example_reconstruction$edge_results)

## ----echo = FALSE, warning = FALSE--------------------------------------------
library(knitr)
dt <- data.frame(Method = c("BRNN", "MT", "MT", "MT", "MT","MT" , "MT", "RF", "RF", "RF", "RF"),
                 Parameter = c("BRNN_neurons", "MT_committees", "MT_neighbors", "MT_rules", "MT_unbiased",
                "MT_extrapolation", "MT_sample", "RF_mtry", "RF_maxnodes", "RF_ntree", "RF_nodesize"), Vector_for_optimization = c("BRNN_neurons_vector", "MT_committees_vector", "MT_neighbors_vector", "MT_rules_vector", "MT_unbiased_vector",
                "MT_extrapolation_vector", "MT_sample_vector", "RF_mtry_vector", "RF_maxnodes_vector", "RF_ntree_vector", "RF_nodesize_vector"))
kable(dt, "html")


## ----results = 'hide'---------------------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load the data
data(example_dataset_1)

example_optimize <- compare_methods(formula = MVA ~  T_APR, dataset = example_dataset_1, k = 5, repeats = 2, optimize = FALSE, BRNN_neurons = 1, MT_committees = 1, MT_neighbors = 0, MT_rules = 100, MT_unbiased = FALSE, MT_extrapolation = 100, MT_sample = 0, RF_mtry = 1, RF_ntree = 100, RF_maxnodes = 20, seed_factor = 5)

## -----------------------------------------------------------------------------
# The data frame of tuning parameters, as defined by the user
kable(example_optimize$parameter_values)

## ----results='hide', message=FALSE, warning=FALSE-----------------------------
# Load packages
library(dendroTools)
library(ggplot2)

# Load example data
data(data_MVA)
data(LJ_daily_temperatures)

# Run daily_response()
example_basic <- daily_response(response = data_MVA,
                                env_data = LJ_daily_temperatures, 
                                row_names_subset = TRUE,
                                lower_limit = 35, upper_limit = 45, 
                                remove_insignificant = FALSE,
                                previous_year = FALSE, 
                                reference_window = "end")

## ----fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 1: Default dendroTools plot for daily_response() output."), message=FALSE, warning=FALSE, include=TRUE----
plot(example_basic)

## ----fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 2: The same plot modified with ggplot2 layers (scale + theme)."), message=FALSE, warning=FALSE, include=TRUE----
plot(example_basic) +
  scale_fill_gradient2(
    name = "cor",
    low = "blue",
    mid = "white",
    high = "red",
    na.value = "white",
    limits = c(-1, 1) # select min-max here
  ) +
  theme_minimal() +
  theme(panel.background = element_blank(),
        plot.background = element_blank(),
        plot.title = element_blank(),
        legend.position = "bottom"
        )

## ----fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 3: Example with modified labels and rotated x-axis text."), message=FALSE, warning=FALSE, include=TRUE----
plot(example_basic) +
  scale_fill_gradient2(
    name = "Correlation",
    low = "blue",
    mid = "white",
    high = "red",
    na.value = "white",
    limits = c(-1, 1)
  ) +
  labs(x = "Season end (DOY)",
       y = "Season length (days)") +
  theme_bw() +
  theme(legend.position = "bottom",
        axis.text.x = element_text(angle = 45, hjust = 1))

## ----results='hide', message=FALSE, warning=FALSE-----------------------------
# Extract calculations (correlation table) from the dmrs object
cor_mat <- example_basic$calculations

# Convert matrix-like object to long format using base R
melted <- as.data.frame(as.table(as.matrix(cor_mat)))
colnames(melted) <- c("season_length", "season_end", "value")

# Convert labels such as "X35" into numeric values (if present)
melted$season_end  <- as.numeric(gsub("X", "", melted$season_end))
melted$season_length <- as.numeric(gsub("X", "", melted$season_length))

# Remove NA values (if any)
melted <- melted[!is.na(melted$value), ]

summary(melted)

## ----fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 4: Heatmap created from scratch using ggplot2 and extracted calculations."), message=FALSE, warning=FALSE, include=TRUE----
ggplot(melted, aes(x = season_end, y = season_length, fill = value)) + 
  geom_tile() +
  scale_y_continuous(expand = c(0, 0)) +
  scale_x_continuous(expand = c(0, 0)) +
  scale_fill_gradient2(
    name = "cor",
    low = "blue",
    mid = "white",
    high = "red",
    na.value = "white",
    limits = c(-1, 1)
  ) +
  xlab("Season end") +
  ylab("Season Length") +
  theme_bw() +
  theme(legend.position = "bottom")

## ----fig.align='center', fig.width=8, fig.height=8, fig.cap=paste("Figure 1: Glimpse of daily temperautres for swit272 site."), message=FALSE, warning=FALSE, include=TRUE----
# Load the dendroTools and ggplot2 R packages
library(dendroTools)
library(ggplot2)

# 1 Load an example data (source: E-OBS)
data("swit272_daily_temperatures")
data("swit272_daily_precipitation")

# 2 Transform data into wide format
swit272_daily_temperatures <- data_transform(swit272_daily_temperatures, format = 'daily', date_format = 'ymd')
swit272_daily_precipitation <- data_transform(swit272_daily_precipitation, format = 'daily', date_format = 'ymd')

# 3 Glimpse daily data
glimpse_daily_data(env_data = swit272_daily_temperatures, na.color = "red") + 
  theme(legend.position = "bottom")

## ----fig.align='center', fig.width=8, fig.height=8, fig.cap=paste("Figure 2: Glimpse of daily precipitation for swit272 site."), message=FALSE, warning=FALSE, include=TRUE----

glimpse_daily_data(env_data = swit272_daily_precipitation, na.color = "red") + 
  theme(legend.position = "bottom")


## ----results = 'hide'---------------------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_MVA)
data(LJ_daily_temperatures)

# Example with fixed width
example_fixed_width <- daily_response(response = data_MVA, env_data = LJ_daily_temperatures,
                                   method = "cor", fixed_width = 60,
                                   row_names_subset = TRUE, remove_insignificant = TRUE,
                                   alpha = 0.05)


## ----fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 3: The MVA parameter contains the optimal temperature signal from March 14 (DOY 73) to May 12 (DOY 132).")----

summary(example_fixed_width)

plot(example_fixed_width, type = 1)


## ----results = 'hide'---------------------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_TRW_1)
data(LJ_daily_temperatures)

# Example negative correlations
data(data_TRW_1)
example_restricted_interval <- daily_response(response = data_TRW_1, env_data = LJ_daily_temperatures,
                                   method = "cor", lower_limit = 55, upper_limit = 65,
                                   previous_year = FALSE, row_names_subset = TRUE, remove_insignificant = TRUE,
                                   alpha = 0.05, day_interval = c(150, 210))

## ----fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 6: Climate-growth correlations calculated only for a subset of time window from DOY 150 to DOY 210"), message=FALSE, warning=FALSE, include=TRUE----

plot(example_restricted_interval, type = 2)


## -----------------------------------------------------------------------------
# Load the dendroTools and brnn R package
library(dendroTools)

# Example of multiproxy analysis
data(example_proxies_1)
data(LJ_daily_temperatures)

# Summary of the example_proxies_1 data frame
summary(example_proxies_1)

## -----------------------------------------------------------------------------
cor(example_proxies_1)

## ----results = 'hide'---------------------------------------------------------
example_multiproxy <- daily_response(response = example_proxies_1, 
                                     env_data = LJ_daily_temperatures, 
                                     method = "lm", metric = "adj.r.squared", 
                                     lower_limit = 63, upper_limit = 67,
                                     skip_window_position = 10,
                                     row_names_subset = TRUE, previous_year = FALSE, 
                                     remove_insignificant = TRUE, alpha = 0.05)

## ----fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 7: The temporal pattern of adjusted r-squared for the multiproxy example. The temporal pattern shows similar result than for the example with MVA only. Therefore, the highest percentage of (adjusted) explained variance is most likely related to MVA variable."), message=FALSE, warning=FALSE, include=TRUE----
plot(example_multiproxy, type = 2)

## ----results = 'hide'---------------------------------------------------------
# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_TRW)
data(KRE_daily_temperatures)

example_reconstruction_lin <- daily_response(response = data_TRW, 
                                             env_data = KRE_daily_temperatures, 
                                             method = "lm", metric = "r.squared", 
                                             lower_limit = 42, upper_limit = 42,
                                             row_names_subset = TRUE, 
                                             temporal_stability_check = "progressive",
                                             cross_validation_type = "randomized", k = 3)

## ----fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 9: The highest r squared was calculated for the period from May 15 to June 27. The aggregated (averaged) daily data for this period is saved in a data frame $optimized_return."), message=FALSE, warning=FALSE, include=TRUE----
plot(example_reconstruction_lin)

## -----------------------------------------------------------------------------
example_reconstruction_lin$temporal_stability
example_reconstruction_lin$cross_validation

## ----fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 10: Linear transfer function"), message=FALSE, warning=FALSE, include=TRUE----
example_reconstruction_lin$transfer_function

## -----------------------------------------------------------------------------
linear_model <- lm(Optimized_return ~ TRW, data = example_reconstruction_lin$optimized_return)
reconstruction <- data.frame(predictions = predict(linear_model, newdata = data_TRW))

## ----fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 11: The reconstructed average temperature May 15 - June 27 with linear model"), message=FALSE, warning=FALSE, include=TRUE----
plot(row.names(data_TRW), reconstruction$predictions, type = "l", xlab = "Year", ylab = "Mean temperature May 15 - Jun 27 [ºC]")

## ----results = 'hide'---------------------------------------------------------

# Example with precipitation and temperatures
partial_cor <- daily_response_seascorr(response = swit272,
                    env_data_primary = swit272_daily_temperatures,
                    env_data_control = swit272_daily_precipitation,
                    row_names_subset = TRUE, fixed_width = 45,
                    remove_insignificant = TRUE, alpha = 0.05, 
                    aggregate_function_env_data_primary = 'mean',
                    aggregate_function_env_data_control = 'sum',
                    pcor_method = "spearman",
                    skip_window_position = 10,
                    boot = FALSE, reference_window = "end")


## ----fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 12: Temporal partern of partial climate-growth correlations calculated with daily_response_seascorr()"), message=FALSE, warning=FALSE, include=TRUE----
summary(partial_cor)
plot(partial_cor, type = 2)

#' smooth_matrix
#'
#' Removes unrealistic values in a matrix and replaces them with mean of
#' values in a window 3 x 3 around the unrealistic value. Unrealistic value is
#' determined by a factor_drop.
#'
#' @param mat a matrix or data.frame
#' @param factor_drop a number that specifies by how many % should a value drop,
#' comparing to two the closest values in a row (i +1 and i -1), to be
#' considered as a unrealistic value.
#' @param repeats an integer that specifies number of repeats of smoothing.
#' Important when there are more unrealistic values one by another.
#'
#' @return a matrix with replaced unrealistic values
#'
#'
#'
#' @examples
#' \donttest{
#' library(dendroTools)
#' data(LJ_daily_temperatures)
#' data(example_proxies_1)
#' Example1 <- daily_response(response = example_proxies_1,
#' env_data = LJ_daily_temperatures, method = "brnn",
#' metric = "r.squared", lower = 250, upper = 251,
#' previous_year = FALSE, brnn_smooth = TRUE, alpha = 0.1,
#' row_names_subset = TRUE)
#' smoothed <- smooth_matrix(mat = Example1[[1]])
#'
#' mat_1 <-  matrix(seq(1.01, 2, by = 0.01)  , ncol = 10, byrow = TRUE)
#' mat_1[5 ,5] <- -1
#' mat_2 <- smooth_matrix(mat_1)
#' }
#' @keywords internal

smooth_matrix <- function(mat, factor_drop = 0.7, repeats = 3){
for (r in 1:repeats){
  for (k in 1:nrow(mat)){

    for (l in 1:(ncol(mat))){

      temp_x <- mean(as.numeric(mat[k, max(1, l - 1):
          min(ncol(mat), l + 1)]), na.rm = TRUE)
      factor <- temp_x * factor_drop
      if (is.na(temp_x) == FALSE) {
        if (is.na(mat[k, l]) == FALSE) {
          if (mat[k, l] < factor) {
            mat[k, l] <- NA
            mat[k, l] <-
              mean(as.matrix(mat[max(1, k - 1):
                  min(nrow(mat), k + 1), max(1, l - 1):
                  min(ncol(mat), l + 1)]), na.rm = T)
          }
        }
      }
    }
  }
}
  return(as.matrix(mat))
}

#' Example of dataset with individual chronologies of MVA and mean April temperature
#'
#' A dataset of individual tree-ring chronologies from a lowland
#' forest in Slovenia. The first row represents a value of a year in 2015.
#'
#' @format A data frame with 56 rows and 54 columns :
#' \describe{
#'   \item{T_Apr}{mean April temperature for Ljubljana}
#'   \item{MVA_1}{Mean vessel area chronology for tree 1}
#'   \item{MVA_2}{Mean vessel area chronology for tree 2 [mm^2]}
#'   \item{MVA_3}{Mean vessel area chronology for tree 3 [mm^2]}
#'   \item{MVA_4}{Mean vessel area chronology for tree 4 [mm^2]}
#'   \item{MVA_5}{Mean vessel area chronology for tree 5 [mm^2]}
#'   \item{MVA_6}{Mean vessel area chronology for tree 6 [mm^2]}
#'   \item{MVA_7}{Mean vessel area chronology for tree 7 [mm^2]}
#'   \item{MVA_8}{Mean vessel area chronology for tree 8 [mm^2]}
#'   \item{MVA_9}{Mean vessel area chronology for tree 9 [mm^2]}
#'   \item{MVA_10}{Mean vessel area chronology for tree 10 [mm^2]}
#' }
#' @source Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"dataset_MVA_individual"

#' Example of dataset with individual chronologies of MVA.
#'
#' A dataset of individual tree-ring chronologies from a lowland
#' forest in Slovenia. The first row represents a value of a year in 2015.
#'
#' @format A data frame with 56 rows and 54 columns :
#' \describe{
#'   \item{MVA_1}{Mean vessel area chronology for tree 1}
#'   \item{MVA_2}{Mean vessel area chronology for tree 2}
#'   \item{MVA_3}{Mean vessel area chronology for tree 3}
#'   \item{MVA_4}{Mean vessel area chronology for tree 4}
#'   \item{MVA_5}{Mean vessel area chronology for tree 5}
#'   \item{MVA_6}{Mean vessel area chronology for tree 6}
#'   \item{MVA_7}{Mean vessel area chronology for tree 7}
#'   \item{MVA_8}{Mean vessel area chronology for tree 8}
#'   \item{MVA_9}{Mean vessel area chronology for tree 9}
#'   \item{MVA_10}{Mean vessel area chronology for tree 10}
#' }
#' @source Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"example_proxies_individual"

#' Tree-ring width (TRW) data from 2012 - 1961
#'
#' A dataset of tree-ring widths (TRW) from a site in Krakovo forest (Slovenia).
#' The first row represents a value of a year in 1961.
#'
#' @format A data frame with 52 rows and 1 variable:
#' \describe{
#'   \item{TRW}{Standardized tree-ring width indices from 2012 - 1961}
#' }
#' @source Tom Levanič, Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"data_TRW_1"

#'
#'
#' Function returns a data frame with row names as years
#'
#' @param data a data frame to be manipulated
#' @param column_year string specifying a column with years
#'
#' @return a data frame with years as row names
#'
#' @export
#'
#' @examples
#' data <- data.frame(years = seq(1950, 2015), observations = rnorm(66))
#' new_data <- years_to_rownames(data = data, column_year = "years")
#'
#' data <- data.frame(observations1 = rnorm(66), years = seq(1950, 2015),
#' observations2 = rnorm(66), observations3 = rnorm(66))
#' new_data <- years_to_rownames(data = data, column_year = "years")

years_to_rownames <- function(data, column_year) {
  data <- data.frame(data) # Data needs to of class data.frame!

  year_index <- grep(column_year, colnames(data))

  names <- colnames(data)
  names <- names[-year_index]

  row.names(data) <- data[, year_index]
  data <- as.data.frame(data[, -as.numeric(year_index), F])

  colnames(data) <- names

  return(data)
}

#' compare_methods
#'
#' Calculates performance metrics for calibration (train) and validation (test)
#' data of different regression methods: multiple linear regression (MLR),
#' artificial neural networks with Bayesian regularization training
#' algorithm (BRNN), (ensemble of) model trees (MT) and random forest of regression
#' trees (RF). With the subset argument, specific methods of interest could be
#' specified. Calculated performance metrics are the correlation coefficient (r),
#' the root mean squared error (RMSE), the root relative squared error (RRSE),
#' the index of agreement (d), the reduction of error (RE), the coefficient of
#' efficiency (CE), the detrended efficiency (DE) and mean bias. For each of the
#' considered methods, there are also residual diagnostic plots available,
#' separately for calibration, holdout and edge data, if applicable.
#'
#' @param formula an object of class "formula" (or one that can be coerced
#' to that class): a symbolic description of the model to be fitted.
#' @param dataset a data frame with dependent and independent variables as
#' columns and (optional) years as row names.
#' @param k number of folds for cross-validation
#' @param repeats number of cross-validation repeats. Should be equal or more
#' than 1
#' @param optimize if set to TRUE (default), the optimal values for the tuning
#' parameters will be selected in a preliminary cross-validation procedure
#' @param dataset_complete optional, a data frame with the full length of tree-ring
#' parameter, which will be used to reconstruct the climate variable specified
#' with the formula argument
#' @param BRNN_neurons number of neurons to be used for the brnn method
#' @param MT_committees an integer:  how many committee models (e.g.  boosting
#' iterations) should be used?
#' @param MT_neighbors how many, if any, neighbors should be used to correct the
#' model predictions
#' @param MT_rules an integer (or NA): define an explicit limit to the number of
#' rules used (NA let’s Cubist decide).
#' @param MT_unbiased a logical: should unbiased rules be used?
#' @param MT_extrapolation a number between 0 and 100: since Cubist uses linear models,
#' predictions can be outside of the outside of the range seen the training set. This
#' parameter controls how much rule predictions are adjusted to be consistent with the
#' training set.
#' @param MT_sample a number between 0 and 99.9:  this is the percentage of the dataset
#' to be randomly selected for model building (not for out-of-bag type evaluation)
#' @param RF_mtry number of variables randomly sampled as candidates at each
#' split
#' @param RF_ntree number of trees to grow. This should not be set to too small
#' a number, to ensure that every input row gets predicted at least a few times
#' @param RF_maxnodes maximum number of terminal nodes trees in the forest can
#' have
#' @param RF_nodesize minimum size of terminal nodes. Setting this number larger
#' causes smaller trees to be grown (and thus take less time).
#' @param seed_factor an integer that will be used to change the seed options
#' for different repeats.
#' @param digits integer of number of digits to be displayed in the final
#' result tables
#' @param edge_share the share of the data to be considered as the edge (extreme) data.
#' This argument could be between 0.10 and 0.50. If the argument is set to 0.10, then
#' the 5 % of the maximal extreme values and 5 % of the minimal extreme values are
#' considered to be the edge data.
#' @param blocked_CV default is FALSE, if changed to TRUE, blocked cross-validation
#' will be used to compare regression methods.
#' @param MLR_stepwise if set to TRUE, stepwise selection of predictors will be used
#' for the MLR method
#' @param stepwise_direction the mode of stepwise search, can be one of "both",
#' "backward", or "forward", with a default of "backward".
#' @param PCA_transformation if set to TRUE, all independent variables will be
#' transformed using PCA transformation.
#' @param log_preprocess if set to TRUE, variables will be transformed with
#' logarithmic transformation before used in PCA
#' @param components_selection character string specifying how to select the Principal
#' Components used as predictors.
#' There are three options: "automatic", "manual" and "plot_selection". If
#' parameter is set to automatic, all scores with eigenvalues above 1 will be
#' selected. This threshold could be changed by changing the
#' eigenvalues_threshold argument. If parameter is set to "manual", user should
#' set the number of components with N_components argument. If component
#' selection is se to "plot_selection", Scree plot will be shown and user must
#' manually enter the number of components used as predictors.
#' @param eigenvalues_threshold threshold for automatic selection of Principal Components
#' @param N_components number of Principal Components used as predictors
#' @param round_bias_cal number of digits for bias in calibration period. Effects
#' the outlook of the final ggplot  of mean bias for calibration data (element 3 of
#' the output list)
#' @param round_bias_val number of digits for bias in validation period. Effects
#' the outlook of the final ggplot of mean bias for validation data (element 4 of
#' the output list)
#' @param n_bins number of bins used for the histograms of mean bias
#' @param methods a vector of strings related to methods that will be compared. A full
#' method vector is methods = c("MLR", "BRNN", "MT", "RF").
#' To use only a subset of methods, pass a vector of methods that you would like to compare.
#' @param tuning_metric a string that specifies what summary metric will be used to select
#' the optimal value of tuning parameters. By default, the argument is set to "RMSE". It is
#' also possible to use "RSquared".
#' @param BRNN_neurons_vector a vector of possible values for BRNN_neurons argument optimization
#' @param MT_committees_vector a vector of possible values for MT_committees argument optimization
#' @param MT_neighbors_vector a vector of possible values for MT_neighbors argument optimization
#' @param MT_rules_vector a vector of possible values for MT_rules argument optimization
#' @param MT_unbiased_vector a vector of possible values for MT_unbiased argument optimization
#' @param MT_extrapolation_vector a vector of possible values for MT_extrapolation argument optimization
#' @param MT_sample_vector a vector of possible values for MT_sample argument optimization
#' @param RF_ntree_vector a vector of possible values for RF_ntree argument optimization
#' @param RF_maxnodes_vector a vector of possible values for RF_maxnodes argument optimization
#' @param RF_mtry_vector a vector of possible values for RF_mtry argument optimization
#' @param RF_nodesize_vector a vector of possible values for RF_nodesize argument optimization
#' @param holdout this argument is used to define observations, which are excluded
#' from the cross-validation and hyperparameters optimization. The holdout argument must be
#' a character with one of the following inputs: “early”, “late” or “manual”. If
#' "early" or "late" characters are specified, then the early or late years will be
#' used as a holdout data. How many of the "early" or "late" years are used as a holdout
#' is specified with the argument holdout_share. If the argument holdout is set to “manual”,
#' then supply a vector of years (or row names) to the argument holdout_manual. Defined
#' years will be used as a holdout. For the holdout data, the same statistical measures are
#' calculated as for the cross-validation. The results for holdout metrics are given in the
#' output element $holdout_results.
#' @param holdout_share the share of the whole dataset to be used as a holdout.
#' Default is 0.10.
#' @param holdout_manual a vector of years (or row names) which will be used as a holdout.
#' calculated as for the cross-validation.
#' @param total_reproducibility logical, default is FALSE. This argument ensures total
#' reproducibility despite the inclusion/exclusion of different methods. By default, the
#' optimization is done only for the methods, that are included in the methods vector. If
#' one method is absent or added, the optimization phase is different, and this affects
#' all the final cross-validation results. By setting the total_reproducibility = TRUE,
#' all methods will be optimized, even though they are not included in the methods vector
#' and the final results will be subset based on the methods vector. Setting the
#' total_reproducibility to TRUE will result in longer optimization phase as well.
#'
#' @return a list with 19 elements:
#' \enumerate{
#'  \item $mean_std - data frame with calculated metrics for the selected \\ regression methods. For each regression method and each calculated metric, mean and standard deviation are given
#'  \item $ranks - data frame with ranks of calculated metrics: mean rank and  share of rank_1 are given
#'  \item $edge_results - data frame with calculated performance metrics for the central-edge test. The central part of the data represents the calibration data, while the edge data, i.e. extreme values, represent the test/validation data. Different regression models are calibrated using the central data and validated for the edge (extreme) data. This test is particularly important to assess the performance of models for the predictions of the extreme data. The share of the edge (extreme) data is defined with the edge_share argument
#'  \item $holdout_results - calculated metrics for the holdout data
#'  \item $bias_cal - ggplot object of mean bias for calibration data
#'  \item $bias_val - ggplot object of mean bias for validation data
#'  \item $transfer_functions - ggplot or plotly object with transfer functions of methods
#'  \item $transfer_functions_together - ggplot or plotly object with transfer functions of methods plotted together
#'  \item $parameter_values - a data frame with specifications of parameters used for different regression methods
#'  \item $PCA_output - princomp object: the result output of the PCA analysis
#'  \item $reconstructions - ggplot object: reconstructed dependent variable based on the dataset_complete argument, facet is used to split plots by methods
#'  \item $reconstructions_together - ggplot object: reconstructed dependent variable based on the dataset_complete argument, all reconstructions are on the same plot
#'  \item $normal_QQ_cal - normal q-q plot for calibration data
#'  \item $normal_QQ_holdout - normal q-q plot for holdout data
#'  \item $normal_QQ_edge- normal q-q plot for edge data
#'  \item $residuals_vs_fitted_cal - residuals vs fitted values plot for calibration data
#'  \item $residuals_vs_fitted_holdout - residuals vs fitted values plot for holdout data
#'  \item $residuals_vs_fitted_edge - residuals vs fitted values plot for edge data
#'  \item $reconstructions_data - raw data that is used for creating reconstruction plots
#'}
#'
#' @export
#'
#' @references
#' Bishop, C.M., 1995. Neural Networks for Pattern Recognition. Oxford
#' University Press, Inc. 482 pp.
#'
#' Breiman, L., 1996. Bagging predictors. Machine Learning 24, 123-140.
#'
#' Breiman, L., 2001. Random forests. Machine Learning 45, 5-32.
#'
#' Burden, F., Winkler, D., 2008. Bayesian Regularization of Neural Networks,
#' in: Livingstone, D.J. (ed.), Artificial Neural Networks: Methods and
#' Applications, vol. 458. Humana Press, Totowa, NJ, pp. 23-42.
#'
#' Hastie, T., Tibshirani, R., Friedman, J.H., 2009. The Elements of
#' Statistical Learning : Data Mining, Inference, and Prediction, 2nd ed.
#' Springer, New York xxii, 745 p. pp.
#'
#' Ho, T.K., 1995. Random decision forests, Proceedings of the Third
#' International Conference on Document Analysis and Recognition Volume 1.
#' IEEE Computer Society, pp. 278-282.
#'
#' Hornik, K., Buchta, C., Zeileis, A., 2009. Open-source machine learning: R
#' meets Weka. Comput. Stat. 24, 225-232.
#'
#' Perez-Rodriguez, P., Gianola, D., 2016. Brnn: Brnn (Bayesian Regularization
#' for Feed-forward Neural Networks). R package version 0.6.
#'
#' Quinlan, J.R., 1992. Learning with Continuous Classes, Proceedings of the
#' 5th Australian Joint Conference on Artificial Intelligence (AI '92). World
#' Scientific, Hobart, pp. 343-348.
#'
#' @examples
#' \donttest{
#'
#' # The examples below are enclosed within donttest{} to minimize the execution
#' # time during R package checks. #'
#'
#' # An example with default settings of machine learning algorithms
#' library(dendroTools)
#' library(ggplot2)
#'
#' data(example_dataset_1)
#' data(dataset_TRW)
#'
#' example_1 <- compare_methods(formula = MVA ~  T_APR,
#' dataset = example_dataset_1, k = 5, repeats = 1, BRNN_neurons = 1,
#' RF_ntree = 100, RF_mtry = 2, RF_maxnodes = 35, seed_factor = 5)
#'
#' # example_1$mean_std
#' # example_1$ranks
#' # example_1$bias_cal
#' # example_1$transfer_functions
#' # example_1$transfer_functions_together
#' # example_1$PCA_output
#' # example_1$parameter_values
#'
#' example_2 <- compare_methods(formula = MVA ~ .,
#' dataset = example_dataset_1, k = 2, repeats = 2,
#' methods = c("MLR", "BRNN", "MT"),
#' optimize = TRUE, MLR_stepwise = TRUE)
#' # example_2$mean_std
#' # example_2$ranks
#' # example_2$bias_val
#' # example_2$transfer_functions
#' # example_2$transfer_functions_together
#' # example_2$parameter_values
#'
#' comparison_TRW <- compare_methods(formula = T_Jun_Jul ~ TRW, dataset = dataset_TRW,
#' k = 3, repeats = 5, optimize = FALSE, methods = c("MLR", "BRNN", "RF", "MT"),
#' seed_factor = 5, dataset_complete = dataset_TRW_complete, MLR_stepwise = TRUE,
#' stepwise_direction = "backward")
#'
#' # comparison_TRW$mean_std
#' # comparison_TRW$bias_val
#' # comparison_TRW$transfer_functions
#' # comparison_TRW$reconstructions
#' # comparison_TRW$reconstructions_together
#' # comparison_TRW$edge_results
#' # comparison_TRW$reconstructions_data
#'
#' }

compare_methods <- function(formula, dataset, k = 10, repeats = 2,
                            optimize = TRUE, dataset_complete = NULL,
                            BRNN_neurons = 1, MT_committees = 1, MT_neighbors = 5,
                            MT_rules = 200, MT_unbiased = TRUE, MT_extrapolation = 100,
                            MT_sample = 0,
                            RF_ntree = 500,
                            RF_maxnodes = 5,
                            RF_mtry  = 1,
                            RF_nodesize = 1,
                            seed_factor = 5,
                            digits = 3, blocked_CV = FALSE,
                            PCA_transformation = FALSE, log_preprocess = TRUE,
                            components_selection = 'automatic',
                            eigenvalues_threshold = 1, N_components = 2,
                            round_bias_cal = 15, round_bias_val = 4,
                            n_bins = 30, edge_share = 0.10,
                            MLR_stepwise = FALSE, stepwise_direction = "backward",
                            methods = c("MLR", "BRNN", "MT", "RF"),
                            tuning_metric = "RMSE",
                            BRNN_neurons_vector = c(1, 2, 3),
                            MT_committees_vector = c(1, 5, 10),
                            MT_neighbors_vector = c(0, 5),
                            MT_rules_vector = c(100, 200),
                            MT_unbiased_vector = c(TRUE, FALSE),
                            MT_extrapolation_vector = c(100),
                            MT_sample_vector = c(0),
                            RF_ntree_vector = c(100, 250, 500),
                            RF_maxnodes_vector = c(5, 10, 20, 25),
                            RF_mtry_vector  = c(1),
                            RF_nodesize_vector = c(1, 5, 10),
                            holdout = NULL, holdout_share = 0.10,
                            holdout_manual = NULL, total_reproducibility = FALSE){

if (k < 2 | k > 26){
  stop(paste0("Selected k is ", k,", but it should be between 2 and 26"))
}

if (repeats > 1 & blocked_CV == TRUE){
  warning("blocked_CV is set to TRUE, repeats argument is ignored")
}

dataset <- data.frame(dataset) # dataset needs to be of class data.frame!

full_methods <- c("MLR", "BRNN", "MT", "RF")

if ("BMT" %in% methods){
  warning(paste0("BMT and MT from RWeka package are now replaced with cubist R function and refered as MT.",
                 "To fit ensemble model tree, use committees argument (MT_committees)."))
}

methods <- sort(methods)

set.seed(seed_factor) # We ensure that the optimization results are always the same

#############################################################################

# Here, empty lists are defined, where calculations will be stored. Empty lists
# for bias are defined separately, since bias should not be averaged. It is
# later given as density plots
list_MLR <- list()
list_BRNN <- list()
list_MT <- list()
list_RF <- list()

# Here, idex of dependent variable is extracted and later used to locate the
# observed values
DepIndex <- grep(as.character(formula[[2]]), colnames(dataset))
DepName <- as.character(formula[[2]])

# If PCA_transformation = TRUE, PCA is performed
if (PCA_transformation == TRUE) {

  # Logarithmic transformation before PCA
  if (log_preprocess == TRUE) {

    dataset_temp <- dataset[,-DepIndex]
    dataset_temp <- data.frame(log(dataset_temp))
  }

  PCA_result <- princomp(dataset_temp, cor = TRUE)

  if (components_selection == 'automatic'){
    subset_vector <- PCA_result$sdev > eigenvalues_threshold
    dataset_temp <- as.data.frame(PCA_result$scores[, subset_vector])
  }

  if (components_selection == 'manual'){
    dataset_temp <- as.data.frame(PCA_result$scores[, 1:N_components])
  }

  if (components_selection == 'plot_selection'){
    plot(PCA_result, type = 'l')

    fun <- function(){
      N_PC <- readline("What number of PC scores should be used as predictors? ")
      return(N_PC)
    }

    N_PC <- fun()
    dataset_temp <- as.data.frame(PCA_result$scores[, 1:as.numeric(N_PC)])
  }

  dataset <- data.frame(dataset[, DepIndex], dataset_temp)
  colnames(dataset)[1] = DepName
  for (i in 2:ncol(dataset)){
    colnames(dataset)[i] <- paste("PC_", i-1, sep = "")
    formula = as.formula(paste(DepName, "~ ."))
  }
} else (PCA_result <- "No PCA result avalialbe!")

# Here we fit a lm model, just to get information about the number of independent variables;
# when formula is used in the form of: y~., we don't know the number of independent variables
# this information is used later
DepIndex <- grep(as.character(formula[[2]]), colnames(dataset))
quazi_mod <- lm(formula, data = dataset)
numIND <- length(quazi_mod[[1]]) - 1
indep_names <- colnames(quazi_mod[[7]][[1]])[-1]
indep_name_1 <- colnames(quazi_mod[[7]][[1]])[-1][1]
indep_name_2 <- colnames(quazi_mod[[7]][[1]])[-1][2]
allnames <- c(as.character(formula[[2]]), indep_names)

# So, sometimes there are variables in dataframe, which are not used as predictors.
# Let's remove them
dataset <- dataset[ names(dataset)[names(dataset) %in% allnames] ]


# Here, mtry is checked. mtry should not be larger than the unmber of independet variables

if(sum(RF_mtry_vector > numIND) > 0){
critical_values <- as.numeric(which(RF_mtry_vector > numIND))
critical_values_inf <- RF_mtry_vector[critical_values]
RF_mtry_vector <- RF_mtry_vector[-critical_values]
warning(paste0("mtry argument for the RF should not be larger than the number of independent variables."),
        " Values ", paste((critical_values_inf), collapse=", "), " for the argument RF_mtry_vector are depreciated!")
}

# Here, we define stop, if edge_share is 0
if (!edge_share > 0 & numIND < 2){
  stop("edge_share should be greater than 0!")
}

# Here, if holdout data is defined, we exclude those observations from cross-validation
# and hyperparameters optimization
if (!is.null(holdout)){
  if (holdout == "early"){
  dataset <- dataset[ order(as.numeric(row.names(dataset)), decreasing = TRUE),]
  dataset_holdout = dataset[1:round((nrow(dataset)*holdout_share),0),]
  dataset = dataset[round(((nrow(dataset)*holdout_share)+1),0):nrow(dataset),]

  } else if (holdout == "late"){
    dataset <- dataset[ order(as.numeric(row.names(dataset)), decreasing = TRUE),]
    dataset_holdout = dataset[(nrow(dataset) - (round2(nrow(dataset)*holdout_share, 0)) + 1):nrow(dataset), ]
    dataset = dataset[1:((nrow(dataset) - (round2(nrow(dataset)*holdout_share, 0)))), ]

  } else if (holdout == "manual"){
    dataset_holdout = dataset[row.names(dataset) %in% holdout_manual, ]
    dataset = dataset[!row.names(dataset) %in% holdout_manual, ]

  } else {
      stop(paste0("The argument holdout is set to ", holdout,". Instead, it should be early, late or manual. ",
                 "Please, see the compare_methods() documentation."))
    }
  }


# Here we use caret package to tune parameters of different methods
# Optimization for randomized procedure

if (optimize == TRUE & blocked_CV == FALSE){

  print("Optimization phase...")

  model = NULL

  if ("BRNN" %in% methods | total_reproducibility == TRUE){

    print("Tuning parameters for the BRNN method...")
    # 2 Optimization for BRNN
    BRNN_neurons <- BRNN_neurons_vector
    hyper_grid <- expand.grid(neurons = BRNN_neurons)

    # Number of potential models in the grid
    num_models <- nrow(hyper_grid)

    # Create an empty list to store models
    grade_models <- list()

    # Write a loop over the rows of hyper_grid to train the grid of models
    for (i in 1:num_models) {

      # Get minsplit, maxdepth values at row i
      neurons <- hyper_grid$neurons[i]

      # cross_validation
      foldi <- seq(1:k)
      foldi <- paste("fold_", foldi)

      #Randomly shuffle the data
      dataset <- dataset[sample(nrow(dataset)), ]

      #Create 10 equally size folds
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      #Perform k fold cross validation
      tuning_vector <- c()

      for (j in 1:k){

        #Segement your data by fold using the which() function
        testIndexes <- which(folds == j, arr.ind = TRUE)
        test <- dataset[testIndexes, ]
        train <- dataset[-testIndexes, ]

        #MLR MODEL
        capture.output(model_temp <- brnn(formula, data = train, neurons = neurons, verbose = FALSE,
                                          tol = 1e-6))
        test_observed <- test[, DepIndex]
        test_predicted <- predict(model_temp, test)

        if (tuning_metric == "RMSE"){
          tuning_vector[j] <- MLmetrics::RMSE(test_predicted, test_observed)
        } else if (tuning_metric == "RSquared"){
          tuning_vector[j] <- cor(test_predicted, test_observed)^2
        } else {
          stop(paste0("tuning_metric argument should be RMSE or RSquared! Instead it is ", tuning_metric))
        }

      }

      grade_models[i] <- mean(tuning_vector)

    }

    grade_list <- unlist(grade_models)

    # Identify the model with smallest validation set RMSE
    if (tuning_metric == "RMSE"){
        best_model <- which.min(grade_list)}
    if (tuning_metric == "RSquared"){
      best_model <- which.max(grade_list)
      }

    best_parameters <- hyper_grid[best_model, ]

    BRNN_neurons <- as.numeric(best_parameters[1])
  }

  if ("MT" %in% methods | total_reproducibility == TRUE){

  print("Tuning parameters for the MT method...")

  # 2 Optimization for MT
    MT_committees <- MT_committees_vector
    MT_neighbors <- MT_neighbors_vector
    MT_rules <- MT_rules_vector
    MT_unbiased <- MT_unbiased_vector
    MT_extrapolation <- MT_extrapolation_vector
    MT_sample <- MT_sample_vector

    hyper_grid <- expand.grid(
    committees = MT_committees,
    neighbors = MT_neighbors,
    rules = MT_rules,
    unbiased = MT_unbiased,
    extrapolation = MT_extrapolation,
    sample = MT_sample)

  # Number of potential models in the grid
  num_models <- nrow(hyper_grid)

  # Create an empty list to store models
  grade_models <- list()

  # Write a loop over the rows of hyper_grid to train the grid of models
  for (i in 1:num_models) {

    # Get minsplit, maxdepth values at row i
    committees <- hyper_grid$committees[i]
    neighbors <- hyper_grid$neighbors[i]
    rules <- hyper_grid$rules[i]
    unbiased <- hyper_grid$unbiased[i]
    extrapolation <- hyper_grid$extrapolation[i]
    sample <- hyper_grid$sample[i]

    # cross_validation
    foldi <- seq(1:k)
    foldi <- paste("fold_", foldi)

    #Randomly shuffle the data
    dataset <- dataset[sample(nrow(dataset)), ]

    #Create 10 equally size folds
    folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

    #Perform k fold cross validation
    tuning_vector <- c()

    for (j in 1:k){

      #Segement your data by fold using the which() function
      testIndexes <- which(folds == j, arr.ind = TRUE)
      test <- dataset[testIndexes, ]
      train <- dataset[-testIndexes, ]

      #MT MODEL
      train_ind <- data.frame(train[, -DepIndex])
      colnames(train_ind) <- indep_names
      test_ind <- data.frame(test[, -DepIndex])
      colnames(test_ind) <- indep_names

      train_dep <- train[, DepIndex]
      test_dep <- test[, DepIndex]

      model_temp <- cubist(x = train_ind, y = train_dep, committees = committees,
                           neighbors = neighbors, cubistControl(rules = rules, unbiased = unbiased,
                                                                extrapolation = extrapolation,
                                                                sample = sample))

      test_observed <- test[, DepIndex]
      test_predicted <- predict(model_temp, test_ind)

      if (tuning_metric == "RMSE"){
        tuning_vector[j] <- MLmetrics::RMSE(test_predicted, test_observed)
      } else if (tuning_metric == "RSquared"){
        tuning_vector[j] <- cor(test_predicted, test_observed)^2
      } else {
        stop(paste0("tuning_metric argument should be RMSE or RSquared! Instead it is ", tuning_metric))
      }

    }

    grade_models[i] <- mean(tuning_vector)

  }

  grade_list <- unlist(grade_models)

  # Identify the model with smallest validation set RMSE
  if (tuning_metric == "RMSE"){
    best_model <- which.min(grade_list)}
  if (tuning_metric == "RSquared"){
    best_model <- which.max(grade_list)
  }

  best_parameters <- hyper_grid[best_model, ]

  MT_committees <- as.numeric(best_parameters[1])
  MT_neighbors <- as.numeric(best_parameters[2])
  MT_rules <- as.numeric(best_parameters[3])
  MT_unbiased <- as.logical(best_parameters[4])
  MT_extrapolation <- as.numeric(best_parameters[5])
  MT_sample <- as.numeric(best_parameters[6])

  }


  if ("RF" %in% methods | total_reproducibility == TRUE){
    # 2 Optimization for RF

    print("Tuning parameters for the RF method...")

    RF_mtry <- RF_mtry_vector
    RF_maxnodes <- RF_maxnodes_vector
    RF_ntree  <- RF_ntree_vector
    RF_nodesize  <- RF_nodesize_vector

    hyper_grid <- expand.grid(mtry = RF_mtry, maxnodes = RF_maxnodes,
                              ntree = RF_ntree,  nodesize = RF_nodesize)

    # Number of potential models in the grid
    num_models <- nrow(hyper_grid)

    # Create an empty list to store models
    grade_models <- list()

    # Write a loop over the rows of hyper_grid to train the grid of models
    for (i in 1:num_models) {

      # Get minsplit, maxdepth values at row i
      mtry <- hyper_grid$mtry[i]
      maxnodes <- hyper_grid$maxnodes[i]
      ntree <- hyper_grid$ntree[i]
      nodesize <- hyper_grid$nodesize[i]

      # cross_validation
      foldi <- seq(1:k)
      foldi <- paste("fold_", foldi)

      #Randomly shuffle the data
      dataset <- dataset[sample(nrow(dataset)), ]

      #Create 10 equally size folds
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      #Perform k fold cross validation
      tuning_vector <- c()

      for (j in 1:k){

        #Segement your data by fold using the which() function
        testIndexes <- which(folds == j, arr.ind = TRUE)
        test <- dataset[testIndexes, ]
        train <- dataset[-testIndexes, ]

        #MLR MODEL


        model_temp <- randomForest(formula, data = train, mtry = mtry, maxnodes = maxnodes,
                                   ntree = ntree, nodesize = nodesize)

        test_observed <- test[, DepIndex]
        test_predicted <- predict(model_temp, test)

        if (tuning_metric == "RMSE"){
          tuning_vector[j] <- MLmetrics::RMSE(test_predicted, test_observed)
        } else if (tuning_metric == "RSquared"){
          tuning_vector[j] <- cor(test_predicted, test_observed)^2
        } else {
          stop(paste0("tuning_metric argument should be RMSE or RSquared! Instead it is ", tuning_metric))
        }

      }

      grade_models[i] <- mean(tuning_vector)

    }

    grade_list <- unlist(grade_models)

    # Identify the model with smallest validation set RMSE
    if (tuning_metric == "RMSE"){
      best_model <- which.min(grade_list)}
    if (tuning_metric == "RSquared"){
      best_model <- which.max(grade_list)
    }

    best_parameters <- hyper_grid[best_model, ]

    RF_mtry <- as.numeric(best_parameters[1])
    RF_maxnodes <- as.numeric(best_parameters[2])
    RF_ntree  <- as.numeric(best_parameters[3])
    RF_nodesize  <- as.numeric(best_parameters[4])

  }

}


###############################################################################
# Optimization for blocked_CV


if (optimize == TRUE & blocked_CV == TRUE){

  print("Optimization phase...")

  model = NULL

  if ("BRNN" %in% methods | total_reproducibility == TRUE){

    print("Tuning parameters for the BRNN method...")
    # 2 Optimization for BRNN
    BRNN_neurons <- BRNN_neurons_vector
    hyper_grid <- expand.grid(neurons = BRNN_neurons)

    # Number of potential models in the grid
    num_models <- nrow(hyper_grid)

    # Create an empty list to store models
    grade_models <- list()

    # Write a loop over the rows of hyper_grid to train the grid of models
    for (i in 1:num_models) {

      # Get minsplit, maxdepth values at row i
      neurons <- hyper_grid$neurons[i]

      # cross_validation
      foldi <- seq(1:k)
      foldi <- paste("fold_", foldi)

      #Randomly shuffle the data
      # dataset <- dataset[sample(nrow(dataset)), ]

      #Create 10 equally size folds
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      #Perform k fold cross validation
      tuning_vector <- c()

      for (j in 1:k){

        #Segement your data by fold using the which() function
        testIndexes <- which(folds == j, arr.ind = TRUE)
        test <- dataset[testIndexes, ]
        train <- dataset[-testIndexes, ]

        #MLR MODEL
        capture.output(model_temp <- brnn(formula, data = train, neurons = neurons, verbose = FALSE))
        test_observed <- test[, DepIndex]
        test_predicted <- predict(model_temp, test)

        if (tuning_metric == "RMSE"){
          tuning_vector[j] <- MLmetrics::RMSE(test_predicted, test_observed)
        } else if (tuning_metric == "RSquared"){
          tuning_vector[j] <- cor(test_predicted, test_observed)^2
        } else {
          stop(paste0("tuning_metric argument should be RMSE or RSquared! Instead it is ", tuning_metric))
        }

      }

      grade_models[i] <- mean(tuning_vector)

    }

    grade_list <- unlist(grade_models)

    # Identify the model with smallest validation set RMSE
    if (tuning_metric == "RMSE"){
      best_model <- which.min(grade_list)}
    if (tuning_metric == "RSquared"){
      best_model <- which.max(grade_list)
    }

    best_parameters <- hyper_grid[best_model, ]

    BRNN_neurons <- as.numeric(best_parameters[1])
  }




  if ("MT" %in% methods | total_reproducibility == TRUE){

    print("Tuning parameters for the MT method...")

    # 2 Optimization for MT
    MT_committees <- MT_committees_vector
    MT_neighbors <- MT_neighbors_vector
    MT_rules <- MT_rules_vector
    MT_unbiased <- MT_unbiased_vector
    MT_extrapolation <- MT_extrapolation_vector
    MT_sample <- MT_sample_vector

    hyper_grid <- expand.grid(
      committees = MT_committees,
      neighbors = MT_neighbors,
      rules = MT_rules,
      unbiased = MT_unbiased,
      extrapolation = MT_extrapolation,
      sample = MT_sample)

    # Number of potential models in the grid
    num_models <- nrow(hyper_grid)

    # Create an empty list to store models
    grade_models <- list()

    # Write a loop over the rows of hyper_grid to train the grid of models
    for (i in 1:num_models) {

      # Get minsplit, maxdepth values at row i
      committees <- hyper_grid$committees[i]
      neighbors <- hyper_grid$neighbors[i]
      rules <- hyper_grid$rules[i]
      unbiased <- hyper_grid$unbiased[i]
      extrapolation <- hyper_grid$extrapolation[i]
      sample <- hyper_grid$sample[i]

      # cross_validation
      foldi <- seq(1:k)
      foldi <- paste("fold_", foldi)

      #Randomly shuffle the data
      #dataset <- dataset[sample(nrow(dataset)), ]

      #Create 10 equally size folds
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      #Perform k fold cross validation
      tuning_vector <- c()

      for (j in 1:k){

        #Segement your data by fold using the which() function
        testIndexes <- which(folds == j, arr.ind = TRUE)
        test <- dataset[testIndexes, ]
        train <- dataset[-testIndexes, ]

        train_ind <- data.frame(train[, -DepIndex])
        colnames(train_ind) <- indep_names
        test_ind <- data.frame(test[, -DepIndex])
        colnames(test_ind) <- indep_names

        train_dep <- train[, DepIndex]
        test_dep <- test[, DepIndex]

        #MT MODEL
        model_temp <- cubist(x = train_ind, y = train_dep, committees = committees,
                             neighbors = neighbors, cubistControl(rules = rules, unbiased = unbiased,
                                                                  extrapolation = extrapolation,
                                                                  sample = sample))

        test_observed <- test[, DepIndex]
        test_predicted <- predict(model_temp, test_ind)

        if (tuning_metric == "RMSE"){
          tuning_vector[j] <- MLmetrics::RMSE(test_predicted, test_observed)
        } else if (tuning_metric == "RSquared"){
          tuning_vector[j] <- cor(test_predicted, test_observed)^2
        } else {
          stop(paste0("tuning_metric argument should be RMSE or RSquared! Instead it is ", tuning_metric))
        }

      }

      grade_models[i] <- mean(tuning_vector)

    }

    grade_list <- unlist(grade_models)

    # Identify the model with smallest validation set RMSE
    if (tuning_metric == "RMSE"){
      best_model <- which.min(grade_list)}
    if (tuning_metric == "RSquared"){
      best_model <- which.max(grade_list)
    }

    best_parameters <- hyper_grid[best_model, ]

    MT_committees <- as.numeric(best_parameters[1])
    MT_neighbors <- as.numeric(best_parameters[2])
    MT_rules <- as.numeric(best_parameters[3])
    MT_unbiased <- as.logical(best_parameters[4])
    MT_extrapolation <- as.numeric(best_parameters[5])
    MT_sample <- as.numeric(best_parameters[6])

  }

  if ("RF" %in% methods | total_reproducibility == TRUE){
    # 2 Optimization for RF

    print("Tuning parameters for the RF method...")

    RF_mtry <- RF_mtry_vector
    RF_maxnodes <- RF_maxnodes_vector
    RF_ntree  <- RF_ntree_vector
    RF_nodesize  <- RF_nodesize_vector

    hyper_grid <- expand.grid(mtry = RF_mtry, maxnodes = RF_maxnodes,
                              ntree = RF_ntree,  nodesize = RF_nodesize)

    # Number of potential models in the grid
    num_models <- nrow(hyper_grid)

    # Create an empty list to store models
    grade_models <- list()

    # Write a loop over the rows of hyper_grid to train the grid of models
    for (i in 1:num_models) {

      # Get minsplit, maxdepth values at row i
      mtry <- hyper_grid$mtry[i]
      maxnodes <- hyper_grid$maxnodes[i]
      ntree <- hyper_grid$ntree[i]
      nodesize <- hyper_grid$nodesize[i]

      # cross_validation
      foldi <- seq(1:k)
      foldi <- paste("fold_", foldi)

      #Randomly shuffle the data
      #dataset <- dataset[sample(nrow(dataset)), ]

      #Create 10 equally size folds
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      #Perform k fold cross validation
      tuning_vector <- c()

      for (j in 1:k){

        #Segement your data by fold using the which() function
        testIndexes <- which(folds == j, arr.ind = TRUE)
        test <- dataset[testIndexes, ]
        train <- dataset[-testIndexes, ]

        #MLR MODEL
        model_temp <- randomForest(formula, data = train, mtry = mtry, maxnodes = maxnodes,
                                   ntree = ntree, nodesize = nodesize)

        test_observed <- test[, DepIndex]
        test_predicted <- predict(model_temp, test)

        if (tuning_metric == "RMSE"){
          tuning_vector[j] <- MLmetrics::RMSE(test_predicted, test_observed)
        } else if (tuning_metric == "RSquared"){
          tuning_vector[j] <- cor(test_predicted, test_observed)^2
        } else {
          stop(paste0("tuning_metric argument should be RMSE or RSquared! Instead it is ", tuning_metric))
        }

      }

      grade_models[i] <- mean(tuning_vector)

    }

    grade_list <- unlist(grade_models)

    # Identify the model with smallest validation set RMSE
    if (tuning_metric == "RMSE"){
      best_model <- which.min(grade_list)}
    if (tuning_metric == "RSquared"){
      best_model <- which.max(grade_list)
    }

    best_parameters <- hyper_grid[best_model, ]

    RF_mtry <- as.numeric(best_parameters[1])
    RF_maxnodes <- as.numeric(best_parameters[2])
    RF_ntree  <- as.numeric(best_parameters[3])
    RF_nodesize  <- as.numeric(best_parameters[4])
  }

}


##################################################################################
##################################################################################
##################################################################################
print("Evaluation of methods...")

# Normal cross-validation with repeats.

if (blocked_CV == FALSE){

# create progress bar

  if(interactive()){

    pb <- txtProgressBar(min = 0, max = repeats, style = 3)

  }

b = 0 # place holder for saving results

for (m in 1:repeats){

foldi <- seq(1:k)
foldi <- paste("fold_", foldi)

#ly shuffle the data
set.seed(seed_factor * m)
dataset <- dataset[sample(nrow(dataset)), ]

#Create 10 equally size folds
folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)


#Perform k fold cross validation

for (j in 1:k){

  b <- b + 1

  #Segement your data by fold using the which() function
  testIndexes <- which(folds == j, arr.ind = TRUE)
  test <- dataset[testIndexes, ]
  train <- dataset[-testIndexes, ]

  #MLR MODEL
  if (MLR_stepwise == FALSE) {
    MLR <- lm(formula, data = train)
    train_predicted <- predict(MLR, train)
    test_predicted <- predict(MLR, test)
    train_observed <- train[, DepIndex]
    test_observed <- test[, DepIndex]
    calculations <- calculate_metrics(train_predicted, test_predicted,
                                      train_observed, test_observed, digits = 15,
                                      formula = formula, test = test)
    list_MLR[[b]] <- calculations

  } else {
    capture.output(MLR <- step(lm(formula = formula, data = train), direction = stepwise_direction))
    train_predicted <- predict(MLR, train)
    test_predicted <- predict(MLR, test)
    train_observed <- train[, DepIndex]
    test_observed <- test[, DepIndex]
    calculations <- calculate_metrics(train_predicted, test_predicted,
                                      train_observed, test_observed, digits = 15,
                                      formula = formula, test = test)
    list_MLR[[b]] <- calculations
  }


  #BRNN Model
  capture.output(BRNN <- brnn(formula, data = train, neurons = BRNN_neurons, verbose = FALSE,
                              tol = 1e-6))
  train_predicted <- predict(BRNN, train)
  test_predicted <- predict(BRNN, test)
  calculations <- calculate_metrics(train_predicted, test_predicted,
                                     train_observed, test_observed, digits = 15,
                                    formula = formula, test = test)
  list_BRNN[[b]] <- calculations

  # Model Trees
  train_ind <- data.frame(train[, -DepIndex])
  colnames(train_ind) <- indep_names
  test_ind <- data.frame(test[, -DepIndex])
  colnames(test_ind) <- indep_names

  train_dep <- train[, DepIndex]
  test_dep <- test[, DepIndex]

  MT_model <- cubist(x = train_ind, y = train_dep, committees = MT_committees,
                     neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                      unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
  train_predicted <- predict(MT_model, train_ind)
  test_predicted <- predict(MT_model, test_ind)
  calculations <- calculate_metrics(train_predicted, test_predicted,
                                    train_observed, test_observed, digits = 15,
                                    formula = formula, test = test)
  list_MT[[b]] <- calculations

  # Random Forest, Regression Tree with random forest, WEKA
  RF_model <- randomForest(formula = formula, data = train, ntree = RF_ntree,
                           maxnodes = RF_maxnodes, mtry  = RF_mtry,
                           nodesize = RF_nodesize)
  train_predicted <- predict(RF_model, train)
  test_predicted <- predict(RF_model, test)
  calculations <- calculate_metrics(train_predicted, test_predicted,
                                     train_observed, test_observed, digits = 15,
                                    formula = formula, test = test)
  list_RF[[b]] <- calculations
}

  if(interactive()){

   setTxtProgressBar(pb, m)

  }

  } # repeats zaključek

if(interactive()){

  close(pb)

}

position <- k * repeats

}




###################################################################################
##### And now the second option: Blocked cross-validation #########################
###################################################################################

if (blocked_CV == TRUE){

  # create progress bar

  if(interactive()){

    pb <- txtProgressBar(min = 0, max = k, style = 3)

  }

  b = 0 # place holder for saving results

  # Here, idex of dependent variable is extracted and later used to locate the
  # observed values
  DepIndex <- grep(as.character(formula[[2]]), colnames(dataset))

  foldi <- seq(1:k)
  #foldi <- paste("fold_", foldi)
  folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

  #Perform k fold cross validation

  for (m in 1:k){

    b <- b + 1
    #Segement your data by fold using the which() function
    testIndexes <- which(folds == m, arr.ind = TRUE)
    test <- dataset[testIndexes, ]
    train <- dataset[-testIndexes, ]

    #MLR MODEL
    if (MLR_stepwise == FALSE) {
      MLR <- lm(formula, data = train)
      train_predicted <- predict(MLR, train)
      test_predicted <- predict(MLR, test)
      train_observed <- train[, DepIndex]
      test_observed <- test[, DepIndex]
      calculations <- calculate_metrics(train_predicted, test_predicted,
                                        train_observed, test_observed, digits = 15,
                                        formula = formula, test = test)
      list_MLR[[b]] <- calculations

    } else {
      capture.output(MLR <- step(lm(formula = formula, data = train), direction = stepwise_direction))
      train_predicted <- predict(MLR, train)
      test_predicted <- predict(MLR, test)
      train_observed <- train[, DepIndex]
      test_observed <- test[, DepIndex]
      calculations <- calculate_metrics(train_predicted, test_predicted,
                                        train_observed, test_observed, digits = 15,
                                        formula = formula, test = test)
      list_MLR[[b]] <- calculations
    }

    #BRNN Model
    capture.output(BRNN <- brnn(formula, data = train, neurons = BRNN_neurons, verbose = FALSE,
                                tol = 1e-6))
    train_predicted <- predict(BRNN, train)
    test_predicted <- predict(BRNN, test)
    calculations <- calculate_metrics(train_predicted, test_predicted,
                                       train_observed, test_observed, digits = 15,
                                      formula = formula, test = test)
    list_BRNN[[b]] <- calculations

    # Model Trees
    train_ind <- data.frame(train[, -DepIndex])
    colnames(train_ind) <- indep_names
    test_ind <- data.frame(test[, -DepIndex])
    colnames(test_ind) <- indep_names

    train_dep <- train[, DepIndex]
    test_dep <- test[, DepIndex]

    MT_model <- cubist(x = train_ind, y = train_dep, committees = MT_committees,
                       neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
    train_predicted <- predict(MT_model, train_ind)
    test_predicted <- predict(MT_model, test_ind)
    calculations <- calculate_metrics(train_predicted, test_predicted,
                                      train_observed, test_observed, digits = 15,
                                      formula = formula, test = test)
    list_MT[[b]] <- calculations


    ##Random Forest
    RF_model <- randomForest(formula = formula, data = train, ntree = RF_ntree,
                             maxnodes = RF_maxnodes, mtry  = RF_mtry,
                             nodesize = RF_nodesize)
    train_predicted <- predict(RF_model, train)
    test_predicted <- predict(RF_model, test)
    calculations <- calculate_metrics(train_predicted, test_predicted,
                                      train_observed, test_observed, digits = 15,
                                      formula = formula, test = test)
    list_RF[[b]] <- calculations

    if(interactive()){

      setTxtProgressBar(pb, m)

    }
  }

  if(interactive()){

    close(pb)

  }
  position <- k
}

###########################################################################################
###########################################################################################
###########################################################################################
# Now the proces of extraction starts
# Here, lists are rearranged and metrics are extracted

listVec <- lapply(list_MLR, c, recursive = TRUE)
m <- do.call(cbind, listVec)
averages <- apply(m, 1, mean, na.rm = TRUE)
std <- apply(m, 1, sd, na.rm = TRUE)
m <- cbind(m, averages, std)
df_MLR <- data.frame(m)
df_MLR_bias <- df_MLR[c(15, 16), c(1: position)]
df_MLR_rank <- df_MLR[-c(15, 16), c(1: position)]
df_MLR_avg <- df_MLR[-c(15, 16), c(position + 1, position + 2)]
rownames(df_MLR_avg) <- c("r_cal", "r_val", "RMSE_cal", "RMSE_val", "RRSE_cal",
                      "RRSE_val", "d_cal", "d_val", "RE_cal", "RE_val",
                      "CE_cal", "CE_val", "DE_cal", "DE_val")

listVec <- lapply(list_BRNN, c, recursive = TRUE)
m <- do.call(cbind, listVec)
averages <- apply(m, 1, mean, na.rm = TRUE)
std <- apply(m, 1, sd, na.rm = TRUE)
m <- cbind(m, averages, std)
df_BRNN <- data.frame(m)
df_BRNN_bias <- df_BRNN[c(15, 16), c(1: position)]
df_BRNN_rank <- df_BRNN[-c(15, 16), c(1: position)]
df_BRNN_avg <- df_BRNN[-c(15, 16), c(position + 1, position + 2)]
rownames(df_BRNN_avg) <- c("r_cal", "r_val", "RMSE_cal", "RMSE_val", "RRSE_cal",
                      "RRSE_val", "d_cal", "d_val", "RE_cal", "RE_val",
                      "CE_cal", "CE_val", "DE_cal", "DE_val")

listVec <- lapply(list_MT, c, recursive = TRUE)
m <- do.call(cbind, listVec)
averages <- apply(m, 1, mean, na.rm = TRUE)
std <- apply(m, 1, sd, na.rm = TRUE)
m <- cbind(m, averages, std)
df_MT <- data.frame(m)
df_MT_bias <- df_MT[c(15, 16), c(1: position)]
df_MT_rank <- df_MT[-c(15, 16), c(1: position)]
df_MT_avg <- df_MT[-c(15, 16), c(position + 1, position + 2)]
rownames(df_MT_avg) <- c("r_cal", "r_val", "RMSE_cal", "RMSE_val", "RRSE_cal",
                      "RRSE_val", "d_cal", "d_val", "RE_cal", "RE_val",
                      "CE_cal", "CE_val", "DE_cal", "DE_val")


listVec <- lapply(list_RF, c, recursive = TRUE)
m <- do.call(cbind, listVec)
averages <- apply(m, 1, mean, na.rm = TRUE)
std <- apply(m, 1, sd, na.rm = TRUE)
m <- cbind(m, averages, std)
df_RF <- data.frame(m)
df_RF_bias <- df_RF[c(15, 16), c(1: position)]
df_RF_rank <- df_RF[-c(15, 16), c(1: position)]
df_RF_avg <- df_RF[-c(15, 16), c(position + 1, position + 2)]
rownames(df_RF_avg) <- c("r_cal", "r_val", "RMSE_cal", "RMSE_val", "RRSE_cal",
                      "RRSE_val", "d_cal", "d_val", "RE_cal", "RE_val",
                      "CE_cal", "CE_val", "DE_cal", "DE_val")


##################################################################################################
##################################################################################################
############################## Here, all data frames are binded together #########################
##################################################################################################

start_position = 0
method_list <- list()
for (i in methods){
  start_position <- start_position + 1
  method_list[[start_position]] <- paste0("df_",i,"_avg")
}

method_vector <- unlist(method_list, use.names=FALSE)

empty_LIST <- list()

for (i in 1:length(method_vector)){
  temp_DF <- get(method_vector[i])
  empty_LIST[[i]] <- temp_DF
}

names(empty_LIST) <- method_vector

df_all_avg <- round(do.call(cbind, empty_LIST), 8)


############################################################################################
############################################################################################
# Calculation of ranks

# First, let's subset the methods, similar to the previous

start_position = 0
method_list <- list()
for (i in methods){
  start_position <- start_position + 1
  method_list[[start_position]] <- paste0("df_",i,"_rank")
}

method_vector <- unlist(method_list, use.names = FALSE)

empty_LIST <- list()

for (i in 1:length(method_vector)){
  temp_DF <- get(method_vector[i])
  empty_LIST[[i]] <- temp_DF
}

names(empty_LIST) <- method_vector

df_all <- round(do.call(rbind, empty_LIST), 8)

# Now, all metrics (except bias) are extracted for calibration and validation
# data.
r_cal <- df_all[c(seq(1, nrow(df_all), by = 14)), ]
r_val <- df_all[c(seq(2, nrow(df_all), by = 14)), ]

RMSE_cal <- df_all[c(seq(3, nrow(df_all), by = 14)), ]
RMSE_val <- df_all[c(seq(4, nrow(df_all), by = 14)), ]

RRSE_cal <- df_all[c(seq(5, nrow(df_all), by = 14)), ]
RRSE_val <- df_all[c(seq(6, nrow(df_all), by = 14)), ]

d_cal <- df_all[c(seq(7, nrow(df_all), by = 14)), ]
d_val <- df_all[c(seq(8, nrow(df_all), by = 14)), ]

RE_cal <- df_all[c(seq(9, nrow(df_all), by = 14)), ]
RE_val <- df_all[c(seq(10, nrow(df_all), by = 14)), ]

CE_cal <- df_all[c(seq(11, nrow(df_all), by = 14)), ]
CE_val <- df_all[c(seq(12, nrow(df_all), by = 14)), ]

DE_cal <- df_all[c(seq(13, nrow(df_all), by = 14)), ]
DE_val <- df_all[c(seq(14, nrow(df_all), by = 14)), ]


# Mean rank and share of rank 1 is calculated
AVG_rank <- data.frame(rowMeans(apply(-r_cal, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-r_cal, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
r_cal_ranks <- cbind(AVG_rank, shareOne)
names(r_cal_ranks) <- c("Mean Rank", "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-r_val, 2, rank, ties.method =  "min")))
shareOne <- data.frame(apply(apply(-r_val, 2, rank, ties.method =  "min"), 1,
                             count_ones) /  position)
r_val_ranks <- cbind(AVG_rank, shareOne)
names(r_val_ranks) <-  c("Mean Rank",  "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(RMSE_cal, 2, rank,
                                      ties.method =  "min")))
shareOne <- data.frame(apply(apply(RMSE_cal, 2, rank, ties.method =  "min"),
                             1, count_ones) /  position)
RMSE_cal_ranks <- cbind(AVG_rank, shareOne)
names(RMSE_cal_ranks) <-  c("Mean Rank", "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(RMSE_val, 2, rank,
                                      ties.method = "min")))
shareOne <- data.frame(apply(apply(RMSE_val, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
RMSE_val_ranks <- cbind(AVG_rank, shareOne)
names(RMSE_val_ranks) <-  c("Mean Rank", "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(RRSE_cal, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(RRSE_cal, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
RRSE_cal_ranks <- cbind(AVG_rank, shareOne)
names(RRSE_cal_ranks) <-  c("Mean Rank", "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(RRSE_val, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(RRSE_val, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
RRSE_val_ranks <- cbind(AVG_rank, shareOne)
names(RRSE_val_ranks) <-  c("Mean Rank", "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-d_cal, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-d_cal, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
d_cal_ranks <- cbind(AVG_rank, shareOne)
names(d_cal_ranks) <-  c("Mean Rank",  "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-d_val, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-d_val, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
d_val_ranks <- cbind(AVG_rank, shareOne)
names(d_val_ranks) <-  c("Mean Rank",  "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-RE_cal, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-RE_cal, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
RE_cal_ranks <- cbind(AVG_rank, shareOne)
names(RE_cal_ranks) <-  c("Mean Rank", "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-RE_val, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-RE_val, 2, rank, ties.method = "min"),
                             1, count_ones) /  position)
RE_val_ranks <- cbind(AVG_rank, shareOne)
names(RE_val_ranks) <-  c("Mean Rank", "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-CE_cal, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-CE_cal, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
CE_cal_ranks <- cbind(AVG_rank, shareOne)
names(CE_cal_ranks) <- c("Mean Rank",  "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-CE_val, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-CE_val, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
CE_val_ranks <- cbind(AVG_rank, shareOne)
names(CE_val_ranks) <-  c("Mean Rank",  "%rank_1")


AVG_rank <- data.frame(rowMeans(apply(-DE_cal, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-DE_cal, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
DE_cal_ranks <- cbind(AVG_rank, shareOne)
names(DE_cal_ranks) <- c("Mean Rank",  "%rank_1")

AVG_rank <- data.frame(rowMeans(apply(-DE_val, 2, rank, ties.method = "min")))
shareOne <- data.frame(apply(apply(-DE_val, 2, rank, ties.method = "min"), 1,
                             count_ones) /  position)
DE_val_ranks <- cbind(AVG_rank, shareOne)
names(DE_val_ranks) <-  c("Mean Rank",  "%rank_1")




# Results are rbinded together
ranks_together <- rbind(r_cal_ranks, r_val_ranks,
                       RMSE_cal_ranks, RMSE_val_ranks,
                       RRSE_cal_ranks, RRSE_val_ranks,
                       d_cal_ranks, d_val_ranks,
                       RE_cal_ranks, RE_val_ranks,
                       CE_cal_ranks, CE_val_ranks,
                       DE_cal_ranks, DE_val_ranks)

# Those variables have to be defined, solution suggest on Stackoverflow.com
BRNN <- NULL
BRNN_AR <- NULL
BRNN_M <- NULL
BRNN_S1 <- NULL
BRNN_SD <- NULL
MLR <- NULL
MLR_AR <- NULL
MLR_M <- NULL
MLR_S1 <- NULL
MLR_SD <- NULL
MT <- NULL
MT_AR <- NULL
MT_S1 <- NULL
MT_SD <- NULL
Metric <- NULL
Period <- NULL
RF <- NULL
RF_AR <- NULL
RF_M <- NULL
RF_S1 <- NULL
RF_SD <- NULL
bias <- NULL
Method <- NULL
value <- NULL
pred <- NULL
method <- NULL
Year <- NULL
reconstruction <- NULL
edge_observation <- NULL
edge_prediction <- NULL
type <- NULL
data_edge_central <- NULL
DE <- NULL
data_holdout_calibration <- NULL
residuals <- NULL
predicted <- NULL

ranks_together$Method <- methods
ranks_together$Period <- c(rep("cal", length(methods)), rep("val", length(methods)))
ranks_together$Metric <- c(rep("r", length(methods) * 2),
                            rep("RMSE", length(methods) * 2),
                            rep("RRSE", length(methods) * 2),
                            rep("d", length(methods) *2),
                            rep("RE", length(methods) * 2),
                            rep("CE", length(methods) * 2),
                            rep("DE", length(methods) * 2))

colnames(ranks_together)[1] <- "Avg_rank"
togeter_AVG_rank <- reshape2::dcast(ranks_together,
                                  formula = Metric + Period ~ Method,
                                  value.var = c("Avg_rank"))
togeter_AVG_rank$Metric  <- factor(togeter_AVG_rank$Metric,
                                    levels = c("r", "RMSE", "RRSE", "d",
                                               "RE", "CE", "DE"))
togeter_AVG_rank <- togeter_AVG_rank[order(togeter_AVG_rank$Metric), ]
togeter_AVG_rank <- dplyr::select(togeter_AVG_rank, Metric, Period, methods)

colnames(ranks_together)[2] <- "Share_rank1"
together_share1 <- reshape2::dcast(ranks_together,
                                 formula = Metric + Period ~ Method,
                                 value.var = c("Share_rank1"))

together_share1$Metric  <- factor(together_share1$Metric,
                                   levels = c("r", "RMSE", "RRSE", "d",
                                              "RE", "CE", "DE"))

together_share1 <- together_share1[order(together_share1$Metric), ]
together_share1 <- dplyr::select(together_share1, Metric, Period, methods)

###############################################################################

temp_string_mean <- paste("Mean", methods)
temp_string_std <- paste("Std", methods)

emptly_list = list()

sp_position = 1
for (i in 1:length(temp_string_mean)){
  emptly_list[sp_position] <- temp_string_mean[i]
  sp_position <- sp_position + 1

  emptly_list[sp_position] <- temp_string_std[i]
  sp_position <- sp_position + 1
}

df_all_avg_colnames <- as.vector(do.call(cbind, emptly_list))
colnames(df_all_avg) <- df_all_avg_colnames

df_all_avg$Period <- c("cal", "val")
df_all_avg$Metric <- c("r", "r", "RMSE", "RMSE", "RRSE", "RRSE",
                            "d", "d", "RE", "RE", "CE", "CE", "DE", "DE")
row.names(df_all_avg) <- NULL

Results_mean_std <- dplyr::select(df_all_avg, Metric, Period, df_all_avg_colnames)

# Here, we organize the rank and share1 data frame



#1 create string for column names
temp_string_rank <- paste("Avg rank", methods)
colnames(togeter_AVG_rank) <- c("Metric", "Period", temp_string_rank)

temp_string_share1 <- paste("%rank_1", methods)
together_share1 <- together_share1[, -c(1,2)]
colnames(together_share1) <- temp_string_share1

ranks <- cbind(togeter_AVG_rank, together_share1)

# We have to create a vector of strings, cobmination avg rank and %rank 1
emptly_list = list()

sp_position = 1
for (i in 1:length(temp_string_rank)){
  emptly_list[sp_position] <- temp_string_rank[i]
  sp_position <- sp_position + 1

  emptly_list[sp_position] <- temp_string_share1[i]
  sp_position <- sp_position + 1
}

Results_ranks <- dplyr::select(ranks, Metric, Period,
                               as.vector(do.call(cbind, emptly_list)))

##################################################################
# Here is a function to calculate bias
df_MLR_bias$Period <- c("Calibration", "Validation")
df_MLR_bias$Method <- "MLR"

df_BRNN_bias$Period <- c("Calibration", "Validation")
df_BRNN_bias$Method <- "BRNN"

df_MT_bias$Period <- c("Calibration", "Validation")
df_MT_bias$Method <- "MT"

df_RF_bias$Period <- c("Calibration", "Validation")
df_RF_bias$Method <- "RF"

# Subset of methods
start_position = 0
method_list <- list()
for (i in methods){
  start_position <- start_position + 1
  method_list[[start_position]] <- paste0("df_",i,"_bias")
}

method_vector <- unlist(method_list, use.names = FALSE)

empty_LIST <- list()

for (i in 1:length(method_vector)){
  temp_DF <- get(method_vector[i])
  empty_LIST[[i]] <- temp_DF
}

names(empty_LIST) <- method_vector

bias_together <- do.call(rbind, empty_LIST)

bias_together <- melt(bias_together, id.vars = c("Period", "Method"))

bias_together_calibration <- dplyr::filter(bias_together, Period == "Calibration")
bias_together_validation <- dplyr::filter(bias_together, Period == "Validation")

bias_together_calibration$value <- round(bias_together_calibration$value, round_bias_cal)

gg_object_cal <- ggplot(bias_together_calibration, aes(value)) +
  geom_histogram(aes(group = Method), bins = n_bins) +
  # geom_density(aes(group = Method)) +
  geom_vline(xintercept = 0) +
  facet_grid(Method ~ ., scales = "free") +
  theme_bw() +
  xlab("bias") +
  theme(legend.position = "NONE", legend.title = element_blank(),
        text = element_text(size = 15))

bias_together_validation$value <- round(bias_together_validation$value, round_bias_val)

gg_object_val <- ggplot(bias_together_validation, aes(value)) +
  geom_histogram(aes(group = Method), bins = n_bins) +
  # geom_density(aes(group = Method)) +
  geom_vline(xintercept = 0) +
  facet_grid(Method ~ .) +
  theme_bw() +
  xlab("bias") +
  theme(legend.position = "NONE", legend.title = element_blank(),
        text = element_text(size = 15))


########################################################################################
# Here, we create transfer functions for each method
if (numIND == 1) {
  Ind_name <- colnames(dataset)[-DepIndex]
  Dep_name <- colnames(dataset)[DepIndex]

  range_max <- max(dataset[,-DepIndex])
  range_min <- min(dataset[,-DepIndex])

  # I want to increase the range for transfer function, to see greater
  # difference among transfer functions
  diff2 <- (range_max - range_min)/4

  full_range <- data.frame(c1 = NA, c2 = seq(range_min - abs(diff2), range_max + abs(diff2), diff2/1000))
  colnames(full_range) <- c(Dep_name, Ind_name)
  #full_range <- select(full_range, colnames(dataset))

#MLR MODEL
MLR <- lm(formula, data = dataset)
predicted_MLR <- data.frame(pred = predict(MLR, full_range), method = "MLR")

#BRNN Model
capture.output(BRNN <- brnn(formula, data = dataset, BRNN_neurons = BRNN_neurons, verbose = FALSE))
predicted_BRNN <- data.frame(pred = predict(BRNN, full_range), method = "BRNN")

# Model Trees
dataset_ind <- data.frame(dataset[, -DepIndex])
colnames(dataset_ind) <- indep_names
full_range_ind <- data.frame(full_range[, -DepIndex])
colnames(full_range_ind) <- indep_names

dataset_dep <- dataset[, DepIndex]
full_range_dep <- full_range[, DepIndex]

MT_model <- cubist(x = dataset_ind, y = dataset_dep, committees = MT_committees,
                   neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                   unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
predicted_MT <- data.frame(pred = predict(MT_model, full_range_ind), method = "MT")

# Random Forest
RF_model <- randomForest(formula = formula, data = dataset, ntree = RF_ntree,
                         maxnodes = RF_maxnodes, mtry  = RF_mtry,
                         nodesize = RF_nodesize)
predicted_RF <- data.frame(pred = predict(RF_model, full_range), method = "RF")

# Subset of methods
start_position = 0
method_list <- list()
for (i in methods){
  start_position <- start_position + 1
  method_list[[start_position]] <- paste0("predicted_",i)
}

method_vector <- unlist(method_list, use.names = FALSE)

empty_LIST <- list()

for (i in 1:length(method_vector)){
  temp_DF <- get(method_vector[i])
  empty_LIST[[i]] <- temp_DF
}

names(empty_LIST) <- method_vector
predictions <- do.call(rbind, empty_LIST)
predictions$range <- full_range[, 2]
colnames(predictions)

journal_theme <- theme_bw() +
  theme(axis.text = element_text(size = 16, face = "bold"),
        axis.title = element_text(size = 18), text = element_text(size = 18),
        plot.title = element_text(size = 16,  face = "bold"),
        legend.position="bottom", legend.title=element_blank())

dataset1 <-dataset
colnames(dataset1)[DepIndex] <- "pred"
colnames(dataset1)[-DepIndex] <- "range"

plot_1 <- ggplot(predictions, aes(x = range, y = pred)) + geom_line() +
  geom_point(data = dataset1, aes(x = range, y = pred)) +
  facet_grid(method~.) +
  xlab("Range of Independent Variable") +
  ylab("Dependent Variable") +
  ggtitle("Transfer Functions") +
  journal_theme

dataset1$method = NA

plot_2 <- ggplot(predictions, aes(x = range, y = pred, group = method, colour = method)) +
  geom_line(aes(x = range, y = pred, group = method, colour = method, linetype = method), size = 1.05) +
  geom_point(data = dataset1, aes(x = range, y = pred)) +
  xlab("Range of Independent Variable") +
  ylab("Dependent Variable") +
  ggtitle("Transfer Functions Plotted Together") +
  journal_theme

} else if (numIND == 2){

  modelList = list()
  plotList = list()
  plotListNames = methods

  if ("BRNN" %in% methods){
    modelList$BRNN = brnn(formula, dataset, neurons = BRNN_neurons)
  }

  if ("MLR" %in% methods){
    modelList$MLR = lm(formula, dataset)
  }

  if ("MT" %in% methods){
    dataset_ind <- data.frame(dataset[, -DepIndex])
    colnames(dataset_ind) <- indep_names
    dataset_dep <- dataset[, DepIndex]

    modelList$MT = cubist(x = dataset_ind, y = dataset_dep, committees = MT_committees,
                          neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                          unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
  }

  if ("RF" %in% methods){

    modelList$RF = randomForest(formula = formula, data = dataset, ntree = RF_ntree,
                                maxnodes = RF_maxnodes, mtry  = RF_mtry,
                                nodesize = RF_nodesize)
  }

  for (i in 1:length(modelList)){

    model = modelList[[i]]

    x1 = dataset[[indep_name_1]]
    x2 = dataset[[indep_name_2]]
    y = dataset[[DepName]]

    grd <- data.frame(X1 = seq(range(x1)[1],range(x1)[2],len=40),
                      X2 = seq(range(x2)[1],range(x2)[2],len=40))

    names(grd) <- c(indep_names)
    grd$pred <- predict(model, newdata=grd)
    grd <- grd[order(grd[[indep_name_1]],grd[[indep_name_2]]),]


    x1 <- unique(grd[[indep_name_1]])
    x2 <- unique(grd[[indep_name_2]])   # shouldn't have used y
    z=matrix(grd$pred,length(x1),length(x2))

    color_workaround <- rep(0,length(grd$pred))
    dim(color_workaround) <- dim(grd$pred)

    thisName = plotListNames[i]
    plotList[[thisName]] = plot_ly(x = x1, y = x2, z = ~z, name =thisName,
                                   scene = paste0('scene',i)) %>%
      add_surface(opacity = 0.5,  showscale=FALSE, hoverinfo = "name", surfacecolor=color_workaround,
                  hoverlabel = list(color = "black")) %>%
      layout(scene = list(xaxis = list(title = indep_name_1),
                          yaxis = list(title = indep_name_2),
                          zaxis = list(title = DepName))) %>%
      add_trace(x = dataset[[indep_name_1]], y = dataset[[indep_name_2]], z = dataset[[DepName]],   mode = "markers", type = "scatter3d",
                marker = list(size = 5, color = "red", symbol = 104), hoverinfo = "name",
                hoverlabel = list(color = "black")) %>%
      #add_annotations(x = 0.5, y = 0.8, text = paste(plotListNames[i]), showarrow = FALSE) %>%
      layout(showlegend = FALSE) %>% plotly_build()

  }

  if (length (plotList) == 1){
    plot_1 <- subplot(plotList)%>%
      layout(scene = list(aspectmode='cube',
                          yaxis = list(title = indep_name_2),
                          xaxis = list(title = indep_name_1),
                          zaxis = list(title = DepName)))
  }

  if (length (plotList) == 2){
    plot_1 <- subplot(plotList)%>%
      layout(scene = list(domain=list(x=c(0,0.5),y=c(0,1)),
                          aspectmode='cube',
                          yaxis = list(title = indep_name_2),
                          xaxis = list(title = indep_name_1),
                          zaxis = list(title = DepName)),
             scene2 = list(domain=list(x=c(0.5,1),y=c(0,1)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)))
  }

  if (length (plotList) == 3){
    plot_1 <- subplot(plotList)%>%
      layout(scene = list(domain=list(x=c(0,0.5),y=c(0.5,1)),
                          aspectmode='cube',
                          yaxis = list(title = indep_name_2),
                          xaxis = list(title = indep_name_1),
                          zaxis = list(title = DepName)),
             scene2 = list(domain=list(x=c(0.5,1),y=c(0.5,1)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)),
             scene3 = list(domain=list(x=c(0,0.5),y=c(0,0.5)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)))
  }

  if (length (plotList) == 4){
    plot_1 <- subplot(plotList)%>%
      layout(scene = list(domain=list(x=c(0,0.5),y=c(0.5,1)),
                          aspectmode='cube',
                          yaxis = list(title = indep_name_2),
                          xaxis = list(title = indep_name_1),
                          zaxis = list(title = DepName)),
             scene2 = list(domain=list(x=c(0.5,1),y=c(0.5,1)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)),
             scene3 = list(domain=list(x=c(0,0.5),y=c(0,0.5)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)),
             scene4 = list(domain=list(x=c(0.5,1),y=c(0,0.5)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)))
  }

  if (length (plotList) == 5){
    plot_1 <- subplot(plotList)%>%
      layout(scene = list(domain=list(x=c(0,0.33),y=c(0.5,1)),
                          aspectmode='cube',
                          yaxis = list(title = indep_name_2),
                          xaxis = list(title = indep_name_1),
                          zaxis = list(title = DepName)),
             scene2 = list(domain=list(x=c(0.33,0.66),y=c(0.5,1)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)),
             scene3 = list(domain=list(x=c(0.66,1),y=c(0.5,1)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)),
             scene4 = list(domain=list(x=c(0,0.33),y=c(0,0.5)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName)),
             scene5 = list(domain=list(x=c(0.33,0.66),y=c(0,0.5)),
                           aspectmode='cube',
                           yaxis = list(title = indep_name_2),
                           xaxis = list(title = indep_name_1),
                           zaxis = list(title = DepName))
             )
  }



  for (i in 1:length(modelList)){

    model = modelList[[i]]

    x1 = dataset[[indep_name_1]]
    x2 = dataset[[indep_name_2]]
    y = dataset[[DepName]]

    grd <- data.frame(X1 = seq(range(x1)[1],range(x1)[2],len=40),
                      X2 = seq(range(x2)[1],range(x2)[2],len=40))

    names(grd) <- c(indep_names)
    grd$pred <- predict(model, newdata=grd)
    grd <- grd[order(grd[[indep_name_1]],grd[[indep_name_2]]),]


    x1 <- unique(grd[[indep_name_1]])
    x2 <- unique(grd[[indep_name_2]])   # shouldn't have used y
    z <- matrix(grd$pred,length(x1),length(x2))

    color_workaround <- rep(i,length(grd$pred))
    dim(color_workaround) <- dim(grd$pred)

    color_vector = c("red", "blue", "green", "orange", "brown")

    thisName = plotListNames[i]

    z1 <- list()
    z1[[1]] <- z
    names(z1) <- thisName


    plotList[[thisName]] = plot_ly(x = x1, y = x2, z = ~z1[[1]], name = thisName,
                                   scene = 'scene') %>%
      add_surface(opacity = 0.5, hoverinfo = "name", surfacecolor = color_workaround, colors = color_vector[i],
                  colorbar =list(title=methods[i], dtick = "tick0", tickvals = c()),
                  hoverlabel = list(color = "black")) %>%
      layout(scene = list(xaxis = list(title = indep_name_1),
                          yaxis = list(title = indep_name_2),
                          zaxis = list(title = DepName))) %>%
      add_trace(x = dataset[[indep_name_1]], y = dataset[[indep_name_2]], z = dataset[[DepName]],   mode = "markers", type = "scatter3d",
                marker = list(size = 5, color= "red", symbol = 104), hoverinfo = "none",
                hoverlabel = list(color = "black")) %>%
      layout(showlegend = FALSE) %>% plotly_build()

  }


  plot_2 <- subplot(plotList)%>%
    layout(scene = list(aspectmode='cube',
                        yaxis = list(title = indep_name_2),
                        xaxis = list(title = indep_name_1),
                        zaxis = list(title = DepName)))





  } else {

  plot_1 <- "transfer functions are not available for regression problems with 2 or more independent variables!"
  plot_2 <- "Transfer functions are not available for regression problems with 2 or more independent variables!"

  }



##### Here both data frames are subset with round_df function #############

Results_mean_std <- round_df(Results_mean_std, digits = digits)
Results_ranks <- round_df(Results_ranks, digits = digits)

# Here, all optimized parameters are saved in a data frame, which will be saved as
# a fifth elemnt of the final_list
parameters <- data.frame(
  Method = c("BRNN", "MT", "MT", "MT", "MT","MT" , "MT", "RF", "RF", "RF", "RF"),
  Parameter = c("BRNN_neurons", "MT_committees", "MT_neighbors", "MT_rules", "MT_unbiased",
                "MT_extrapolation", "MT_sample", "RF_mtry", "RF_maxnodes", "RF_ntree", "RF_nodesize"),
  Considered_values = c(
    as.character(paste0(BRNN_neurons_vector, collapse=", ")),
    as.character(paste0(MT_committees_vector, collapse=", ")),
    as.character(paste0(MT_neighbors_vector, collapse=", ")),
    as.character(paste0(MT_rules_vector, collapse=", ")),
    as.character(paste0(MT_unbiased_vector, collapse=", ")),
    as.character(paste0(MT_extrapolation_vector, collapse=", ")),
    as.character(paste0(MT_sample_vector, collapse=", ")),
    as.character(paste0(RF_mtry_vector, collapse=", ")),
    as.character(paste0(RF_maxnodes_vector, collapse=", ")),
    as.character(paste0(RF_ntree_vector, collapse=" ")),
    as.character(paste0(RF_nodesize_vector, collapse=" "))
    ),
  Value = c(BRNN_neurons, MT_committees, MT_neighbors, MT_rules,
            ifelse(MT_unbiased == 1, as.character("TRUE"), as.character("FALSE")),
            MT_extrapolation, MT_sample, RF_mtry, RF_maxnodes, RF_ntree, RF_nodesize))

colnames(parameters) = c("Method", "Parameter", "Considered values", "Selected value")

parameters <- parameters[parameters$Method %in% methods,]

if (optimize == FALSE){
  parameters[, 3] <- NULL

}

                 ############################################################
                 # The reconstruction of climate based on all methods used  #
                 ############################################################
if (is.null(dataset_complete) == FALSE){
MLR_model <- lm(formula, data = dataset)
reconstruction_MLR <- data.frame(reconstruction = predict(MLR_model, dataset_complete), method = "MLR")

#BRNN Model
capture.output(BRNN_model <- brnn(formula, data = dataset, BRNN_neurons = BRNN_neurons, verbose = FALSE))
reconstruction_BRNN <- data.frame(reconstruction = predict(BRNN_model, dataset_complete), method = "BRNN")

# Model Trees
dataset_ind <- data.frame(dataset[, -DepIndex])
colnames(dataset_ind) <- indep_names
dataset_complete_ind <- data.frame(dataset_complete)
colnames(dataset_complete_ind) <- indep_names

dataset_dep <- dataset[, DepIndex]

MT_model <- cubist(x = dataset_ind, y = dataset_dep, committees = MT_committees,
                   neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                   unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
reconstruction_MT <- data.frame(reconstruction = predict(MT_model, dataset_complete_ind), method = "MT")


# Random Forest
RF_model <- randomForest(formula = formula, data = dataset, ntree = RF_ntree,
                         maxnodes = RF_maxnodes, mtry  = RF_mtry,
                         nodesize = RF_nodesize)

reconstruction_RF <- data.frame(reconstruction = predict(RF_model, dataset_complete), method = "RF")



# Subset of methods
start_position = 0
method_list <- list()
for (i in methods){
  start_position <- start_position + 1
  method_list[[start_position]] <- paste0("reconstruction_",i)
}

method_vector <- unlist(method_list, use.names = FALSE)

empty_LIST <- list()

for (i in 1:length(method_vector)){
  temp_DF <- get(method_vector[i])
  empty_LIST[[i]] <- temp_DF
}

names(empty_LIST) <- method_vector
reconstructions <- do.call(rbind, empty_LIST)
reconstructions$Year = as.numeric(row.names(dataset_complete))

recon_df <- reconstructions

# Add plots
plot_3 <- ggplot(reconstructions, aes(x = Year, y = reconstruction, group = method, colour = method)) +
  geom_line() +
  ylab("Reconstruction") +
  ggtitle("Reconstruction of dependent variable") +
  journal_theme

plot_4 <- ggplot(reconstructions, aes(x = Year, y = reconstruction, group = method)) +
  geom_line() +
  ylab("Reconstruction") +
  ggtitle("Reconstruction of dependent variable") +
  facet_wrap(~method, ncol = 1) +
  journal_theme +
  theme(legend.position = 'none')
} else {
  plot_3 <- "The dataset full argument is not supplied, therefore reconstrucions are not calculated"
  plot_4 <- "The dataset full argument is not supplied, therefore reconstrucions are not calculated"
  recon_df <- "The dataset full is not supplied, so reconstructed data is not available"
}


##########################################################
#   The comparison of methods, only the edge instances     #
############################################################
# Here, I extract the "edge data", I calibrate models using the central part of the data and
# afterwards use it on the edge data to see, how methods perform in modeling
if (numIND == 1 & edge_share > 0) {
edge_factor <- round2(nrow(dataset)*(edge_share/2),0)

dataset_max <- dplyr::arrange(dataset, desc(dataset[, -DepIndex]))[1:edge_factor,]
dataset_min <- dplyr::arrange(dataset, desc(dataset[, -DepIndex]))[(nrow(dataset)-edge_factor+1):nrow(dataset),]
dataset_edges <- rbind(dataset_max, dataset_min)
dataset_central <- dplyr::arrange(dataset, desc(dataset[, -DepIndex]))[(edge_factor+1):(nrow(dataset)-edge_factor),]

   MLR_model <- lm(formula, data = dataset_central)
   edge_prediction <- data.frame(value = predict(MLR_model, dataset_edges), data_edge_central = "edge", type = "predicted", method = "MLR")
   central_prediction <- data.frame(value = predict(MLR_model, dataset_central), data_edge_central = "central", type = "predicted", method = "MLR")
   edge_observed <- data.frame(value = dataset_edges[, DepIndex], data_edge_central = "edge", type = "observed", method = "MLR")
   central_observed <- data.frame(value = dataset_central[, DepIndex], data_edge_central = "central", type = "observed", method = "MLR")

  MLR_central_edge <- rbind(edge_prediction, central_prediction, edge_observed, central_observed)

  residuals_edge_MLR <- data.frame(residuals = edge_prediction$value - edge_observed$value,
                                      predicted = edge_prediction$value, method = "MLR")

  test_1 <- data.frame(dataset_edges[, DepIndex])
  colnames(test_1) <- "DE_TRICK"
  test_1$sequence <- seq(1:nrow(dataset_edges))
  MLR_DE <- lm(formula = DE_TRICK~., data = test_1)
  DE_predicted <- predict(MLR_DE, test_1)

  #BRNN Model
  capture.output(BRNN_model <- brnn(formula, data = dataset_central, BRNN_neurons = BRNN_neurons, verbose = FALSE))
  edge_prediction <- data.frame(value = predict(BRNN_model, dataset_edges), data_edge_central = "edge", type = "predicted", method = "BRNN")
  central_prediction <- data.frame(value = predict(BRNN_model, dataset_central), data_edge_central = "central", type = "predicted", method = "BRNN")
  edge_observed <- data.frame(value = dataset_edges[, DepIndex], data_edge_central = "edge", type = "observed", method = "BRNN")
  central_observed <- data.frame(value = dataset_central[, DepIndex], data_edge_central = "central", type = "observed", method = "BRNN")
  BRNN_central_edge <- rbind(edge_prediction, central_prediction, edge_observed, central_observed)

  residuals_edge_BRNN <- data.frame(residuals = edge_prediction$value - edge_observed$value,
                                   predicted = edge_prediction$value, method = "BRNN")

  # Model Trees
  dataset_central_ind <- data.frame(dataset_central[, -DepIndex])
  colnames(dataset_central_ind) <- indep_names
  dataset_edges_ind <- data.frame(dataset_edges[, -DepIndex])
  colnames(dataset_edges_ind) <- indep_names

  dataset_central_dep <- dataset_central[, DepIndex]
  dataset_edges_dep <- dataset_edges[, DepIndex]

  MT_model <- cubist(x = dataset_central_ind, y = dataset_central_dep, committees = MT_committees,
                     neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                     unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
  edge_prediction <- data.frame(value = predict(MT_model, dataset_edges_ind), data_edge_central = "edge", type = "predicted", method = "MT")
  central_prediction <- data.frame(value = predict(MT_model, dataset_central_ind), data_edge_central = "central", type = "predicted", method = "MT")
  edge_observed <- data.frame(value = dataset_edges[, DepIndex], data_edge_central = "edge", type = "observed", method = "MT")
  central_observed <- data.frame(value = dataset_central[, DepIndex], data_edge_central = "central", type = "observed", method = "MT")
  MT_central_edge <- rbind(edge_prediction, central_prediction, edge_observed, central_observed)

  residuals_edge_MT <- data.frame(residuals = edge_prediction$value - edge_observed$value,
                                   predicted = edge_prediction$value, method = "MT")

  # Random Forest
  RF_model <- randomForest(formula = formula, data = dataset_central, ntree = RF_ntree, maxnodes = RF_maxnodes, mtry  = RF_mtry,
                           nodesize = RF_nodesize)

  edge_prediction <- data.frame(value = predict(RF_model, dataset_edges), data_edge_central = "edge", type = "predicted", method = "RF")
  central_prediction <- data.frame(value = predict(RF_model, dataset_central), data_edge_central = "central", type = "predicted", method = "RF")
  edge_observed <- data.frame(value = dataset_edges[, DepIndex], data_edge_central = "edge", type = "observed", method = "RF")
  central_observed <- data.frame(value = dataset_central[, DepIndex], data_edge_central = "central", type = "observed", method = "RF")
  RF_central_edge <- rbind(edge_prediction, central_prediction, edge_observed, central_observed)

  residuals_edge_RF <- data.frame(residuals = edge_prediction$value - edge_observed$value,
                                   predicted = edge_prediction$value, method = "RF")

  edge_central_data <- rbind(MLR_central_edge,
                             BRNN_central_edge,
                             MT_central_edge,
                             RF_central_edge)

  # Here i define the residual dataframe for the holdout
  residuals_edge = rbind(residuals_edge_MLR,
                            residuals_edge_BRNN,
                            residuals_edge_MT,
                            residuals_edge_RF)

  edge_central_data$DE = 0

  DE_data_MLR <- data.frame(value = DE_predicted, data_edge_central = "edge", type = "predicted", method = "MLR", DE = 1)
  DE_data_BRNN <- data.frame(value = DE_predicted, data_edge_central = "edge", type = "predicted", method = "BRNN", DE = 1)
  DE_data_MT <- data.frame(value = DE_predicted, data_edge_central = "edge", type = "predicted", method = "MT", DE = 1)
  DE_data_RF <- data.frame(value = DE_predicted, data_edge_central = "edge", type = "predicted", method = "RF", DE = 1)

  DE_data <- rbind(DE_data_MLR,DE_data_BRNN,DE_data_MT,DE_data_RF)

  edge_central_data <- rbind(edge_central_data, DE_data)

  edge_central_data <- dplyr::filter(edge_central_data, method %in% methods)

  edge_results <- edge_central_data %>%
                    group_by(method) %>%
                    summarise(r_central = cor(value[data_edge_central == "central" & type == "predicted"  & DE == 0 ],
                                          value[data_edge_central == "central" & type == "observed" & DE == 0 ]),
                             r_edge = cor(value[data_edge_central == "edge" & type == "predicted" & DE == 0 ],
                                      value[data_edge_central == "edge" & type == "observed" & DE == 0 ]),

                             RMSE_central = MLmetrics::RMSE(value[data_edge_central == "central" & type == "predicted" & DE == 0 ],
                                                            value[data_edge_central == "central" & type == "observed" & DE == 0 ]),
                             RMSE_edge =  MLmetrics::RMSE(value[data_edge_central == "edge" & type == "predicted" & DE == 0 ],
                                                          value[data_edge_central == "edge" & type == "observed" & DE == 0 ]),

                             RRSE_central = MLmetrics::RRSE(value[data_edge_central == "central" & type == "predicted" & DE == 0 ],
                                                            value[data_edge_central == "central" & type == "observed" & DE == 0 ]),
                             RRSE_edge = MLmetrics::RRSE(value[data_edge_central == "edge" & type == "predicted" & DE == 0 ],
                                                         value[data_edge_central == "edge" & type == "observed" & DE == 0 ]),
                             d_central  = 1 - (sum((value[data_edge_central == "central" & type == "observed" & DE == 0 ] -
                                                      value[data_edge_central == "central" & type == "predicted" & DE == 0 ]) ^ 2)) /
                               sum((abs(value[data_edge_central == "central" & type == "predicted" & DE == 0 ] -
                                          mean(value[data_edge_central == "central" & type == "observed" & DE == 0 ])) +
                                      abs(value[data_edge_central == "central" & type == "observed" & DE == 0 ] -
                                            mean(value[data_edge_central == "central" & type == "observed" & DE == 0 ]))) ^ 2),
                             d_edge = 1 - (sum((value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                                  value[data_edge_central == "edge" & type == "predicted" & DE == 0 ]) ^ 2)) /
                               sum((abs(value[data_edge_central == "edge" & type == "predicted" & DE == 0 ] -
                                          mean(value[data_edge_central == "edge" & type == "observed" & DE == 0 ])) +
                                      abs(value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                            mean(value[data_edge_central == "edge" & type == "observed" & DE == 0 ]))) ^ 2),

                             RE_edge = 1 - (sum((value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                                   value[data_edge_central == "edge" & type == "predicted" & DE == 0 ]) ^ 2) /
                                              sum((value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                                     mean(value[data_edge_central == "central" & type == "observed" & DE == 0 ])) ^ 2)),

                             CE_edge  = 1 - (sum((value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                                    value[data_edge_central == "edge" & type == "predicted" & DE == 0 ]) ^ 2) /
                                               sum((value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                                      mean(value[data_edge_central == "edge" & type == "observed" & DE == 0 ])) ^ 2)),
                             DE_edge = 1 - (sum((value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                                   value[data_edge_central == "edge" & type == "predicted" & DE == 0 ]) ^ 2) /
                                              sum((value[data_edge_central == "edge" & type == "observed" & DE == 0 ] -
                                                     value[data_edge_central == "edge" & type == "predicted" & DE == 1 ]) ^ 2))


                               )


# summary(edge_central_data)
# test_observed: edge_central_data$value[edge_central_data$data_edge_central == "edge" & edge_central_data$type == "predicted" & edge_central_data$DE == 1 & edge_central_data$method == "MLR"]
# test_predicted: value[data_edge_central == "edge" & type == "predicted"]
# train_observed: value[data_edge_central == "central" & type == "observed"]
# train_predicted: value[data_edge_central == "central" & type == "predicted"]



  edge_results_t <- t(edge_results[,-1])
  colnames(edge_results_t) <- edge_results$method
  edge_results_t <- round(edge_results_t, digits)
  edge_results_t <- data.frame(edge_results_t, Period =  c("cal_central", "val_edge",
                                                           "cal_central", "val_edge",
                                                           "cal_central", "val_edge",
                                                           "cal_central", "val_edge",
                                                           "val_edge","val_edge",
                                                           "val_edge"
                                                            ),
                               Metric = c("r", "r", "RMSE", "RMSE", "RRSE", "RRSE",
                             "d", "d", "RE", "CE", "DE"))
  edge_results_t <- select(edge_results_t, Metric, Period, methods)
  row.names(edge_results_t) <- NULL


} else {
  edge_results_t <- "No edge data is available for regression problems with more than 1 independent variable."
}

# Here, we once again subset the data frames with preformance metrics, to exclude
# RE, CE and DE for the calibration data
Results_mean_std <- Results_mean_std[-c(9, 11, 13), ]
Results_ranks <- Results_ranks[-c(9, 11, 13), ]




##########################################################
#   holdout calculations    #
############################################################

if (!is.null(holdout)) {

  MLR_model <- lm(formula, data = dataset)
  holdout_prediction <- data.frame(value = predict(MLR_model, dataset_holdout), data_holdout_calibration = "holdout", type = "predicted", method = "MLR")
  calibration_prediction <- data.frame(value = predict(MLR_model, dataset), data_holdout_calibration = "calibration", type = "predicted", method = "MLR")
  holdout_observed <- data.frame(value = dataset_holdout[, DepIndex], data_holdout_calibration = "holdout", type = "observed", method = "MLR")
  calibration_observed <- data.frame(value = dataset[, DepIndex], data_holdout_calibration = "calibration", type = "observed", method = "MLR")

  residuals_holdout_MLR <- data.frame(residuals = holdout_prediction$value - holdout_observed$value,
                                      predicted = holdout_prediction$value, method = "MLR")

  MLR_calibration_holdout <- rbind(holdout_prediction, calibration_prediction, holdout_observed, calibration_observed)

  test_1 <- data.frame(dataset_holdout[, DepIndex])
  colnames(test_1) <- "DE_TRICK"
  test_1$sequence <- seq(1:nrow(dataset_holdout))
  MLR_DE <- lm(formula = DE_TRICK~., data = test_1)
  DE_predicted <- predict(MLR_DE, test_1)

  #BRNN Model
  capture.output(BRNN_model <- brnn(formula, data = dataset, BRNN_neurons = BRNN_neurons, verbose = FALSE))
  holdout_prediction <- data.frame(value = predict(BRNN_model, dataset_holdout), data_holdout_calibration = "holdout", type = "predicted", method = "BRNN")
  calibration_prediction <- data.frame(value = predict(BRNN_model, dataset), data_holdout_calibration = "calibration", type = "predicted", method = "BRNN")
  holdout_observed <- data.frame(value = dataset_holdout[, DepIndex], data_holdout_calibration = "holdout", type = "observed", method = "BRNN")
  calibration_observed <- data.frame(value = dataset[, DepIndex], data_holdout_calibration = "calibration", type = "observed", method = "BRNN")
  BRNN_calibration_holdout <- rbind(holdout_prediction, calibration_prediction, holdout_observed, calibration_observed)

  residuals_holdout_BRNN <- data.frame(residuals = holdout_prediction$value - holdout_observed$value,
                                       predicted = holdout_prediction$value, method = "BRNN")

  # Model Trees
  dataset_ind <- data.frame(dataset[, -DepIndex])
  colnames(dataset_ind) <- indep_names
  dataset_holdout_ind <- data.frame(dataset_holdout[, -DepIndex])
  colnames(dataset_holdout_ind) <- indep_names

  dataset_dep <- dataset[, DepIndex]
  dataset_holdout_dep <- dataset_holdout[, DepIndex]

  MT_model <- cubist(x = dataset_ind, y = dataset_dep, committees = MT_committees,
                     neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                     unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
  holdout_prediction <- data.frame(value = predict(MT_model, dataset_holdout_ind), data_holdout_calibration = "holdout", type = "predicted", method = "MT")
  calibration_prediction <- data.frame(value = predict(MT_model, dataset_ind), data_holdout_calibration = "calibration", type = "predicted", method = "MT")
  holdout_observed <- data.frame(value = dataset_holdout[, DepIndex], data_holdout_calibration = "holdout", type = "observed", method = "MT")
  calibration_observed <- data.frame(value = dataset[, DepIndex], data_holdout_calibration = "calibration", type = "observed", method = "MT")
  MT_calibration_holdout <- rbind(holdout_prediction, calibration_prediction, holdout_observed, calibration_observed)

  residuals_holdout_MT <- data.frame(residuals = holdout_prediction$value - holdout_observed$value,
                                     predicted = holdout_prediction$value, method = "MT")

  # Random Forest
  RF_model <- randomForest(formula = formula, data = dataset, mtry = RF_mtry,
                           maxnodes = RF_maxnodes, ntree = RF_ntree, nodesize = RF_nodesize)

  holdout_prediction <- data.frame(value = predict(RF_model, dataset_holdout), data_holdout_calibration = "holdout", type = "predicted", method = "RF")
  calibration_prediction <- data.frame(value = predict(RF_model, dataset), data_holdout_calibration = "calibration", type = "predicted", method = "RF")
  holdout_observed <- data.frame(value = dataset_holdout[, DepIndex], data_holdout_calibration = "holdout", type = "observed", method = "RF")
  calibration_observed <- data.frame(value = dataset[, DepIndex], data_holdout_calibration = "calibration", type = "observed", method = "RF")
  RF_calibration_holdout <- rbind(holdout_prediction, calibration_prediction, holdout_observed, calibration_observed)

  holdout_calibration_data <- rbind(MLR_calibration_holdout,
                             BRNN_calibration_holdout,
                             MT_calibration_holdout,
                             RF_calibration_holdout)

  residuals_holdout_RF <- data.frame(residuals = holdout_prediction$value - holdout_observed$value,
                                     predicted = holdout_prediction$value, method = "RF")

  # Here i define the residual dataframe for the holdout
  residuals_holdout = rbind(residuals_holdout_MLR,
                            residuals_holdout_BRNN,
                            residuals_holdout_MT,
                            residuals_holdout_RF)

  residuals_holdout <- filter(residuals_holdout, method %in% methods)


  holdout_calibration_data$DE = 0

  DE_data_MLR <- data.frame(value = DE_predicted, data_holdout_calibration = "holdout", type = "predicted", method = "MLR", DE = 1)
  DE_data_BRNN <- data.frame(value = DE_predicted, data_holdout_calibration = "holdout", type = "predicted", method = "BRNN", DE = 1)
  DE_data_MT <- data.frame(value = DE_predicted, data_holdout_calibration = "holdout", type = "predicted", method = "MT", DE = 1)
  DE_data_RF <- data.frame(value = DE_predicted, data_holdout_calibration = "holdout", type = "predicted", method = "RF", DE = 1)

  DE_data <- rbind(DE_data_MLR,DE_data_BRNN,DE_data_MT,DE_data_RF)

  holdout_calibration_data <- rbind(holdout_calibration_data, DE_data)

  holdout_calibration_data <- dplyr::filter(holdout_calibration_data, method %in% methods)

  holdout_results <- holdout_calibration_data %>%
    group_by(method) %>%
    summarise(r_calibration = cor(value[data_holdout_calibration == "calibration" & type == "predicted"  & DE == 0 ],
                              value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ]),
              r_holdout = cor(value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ],
                           value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ]),

              RMSE_calibration = MLmetrics::RMSE(value[data_holdout_calibration == "calibration" & type == "predicted" & DE == 0 ],
                                             value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ]),
              RMSE_holdout =  MLmetrics::RMSE(value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ],
                                           value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ]),

              RRSE_calibration = MLmetrics::RRSE(value[data_holdout_calibration == "calibration" & type == "predicted" & DE == 0 ],
                                             value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ]),
              RRSE_holdout = MLmetrics::RRSE(value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ],
                                          value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ]),
              d_calibration  = 1 - (sum((value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ] -
                                       value[data_holdout_calibration == "calibration" & type == "predicted" & DE == 0 ]) ^ 2)) /
                sum((abs(value[data_holdout_calibration == "calibration" & type == "predicted" & DE == 0 ] -
                           mean(value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ])) +
                       abs(value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ] -
                             mean(value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ]))) ^ 2),
              d_holdout = 1 - (sum((value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                                   value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ]) ^ 2)) /
                sum((abs(value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ] -
                           mean(value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ])) +
                       abs(value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                             mean(value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ]))) ^ 2),

              RE_holdout = 1 - (sum((value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                                    value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ]) ^ 2) /
                               sum((value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                                      mean(value[data_holdout_calibration == "calibration" & type == "observed" & DE == 0 ])) ^ 2)),

              CE_holdout  = 1 - (sum((value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                                     value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ]) ^ 2) /
                                sum((value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                                       mean(value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ])) ^ 2)),
              DE_holdout = 1 - (sum((value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                                    value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 0 ]) ^ 2) /
                               sum((value[data_holdout_calibration == "holdout" & type == "observed" & DE == 0 ] -
                                      value[data_holdout_calibration == "holdout" & type == "predicted" & DE == 1 ]) ^ 2))


    )


  holdout_results_t <- t(holdout_results[,-1])
  colnames(holdout_results_t) <- holdout_results$method
  holdout_results_t <- round(holdout_results_t, digits)
  holdout_results_t <- data.frame(holdout_results_t, Period =  c("calibration", "holdout",
                                                           "calibration", "holdout",
                                                           "calibration", "holdout",
                                                           "calibration", "holdout",
                                                           "holdout","holdout",
                                                           "holdout"
  ),
  Metric = c("r", "r", "RMSE", "RMSE", "RRSE", "RRSE",
             "d", "d", "RE", "CE", "DE"))
  holdout_results_t <- select(holdout_results_t, Metric, Period, methods)
  row.names(holdout_results_t) <- NULL


} else {
  holdout_results_t <- "No holdout data was defined."
}

###################### Residual diagnostic ###################################################################
residuals_df <- data.frame(residuals = c(), predicted = c(), method = c())

data_observed <- dataset[,DepIndex]

#MLR model
if ("MLR" %in% methods){
MLR_model <- lm(formula, data = dataset)
MLR_predicted <- predict(MLR_model, dataset)
residuals_MLR <- MLR_predicted - data_observed
LOESS_MLR <- loess.smooth(MLR_predicted, residuals_MLR)
residuals_df_MLR <- data.frame(residuals = residuals_MLR,
                               predicted = MLR_predicted,
                               method = "MLR")
residuals_df <- rbind(residuals_df, residuals_df_MLR)

}

# BRNN model
if ("BRNN" %in% methods){
capture.output(BRNN_model <- brnn(formula, data = dataset, BRNN_neurons = BRNN_neurons, verbose = FALSE))
BRNN_predicted <- predict(BRNN_model, dataset)
residuals_BRNN <- BRNN_predicted - data_observed
LOESS_BRNN <- loess.smooth(BRNN_predicted, residuals_BRNN)
residuals_df_BRNN <- data.frame(residuals = residuals_BRNN,
                                predicted = BRNN_predicted,
                                method = "BRNN")
residuals_df <- rbind(residuals_df, residuals_df_BRNN)

}

# MT model
if ("MF" %in% methods){
dataset_ind <- data.frame(dataset[, -DepIndex])
colnames(dataset_ind) <- indep_names
dataset_dep <- dataset[, DepIndex]
MT_model <- cubist(x = dataset_ind, y = dataset_dep, committees = MT_committees,
                   neighbors = MT_neighbors, cubistControl(rules = MT_rules,
                   unbiased = MT_unbiased, extrapolation = MT_extrapolation, sample = MT_sample))
MT_predicted <- predict(MT_model, dataset_ind)
residuals_MT <- MT_predicted - data_observed
LOESS_MT <- loess.smooth(MT_predicted, residuals_MT)
residuals_df_MT <- data.frame(residuals = residuals_MT,
                              predicted = MT_predicted,
                              method = "MT")
residuals_df <- rbind(residuals_df, residuals_df_MT)

}

# RF model
if ("RF" %in% methods){
  RF_model <- randomForest(formula = formula, data = dataset, ntree = RF_ntree,
                           maxnodes = RF_maxnodes, mtry  = RF_mtry,
                           nodesize = RF_nodesize)
  RF_predicted <- predict(RF_model, dataset)
  residuals_RF <- RF_predicted - data_observed
  LOESS_RF <- loess.smooth(RF_predicted, residuals_RF)
  residuals_df_RF <- data.frame(residuals = residuals_RF,
                                predicted = RF_predicted,
                                method = "RF")
  residuals_df <- rbind(residuals_df, residuals_df_RF)
}

# Here we create plots for calibration

Normal_QQ_cal1 <- ggplot(residuals_df, aes(sample = residuals)) +
                    facet_grid(method~.)+ stat_qq() + stat_qq_line() +
                    xlab("Theoretical quantiles") +
                    ylab("Residuals") +
                    ggtitle("Normal Q-Q plot, Calibration data") +
                    theme_bw() +
                    theme(text = element_text(size = 15))

Residuals_vs_fitted_cal1 <- ggplot(residuals_df, aes(y = residuals, x = predicted)) +
  geom_point() +
  stat_smooth(method="loess", se = FALSE)+geom_hline(yintercept=0, col="red", linetype="dashed") +
  facet_grid(method ~ .) +
  xlab("Fitted values") +
  ylab("Residuals") +
  ggtitle("Residual vs Fitted Plot, Calibration data") +
  theme_bw() +
  theme(text = element_text(size = 15), axis.title.y = element_blank())

# Residual plots for the holdout data
if (!is.null(holdout)) {
  Normal_QQ_holdout1 <- ggplot(residuals_holdout, aes(sample = residuals)) +
  facet_grid(method~.)+ stat_qq() + stat_qq_line() +
  xlab("Theoretical quantiles") +
  ylab("Residuals") +
  ggtitle("Normal Q-Q plot, Holdout data") +
  theme_bw() +
  theme(text = element_text(size = 15))

  Residuals_vs_fitted_holdout1 <- ggplot(residuals_holdout, aes(y = residuals, x = predicted)) +
  geom_point() +
  stat_smooth(method="loess", se = FALSE)+geom_hline(yintercept=0, col="red", linetype="dashed") +
  facet_grid(method ~ .) +
  xlab("Fitted values") +
  ylab("Residuals") +
  ggtitle("Residual vs Fitted Plot, Holdout data") +
  theme_bw() +
  theme(text = element_text(size = 15), axis.title.y = element_blank())

} else {
  Normal_QQ_holdout1 <- "No holdout data was defined."
  Residuals_vs_fitted_holdout1 <- "No holdout data was defined."
}

# Residual plots for the edge data
if (numIND < 2 & edge_share > 0){
  Normal_QQ_edge1 <- ggplot(residuals_edge, aes(sample = residuals)) +
    facet_grid(method~.)+ stat_qq() + stat_qq_line() +
    xlab("Theoretical quantiles") +
    ylab("Residuals") +
    ggtitle("Normal Q-Q plot, edge data") +
    theme_bw() +
    theme(text = element_text(size = 15))

  Residuals_vs_fitted_edge1 <- ggplot(residuals_edge, aes(y = residuals, x = predicted)) +
    geom_point() +
    stat_smooth(method="loess", se = FALSE)+geom_hline(yintercept=0, col="red", linetype="dashed") +
    facet_grid(method ~ .) +
    xlab("Fitted values") +
    ylab("Residuals") +
    ggtitle("Residual vs Fitted Plot, edge data") +
    theme_bw() +
    theme(text = element_text(size = 15), axis.title.y = element_blank())
} else {
  Residuals_vs_fitted_edge1 <- "No edge data is available for regression problems with 2 or more independent variables."
  Normal_QQ_edge1 <- "No edge data is available for regression problems with 2 or more independent variables."
}


  #####################################################################

# If Calibration and Validation data should be returned, then this is our final results
  final_list <- list(mean_std = Results_mean_std, ranks = Results_ranks,
                     edge_results = edge_results_t,
                     holdout_results = holdout_results_t,
                     bias_cal = suppressMessages(gg_object_cal),
                     bias_val = suppressMessages(gg_object_val),
                     transfer_functions = suppressMessages(plot_1),
                     transfer_functions_together = suppressMessages(plot_2),
                     parameter_values = parameters,
                     PCA_output = PCA_result,
                     reconstructions = suppressMessages(plot_4),
                     reconstructions_together = suppressMessages(plot_3),
                     normal_QQ_cal = suppressMessages(Normal_QQ_cal1),
                     normal_QQ_holdout = suppressMessages(Normal_QQ_holdout1),
                     normal_QQ_edge = suppressMessages(Normal_QQ_edge1),
                     residuals_vs_fitted_cal = suppressMessages(Residuals_vs_fitted_cal1),
                     residuals_vs_fitted_holdout = suppressMessages(Residuals_vs_fitted_holdout1),
                     residuals_vs_fitted_edge = suppressMessages(Residuals_vs_fitted_edge1),
                     reconstructions_data = recon_df
                     )

return(final_list) # Return the final list

}

#' boot_f_cor
#'
#' Generate R bootstrap replicates of a statistic applied to data.
#'
#' @param data A vector, matrix, or data frame containing the input data.
#' If a matrix or data frame is provided, each row is treated as a separate
#' multivariate observation.
#' @param indices A numeric or integer vector specifying the indices to be
#' utilized in the calculation. These indices determine which elements or rows
#' of the 'data' are to be included in the analysis
#' @param cor.type A string specifying the method of correlation to be applied
#' in the bootstrap procedure. Available options are "pearson" (default),
#' "kendall", or "spearman".
#'
#' @return A matrix with bootstrapped estimates of correlation coefficients
#'
#' @references https://www.datacamp.com/community/tutorials/bootstrap-r
#' @keywords internal
#'
boot_f_cor <- function(data, indices, cor.type){
  dt <- data[indices,]
    c(
      cor(dt[,1], dt[,2], method = cor.type),
      median(dt[,1]),
      median(dt[,2])
    )

  }

#' Monthly sums of precipitation for Ljubljana from 2018 - 1900. Tidy format.
#'
#' A dataset of monthly sums of precipitations in Ljubljana (Slovenia). The first row
#' represents precipitation sum for January 1900.
#'
#' @format A data frame with 1417 rows and 3 variables:
#' \describe{
#'   \item{Year}{year}
#'   \item{Month}{Month}
#'   \item{Precipitation}{Sum of precipitation}
#' }
#' @source \url{http://climexp.knmi.nl/start.cgi}
#' @export
"LJ_monthly_precipitation"

#' monthly_response_seascorr
#'
#' Function calculates all possible partial correlation coefficients between
#' tree-ring chronology and monthly environmental (usually climate) data.
#' All calculated (partial) correlation coefficients are stored in a matrix.
#' The location of stored correlation in the matrix is indicating a window
#' width (row names) and a location in a matrix of monthly sequences of
#' environmental data (column names).
#'
#' @param response a data frame with tree-ring proxy variable and (optional)
#' years as row names. Row.names should be matched with those from env_data_primary
#' and env_data_control data frame. If not, set the row_names_subset argument to
#' TRUE.
#' @param env_data_primary primary data frame of monthly sequences of environmental
#' data as columns and years as row names. Each row represents a year and
#' each column represents a day of a year. Row.names should be matched with
#' those from the response data frame. If not, set the argument row_names_subset
#' to TRUE. Alternatively, env_data_primary could be a tidy data with three columns,
#' i.e. Year, Month and third column representing values of mean temperatures,
#' sum of precipitation etc. If tidy data is passed to the function, set the argument
#' tidy_env_data_primary to TRUE.
#' @param env_data_control a data frame of monthly sequences of environmental data as
#' columns and years as row names. This data is used as control for calculations of
#' partial correlation coefficients. Each row represents a year and each column
#' represents a day of a year. Row.names should be matched with those from the
#' response data frame. If not, set the row_names_subset argument to TRUE.
#' Alternatively, env_data_control could be a tidy data with three columns,
#' i.e. Year, Month and third column representing values of mean temperatures, sum
#' of precipitation etc. If tidy data is passed to the function, set the argument
#' tidy_env_data_control to TRUE.
#' @param pcor_method a character string indicating which partial correlation
#' coefficient is to be computed. One of "pearson" (default), "kendall", or
#' "spearman", can be abbreviated.
#' @param lower_limit lower limit of window width (i.e. number of consecutive months
#' to be used for calculations)
#' @param upper_limit upper limit of window width (i.e. number of consecutive months
#' to be used for calculations)
#' @param fixed_width fixed width used for calculations (i.e. number of consecutive
#' months to be used for calculations)
#' @param previous_year logical. If TRUE, previous-year climate data are included
#' in the analysis. If FALSE, no previous-year climate data are included and
#' number_previous_years is ignored.
#' @param number_previous_years integer between 1 and 5 specifying how many
#' previous years should be included in the environmental matrices when
#' previous_year = TRUE. For example, number_previous_years = 2 uses climate
#' data from years t - 2, t - 1 and t for response year t. If NULL and
#' previous_year = TRUE, one previous year is included.
#' @param reference_window character string, the reference_window argument describes,
#' how each calculation is referred. There are two different options: 'start'
#' (default) and 'end'. If the reference_window argument is set to 'start',
#' then each calculation is related to the starting month of window. If the
#' reference_window argument is set to 'end', then each calculation is related
#' to the ending day of window calculation.
#' @param remove_insignificant if set to TRUE, removes all correlations bellow
#' the significant threshold level, based on a selected alpha.
#' @param alpha significance level used to remove insignificant calculations.
#' @param row_names_subset if set to TRUE, row.names are used to subset
#' env_data_primary, env_data_control and response data frames. Only years from
#' all three data frames are kept.
#' @param aggregate_function_env_data_primary character string specifying how the
#' monthly data from env_data_primary should be aggregated. The default is 'mean',
#' the other options are 'median', 'sum' and 'quantile'.
#' @param quantile_prob_env_data_primary numeric value between 0 and 1 specifying
#' the quantile probability used when aggregate_function_env_data_primary =
#' 'quantile'. For example, 0.95 calculates the 95th percentile. The default is 0.5.
#' @param aggregate_function_env_data_control character string specifying how the
#' monthly data from env_data_control should be aggregated. The default is 'mean',
#' the other options are 'median', 'sum' and 'quantile'.
#' @param quantile_prob_env_data_control numeric value between 0 and 1 specifying
#' the quantile probability used when aggregate_function_env_data_control =
#' 'quantile'. For example, 0.95 calculates the 95th percentile. The default is 0.5.
#' @param temporal_stability_check character string, specifying, how temporal stability
#' between the optimal selection and response variable(s) will be analysed. Current
#' possibilities are "sequential", "progressive" and "running_window". Sequential check
#' will split data into k splits and calculate selected metric for each split. Progressive
#' check will split data into k splits, calculate metric for the first split and then
#' progressively add 1 split at a time and calculate selected metric. For running window,
#' select the length of running window with the k_running_window argument.
#' @param k_running_window the length of running window for temporal stability check.
#' Applicable only if temporal_stability argument is set to running window.
#' @param k integer, number of breaks (splits) for temporal stability
#' @param subset_years a subset of years to be analyzed. Should be given in the form of
#' subset_years = c(1980, 2005)
#' @param ylimits limit of the y axes for plot_extreme. It should be given in
#' the form of: ylimits = c(0,1)
#' @param seed optional seed argument for reproducible results
#' @param tidy_env_data_primary if set to TRUE, env_data_primary should be inserted as a
#' data frame with three columns: "Year", "Month", "Precipitation/Temperature/etc."
#' @param tidy_env_data_control if set to TRUE, env_data_control should be inserted as a
#' data frame with three columns: "Year", "Month", "Precipitation/Temperature/etc."
#' @param boot logical, if TRUE, bootstrap procedure will be used to calculate
#' partial correlation coefficients
#' @param boot_n The number of bootstrap replicates
#' @param boot_ci_type A character string representing the type of bootstrap intervals
#' required. The value should be any subset of the values c("norm","basic", "stud",
#' "perc", "bca").
#' @param boot_conf_int A scalar or vector containing the confidence level(s) of
#' the required interval(s)
#' @param month_interval a vector of two values defining the interval of months
#' used for calculations. Positive values indicate months in the current year.
#' Negative values indicate months in the previous-year block and are used only
#' when previous_year = TRUE. If previous_year = FALSE and negative values are
#' supplied, month_interval is ignored and the analysis is performed for the
#' current year only using month_interval = c(1, 12). If number_previous_years > 1,
#' the previous-year block starts with the earliest included previous year. For
#' example, previous_year = TRUE, number_previous_years = 2 and
#' month_interval = c(-1, 12) analyses the full sequence from January of year
#' t - 2 to December of year t.
#' @param dc_method a character string to determine the method to detrend climate
#' data.  Possible values are "none" (default) and "SLD" which refers to Simple
#' Linear Detrending
#' @param pcor_na_use an optional character string giving a method for computing
#' covariances in the presence of missing values for partial correlation
#' coefficients. This must be (an abbreviation of) one of the strings "all.obs",
#' "everything", "complete.obs", "na.or.complete", or "pairwise.complete.obs"
#' (default). See also the documentation for the base partial.r in psych R package
#'
#' @return a list with 17 elements:
#' \enumerate{
#'  \item $calculations - a matrix with calculated metrics
#'  \item $method - the character string of a method
#'  \item $metric - the character string indicating the metric used for calculations
#'  \item $analysed_period - the character string specifying the analysed period based on the information from row names. If there are no row names, this argument is given as NA
#'  \item $optimized_return - data frame with two columns, response variable and aggregated (averaged) monthly data that return the optimal results. This data.frame could be directly used to calibrate a model for climate reconstruction
#'  \item $optimized_return_all - a data frame with aggregated monthly data, that returned the optimal result for the entire env_data_primary (and not only subset of analysed years)
#'  \item $transfer_function - a ggplot object: scatter plot of optimized return and a transfer line of the selected method
#'  \item $temporal_stability - a data frame with calculations of selected metric for different temporal subsets
#'  \item $cross_validation - not available for partial correlation method
#'  \item $plot_heatmap - ggplot2 object: a heatmap of calculated metrics
#'  \item $plot_extreme - ggplot2 object: line plot of a row with the highest value in a matrix of calculated metrics
#'  \item $type - the character string describing type of analysis: monthly or monthly
#'  \item $reference_window - character string, which reference window was used for calculations
#'  \item $aggregated_climate_primary - matrix with all aggregated climate series of primary data
#'  \item $aggregated_climate_control - matrix with all aggregated climate series of control data
#'  \item $previous_year - logical indicating whether previous-year climate data were used
#'  \item $number_previous_years - integer indicating how many previous years were used
#'}
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Load the dendroTools R package
#' library(dendroTools)
#'
#' # Load data
#' data(data_MVA)
#' data(data_TRW)
#' data(data_TRW_1)
#' data(example_proxies_individual)
#' data(example_proxies_1)
#' data(LJ_monthly_temperatures)
#' data(LJ_monthly_precipitation)
#'
#' # 1 Basic example
#' example_basic <- monthly_response_seascorr(response = data_MVA,
#'    fixed_width = 11,
#'    env_data_primary = LJ_monthly_temperatures,
#'    env_data_control = LJ_monthly_precipitation,
#'    row_names_subset = TRUE,
#'    remove_insignificant = TRUE,
#'    reference_window = "start",
#'    aggregate_function_env_data_primary = 'median',
#'    aggregate_function_env_data_control = 'median',
#'    alpha = 0.05, pcor_method = "spearman",
#'    tidy_env_data_primary = FALSE,
#'    tidy_env_data_control = TRUE,
#'    previous_year = TRUE)
#'
#' # summary(example_basic)
#' # plot(example_basic, type = 1)
#' # plot(example_basic, type = 2)
#' # example_basic$optimized_return
#' # example_basic$optimized_return_all
#' # example_basic$temporal_stability
#'
#' # 2 Extended example
#' example_extended <- monthly_response_seascorr(response = data_MVA,
#'    env_data_primary = LJ_monthly_temperatures,
#'    env_data_control = LJ_monthly_precipitation,
#'    row_names_subset = TRUE, dc_method = "SLD",
#'    remove_insignificant = FALSE,
#'    aggregate_function_env_data_primary = 'mean',
#'    aggregate_function_env_data_control = 'mean',
#'    alpha = 0.05, pcor_na_use = "pairwise.complete",
#'    reference_window = "end",
#'    tidy_env_data_primary = FALSE,
#'    tidy_env_data_control = TRUE)
#'
#' # summary(example_extended)
#' # plot(example_extended, type = 1)
#' # plot(example_extended, type = 2)
#' # example_extended$optimized_return
#' # example_extended$optimized_return_all
#'
#' # 3 Example using quantiles for primary and control variables
#' example_quantile <- monthly_response_seascorr(
#'    response = data_MVA,
#'    env_data_primary = LJ_monthly_temperatures,
#'    env_data_control = LJ_monthly_precipitation,
#'    row_names_subset = TRUE,
#'    fixed_width = 3,
#'    previous_year = TRUE,
#'    aggregate_function_env_data_primary = "quantile",
#'    quantile_prob_env_data_primary = 0.50,
#'    aggregate_function_env_data_control = "quantile",
#'    quantile_prob_env_data_control = 0.50,
#'    tidy_env_data_primary = FALSE,
#'    tidy_env_data_control = TRUE)
#'
#'  # summary(example_quantile)
#'
#' }

monthly_response_seascorr <- function(response, env_data_primary, env_data_control,
                                      previous_year = FALSE, number_previous_years = NULL,
                                      pcor_method = "pearson",
                                      remove_insignificant = TRUE, lower_limit = 1,
                                      upper_limit = 12, fixed_width = 0,
                                      alpha = .05, row_names_subset = FALSE,
                                      reference_window = "start",
                                      aggregate_function_env_data_primary = 'mean',
                                      quantile_prob_env_data_primary = 0.5,
                                      aggregate_function_env_data_control = 'mean',
                                      quantile_prob_env_data_control = 0.5,
                                      temporal_stability_check = "sequential", k = 2,
                                      k_running_window = 30,
                                      subset_years = NULL,
                                      ylimits = NULL, seed = NULL, tidy_env_data_primary = FALSE,
                                      tidy_env_data_control = FALSE,  boot = FALSE, boot_n = 1000,
                                      boot_ci_type = "norm", boot_conf_int = 0.95,
                                      month_interval = NULL,
                                      dc_method = NULL,
                                      pcor_na_use = "pairwise.complete") {

  ##############################################################################
  # 1 month interval is organized

  months_per_year <- 12L

  if (reference_window == "middle"){

    stop(paste0("reference_window should be 'start' or 'end'. 'middle' reference_window is not implemented for the monthly_response_seascorr"))
  }

  # previous_year is the master switch for previous-year analyses.
  # If previous_year = FALSE, number_previous_years is ignored.
  if (!is.logical(previous_year) ||
      length(previous_year) != 1 ||
      is.na(previous_year)) {
    stop("previous_year must be either TRUE or FALSE.")
  }

  if (isFALSE(previous_year)) {

    if (!is.null(number_previous_years) &&
        is.numeric(number_previous_years) &&
        length(number_previous_years) == 1 &&
        !is.na(number_previous_years) &&
        number_previous_years > 0) {
      warning(paste0("number_previous_years is ignored because ",
                     "previous_year = FALSE."))
    }

    number_previous_years <- 0L

  } else {

    # Backward compatibility:
    # previous_year = TRUE still means one previous year unless
    # number_previous_years is explicitly supplied.
    if (is.null(number_previous_years)) {
      number_previous_years <- 1L
    }

    if (!is.numeric(number_previous_years) ||
        length(number_previous_years) != 1 ||
        is.na(number_previous_years) ||
        number_previous_years %% 1 != 0 ||
        number_previous_years < 1 ||
        number_previous_years > 5) {
      stop(paste0("When previous_year = TRUE, number_previous_years must be ",
                  "a single integer between 1 and 5."))
    }

    number_previous_years <- as.integer(number_previous_years)
  }

  year_block_width <- months_per_year * (number_previous_years + 1L)

  # Default interval:
  # current year only: c(1, 12)
  # previous-year analysis: c(-1, 12), i.e. from January of the earliest
  # included previous year to December of the current year.
  if (is.null(month_interval)) {
    month_interval <- if (isTRUE(previous_year)) c(-1, months_per_year) else c(1, months_per_year)
  }

  if (!is.numeric(month_interval) ||
      length(month_interval) != 2 ||
      any(is.na(month_interval))) {
    stop("month_interval must be a numeric vector of length 2.")
  }

  offset_start <- month_interval[1]
  offset_end <- month_interval[2]

  if (offset_start == 0 || offset_end == 0) {
    stop("month_interval cannot contain 0. Use negative values for previous-year months and positive values for current-year months.")
  }

  # Negative values in month_interval would imply previous-year climate data.
  # If previous_year = FALSE, previous-year data are not used. In this case,
  # ignore month_interval and use the full current year.
  if (isFALSE(previous_year) && (offset_start < 0 || offset_end < 0)) {

    warning(paste0("Negative values were supplied in month_interval, but ",
                   "previous_year = FALSE. The month_interval argument is ignored ",
                   "and the analysis is performed for the current year only ",
                   "using month_interval = c(1, 12)."))

    month_interval <- c(1, months_per_year)
    offset_start <- month_interval[1]
    offset_end <- month_interval[2]
  }

  # If both limits are positive while previous_year = TRUE, previous years are
  # not actually included in the selected interval. In that case, keep the
  # analysis current-year only.
  if (offset_start > 0 && offset_end > 0 && isTRUE(previous_year)) {

    number_previous_years <- 0L
    previous_year <- FALSE
    year_block_width <- months_per_year

    warning(paste0("Previous years are not included in selected month_interval. ",
                   "The analysis is performed for the current year only."))
  }

  # Convert the user-facing month_interval to column positions in the lagged matrix.
  # For number_previous_years = 2, the internal matrix is:
  # columns 1:12     = year t - 2
  # columns 13:24    = year t - 1
  # columns 25:36    = year t
  if (offset_start < 0 && offset_end < 0) {

    offset_start <- abs(offset_start)
    offset_end <- abs(offset_end)

  } else if (offset_start < 0 && offset_end > 0) {

    offset_start <- abs(offset_start)
    offset_end <- offset_end + months_per_year * number_previous_years

  } else if (offset_start > 0 && offset_end > 0) {

    # Current-year-only interval; already in current-year coordinates.
    offset_start <- offset_start
    offset_end <- offset_end

  } else {

    stop("month_interval must not run from current-year months back to previous-year months.")
  }

  if (offset_start > offset_end) {
    stop("month_interval is invalid after conversion. The start of the interval is after the end.")
  }

  if (offset_start < 1 || offset_end > year_block_width) {
    stop(paste0("month_interval is outside the available climate sequence. ",
                "For number_previous_years = ", number_previous_years,
                ", the available width is ", year_block_width, " months."))
  }

  # Calculate the max_window allowed
  max_window <- offset_end - offset_start + 1

  # If max_window is smaller than upper_limit, upper_limit must be reduced
  if (upper_limit > max_window){

    upper_limit <- max_window

    if (fixed_width == 0){
      warning(paste0("The upper_limit is outside your month_interval and",
                     " therefore reduced to the maximum allowed: ", max_window, "."))
    }
  }

  if (lower_limit > max_window){
    lower_limit <- max_window

    if (fixed_width == 0){
      warning(paste0("The lower_limit is outside your month_interval and",
                     " therefore reduced to the minimum allowed: ", max_window, "."))
    }
  }

  # Also correction for fixed_window approach - can only be ERROR
  if (fixed_width > max_window){

    stop(paste0("The selected fixed_width is outside your month_interval. ",
                "Decrease the fixed_width argument to at least: ", max_window, "."))
  }

  # offset_end is converted to the number of months that must be skipped at the
  # end of the full environmental matrix.
  offset_end <- year_block_width - offset_end

  ################################################################################

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Defining global variables
  median <- NULL
  proxy <- NULL
  optimized_return <- NULL
  transfer_f <- NULL
  journal_theme <- NULL
  CV <- NULL
  Period <- NULL
  Years <- NULL
  yearABC <- NULL
  RMSE <- NULL
  RE <- NULL
  CE <- NULL
  DE <- NULL
  d <- NULL
  env_data_primary_control <- NULL
  metric <- NULL
  temporal_matrix_lower <- NULL
  temporal_matrix_upper <- NULL

  # Check selected aggregation functions
  allowed_aggregate_functions <- c("mean", "median", "sum", "quantile")

  check_aggregate_function <- function(aggregate_function, argument_name) {
    if (!(aggregate_function %in% allowed_aggregate_functions)) {
      stop(paste0(
        argument_name, " is '", aggregate_function,
        "'. Instead it should be one of: ",
        paste(allowed_aggregate_functions, collapse = ", "), "."
      ))
    }
  }

  check_quantile_prob <- function(quantile_prob, argument_name) {
    if (!is.numeric(quantile_prob) ||
        length(quantile_prob) != 1 ||
        is.na(quantile_prob) ||
        quantile_prob < 0 ||
        quantile_prob > 1) {
      stop(paste0(argument_name, " must be a single numeric value between 0 and 1."))
    }
  }

  check_aggregate_function(aggregate_function_env_data_primary,
                           "aggregate_function_env_data_primary")
  check_aggregate_function(aggregate_function_env_data_control,
                           "aggregate_function_env_data_control")
  check_quantile_prob(quantile_prob_env_data_primary,
                      "quantile_prob_env_data_primary")
  check_quantile_prob(quantile_prob_env_data_control,
                      "quantile_prob_env_data_control")

  # Internal helper for aggregating monthly data across rows
  aggregate_monthly_window <- function(x, aggregate_function, quantile_prob) {

    x <- data.frame(x)

    if (aggregate_function == "mean") {
      return(rowMeans(x, na.rm = TRUE))
    }

    if (aggregate_function == "median") {
      return(apply(x, 1, median, na.rm = TRUE))
    }

    if (aggregate_function == "sum") {
      return(apply(x, 1, sum, na.rm = TRUE))
    }

    if (aggregate_function == "quantile") {
      return(apply(
        x, 1, quantile,
        probs = quantile_prob,
        na.rm = TRUE,
        names = FALSE,
        type = 7
      ))
    }
  }

  aggregate_monthly_window_primary <- function(x) {
    aggregate_monthly_window(x,
                             aggregate_function = aggregate_function_env_data_primary,
                             quantile_prob = quantile_prob_env_data_primary)
  }

  aggregate_monthly_window_control <- function(x) {
    aggregate_monthly_window(x,
                             aggregate_function = aggregate_function_env_data_control,
                             quantile_prob = quantile_prob_env_data_control)
  }


  # Internal helper for constructing a multi-year climate matrix.
  # Rows are current response years. Columns are ordered from the oldest previous
  # year to the current year.
  build_lagged_env_data <- function(x, n_previous_years) {

    x <- data.frame(x, check.names = FALSE)

    if (n_previous_years == 0L) {
      return(x)
    }

    env_years <- suppressWarnings(as.integer(row.names(x)))

    if (any(is.na(env_years))) {
      stop("For previous-year analyses, row.names of environmental data must be calendar years.")
    }

    if (any(duplicated(env_years))) {
      stop("Duplicated years are present in row.names of environmental data.")
    }

    row.names(x) <- as.character(env_years)
    x <- x[order(env_years), , drop = FALSE]
    env_years <- as.integer(row.names(x))

    current_years <- env_years[
      vapply(env_years, function(y) {
        all((y - n_previous_years):y %in% env_years)
      }, logical(1))
    ]

    if (length(current_years) == 0L) {
      stop(paste0("No years in environmental data have the required ",
                  n_previous_years, " previous year(s)."))
    }

    lag_blocks <- lapply(seq(n_previous_years, 0L), function(lag_i) {

      block <- x[as.character(current_years - lag_i), , drop = FALSE]
      row.names(block) <- as.character(current_years)
      colnames(block) <- paste0("Y", -lag_i, "_", colnames(x))

      block
    })

    out <- do.call(cbind, lag_blocks)
    row.names(out) <- as.character(current_years)

    data.frame(out, check.names = FALSE)
  }

  # lower_limit = 1
  # upper_limit = 12
  # fixed_width = 0

  if (reference_window == "middle"){

    stop(paste0("reference_window should be 'start' or 'end'. 'middle' reference_window is not implemented for the monthly_response"))
  }

  # if fixed width is not 0, then change lower and upper limits, just to avoid
  # error messages
  if (fixed_width != 0){
    lower_limit = 1
    upper_limit = 12
  }

  # If there is a column name samp.depth in response data frame, warning is given
  if ("samp.depth" %in% colnames(response)){

    samp.depth_index <- grep("samp.depth", colnames(response))
    response <- response[, -samp.depth_index,F]

    warning("Removed the samp.depth from response data frame")
  }

  # If there is more than 2 columns in response data frame, give a warning
  if (ncol(response) > 1){
    warning(paste0("Your response data frame has more than 1 column! Are you doing a multiproxy research?",
                   " If so, OK. If not, check your response data frame!"))
  }


  # If env_data_control is given in tidy version, transformation is needed
  if (tidy_env_data_control == TRUE){

    n_col_tidy_DF <- ncol(env_data_control)
    colnames_tidy_DF <- colnames(env_data_control)

    if (ncol(env_data_control) != 3){
      stop(paste("env_data_control was inserted in tidy version (tidy_env_data_control is set to TRUE).",
                 "env_data_control should have 3 columns, but it has", n_col_tidy_DF, "instead!"))
    }

    if (colnames_tidy_DF[1] != "Year"){
      stop(paste("env_data_control was inserted in tidy version (tidy_env_data_control is set to TRUE).",
                 "The first column name of the env_data_control should be 'Year', but it is",
                 colnames_tidy_DF[1], "instead!"))
    }

    if (colnames_tidy_DF[2] != "Month"){
      stop(paste("env_data_control was inserted in tidy version (tidy_env_data_control is set to TRUE).",
                 "The second column name of the env_data_control should be 'Month', but it is",
                 colnames_tidy_DF[2], "instead!"))
    }

    value_variable = colnames(env_data_control)[3]
    env_data_control <- dcast(env_data_control, Year~Month, value.var = value_variable)
    env_data_control <- years_to_rownames(env_data_control, "Year")
  }


  # If env_data_primary is given in tidy version, transformation is needed
  if (tidy_env_data_primary == TRUE){

    n_col_tidy_DF <- ncol(env_data_primary)
    colnames_tidy_DF <- colnames(env_data_primary)

    if (ncol(env_data_primary) != 3){
      stop(paste("env_data_primary was inserted in tidy version (tidy_env_data_primary is set to TRUE).",
                 "env_data_primary should have 3 columns, but it has", n_col_tidy_DF, "instead!"))
    }

    if (colnames_tidy_DF[1] != "Year"){
      stop(paste("env_data_primary was inserted in tidy version (tidy_env_data_primary is set to TRUE).",
                 "The first column name of the env_data_primary should be 'Year', but it is",
                 colnames_tidy_DF[1], "instead!"))
    }

    if (colnames_tidy_DF[2] != "Month"){
      stop(paste("env_data_primary was inserted in tidy version (tidy_env_data_primary is set to TRUE).",
                 "The second column name of the env_data_primary should be 'Month', but it is",
                 colnames_tidy_DF[2], "instead!"))
    }

    value_variable = colnames(env_data_primary)[3]
    env_data_primary <- dcast(env_data_primary, Year~Month, value.var = value_variable)
    env_data_primary <- years_to_rownames(env_data_primary, "Year")
  }



  # PART 1 - general data arrangements, warnings and stops
  # Both objects (response, env_data_primary and env_data_control) are converted
  # to data frames.
  response <- data.frame(response)
  env_data_primary <- data.frame(env_data_primary)
  env_data_control <- data.frame(env_data_control)

  # Here we save the original response data that will be used later
  response_original <- response

  # Previous-year analyses require alignment by calendar year names. This avoids
  # accidental row-position mismatches after adding lagged climate years.
  if (number_previous_years > 0L && row_names_subset == FALSE) {
    row_names_subset <- TRUE
    warning(paste0("Previous-year analyses require alignment by year names. ",
                   "row_names_subset is set to TRUE."))
  }

  # Build current-year or multi-year environmental matrices.
  # For number_previous_years = 0 these return the data unchanged.
  env_data_primary <- build_lagged_env_data(env_data_primary, number_previous_years)
  env_data_control <- build_lagged_env_data(env_data_control, number_previous_years)

  # These objects are used later for optimized_return_all. They should represent
  # the full lagged climate matrices before subset_years is applied.
  env_data_primary_original <- env_data_primary
  env_data_control_original <- env_data_control

  # For metric calculations, all objects need to have the same length,
  # with the exception when row_names_subset is set to TRUE.
  if (nrow(response) != nrow(env_data_primary) & row_names_subset == FALSE)
    stop(paste0("Length of env_data_primary and response records differ",
                " You can use row_names_subset = TRUE"))

  if (nrow(response) != nrow(env_data_control) & row_names_subset == FALSE)
    stop(paste0("Length of env_data_control and response records differ",
                " You can use row_names_subset = TRUE"))

  #######################################################
  # Rules for selected window limits

  if (fixed_width < 0 | fixed_width > year_block_width)
    stop(paste0("fixed_width should be between 0 and ", year_block_width))

  # Correlations could be calculated only for one variable
  if (ncol(response) > 1)
    stop(paste("More than 1 variable in response data frame not suitable ",
               "for 'pcor' method'"))

  if (lower_limit > upper_limit)
    stop("lower_limit can not be higher than upper_limit!")

  if (lower_limit > year_block_width | lower_limit < 1)
    stop(paste0("lower_limit out of bounds! It should be between 1 and ",
                year_block_width))

  if (upper_limit > year_block_width | upper_limit < 1)
    stop(paste0("upper_limit out of bounds! It should be between 1 and ",
                year_block_width))

  # Make sure the selected method is appropriate
  if (!is.null(dc_method)){

    if (!(dc_method %in% c("SLD"))){

      stop(paste0('dc_method should be SLD but instead it is:',dc_method))

    }
  }

  # Warn users in case of missing values. The original threshold was 8 missing
  # months for one 12-month climate year, so it is scaled when multiple
  # previous years are included.
  missing_month_threshold <- 8 * (number_previous_years + 1L)

  # A) env_data_primary
  env_temp_primary <- env_data_primary[row.names(env_data_primary) %in% row.names(response),]

  if (!is.null(subset_years)){
    lower_subset <- subset_years[1]
    upper_subset <- subset_years[2]

    subset_seq <- seq(lower_subset, upper_subset)
    env_temp_primary <- subset(env_temp_primary, row.names(env_temp_primary) %in% subset_seq)
  }

  na_problem <- data.frame(na_sum = rowSums(is.na(env_temp_primary)))
  na_problem <- na_problem[na_problem$na_sum > missing_month_threshold, , F]
  problematic_years <- paste0(row.names(na_problem), sep = "", collapse=", ")

  if (nrow(na_problem) > 0){

    warning(paste0("Problematic years with missing values are present in env_data_primary: ", problematic_years))

  }

  # B) env_data_control
  env_temp_conotrol <- env_data_control[row.names(env_data_control) %in% row.names(response),]

  if (!is.null(subset_years)){
    lower_subset <- subset_years[1]
    upper_subset <- subset_years[2]

    subset_seq <- seq(lower_subset, upper_subset)
    env_temp_conotrol <- subset(env_temp_conotrol, row.names(env_temp_conotrol) %in% subset_seq)
  }

  na_problem <- data.frame(na_sum = rowSums(is.na(env_temp_conotrol)))
  na_problem <- na_problem[na_problem$na_sum > missing_month_threshold, , F]
  problematic_years <- paste0(row.names(na_problem), sep = "", collapse=", ")

  if (nrow(na_problem) > 0){

    warning(paste0("Problematic years with missing values are present in env_data_control: ", problematic_years))

  }

  ##############################################################################

  # Previous-year climate data have already been added by
  # build_lagged_env_data().

  # If row_names_subset == TRUE, data is subset and ordered based on matching
  # row.names. Additionally, number of characters in row.names is checked.
  # There should be at least three characters (assuming years before 100 will
  # never be analysed, there is no such environmental data available)
  if (row_names_subset == TRUE & nchar(row.names(env_data_primary)[1]) >= 3){

    env_data_primary$yearABC <- row.names(env_data_primary)
    env_data_control$yearABC <- row.names(env_data_control)
    response$yearABC <- row.names(response)

    intersects <- intersect(intersect(response$yearABC, env_data_control$yearABC), env_data_primary$yearABC)

    response <- dplyr::filter(response, yearABC %in% intersects)
    response <- dplyr::arrange(response, -as.numeric(yearABC))
    response <- years_to_rownames(response, "yearABC")

    env_data_primary <- dplyr::filter(env_data_primary, yearABC %in% intersects)
    env_data_primary <- dplyr::arrange(env_data_primary, -as.numeric(yearABC))
    env_data_primary <- years_to_rownames(env_data_primary, "yearABC")

    env_data_control <- dplyr::filter(env_data_control, yearABC %in% intersects)
    env_data_control <- dplyr::arrange(env_data_control, -as.numeric(yearABC))
    env_data_control <- years_to_rownames(env_data_control, "yearABC")

  }

  # if row.names of env_data_primary and the response data frames are not equal,
  # warning is given.
  if (sum(row.names(env_data_primary) == row.names(response)) != nrow(env_data_primary)) {
    warning("row.names between env_data_primary and response do not match!")
  }

  # If row_names_subset == TRUE, but row.names does not appear to be years,
  # error is given.
  if (row_names_subset == TRUE & nchar(row.names(env_data_primary)[1]) < 3){
    stop(paste("row.names does not appear to be years!",
               "At least three characters needed!"))
  }

  # Subset of years
  if (!is.null(subset_years)){
    lower_subset <- subset_years[1]
    upper_subset <- subset_years[2]

    if (lower_subset > upper_subset){
      stop("Change the order of elements in the subset_years argument! First element should be lower than the second!")
    }

    subset_seq <- seq(lower_subset, upper_subset)


    if (any(!(subset_seq %in% row.names(response)))){

      stop(paste0("Undefined columns selected. Subset years don't exist",
                  " in the response data frame. Change the subset_years argument"))
    }

    if (any(!(subset_seq %in% row.names(env_data_primary)))){

      stop(paste0("Undefined columns selected. Subset years don't exist",
                  " in the env_data_primary data frame. Change the subset_years argument"))
    }

    if (any(!(subset_seq %in% row.names(env_data_control)))){

      stop(paste0("Undefined columns selected. Subset years don't exist",
                  "in the env_data_control data frame. Change the subset_years argument"))
    }

    response <- subset(response, row.names(response) %in% subset_seq)
    env_data_primary <- subset(env_data_primary, row.names(env_data_primary) %in% subset_seq)
    env_data_control <- subset(env_data_control, row.names(env_data_control) %in% subset_seq)
  }

  # NA values are not allowed and must be removed from response data.frame
  # exception if cor_na_us accounts for missing values
  if (sum(is.na(response)) > 0 & !(pcor_na_use %in% c("complete.obs", "na.or.complete", "pairwise.complete.obs"))){

    prob_year <- row.names(response[is.na(response), , drop = F])

    stop(paste0("NA is not allowed in response data frame. ",
                "Problematic year is ", prob_year))

  }


  # PART 2 - the calculation of day-wise correlations
  # A) fixed window approach
  # B) Variable window approach

  # these are lists for climate and and holder for saving mm1 and mm2
  list_climate_primary <- list()
  list_climate_control <- list()

  mm1 <- 1
  mm2 <- 1

  # A) The fixed window approach
  if (fixed_width != 0) {

    # This is an empty matrix, currently filled with NA's
    # Latter, calculations will be saved in this matrix
    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = (ncol(env_data_primary) - fixed_width) + 1)
    } else if (reference_window == 'end') {
      temporal_matrix <- matrix(NA, nrow = 1, ncol = (ncol(env_data_primary)))
    } else if (reference_window == 'middle') {
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = round2((ncol(env_data_primary) - fixed_width +
                                                 1 + fixed_width/2 ),0))
    }

    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    if (fixed_width != max_window){

      if(interactive()){

        pb <- txtProgressBar(min = 0, max = (ncol(env_data_primary) - fixed_width - offset_end - offset_start + 1),
                             style = 3)}

    }

    b = 0

    # An iterating loop. In each iteration x is calculated and represents
    # response (dependent) variable. X is a moving average. Window width of
    # a moving window is fixed_width. Next, partial correlation is stored in
    # temporal matrix.

    for (j in (0 + offset_start -1): (ncol(env_data_primary) - max((fixed_width + offset_end), offset_end))) {

      b = b + 1

      x1 <- aggregate_monthly_window_primary(
        env_data_primary[1:nrow(env_data_primary),
                         (1 + j):(j + fixed_width),
                         drop = FALSE]
      )

      x2 <- aggregate_monthly_window_control(
        env_data_control[1:nrow(env_data_control),
                         (1 + j):(j + fixed_width),
                         drop = FALSE]
      )

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          tmp_model <- lm(x1 ~ seq(1:length(x1)))
          tmp_pred <- predict(tmp_model)

          if (length(x1) != length(tmp_pred)) {
            warning("Note missing values in your env_data")
          }

          tmp_res <- suppressWarnings(x1 - tmp_pred)

          x1 <- data.frame(x1 = tmp_res/sd(tmp_res, na.rm = TRUE))

        }

      } else {

        x1 <- matrix(x1, nrow = nrow(env_data_primary), ncol = 1)

      }

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          tmp_model <- lm(x2 ~ seq(1:length(x2)))
          tmp_pred <- predict(tmp_model)

          if (length(x2) != length(tmp_pred)) {
            warning("Note missing values in your env_data")
          }

          tmp_res <- suppressWarnings(x2 - tmp_pred)

          x2 <- data.frame(x2 = tmp_res/sd(tmp_res, na.rm = TRUE))

        }

      } else {

        x2 <- matrix(x2, nrow = nrow(env_data_primary), ncol = 1)

      }

      my_temporal_data <- cbind(response[, 1], x1, x2)
      colnames(my_temporal_data) <- c("x", "y", "z")

      x1_list <- x1
      colnames(x1_list) <- paste0(j + 1, "_" ,j + fixed_width)
      row.names(x1_list) <- row.names(env_data_primary)
      list_climate_primary[[mm1]] <- x1_list
      mm1 = mm1 + 1

      x2_list <- x2
      colnames(x2_list) <- paste0(j + 1, "_" ,j + fixed_width)
      row.names(x2_list) <- row.names(env_data_control)
      list_climate_control[[mm2]] <- x2_list
      mm2 = mm2 + 1

      if (boot == FALSE){

        temporal_correlation <- partial.r(data=my_temporal_data, x=c("x","y"), y="z",
                                          use=pcor_na_use,method = pcor_method)[2]

        if (reference_window == 'start'){
          temporal_matrix[1, j + 1] <- as.numeric(temporal_correlation)[1]
        } else if (reference_window == 'end'){
          temporal_matrix[1, j + fixed_width] <- as.numeric(temporal_correlation)[1]
        } else if (reference_window == 'middle'){
          temporal_matrix[1, round2(j + 1 + fixed_width/2, 0)] <- as.numeric(temporal_correlation)[1]
        }

      } else if (boot == TRUE){

        calc <- boot(data = my_temporal_data, statistic = boot_f_pcor, R = boot_n, cor.type = pcor_method)

        temporal_correlation <- colMeans(calc$t)[1]
        ci_int <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type), silent = TRUE)

        if (class(ci_int)[[1]] == "try-error"){

          temporal_lower <- NA
          temporal_upper <- NA

        } else {

          if (boot_ci_type == "norm"){

            temporal_lower <- ci_int$norm[2]
            temporal_upper <- ci_int$norm[3]

          } else if (boot_ci_type == "perc"){

            temporal_lower <- ci_int$perc[4]
            temporal_upper <- ci_int$perc[5]

          } else if (boot_ci_type == "stud") {

            temporal_lower <- ci_int$student[4]
            temporal_upper <- ci_int$student[5]

          } else if (boot_ci_type == "basic") {

            temporal_lower <- ci_int$basic[4]
            temporal_upper <- ci_int$basic[5]

          } else if (boot_ci_type == "bca") {

            temporal_lower <- ci_int$bca[4]
            temporal_upper <- ci_int$bca[5]

          } else {

            stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

          }
        }

        if (reference_window == 'start'){
          temporal_matrix[1, j + 1] <- temporal_correlation
          temporal_matrix_lower[1, j + 1] <- temporal_lower
          temporal_matrix_upper[1, j + 1] <- temporal_upper
        } else if (reference_window == 'end'){
          temporal_matrix[1, j + fixed_width] <- temporal_correlation
          temporal_matrix_lower[1, j + fixed_width] <- temporal_lower
          temporal_matrix_upper[1, j + fixed_width] <- temporal_upper
        } else if (reference_window == 'middle'){
          temporal_matrix[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_correlation
          temporal_matrix_lower[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_lower
          temporal_matrix_upper[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_upper
        }

      } else {
        print(paste0("boot should be TRUE or FALSE, instead it is ", boot))
      }

      if(interactive()){

        if (fixed_width != max_window){setTxtProgressBar(pb, b)}

      }
    }

    if(interactive()){

      if (fixed_width != max_window){close(pb)}

    }

    # temporal_matrix is given rownames and colnames. Rownames represent a
    # window width used for calculations. Colnames represent the position of
    # moving window in a original env_data_primary data frame.
    row.names(temporal_matrix) <- fixed_width
    row.names(temporal_matrix_lower) <- fixed_width
    row.names(temporal_matrix_upper) <- fixed_width

    temporal_colnames <- as.vector(seq(from = 1,
                                       to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames
  }


  # B fixed_width == 0, in this case, lower_limit and upper_limit arguments
  # will be used to define window width of a moving window.

  if (fixed_width == 0) {

    # This is an empty matrix, currently filled with NA's
    # Latter, calculations will be saved in this matrix

    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data_primary) - lower_limit) + 1)
    } else if (reference_window == 'end'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data_primary)))
    } else if (reference_window == 'middle'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = round2((ncol(env_data_primary) - lower_limit +
                                                 1 + lower_limit/2 ),0))
    }

    # An iterating double loop: 1 outer loop) iterating from lower_limit to
    # upper_limit defines window width used for a moving window. 2) inner loop
    # defines the starting position of a moving window.
    # In each iteration, x is calculated and represents a response (dependent)
    # variable. x is a moving average, based on rowMeans/apply function.
    # Next, statistical metric is calculated based on a selected method (pcor).
    # Calculation is stored in temporal matrix in a proper place. The position of
    # stored calculation is informative later used for indicating optimal values.

    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    if (upper_limit != lower_limit){

      if(interactive()){

        pb <- txtProgressBar(min = 0, max = (upper_limit - lower_limit), style = 3)

      }
    }

    b = 0

    for (K in lower_limit:upper_limit) {

      b = b + 1

      for (j in (0 + offset_start -1): (ncol(env_data_primary) - max((K + offset_end), offset_end))) {

        x1 <- aggregate_monthly_window_primary(
          env_data_primary[1:nrow(env_data_primary),
                           (1 + j):(j + K),
                           drop = FALSE]
        )

        x2 <- aggregate_monthly_window_control(
          env_data_control[1:nrow(env_data_control),
                           (1 + j):(j + K),
                           drop = FALSE]
        )

        if (!is.null(dc_method)){

          if (dc_method == "SLD"){

            tmp_model <- lm(x1 ~ seq(1:length(x1)))
            tmp_pred <- predict(tmp_model)

            if (length(x1) != length(tmp_pred)) {
              warning("Note missing values in your env_data")
            }

            tmp_res <- suppressWarnings(x1 - tmp_pred)

            x1 <- data.frame(x1 = tmp_res/sd(tmp_res, na.rm = TRUE))

          }

        } else {

          x1 <- matrix(x1, nrow = nrow(env_data_primary), ncol = 1)

        }

        if (!is.null(dc_method)){

          if (dc_method == "SLD"){

            tmp_model <- lm(x2 ~ seq(1:length(x2)))
            tmp_pred <- predict(tmp_model)

            if (length(x2) != length(tmp_pred)) {
              warning("Note missing values in your env_data")
            }

            tmp_res <- suppressWarnings(x2 - tmp_pred)

            x2 <- data.frame(x2 = tmp_res/sd(tmp_res, na.rm = TRUE))

          }

        } else {

          x2 <- matrix(x2, nrow = nrow(env_data_primary), ncol = 1)

        }

        my_temporal_data <- cbind(response[, 1], x1, x2)
        colnames(my_temporal_data) <- c("x", "y", "z")

        x1_list <- x1
        colnames(x1_list) <- paste0(j + 1, "_" ,j + K)
        row.names(x1_list) <- row.names(env_data_primary)
        list_climate_primary[[mm1]] <- x1_list
        mm1 = mm1 + 1

        x2_list <- x2
        colnames(x2_list) <- paste0(j + 1, "_" ,j + K)
        row.names(x2_list) <- row.names(env_data_control)
        list_climate_control[[mm2]] <- x2_list
        mm2 = mm2 + 1

        if (boot == FALSE){

          temporal_correlation <- partial.r(data=my_temporal_data, x=c("x","y"), y="z",
                                            use=pcor_na_use,method = pcor_method)[2]

          if (reference_window == 'start'){

            temporal_matrix[(K - lower_limit) + 1, j + 1] <- as.numeric(temporal_correlation)[1]

          } else if (reference_window == 'end'){

            temporal_matrix[(K - lower_limit) + 1, j + K] <- as.numeric(temporal_correlation)[1]

          } else if (reference_window == 'middle'){

            temporal_matrix[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- as.numeric(temporal_correlation)[1]

          }

        } else if (boot == TRUE){

          calc <- boot(data = my_temporal_data, statistic = boot_f_pcor, R = boot_n, cor.type = pcor_method)

          temporal_correlation <- colMeans(calc$t)[1]
          ci_int <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type), silent = TRUE)

          if (class(ci_int)[[1]] == "try-error"){

            temporal_lower <- NA
            temporal_upper <- NA

          } else {
            if (boot_ci_type == "norm"){

              temporal_lower <- ci_int$norm[2]
              temporal_upper <- ci_int$norm[3]

            } else if (boot_ci_type == "perc"){

              temporal_lower <- ci_int$perc[4]
              temporal_upper <- ci_int$perc[5]

            } else if (boot_ci_type == "stud") {

              temporal_lower <- ci_int$student[4]
              temporal_upper <- ci_int$student[5]

            } else if (boot_ci_type == "basic") {

              temporal_lower <- ci_int$basic[4]
              temporal_upper <- ci_int$basic[5]

            } else if (boot_ci_type == "bca") {

              temporal_lower <- ci_int$bca[4]
              temporal_upper <- ci_int$bca[5]

            } else {

              stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

            }
          }


          if (reference_window == 'start'){
            temporal_matrix[(K - lower_limit) + 1, j + 1] <- temporal_correlation
            temporal_matrix_lower[(K - lower_limit) + 1, j + 1] <- temporal_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + 1] <- temporal_upper
          } else if (reference_window == 'end'){
            temporal_matrix[(K - lower_limit) + 1, j + K] <- temporal_correlation
            temporal_matrix_lower[(K - lower_limit) + 1, j + K] <- temporal_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + K] <- temporal_upper
          } else if (reference_window == 'middle'){
            temporal_matrix[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_correlation
            temporal_matrix_lower[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_lower
            temporal_matrix_upper[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_upper
          }

        } else {
          print(paste0("boot should be TRUE or FALSE, instead it is ", boot))
        }
      }

      if(interactive()){

        if (upper_limit != lower_limit){setTxtProgressBar(pb, b)}

      }
    }

    if(interactive()){

      if (upper_limit != lower_limit){close(pb)}

    }

    # temporal_matrix is given rownames and colnames. Rownames represent a
    # window width used fot calculations. Colnames represent the position of
    # moving window in a original env_data_primary data frame.
    temporal_rownames <- as.vector(seq(from = lower_limit, to = upper_limit,
                                       by = 1))
    row.names(temporal_matrix) <- temporal_rownames
    row.names(temporal_matrix_lower) <- temporal_rownames
    row.names(temporal_matrix_upper) <- temporal_rownames

    temporal_colnames <- as.vector(seq(from = 1,
                                       to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames
  }

  # To enhance the visualization, insignificant values
  # are removed if remove_insignificant == TRUE
  if (remove_insignificant == TRUE){
    critical_threshold_cor <- critical_r(nrow(response), alpha = alpha)
    critical_threshold_cor2 <- critical_threshold_cor ^ 2

    temporal_matrix[abs(temporal_matrix) < abs(critical_threshold_cor)] <- NA

  }

  if(is.finite(mean(temporal_matrix, na.rm = TRUE)) == FALSE){

    warning("All calculations are insignificant! Please change the alpha argument.")

    final_list <- list(calculations = temporal_matrix,
                       method = "pcor",
                       metric = pcor_method, analysed_period = NA,
                       optimized_return = NA,
                       optimized_return_all = NA,
                       transfer_function = NA, temporal_stability = NA,
                       cross_validation = NA,
                       plot_heatmap = NA,
                       plot_extreme = NA,
                       type = "monthly",
                       reference_window = reference_window,
                       boot_lower = temporal_matrix_lower,
                       boot_upper = temporal_matrix_upper,
                       aggregated_climate_primary = NA,
                       aggregated_climate_control = NA,
                       previous_year = previous_year,
                       number_previous_years = number_previous_years)

    class(final_list) <- 'dmrs'

    return(final_list)

  } else {

    ########################################################################
    # PART 4: Final list is being created and returned as a function output#
    ########################################################################

    # The first three elements of the final list are already created: calculated
    # values, method and metric used.
    # Here we create the fourth element: the optimal sequence of days that
    # returns the best selected statistical metric. We name it optimal_return.

    # In case of negative correlations, different strategy is applied.
    # For more detailed description see plot_extreme()

    if(is.finite(mean(temporal_matrix, na.rm = TRUE)) == FALSE){
      stop("All calculations are insignificant! Change the alpha argument!")
    }

    overall_max <- max(temporal_matrix, na.rm = TRUE)
    overall_min <- min(temporal_matrix, na.rm = TRUE)

    # absolute vales of overall_maximum and overall_minimum are compared and
    # one of the following two if functions is used
    # There are unimportant warnings produced:
    # no non-missing arguments to max; returning -Inf

    if ((abs(overall_max) >= abs(overall_min)) == TRUE) {

      # maximum value is located. Row indeces are needed to query information
      # about the window width used to calculate the maximum. Column name is
      # needed to query the starting day.
      max_calculation <- max(temporal_matrix, na.rm = TRUE)

      max_result <- suppressWarnings(which.max(apply(temporal_matrix,
                                                     MARGIN = 2, max,
                                                     na.rm = TRUE)))
      plot_column <- max_result
      max_index <- which.max(temporal_matrix[, names(max_result)])
      row_index <- row.names(temporal_matrix)[max_index]
    }

    if ((abs(overall_max) < abs(overall_min)) == TRUE) {

      max_calculation <- min(temporal_matrix, na.rm = TRUE)

      min_result <- suppressWarnings(which.min(apply(temporal_matrix,
                                                     MARGIN = 2, min,
                                                     na.rm = TRUE)))
      plot_column <- min_result
      min_index <- which.min(temporal_matrix[, names(min_result)])
      row_index <- row.names(temporal_matrix)[min_index]
    }


    ###############################################################

    # The fourth return element is being created: rowMeans/apply of optimal sequence.
    # Here we consider options based on reference_window.

    # 1. reference window = "start"
    if (reference_window == 'start'){

      x1 <- data.frame(aggregate_monthly_window_primary(
        env_data_primary[, as.numeric(plot_column):
                           (as.numeric(plot_column) + as.numeric(row_index) - 1),
                         drop = FALSE]
      ))

      x2 <- data.frame(aggregate_monthly_window_control(
        env_data_control[, as.numeric(plot_column):
                           (as.numeric(plot_column) + as.numeric(row_index) - 1),
                         drop = FALSE]
      ))

      ## Once again, the same procedure, to get the optimal sequence, but this time for whole data, not only
      # for the analysed period.

      x1_original <- data.frame(aggregate_monthly_window_primary(
        env_data_primary_original[, as.numeric(plot_column):
                                    (as.numeric(plot_column) + as.numeric(row_index) - 1),
                                  drop = FALSE]
      ))

      x2_original <- data.frame(aggregate_monthly_window_control(
        env_data_control_original[, as.numeric(plot_column):
                                    (as.numeric(plot_column) + as.numeric(row_index) - 1),
                                  drop = FALSE]
      ))

    }

    # Option 2, reference window = "end"
    if (reference_window == 'end'){

      x1 <- data.frame(aggregate_monthly_window_primary(
        env_data_primary[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                           as.numeric(plot_column),
                         drop = FALSE]
      ))

      x2 <- data.frame(aggregate_monthly_window_control(
        env_data_control[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                           as.numeric(plot_column),
                         drop = FALSE]
      ))

      ## Once again, the same procedure, to get the optimal sequence, but this time for whole data, not only
      # for the analysed period.

      x1_original <- data.frame(aggregate_monthly_window_primary(
        env_data_primary_original[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                                    as.numeric(plot_column),
                                  drop = FALSE]
      ))

      x2_original <- data.frame(aggregate_monthly_window_control(
        env_data_control_original[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                                    as.numeric(plot_column),
                                  drop = FALSE]
      ))

    }

    # Third option: reference window = "middle"
    if (reference_window == 'middle'){

      if (as.numeric(row_index) %% 2 == 0){
        adjustment_1 <- 0
        adjustment_2 <- 1
      } else {
        adjustment_1 <- 1
        adjustment_2 <- 2
      }

      x1 <- data.frame(aggregate_monthly_window_primary(
        env_data_primary[, (round2((as.numeric(plot_column) - as.numeric(row_index) / 2)) - adjustment_1):
                           (round2((as.numeric(plot_column) + as.numeric(row_index) / 2)) - adjustment_2),
                         drop = FALSE]
      ))

      x2 <- data.frame(aggregate_monthly_window_control(
        env_data_control[, (round2((as.numeric(plot_column) - as.numeric(row_index) / 2)) - adjustment_1):
                           (round2((as.numeric(plot_column) + as.numeric(row_index) / 2)) - adjustment_2),
                         drop = FALSE]
      ))

      ## Once again, the same procedure, to get the optimal sequence, but this time for whole data, not only
      # for the analysed period.

      x1_original <- data.frame(aggregate_monthly_window_primary(
        env_data_primary_original[, (round2((as.numeric(plot_column) - as.numeric(row_index) / 2)) - adjustment_1):
                                    (round2((as.numeric(plot_column) + as.numeric(row_index) / 2)) - adjustment_2),
                                  drop = FALSE]
      ))

      x2_original <- data.frame(aggregate_monthly_window_control(
        env_data_control_original[, (round2((as.numeric(plot_column) - as.numeric(row_index) / 2)) - adjustment_1):
                                    (round2((as.numeric(plot_column) + as.numeric(row_index) / 2)) - adjustment_2),
                                  drop = FALSE]
      ))

    }

    # Final output list

    if (!is.null(dc_method)){

      if (dc_method == "SLD"){

        x1 <- x1[,1]
        tmp_model <- lm(x1 ~ seq(1:length(x1)))
        tmp_pred <- predict(tmp_model)

        if (length(x1) != length(tmp_pred)) {
          warning("Note missing values in your env_data")
        }

        tmp_res <- suppressWarnings(x1 - tmp_pred)

        x1 <- data.frame(x1 = tmp_res/sd(tmp_res, na.rm = TRUE))

        x2 <- x2[,1]
        tmp_model <- lm(x2 ~ seq(1:length(x2)))
        tmp_pred <- predict(tmp_model)

        if (length(x2) != length(tmp_pred)) {
          warning("Note missing values in your env_data")
        }

        tmp_res <- x2 - tmp_pred

        x2 <- data.frame(x2 = tmp_res/sd(tmp_res, na.rm = TRUE))

      }
    }


    x1_full <- cbind(response, x1, x2)
    colnames(x1_full)[ncol(x1_full)-1] <- "Optimized_return_primary"
    colnames(x1_full)[ncol(x1_full)] <- "Optimized_return_control"

    colnames(x1) <- "Optimized.rowNames.primary"
    colnames(x2) <- "Optimized.rowNames.control"


    # original

    if (!is.null(dc_method)){

      if (dc_method == "SLD"){

        x1_original <- x1_original[,1]
        tmp_model <- lm(x1_original ~ seq(1:length(x1_original)))
        tmp_pred <- predict(tmp_model)
        tmp_res <- suppressWarnings(x1_original - tmp_pred)
        x1_original <- data.frame(x1_original = tmp_res/sd(tmp_res, na.rm = TRUE))

        x2_original <- x2_original[,1]
        tmp_model <- lm(x2_original ~ seq(1:length(x2_original)))
        tmp_pred <- predict(tmp_model)
        tmp_res <- suppressWarnings(x2_original - tmp_pred)
        x2_original <- data.frame(x2_original = tmp_res/sd(tmp_res, na.rm = TRUE))

      }
    }

    x1_full_original <- merge(x1_original, x2_original, by = 0, all = TRUE)
    colnames(x1_full_original)[1] <- "year"
    x1_full_original <- years_to_rownames(x1_full_original, "year")
    colnames(x1_full_original)[ncol(x1_full_original)-1] <- "Optimized_return_primary"
    colnames(x1_full_original)[ncol(x1_full_original)] <- "Optimized_return_control"

    colnames(x1) <- "Optimized.rowNames"

    my_temporal_data <- cbind(x1_full[,1], x1_full[,2], x1_full[,3])
    colnames(my_temporal_data) <- c("x", "y", "z")
    test_calculation <- partial.r(data=my_temporal_data, x=c("x","y"), y="z", use=pcor_na_use,method = pcor_method)[2]

    test_logical <- as.numeric(max_calculation) == as.numeric(test_calculation)

    # Element 5
    # Here we create the fifth element of the final list: Analysed period in the
    # form of min(year) - max(year), e.g. 1950 - 2015
    min_env_data_primary <- min(as.numeric(row.names(env_data_primary)))
    min_response <- min(as.numeric(row.names(response)))

    max_env_data_primary <- max(as.numeric(row.names(env_data_primary)))
    max_response <- max(as.numeric(row.names(response)))

    min_together <- min(min_env_data_primary, min_response)
    max_together <- min(max_env_data_primary, max_response)


    analysed_period <- paste(as.character(min_together),
                             as.character(max_together),
                             sep = " - ")
    if (nchar(analysed_period) < 9) {
      analysed_period <- NA
    }

    # Here, the transfer function is being created
    transfer_data = data.frame(proxy = response[,1], optimized_return =x1[,1])
    lm_model = lm(optimized_return ~ proxy, data = transfer_data)
    full_range = data.frame(proxy = seq(from = min(response[,1], na.rm = TRUE), to = max(response[,1], na.rm = TRUE), length.out = 100))
    full_range$transfer_f = predict(lm_model, full_range)

    # String for titles

    title_string <- "Partial Correlation Coefficients"

    # The definition of theme
    journal_theme <- theme_bw() +
      theme(axis.text = element_text(size = 16, face = "bold"),
            axis.title = element_text(size = 18), text = element_text(size = 18),
            plot.title = element_text(size = 16,  face = "bold"))

    p1 <- ggplot(transfer_data, aes(proxy, optimized_return)) +
      geom_point() +
      geom_line(aes(proxy, transfer_f), full_range) +
      journal_theme +
      ggtitle(paste("Analysed Period:", analysed_period, "\nMethod:", title_string))

    # If there is more than one independent variable in the model,
    # transfer function is not given, since we should return a 3d model
    if (ncol(response) > 1){
      p1 <- "No transfer function is created for two or more response variables"
    }

    #######################################################################
    ############## The temporal stability of optimized_return #############
    #######################################################################

    dataset = x1_full

    empty_list = list()
    empty_list_period = list()
    empty_list_significance = list()

    temporal_stability <- data.frame()

    # 1. Progressive stability check
    if (temporal_stability_check == "progressive"){

      foldi <- seq(1:k)
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      for (m in 1:k){

        #Segement your data by fold using the which() function
        trainIndexes <- which(folds <= m, arr.ind = TRUE)
        dataset_temp <- dataset[trainIndexes, ]
        MAKS <- max(as.numeric(row.names(dataset_temp)))
        MIN <- min(as.numeric(row.names(dataset_temp)))
        empty_list_period[[m]] <- paste(MIN, "-", MAKS)

        par.r <- partial.r(data=dataset_temp, x=c(1,2), y=3, use=pcor_na_use,method = pcor_method)
        calculation <- par.r[2]
        sig <- corr.p(par.r,n = nrow(dataset_temp - 2))$p[2]

        empty_list_significance[[m]] <- sig
        empty_list[[m]] <- calculation
        colname = "partial correlation"

      }
      m1 <- do.call(rbind, empty_list)
      m2 <- do.call(rbind, empty_list_period)
      m3 <- do.call(rbind, empty_list_significance)

      temporal_stability <- data.frame(cbind(m2, format(round(m1, 3), nsmall = 3), format(round(m3, 4), nsmall = 3)))
      colnames(temporal_stability) <-c("Period", colname, "p value")
      temporal_stability
    }

    # 2. Sequential stability check
    if (temporal_stability_check == "sequential"){

      foldi <- seq(1:k)
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      for (m in 1:k){

        #Segement your data by fold using the which() function
        trainIndexes <- which(folds == m, arr.ind = TRUE)
        dataset_temp <- dataset[trainIndexes, ]

        MAKS <- max(as.numeric(row.names(dataset_temp)))
        MIN <- min(as.numeric(row.names(dataset_temp)))
        empty_list_period[[m]] <- paste(MIN, "-", MAKS)

        par.r <- partial.r(data=dataset_temp, x=c(1,2), y=3, use=pcor_na_use,method = pcor_method)
        calculation <- par.r[2]
        sig <- corr.p(par.r,n = nrow(dataset_temp - 2))$p[2]

        empty_list_significance[[m]] <- sig
        empty_list[[m]] <- calculation
        colname = "partial correlation"

      }

      m1 <- do.call(rbind, empty_list)
      m2 <- do.call(rbind, empty_list_period)
      m3 <- do.call(rbind, empty_list_significance)

      temporal_stability <- data.frame(cbind(m2, format(round(m1, 3), nsmall = 3), format(round(m3, 4), nsmall = 3)))
      colnames(temporal_stability) <-c("Period", colname, "p value")
      temporal_stability

    }


    # 3. running_window
    if (temporal_stability_check == "running_window"){

      k_end <- nrow(dataset)
      place_list <- 1
      empty_list_datasets <- list()
      empty_list_period <- list()

      empty_list <- list()
      empty_list_significance <- list()

      place_list = 1

      if (k_end < k_running_window){
        stop("k_running_window is less than the number of analysed years. Reduce the argument k_running_window")
      }

      for (w in 0:(k_end - k_running_window)){

        dataset_temp <- dataset[(1+w):(k_running_window + w),]
        empty_list_datasets[[place_list]] <- dataset_temp
        MAKS <- max(as.numeric(row.names(dataset_temp)))
        MIN <- min(as.numeric(row.names(dataset_temp)))
        empty_list_period[[place_list]] <- paste(MIN, "-", MAKS)
        place_list <- place_list + 1

      }

      for (m in 1:length(empty_list_datasets)){

        dataset_temp <- empty_list_datasets[[m]]

        par.r <- partial.r(data=dataset_temp, x=c(1,2), y=3, use=pcor_na_use,method = pcor_method)
        calculation <- par.r[2]
        sig <- corr.p(par.r,n = nrow(dataset_temp - 2))$p[2]

        empty_list_significance[[m]] <- sig
        empty_list[[m]] <- calculation
        colname = "partial correlation"

      }
      m1 <- do.call(rbind, empty_list)
      m2 <- do.call(rbind, empty_list_period)
      m3 <- do.call(rbind, empty_list_significance)

      temporal_stability <- data.frame(cbind(m2, format(round(m1, 3), nsmall = 3), format(round(as.numeric(m3), digits = 3), nsmall = 3)))
      colnames(temporal_stability) <-c("Period", colname, "p value")
      temporal_stability
    }

    ################################################################
    #### Here the final list is being filled with six elements #####
    ################################################################

    final_list <- list(calculations = temporal_matrix, method = "pcor",
                       metric = pcor_method, analysed_period = analysed_period,
                       optimized_return = x1_full,
                       optimized_return_all = x1_full_original,
                       transfer_function = p1, temporal_stability = temporal_stability,
                       cross_validation = NA,
                       aggregated_climate_primary = do.call(cbind, list_climate_primary),
                       aggregated_climate_control = do.call(cbind, list_climate_control),
                       previous_year = previous_year,
                       number_previous_years = number_previous_years)

    plot_heatmapA <- plot_heatmap(final_list, reference_window = reference_window, type = "monthly")
    plot_extremeA <- plot_extreme(final_list, ylimits = ylimits, reference_window = reference_window, type = "monthly")

    final_list <- list(calculations = temporal_matrix, method = "pcor",
                       metric = pcor_method, analysed_period = analysed_period,
                       optimized_return = x1_full,
                       optimized_return_all = x1_full_original,
                       transfer_function = p1, temporal_stability = temporal_stability,
                       cross_validation = NA,
                       plot_heatmap = plot_heatmapA,
                       plot_extreme = plot_extremeA,
                       type = "monthly",
                       reference_window = reference_window,
                       boot_lower = temporal_matrix_lower,
                       boot_upper = temporal_matrix_upper,
                       aggregated_climate_primary = do.call(cbind, list_climate_primary),
                       aggregated_climate_control = do.call(cbind, list_climate_control),
                       previous_year = previous_year,
                       number_previous_years = number_previous_years)

    class(final_list) <- "dmrs"

    return(final_list)

  }
}

#' Example of dataset as required for compare_methods()
#'
#' A dataset of Mean Vessel Area (MVA) tree-ring parameter from a lowland
#' forest in Slovenia. The first row represents a value of a year in 2012.
#'
#' @format A data frame with 58 rows and 3 columns :
#' \describe{
#'   \item{MVA}{Mean Vessel Area measurements from 2012 - 1955}
#'   \item{T_APR}{Mean April temperatures from 2012 - 1955}
#'   \item{T_aug_sep}{Mean August-September temperatures from preceding
#'   growing season from 2012 - 1955}
#' }
#' @source Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"example_dataset_1"

#' Daily precipitation for swit272 chronology
#'
#' Sum of daily precipitation in millimeters for the period 1950 - 2019.
#' This gridded E-OBS data on 0.1° regular grid, version 20e.
#' Extracted data is for the grid point with lon = 9.75 and lat = 46.45.
#'
#' @format A data frame with 25414 rows and 2 variables:
#' \describe{
#'   \item{date}{character string describing date}
#'   \item{p_sum}{mean temperature}
#' }
#' @source \url{https://www.ecad.eu/download/ensembles/download.php}
#'
#' @details We acknowledge the E-OBS dataset from the EU-FP6 project UERRA (http://www.uerra.eu) and the Copernicus Climate Change Service, and the data providers in the ECA&D project (https://www.ecad.eu). Cornes, R., G. van der Schrier, E.J.M. van den Besselaar, and P.D. Jones. 2018: An Ensemble Version of the E-OBS Temperature and Precipitation Datasets, J. Geophys. Res. Atmos., 123. doi:10.1029/2017JD028200
#'
#' @export
"swit272_daily_precipitation"

#' Monthly mean air temperatures for Ljubljana from 2015 - 1900
#'
#' A dataset of monthly mean air temperatures in Ljubljana (Slovenia). The
#' first row represents temperatures in 2015. The first column represents
#' mean January temperature, the second column represents mean February
#' temperature. etc. Row names represent year.
#'
#' @format A data frame with 116 rows and 12 variables:
#' \describe{
#'   \item{Jan}{Mean monthly air temperature for January from 1900 to 2015}
#'   \item{Feb}{Mean monthly air temperature for February from 1900 to 2015}
#'   \item{Mar}{Mean monthly air temperature for March from 1900 to 2015}
#'   \item{Apr}{Mean monthly air temperature for April from 1900 to 2015}
#'   \item{May}{Mean monthly air temperature for May from 1900 to 2015}
#'   \item{Jun}{Mean monthly air temperature for June from 1900 to 2015}
#'   \item{Jul}{Mean monthly air temperature for July from 1900 to 2015}
#'   \item{Aug}{Mean monthly air temperature for August from 1900 to 2015}
#'   \item{Sep}{Mean monthly air temperature for September from 1900 to 2015}
#'   \item{Oct}{Mean monthly air temperature for October from 1900 to 2015}
#'   \item{Nov}{Mean monthly air temperature for November from 1900 to 2015}
#'   \item{Dec}{Mean monthly air temperature for December from 1900 to 2015}
#' }
#' @source \url{https://meteo.arso.gov.si/met/sl/archive/}
#' @export
"LJ_monthly_temperatures"

#' plot_extreme
#'
#' Graphs a line or bar plot of a row with the highest metric in a matrix,
#' produced by \code{\link{daily_response}} or \code{\link{monthly_response}}
#' functions. Bar plot is drawn for monthly_response(), while for daily_response,
#' line plot is produced.
#'
#' @param result_daily_response a list with objects as produced by
#' daily_response or monthly_response function
#' @param title logical, if set to FALSE, no plot title is displayed
#' @param ylimits limit of the y axes. It should be given as ylimits = c(0,1)
#' @param reference_window character string, the reference_window argument describes,
#' how each calculation is referred. There are three different options: 'start'
#' (default), 'end' and 'middle'. If available, the value stored in
#' result_daily_response$reference_window is used.
#' @param type the character string describing type of analysis: daily or monthly
#' @param show_year_separators logical. If TRUE, dashed vertical lines are
#' added between relative-year blocks. The default is TRUE.
#'
#' @return A ggplot2 object containing the plot display
#'
#' @keywords internal

plot_extreme <- function(result_daily_response,
                         title = TRUE,
                         ylimits = NULL,
                         reference_window = "start",
                         type = "daily",
                         show_year_separators = TRUE) {

  # This needs to be set to provide results in English language
  Sys.setlocale("LC_TIME", "C")

  Value <- NULL
  BarColour <- NULL

  # Prefer metadata stored in the result object
  if (!is.null(result_daily_response$reference_window)) {
    reference_window <- result_daily_response$reference_window
  }

  reference_position_label <- if (reference_window == "start") {
    "start of window"
  } else if (reference_window == "end") {
    "end of window"
  } else if (reference_window == "middle") {
    "middle of window"
  } else {
    reference_window
  }

  # ---------------------------------------------------------------------------
  # Helper functions
  # ---------------------------------------------------------------------------

  round2_local <- function(x, digits = 0) {
    if (exists("round2", mode = "function")) {
      round2(x, digits)
    } else {
      round(x, digits)
    }
  }

  make_daily_date_labels <- function() {

    doy <- seq(1:366)
    date <- seq(as.Date("2013-01-01"),
                as.Date("2013-12-31"),
                by = "+1 day")

    # Original dendroTools convention: 366 columns are possible.
    date[366] <- as.Date("2015-12-31")
    date <- format(date, "%b %d")

    data.frame(doy = doy, date = date, stringsAsFactors = FALSE)
  }

  get_daily_position_info <- function(position,
                                      number_previous_years,
                                      days_per_year = 366L) {

    block_index <- ceiling(position / days_per_year)
    doy <- ((position - 1L) %% days_per_year) + 1L

    if (number_previous_years == 0L) {

      relative_year_number <- 0L
      relative_year_label <- "Y"

    } else {

      relative_year_number <- block_index - (number_previous_years + 1L)

      relative_year_label <- if (relative_year_number == 0L) {
        "Y"
      } else {
        paste0("Y", relative_year_number)
      }
    }

    list(
      doy = doy,
      relative_year_number = relative_year_number,
      relative_year_label = relative_year_label
    )
  }

  get_monthly_position_info <- function(position,
                                        number_previous_years,
                                        months_per_year = 12L) {

    block_index <- ceiling(position / months_per_year)
    month_number <- ((position - 1L) %% months_per_year) + 1L

    if (number_previous_years == 0L) {

      relative_year_number <- 0L
      relative_year_label <- "Y"

    } else {

      relative_year_number <- block_index - (number_previous_years + 1L)

      relative_year_label <- if (relative_year_number == 0L) {
        "Y"
      } else {
        paste0("Y", relative_year_number)
      }
    }

    list(
      month_number = month_number,
      relative_year_number = relative_year_number,
      relative_year_label = relative_year_label
    )
  }

  # Function to find the longest non-NA range
  longest_non_na_sequence <- function(x) {

    max_length <- 0
    current_length <- 0

    for (value_temp in x) {

      if (!is.na(value_temp)) {

        current_length <- current_length + 1

      } else {

        if (current_length > max_length) {
          max_length <- current_length
        }

        current_length <- 0
      }
    }

    max(max_length, current_length)
  }

  # ---------------------------------------------------------------------------
  # Extract matrix and find optimal row/column
  # ---------------------------------------------------------------------------

  result_matrix <- as.matrix(result_daily_response[[1]])

  if (!any(is.finite(result_matrix))) {
    return("All calculations are insignificant! No plot output available.")
  }

  result_daily_element1 <- data.frame(result_daily_response[[1]],
                                      check.names = FALSE)

  n_matrix_cols <- ncol(result_matrix)

  overall_max <- max(result_matrix, na.rm = TRUE)
  overall_min <- min(result_matrix, na.rm = TRUE)

  if (abs(overall_max) >= abs(overall_min)) {

    calculated_value <- overall_max
    optimal_position <- which(result_matrix == overall_max, arr.ind = TRUE)[1, ]

  } else {

    calculated_value <- overall_min
    optimal_position <- which(result_matrix == overall_min, arr.ind = TRUE)[1, ]
  }

  row_position <- optimal_position[1]
  plot_column_source <- optimal_position[2]
  plot_column <- plot_column_source

  row_index <- row.names(result_daily_element1)[row_position]

  if (is.null(row_index) || is.na(row_index)) {
    row_index <- as.character(row_position)
  }

  optimal_window_width <- as.numeric(row_index)
  calculated_metric <- round(calculated_value, 3)

  temporal_vector <- as.numeric(result_matrix[row_position, ])

  plot_df <- data.frame(
    Position = seq_along(temporal_vector),
    Value = temporal_vector
  )

  # ---------------------------------------------------------------------------
  # General labels
  # ---------------------------------------------------------------------------

  if (result_daily_response[[2]] == "cor") {

    y_lab <- "Correlation Coefficient"

  } else if (result_daily_response[[2]] == "pcor") {

    y_lab <- "Partial Correlation Coefficient"

  } else if (result_daily_response[[3]] == "r.squared") {

    y_lab <- "Explained Variance"

  } else if (result_daily_response[[3]] == "adj.r.squared") {

    y_lab <- "Adjusted Explained Variance"

  } else {

    y_lab <- "Calculated Metric"
  }

  if (result_daily_response[[2]] == "cor") {

    method_string <- paste0("\nMethod: Correlation Coefficient (",
                            result_daily_response[[3]], ")")

  } else if (result_daily_response[[2]] == "pcor") {

    method_string <- paste0("\nMethod: Partial Correlation Coefficient (",
                            result_daily_response[[3]], ")")

  } else if (result_daily_response[[2]] == "lm") {

    method_string <- "\nMethod: Linear Regression"

  } else if (result_daily_response[[2]] == "brnn") {

    method_string <- "\nMethod: ANN with Bayesian Regularization"

  } else {

    method_string <- paste0("\nMethod: ", result_daily_response[[2]])
  }

  period_string <- paste0("Analysed Period: ", result_daily_response[[4]])
  optimal_window_string <- paste0("\nOptimal Window Width: ",
                                  optimal_window_width)

  optimal_calculation <- paste0("\nOptimal ", y_lab, ": ",
                                calculated_metric)

  # ---------------------------------------------------------------------------
  # Determine optimal start/end positions
  # ---------------------------------------------------------------------------

  if (optimal_window_width %% 2 == 0) {

    adjustment_1 <- 0
    adjustment_2 <- 1

  } else {

    adjustment_1 <- 1
    adjustment_2 <- 2
  }

  if (reference_window == "start") {

    optimal_start_position <- plot_column_source
    optimal_end_position <- plot_column_source + optimal_window_width - 1

  } else if (reference_window == "end") {

    optimal_start_position <- plot_column_source - optimal_window_width + 1
    optimal_end_position <- plot_column_source

  } else if (reference_window == "middle") {

    optimal_start_position <- round2_local(
      plot_column_source - optimal_window_width / 2, 0
    ) - adjustment_1

    optimal_end_position <- round2_local(
      plot_column_source + optimal_window_width / 2, 0
    ) - adjustment_2

  } else {

    stop("reference_window must be 'start', 'end' or 'middle'.")
  }

  # ---------------------------------------------------------------------------
  # Theme
  # ---------------------------------------------------------------------------

  journal_theme <- theme_bw() +
    theme(axis.text = element_text(size = 16, face = "bold"),
          axis.title = element_text(size = 18),
          text = element_text(size = 18),
          plot.title = element_text(size = 16, face = "bold"))

  if (title == FALSE) {
    journal_theme <- journal_theme +
      theme(plot.title = element_blank())
  }

  # ---------------------------------------------------------------------------
  # Daily plot
  # ---------------------------------------------------------------------------

  if (type == "daily") {

    days_per_year <- 366L

    if (!is.null(result_daily_response$number_previous_years)) {

      number_previous_years <- result_daily_response$number_previous_years

    } else {

      number_previous_years <- ceiling(n_matrix_cols / days_per_year) - 1L
    }

    if (is.na(number_previous_years) || number_previous_years < 0L) {
      number_previous_years <- 0L
    }

    number_previous_years <- as.integer(number_previous_years)

    full_sequence_width <- days_per_year * (number_previous_years + 1L)

    optimal_start_position <- max(1, optimal_start_position)
    optimal_end_position <- min(full_sequence_width, optimal_end_position)
    plot_column_source <- max(1, min(full_sequence_width,
                                     plot_column_source))

    date_codes <- make_daily_date_labels()

    daily_label <- function(position) {

      position_info <- get_daily_position_info(
        position = position,
        number_previous_years = number_previous_years,
        days_per_year = days_per_year
      )

      date_label <- date_codes$date[position_info$doy]

      if (number_previous_years > 0L) {
        paste0(position_info$relative_year_label, " ", date_label)
      } else {
        date_label
      }
    }

    reference_info <- get_daily_position_info(
      position = plot_column_source,
      number_previous_years = number_previous_years,
      days_per_year = days_per_year
    )

    if (reference_window == "start") {

      reference_string <- paste0("\nStarting Day of Optimal Window Width: Day ",
                                 reference_info$doy)

    } else if (reference_window == "end") {

      reference_string <- paste0("\nEnding Day of Optimal Window Width: Day ",
                                 reference_info$doy)

    } else if (reference_window == "middle") {

      reference_string <- paste0("\nMiddle Day of Optimal Window Width: Day ",
                                 reference_info$doy)
    }

    if (number_previous_years > 0L) {
      reference_string <- paste0(reference_string,
                                 " of ",
                                 reference_info$relative_year_label)
    }

    if (optimal_window_width == 1) {

      Optimal_string <- paste0("\nOptimal Selection: ",
                               daily_label(optimal_start_position))

    } else {

      Optimal_string <- paste0("\nOptimal Selection: ",
                               daily_label(optimal_start_position),
                               " - ",
                               daily_label(optimal_end_position))
    }

    # Axis breaks and labels, same logic as plot_heatmap()
    year_labels <- if (number_previous_years == 0L) {
      "Y"
    } else {
      c(paste0("Y-", number_previous_years:1), "Y")
    }

    year_start <- seq(1,
                      by = days_per_year,
                      length.out = length(year_labels))

    x_breaks <- c()
    x_labels <- c()

    for (i in seq_along(year_start)) {

      start_i <- year_start[i]

      candidate_breaks <- c(start_i,
                            start_i + 99,
                            start_i + 199,
                            start_i + 299)

      candidate_breaks <- candidate_breaks[
        candidate_breaks <= n_matrix_cols
      ]

      candidate_labels <- c(year_labels[i], "100", "200", "300")
      candidate_labels <- candidate_labels[seq_along(candidate_breaks)]

      x_breaks <- c(x_breaks, candidate_breaks)
      x_labels <- c(x_labels, candidate_labels)
    }

    year_separator_positions <- year_start[-1] - 0.5
    year_separator_positions <- year_separator_positions[
      year_separator_positions <= n_matrix_cols
    ]

    longest_sequence <- longest_non_na_sequence(plot_df$Value)

    if (longest_sequence > 1) {

      final_plot <- suppressWarnings(
        ggplot(plot_df, aes(x = Position, y = Value)) +
          geom_line(size = 1.2) +
          geom_vline(xintercept = plot_column_source,
                     colour = "red",
                     size = 0.8) +
          scale_x_continuous(expand = c(0, 0),
                             breaks = x_breaks,
                             labels = x_labels) +
          scale_y_continuous(limits = ylimits) +
          journal_theme
      )

    } else {

      final_plot <- suppressWarnings(
        ggplot(plot_df, aes(x = Position, y = Value)) +
          geom_point() +
          geom_vline(xintercept = plot_column_source,
                     colour = "red",
                     size = 0.8) +
          scale_x_continuous(expand = c(0, 0),
                             breaks = x_breaks,
                             labels = x_labels) +
          scale_y_continuous(limits = ylimits) +
          journal_theme
      )
    }

    if (show_year_separators && number_previous_years > 0L) {

      final_plot <- final_plot +
        geom_vline(xintercept = year_separator_positions,
                   size = 0.4,
                   linetype = "dashed",
                   colour = "grey40")
    }

    # Annotation placement
    y_min <- min(plot_df$Value, na.rm = TRUE)
    y_max <- max(plot_df$Value, na.rm = TRUE)
    y_span <- y_max - y_min

    if (!is.finite(y_span) || y_span == 0) {
      y_span <- abs(y_max)

      if (!is.finite(y_span) || y_span == 0) {
        y_span <- 1
      }
    }

    label_x <- plot_column_source + max(10, round(n_matrix_cols * 0.015))
    label_x <- min(label_x, n_matrix_cols)

    reference_label <- if (number_previous_years > 0L) {
      paste0(reference_info$relative_year_label,
             " DOY ",
             reference_info$doy)
    } else {
      paste0("DOY ", reference_info$doy)
    }

    final_plot <- final_plot +
      annotate("label",
               label = as.character(calculated_metric),
               y = calculated_metric,
               x = label_x) +
      annotate("label",
               label = reference_label,
               y = y_min + 0.15 * y_span,
               x = label_x)

    x_lab <- if (number_previous_years > 0L) {
      paste0("Relative year and DOY (", reference_position_label, ")")
    } else {
      paste0("DOY (", reference_position_label, ")")
    }

    final_plot <- final_plot +
      ggtitle(paste0(period_string,
                     method_string,
                     optimal_calculation,
                     optimal_window_string,
                     " Days",
                     reference_string,
                     Optimal_string)) +
      xlab(x_lab) +
      ylab(y_lab)
  }

  # ---------------------------------------------------------------------------
  # Monthly plot
  # ---------------------------------------------------------------------------

  if (type == "monthly") {

    months_per_year <- 12L

    month_names <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
                     "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")

    if (!is.null(result_daily_response$number_previous_years)) {

      number_previous_years <- result_daily_response$number_previous_years

    } else {

      number_previous_years <- ceiling(n_matrix_cols / months_per_year) - 1L
    }

    if (is.na(number_previous_years) || number_previous_years < 0L) {
      number_previous_years <- 0L
    }

    number_previous_years <- as.integer(number_previous_years)

    full_sequence_width <- months_per_year * (number_previous_years + 1L)

    optimal_start_position <- max(1, optimal_start_position)
    optimal_end_position <- min(full_sequence_width, optimal_end_position)
    plot_column_source <- max(1, min(full_sequence_width,
                                     plot_column_source))

    monthly_label <- function(position) {

      position_info <- get_monthly_position_info(
        position = position,
        number_previous_years = number_previous_years,
        months_per_year = months_per_year
      )

      month_label <- month_names[position_info$month_number]

      if (number_previous_years > 0L) {
        paste0(position_info$relative_year_label, " ", month_label)
      } else {
        month_label
      }
    }

    reference_info <- get_monthly_position_info(
      position = plot_column_source,
      number_previous_years = number_previous_years,
      months_per_year = months_per_year
    )

    if (reference_window == "start") {

      reference_string <- paste0("\nStarting Month of Optimal Window Width: Month ",
                                 reference_info$month_number)

    } else if (reference_window == "end") {

      reference_string <- paste0("\nEnding Month of Optimal Window Width: Month ",
                                 reference_info$month_number)

    } else if (reference_window == "middle") {

      reference_string <- paste0("\nMiddle Month of Optimal Window Width: Month ",
                                 reference_info$month_number)
    }

    if (number_previous_years > 0L) {
      reference_string <- paste0(reference_string,
                                 " of ",
                                 reference_info$relative_year_label)
    }

    if (optimal_window_width == 1) {

      Optimal_string <- paste0("\nOptimal Selection: ",
                               monthly_label(optimal_start_position))

    } else {

      Optimal_string <- paste0("\nOptimal Selection: ",
                               monthly_label(optimal_start_position),
                               " - ",
                               monthly_label(optimal_end_position))
    }

    if (optimal_window_width == 1) {
      month_string <- " Month"
    } else {
      month_string <- " Months"
    }

    optimal_window_string <- paste0("\nOptimal Window Width: ",
                                    optimal_window_width,
                                    month_string)

    year_labels <- if (number_previous_years == 0L) {
      "Y"
    } else {
      c(paste0("Y-", number_previous_years:1), "Y")
    }

    year_start <- seq(1,
                      by = months_per_year,
                      length.out = length(year_labels))

    x_breaks <- c()
    x_labels <- c()

    for (i in seq_along(year_start)) {

      start_i <- year_start[i]

      candidate_breaks <- start_i:(start_i + months_per_year - 1)
      candidate_breaks <- candidate_breaks[
        candidate_breaks <= n_matrix_cols
      ]

      candidate_labels <- month_names[seq_along(candidate_breaks)]

      if (number_previous_years > 0L && length(candidate_labels) > 0) {
        candidate_labels[1] <- year_labels[i]
      }

      x_breaks <- c(x_breaks, candidate_breaks)
      x_labels <- c(x_labels, candidate_labels)
    }

    year_separator_positions <- year_start[-1] - 0.5
    year_separator_positions <- year_separator_positions[
      year_separator_positions <= n_matrix_cols
    ]

    plot_df$BarColour <- ifelse(plot_df$Position == plot_column_source,
                                "optimal", "other")

    final_plot <- suppressWarnings(
      ggplot(plot_df, aes(x = Position, y = Value, fill = BarColour)) +
        geom_col() +
        geom_hline(yintercept = 0) +
        scale_x_continuous(expand = c(0, 0),
                           breaks = x_breaks,
                           labels = x_labels) +
        scale_y_continuous(limits = ylimits) +
        scale_fill_manual(values = c("other" = "grey50",
                                     "optimal" = "red"),
                          guide = "none") +
        journal_theme
    )

    if (show_year_separators && number_previous_years > 0L) {

      final_plot <- final_plot +
        geom_vline(xintercept = year_separator_positions,
                   size = 0.4,
                   linetype = "dashed",
                   colour = "grey40")
    }

    x_lab <- if (number_previous_years > 0L) {
      paste0("Relative year and month (", reference_position_label, ")")
    } else {
      paste0("Month (", reference_position_label, ")")
    }

    final_plot <- final_plot +
      annotate("label",
               label = as.character(calculated_metric),
               y = calculated_metric,
               x = plot_column_source) +
      xlab(x_lab) +
      ylab(y_lab) +
      ggtitle(paste0(period_string,
                     method_string,
                     optimal_calculation,
                     optimal_window_string,
                     reference_string,
                     Optimal_string))
  }

  final_plot
}

#' The complete dataset of standardized tree-ring chronology from Albania
#'
#' A dataset with a tree-ring width (TRW) chronology of Pinus nigra from Albania This TRW
#' chronology has a span of 551 years (period 2009 - 1459) and was already used to reconstruct
#' summer temperatures by Levanič et al. (2015). In this paper, all the details about sample
#' replication, site description and correlation statistics are described.
#'
#'
#' @format A data frame with 551 rows and 1 variable:
#' \describe{
#'   \item{TRW}{Standardised tree-ring width chronology of Pinus nigra from Albania}
#' }
#' @source Levanič, T., Poljanšek, S., Toromani, E., 2015. Early summer temperatures reconstructed from
#' black pine (Pinus nigra Arnold) tree-ring widths from Albania. The Holocene 25, 469-481.
#'
#' @export
"dataset_TRW_complete"

#' Daily precipitation for Ljubljana from 2017 - 1900
#'
#' A dataset of daily sum of precipitation [mm] in Ljubljana (Slovenia).
#' The first row represents precipitation in 1900 on DOY 1.
#'
#' @format A data frame with 43067 rows and 3 variables:
#' \describe{
#'   \item{Year}{year}
#'   \item{DOY}{day of year}
#'   \item{Precipitation}{Sum of precipitation in mm}
#' }
#' @source \url{http://climexp.knmi.nl/start.cgi}
#' @export
"LJ_daily_precipitation"

#' calculate_metrics
#'
#' Calculates performance metrics for train and test data. Calculated
#' performance metrics are correlation coefficient (r), root mean squared
#' error (RMSE), root relative squared error (RRSE), index of agreement
#' (d), reduction of error (RE), coefficient of efficiency (CE), detrended
#' efficiency (DE) and bias.
#'
#' @param train_predicted a vector indicating predicted data for training set
#' @param test_predicted a vector indicating predicted data for testing set
#' @param train_observed a vector indicating observed data for training set
#' @param test_observed a vector indicating observed data for training set
#' @param digits integer of number of digits to be displayed
#' @param formula an object of class "formula" (or one that can be coerced
#' to that class): a symbolic description of the model to be fitted. This
#' additional argument is needed to calculate DE metrics.
#' @param test data frame with test data.
#' @return a data frame of calculated test and train metrics
#' @export
#'
#' @references
#' Briffa, K.R., Jones, P.D., Pilcher, J.R., Hughes, M.K., 1988. Reconstructing
#' summer temperatures in northern Fennoscandinavia back to A.D.1700 using tree
#' ring data from Scots Pine. Arct. Alp. Res. 20, 385-394.
#'
#' Fritts, H.C., 1976. Tree Rings and Climate. Academic Press, London 567 pp.
#'
#' Lorenz, E.N., 1956. Empirical Orthogonal Functions and Statistical Weather
#' Prediction. Massachusetts Institute of Technology, Department of Meteorology.
#'
#' Willmott, C.J., 1981. On the validation of models. Phys. Geogr. 2, 184-194.
#'
#' Witten, I.H., Frank, E., Hall, M.A., 2011. Data Mining: Practical Machine
#' Learning Tools and Techniques, 3rd ed. Morgan Kaufmann Publishers, Burlington
#' 629 pp.
#'
#' @examples
#' \donttest{
#' data(example_dataset_1)
#' test_data <- example_dataset_1[1:30, ]
#' train_data <- example_dataset_1[31:55, ]
#' lin_mod <- lm(MVA ~., data = train_data)
#' train_predicted <- predict(lin_mod, train_data)
#' test_predicted <- predict(lin_mod, test_data)
#' train_observed <- train_data[, 1]
#' test_observed <- test_data[, 1]
#' calculate_metrics(train_predicted, test_predicted, train_observed,
#' test_observed, test = test_data, formula = MVA ~.)
#'
#' test_data <- example_dataset_1[1:20, ]
#' train_data <- example_dataset_1[21:55, ]
#' library(brnn)
#' lin_mod <- brnn(MVA ~., data = train_data)
#' train_predicted <- predict(lin_mod, train_data)
#' test_predicted <- predict(lin_mod, test_data)
#' train_observed <- train_data[, 1]
#' test_observed <- test_data[, 1]
#' calculate_metrics(train_predicted, test_predicted, train_observed,
#' test_observed, test = test_data, formula = MVA ~.)
#' }

calculate_metrics <- function(train_predicted, test_predicted,
                               train_observed, test_observed, digits = 4, formula, test){

  # Calculating metrics for train (calibration data)
  train_cor <- cor(train_predicted, train_observed)
  train_RMSE <- MLmetrics::RMSE(train_predicted, train_observed)
  train_RRSE <- MLmetrics::RRSE(train_predicted, train_observed)
  train_d <- 1 - (sum((train_observed - train_predicted) ^ 2)) /
    sum((abs(train_predicted - mean(train_observed)) +
             abs(train_observed - mean(train_observed))) ^ 2)
  bias_train <- mean(train_observed) - mean(train_predicted)

  train_CE <- NA
  train_RE <- NA
  train_DE <- NA

  train_metrics <- data.frame(cbind(train_cor, train_RMSE, train_RRSE,
                                     train_d, train_RE, train_CE, train_DE, bias_train))
  row.names(train_metrics) <- c(deparse(substitute(train_predicted)))
  colnames(train_metrics) <- c("cor", "RMSE", "RRSE", "d", "RE", "CE", "DE", "bias")

  #Calculations for test (validation) data
  test_cor <- cor(test_observed, test_predicted)
  test_RMSE <- MLmetrics::RMSE(test_predicted, test_observed)
  test_d <- 1 - (sum((test_observed - test_predicted) ^ 2)) /
    sum((abs(test_predicted - mean(test_observed)) +
             abs(test_observed - mean(test_observed))) ^ 2)

  test_RRSE <- MLmetrics::RRSE(test_predicted, test_observed)
  bias_test <- mean(test_observed) - mean(test_predicted)


  test_RE <- 1 - (sum((test_observed - test_predicted) ^ 2) /
                           sum((test_observed - mean(train_observed)) ^ 2))

  test_CE <- 1 - (sum((test_observed - test_predicted) ^ 2) /
                           sum((test_observed - mean(test_observed)) ^ 2))

  ###
  DepIndex <- grep(as.character(formula[[2]]), colnames(test))
  DepName <- colnames(test)[DepIndex]

  test_1 <- data.frame(test[, DepIndex])
  colnames(test_1) <- "DE_TRICK"
  test_1$sequence <- seq(1:nrow(test))

  MLR_DE <- lm(formula = DE_TRICK~., data = test_1)

  DE_predicted <- predict(MLR_DE, test_1)

  test_DE <- 1 - (sum((test_observed - test_predicted) ^ 2) /
                    sum((test_observed - DE_predicted) ^ 2))
  ##

  test_metrics <- data.frame(cbind(test_cor, test_RMSE, test_RRSE, test_d,
                                    test_RE, test_CE, test_DE, bias_test))
  row.names(test_metrics) <- c(deparse(substitute(test_predicted)))
  colnames(test_metrics) <- c("cor", "RMSE", "RRSE", "d", "RE", "CE", "DE", "bias")

  metrics <- round(rbind(train_metrics, test_metrics),  digits)
  row.names(metrics) <- c("train", "test")

  metrics
}

#' boot_f_brnn
#'
#' Generate R bootstrap replicates of a brnn function applied to data. Function is based on boot() from
#' boot R package
#'
#' @param data data frame with variables for model fitting
#' @param index indices to be used for calculation
#' @param brnn.formula object of class formula to be passed into brnn function
#'
#' @return A matrix with bootstrapped estimates of r squared and adjusted r squared
#'
#' @references https://www.datacamp.com/community/tutorials/bootstrap-r
#' @keywords internal
#'

boot_f_brnn <- function(data, index, brnn.formula, neurons = 3) {

  capture.output(bsFit <- try(brnn(as.formula(brnn.formula), data = data[index,], neurons = neurons, tol = 1e-6), silent = TRUE))

  temporal_bsFit_predictions <- try(predict.brnn(bsFit, data[index,]), silent = TRUE)

  if (class(bsFit)[[1]] != "try-error"){

    temporal_r_squared <- 1 - (sum((data[index,][, 1] - temporal_bsFit_predictions) ^ 2) /
                                 sum((data[index,][, 1] - mean(data[index,][, 1])) ^ 2))
    temporal_adj_r_squared <- 1 - ((1 - temporal_r_squared) *
                                     ((nrow(data[index,]) - 1)) /
                                     (nrow(data[index,]) - ncol(as.data.frame(data[index,])) -  1 + 1))

  }

  c(
    temporal_r_squared,
    temporal_adj_r_squared
  )

}

#' round_df
#'
#' Round all numeric columns in a data frame.
#'
#' @param df a data frame
#' @param digits number of digits for the round function
#'
#' @return data frame with rounded values
#'
#'
#' @examples
#' ID <- c("a", "b", "c", "d", "e")
#' Value1 <- as.numeric(c("3.4", "6.4", "8.7", "1.1", "0.1"))
#' Value2 <- as.numeric(c("8.2", "1.7", "6.4", "1.9", "10.3"))
#' df <- data.frame(ID, Value1, Value2, stringsAsFactors = FALSE)
#' # round_df(df, digits = 0)
#'
#' @references
#' https://stackoverflow.com/questions/9063889/how-to-round-a-data-frame-in-r-that-contains-some-character-variables
#' @keywords internal

round_df <- function(df, digits = 3) {
  nums <- vapply(df, is.numeric, FUN.VALUE = logical(1))

  df[,nums] <- round(df[,nums], digits = digits)

  (df)
}

#' @method plot dmrs
#' @export

plot.dmrs <- function(x, ..., type = 2){

if (type == 1){

  plot_out <- x[["plot_extreme"]]

} else if (type == 2){

  plot_out <- x[["plot_heatmap"]]

} else {

  stop(paste0("type should be 1 or 2, but instead it is ", type, "!"))
}

  return(plot_out)

}

#' critical_r
#'
#' Calculates critical value of Pearson correlation coefficient
#' for a selected alpha.
#'
#' @param n number of observations
#' @param alpha significance level
#'
#' @return calculated critical value of Pearson correlation coefficient
#'
#' @export
#'
#' @examples
#' threshold_1 <- critical_r(n = 55, alpha = 0.01)
#' threshold_2 <- critical_r(n = 55, alpha = 0.05)

critical_r <- function(n, alpha = .05) {
  df <- n - 2
  critical_t <- qt(alpha / 2, df, lower.tail = FALSE)
  critical_r_value <- sqrt((critical_t ^ 2) / ((critical_t ^ 2) + df))
  return(critical_r_value)
}

#' daily_response
#'
#' Function calculates all possible values of a selected statistical metric
#' between one or more response variables and daily sequences of environmental
#' data. Calculations are based on moving window which is defined with two
#' arguments: window width and a location in a matrix of daily sequences of
#' environmental data. Window width could be fixed (use fixed_width) or
#' variable width (use lower_limit and upper_limit arguments). In this case,
#' all window widths between lower and upper limit will be used. All calculated
#' metrics are stored in a matrix. The location of stored calculated metric
#' in the matrix is indicating a window width (row names) and a location in a
#' matrix of daily sequences of environmental data (column names).
#'
#' @param response a data frame with tree-ring proxy variables as columns and
#' (optional) years as row names. Row.names should be matched with those from a
#' env_data data frame. If not, set row_names_subset = TRUE.
#' @param env_data a data frame of daily sequences of environmental data as
#' columns and years as row names. Each row represents a year and
#' each column represents a day of a year. Row.names should be matched with
#' those from a response data frame. If not, set row_names_subset = TRUE.
#' Alternatively, env_data could be a tidy data with three columns,
#' i.e. Year, DOY and third column representing values of mean temperatures,
#' sum of precipitation etc. If tidy data is passed to the function, set the argument
#' tidy_env_data to TRUE.
#' @param method a character string specifying which method to use. Current
#' possibilities are "cor" (default), "lm" and "brnn".
#' @param cor_method a character string indicating which correlation
#' coefficient is to be computed. One of "pearson" (default), "kendall", or
#' "spearman".
#' @param metric a character string specifying which metric to use. Current
#' possibilities are "r.squared" and "adj.r.squared". If method = "cor",
#' metric is not relevant.
#' @param lower_limit lower limit of window width
#' @param upper_limit upper limit of window width
#' @param fixed_width fixed width used for calculation. If fixed_width is
#' assigned a value, upper_limit and lower_limit will be ignored
#' @param previous_year logical. If TRUE, previous-year climate data are included
#' in the analysis. If FALSE, no previous-year climate data are included and
#' number_previous_years is ignored.
#' @param number_previous_years integer between 1 and 5 specifying how many
#' previous years should be included in the environmental matrix when
#' previous_year = TRUE. For example, number_previous_years = 2 uses climate
#' data from years t - 2, t - 1 and t for response year t. If NULL and
#' previous_year = TRUE, one previous year is included.
#' @param neurons positive integer that indicates the number of neurons used
#'  for brnn method
#' @param brnn_smooth if set to TRUE, a smoothing algorithm is applied that
#' removes unrealistic calculations which are a result of neural net failure.
#' @param remove_insignificant if set to TRUE, removes all correlations bellow
#' the significant threshold level, based on a selected alpha. For "lm" and
#' "brnn" method, squared correlation is used as a threshold
#' @param alpha significance level used to remove insignificant calculations.
#' @param row_names_subset if set to TRUE, row.names are used to subset
#' env_data and response data frames. Only years from both data frames are
#' kept.
#' @param aggregate_function character string specifying how the daily data
#' should be aggregated. The default is 'mean', the other options are 'median',
#' 'sum', 'min', 'max' and 'quantile'.
#' @param quantile_prob numeric value between 0 and 1 specifying the quantile
#' probability used when aggregate_function = 'quantile'. For example,
#' quantile_prob = 0.95 calculates the 95th percentile. The default is 0.5.
#' @param temporal_stability_check character string, specifying, how temporal stability
#' between the optimal selection and response variable(s) will be analysed. Current
#' possibilities are "sequential", "progressive" and "running_window". Sequential check
#' will split data into k splits and calculate selected metric for each split. Progressive
#' check will split data into k splits, calculate metric for the first split and then
#' progressively add 1 split at a time and calculate selected metric. For running window,
#' select the length of running window with the k_running_window argument.
#' @param k_running_window the length of running window for temporal stability check.
#' Applicable only if temporal_stability argument is set to running window.
#' @param k integer, number of breaks (splits) for temporal stability and cross validation
#' analysis.
#' @param cross_validation_type character string, specifying, how to perform cross validation
#' between the optimal selection and response variables. If the argument is set to "blocked",
#' years will not be shuffled. If the argument is set to "randomized", years will be shuffled.
#' @param subset_years a subset of years to be analyzed. Should be given in the form of
#' subset_years = c(1980, 2005)
#' @param ylimits limit of the y axes for plot_extreme. It should be given in
#' the form of: ylimits = c(0,1)
#' @param seed optional seed argument for reproducible results
#' @param tidy_env_data if set to TRUE, env_data should be inserted as a data frame with three
#' columns: "Year", "DOY", "Precipitation/Temperature/etc."
#' @param reference_window character string, the reference_window argument describes,
#' how each calculation is referred. There are three different options: 'start'
#' (default), 'end' and 'middle'. If the reference_window argument is set to 'start',
#' then each calculation is related to the starting day of window. If the
#' reference_window argument is set to 'middle', each calculation is related to the
#' middle day of window calculation. If the reference_window argument is set to
#' 'end', then each calculation is related to the ending day of window calculation.
#' For example, if we consider correlations with window from DOY 15 to DOY 35. If
#' reference window is set to  'start', then this calculation will be related to the
#' DOY 15. If the reference window is set to 'end', then this calculation will be
#' related to the DOY 35. If the reference_window is set to 'middle', then this
#' calculation is related to DOY 25.
#' The optimal selection, which describes the optimal consecutive days that returns
#' the highest calculated metric and is obtained by the $plot_extreme output, is the
#' same for all three reference windows.
#' @param boot logical, if TRUE, bootstrap procedure will be used to calculate
#' estimates correlation coefficients, R squared or adjusted R squared metrices
#' @param boot_n The number of bootstrap replicates
#' @param boot_ci_type A character string representing the type of bootstrap intervals
#' required. The value should be any subset of the values c("norm","basic", "stud",
#' "perc", "bca").
#' @param boot_conf_int A scalar or vector containing the confidence level(s) of
#' the required interval(s)
#' @param day_interval a vector of two values defining the interval of days used
#' for calculations. Positive values indicate days in the current year. Negative
#' values indicate days in the previous-year block and are used only when
#' previous_year = TRUE. If previous_year = FALSE and negative values are
#' supplied, day_interval is ignored, a warning is issued, and the analysis is
#' performed for the current year only using day_interval = c(1, 366). If
#' number_previous_years > 1, the previous-year block starts with the earliest
#' included previous year. For example, previous_year = TRUE,
#' number_previous_years = 2 and day_interval = c(-1, 366) analyses the full
#' sequence from DOY 1 of year t - 2 to DOY 366 of year t.
#' @param dc_method a character string to determine the method to detrend climate
#' data.  Possible values are "none" (default) and "SLD" which refers to Simple
#' Linear Detrending
#' @param cor_na_use an optional character string giving a method for computing
#' covariances in the presence of missing values for correlation coefficients.
#' This must be (an abbreviation of) one of the strings "everything" (default),
#' "all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs". See
#' also the documentation for the base cor() function.
#' @param skip_window_length an integer specifying the frequency of window
#' selection for the calculations of climate-growth relationships. The default
#' value is 1, indicating that every window is included in the calculations.
#' When set to a value greater than 1, the function selectively processes
#' windows at regular intervals defined by this parameter. For instance, if
#' skip_window_length = 2, the function processes every second window.
#' Similarly, if skip_window_length = 3, every third window is processed,
#' skipping two windows in between each selected one. This parameter allows for
#' controlling the granularity of the analysis and can help in reducing
#' computation time by focusing on a subset of the data.
#' @param skip_window_position an integer specifying the frequency of window
#' positions used in the calculations of climate-growth relationships. The
#' default value is 1, indicating that every window position is included in the
#' calculations. When set to a value greater than 1, the function selectively
#' processes window positions at regular intervals defined by this parameter.
#' For instance, if skip_window_position = 2, the function processes every
#' second window position. Similarly, if skip_window_position = 3, every third
#' window position is processed, skipping two positions in between each selected
#' one. This parameter allows for controlling the granularity of the analysis
#' and can help in reducing computation time by focusing on a subset of the data.
#'
#' @return a list with 19 elements:
#' \enumerate{
#'  \item $calculations - a matrix with calculated metrics
#'  \item $method - the character string of a method
#'  \item $metric - the character string indicating the metric used for calculations
#'  \item $analysed_period - the character string specifying the analysed period based on the information from row names. If there are no row names, this argument is given as NA
#'  \item $optimized_return - data frame with two columns, response variable and aggregated (averaged) daily data that return the optimal results. This data.frame could be directly used to calibrate a model for climate reconstruction
#'  \item $optimized_return_all - a data frame with aggregated daily data, that returned the optimal result for the entire env_data (and not only subset of analysed years)
#'  \item $transfer_function - a ggplot object: scatter plot of optimized return and a transfer line of the selected method
#'  \item $temporal_stability -  a data frame with calculations of selected metric for different temporal subsets
#'  \item $cross_validation - a data frame with cross validation results
#'  \item $plot_heatmap - ggplot2 object: a heatmap of calculated metrics
#'  \item $plot_extreme - ggplot2 object: line plot of a row with the highest value in a matrix of calculated metrics
#'  \item $type - the character string describing type of analysis: daily or monthly
#'  \item $reference_window - character string, which reference window was used for calculations
#'  \item $boot_lower - matrix with lower limit of confidence intervals of bootstrap calculations
#'  \item $boot_upper - matrix with upper limit of confidence intervals of bootstrap calculations
#'  \item $aggregated_climate - matrix with all aggregated climate series
#'  \item $previous_year - logical indicating whether previous-year climate data were used
#'  \item $number_previous_years - integer indicating how many previous years were used
#'}
#' @export
#'
#' @examples
#' \donttest{
#'
#' # The examples below are enclosed within donttest{} to minimize the execution
#' # time during R package checks. Additionally, all examples include the
#' # parameters `skip_window_length` and `skip_window_position`, which limit the
#' # number of combinations evaluated in climate-growth correlation calculations.
#' # To explore all possible combinations, users should set both parameters to 1.
#'
#' # Load the dendroTools R package
#' library(dendroTools)
#'
#' # Load data
#' data(data_MVA)
#' data(data_TRW)
#' data(data_TRW_1)
#' data(example_proxies_individual)
#' data(example_proxies_1)
#' data(LJ_daily_temperatures)
#'
#' example_basic <- daily_response(response = data_MVA,
#'                           env_data = LJ_daily_temperatures,
#'                           row_names_subset = TRUE,
#'                           fixed_width = 25,
#'                           lower_limit = 35, upper_limit = 45,
#'                           remove_insignificant = FALSE,
#'                           aggregate_function = 'median',
#'                           alpha = 0.05, cor_method = "spearman",
#'                           previous_year = FALSE, boot = TRUE,
#'                           boot_n = 10,
#'                           skip_window_length = 50,
#'                           skip_window_position = 50,
#'                           reference_window = "end", k = 5,
#'                           dc_method = "SLD",
#'                           day_interval = c(1, 250))
#'
#' # 1 Example with fixed width. Lower and upper limits are ignored.
#' example_daily_response <- daily_response(response = data_MVA,
#'     env_data = LJ_daily_temperatures,
#'     method = "cor", fixed_width = 40, cor_method = "spearman",
#'     row_names_subset = TRUE, previous_year = TRUE,
#'     remove_insignificant = TRUE, boot = TRUE,
#'     alpha = 0.005, aggregate_function = 'mean',
#'     day_interval = c(-100, 250), skip_window_length = 100,
#'     reference_window = "start", skip_window_position = 100)
#'
#' # summary(example_daily_response)
#' # plot(example_daily_response, type = 1)
#' # plot(example_daily_response, type = 2)
#'
#' # 2 Example for past and present. Use subset_years argument.
#' example_MVA_early <- daily_response(response = data_MVA,
#'     env_data = LJ_daily_temperatures, cor_method = "kendall",
#'     method = "lm", lower_limit = 21, upper_limit = 91,
#'     row_names_subset = TRUE, previous_year = TRUE,
#'     remove_insignificant = TRUE, alpha = 0.05,
#'     subset_years = c(1940, 1980),
#'     fixed_width = 45,
#'     aggregate_function = 'sum',
#'     skip_window_length = 50,
#'     skip_window_position = 50)
#'
#' example_MVA_late <- daily_response(response = data_MVA,
#'     env_data = LJ_daily_temperatures,
#'     method = "cor", lower_limit = 21, upper_limit = 60,
#'     row_names_subset = TRUE, previous_year = TRUE,
#'     remove_insignificant = TRUE, alpha = 0.05,
#'     subset_years = c(1981, 2010),
#'     skip_window_length = 50,
#'     skip_window_position = 50)
#'
#' # plot(example_MVA_early, type = 1)
#' # plot(example_MVA_late, type = 1)
#' # plot(example_MVA_early, type = 2)
#' # plot(example_MVA_late, type = 2)
#'
#' # 3 Example with negative correlations
#' example_neg_cor <- daily_response(response = data_TRW_1,
#'     env_data = LJ_daily_temperatures, previous_year = TRUE,
#'     method = "cor", lower_limit = 21, upper_limit = 90,
#'     row_names_subset = TRUE, remove_insignificant = TRUE,
#'     alpha = 0.05, skip_window_length = 50,
#'     skip_window_position = 50)
#'
#' # summary(example_neg_cor)
#' # plot(example_neg_cor, type = 1)
#' # plot(example_neg_cor, type = 2)
#'
#' # 4 Example of multiproxy analysis
#' # summary(example_proxies_1)
#' # cor(example_proxies_1)
#'
#' example_multiproxy <- daily_response(response = example_proxies_1,
#'    env_data = LJ_daily_temperatures,
#'    method = "lm", metric = "adj.r.squared",
#'    lower_limit = 21, upper_limit = 180,
#'    row_names_subset = TRUE, previous_year = FALSE,
#'    remove_insignificant = TRUE, alpha = 0.05,
#'    skip_window_length = 50,
#'    skip_window_position = 50)
#'
#' # plot(example_multiproxy, type = 1)
#'
#' # 5 Example to test the temporal stability
#' example_MVA_ts <- daily_response(response = data_MVA,
#'    env_data = LJ_daily_temperatures, method = "brnn",
#'    lower_limit = 100, metric = "adj.r.squared", upper_limit = 180,
#'    row_names_subset = TRUE, remove_insignificant = TRUE, alpha = 0.05,
#'    temporal_stability_check = "running_window", k_running_window = 10,
#'    skip_window_length = 50, skip_window_position = 50)
#'
#' # Check the results for temporal stability
#' # example_MVA_ts$temporal_stability
#'
#' # 6 Example with nonlinear brnn estimation
#' example_brnn <- daily_response(response = data_MVA,
#'    env_data = LJ_daily_temperatures, method = "brnn", boot = FALSE,
#'    lower_limit = 100, metric = "adj.r.squared", upper_limit = 101,
#'    row_names_subset = TRUE, remove_insignificant = TRUE, boot_n = 10,
#'    skip_window_length = 50, skip_window_position = 50)
#'
#' # summary(example_brnn)
#'
#' # Example using quantiles for the aggregation
#' example_q95 <- daily_response(
#' response = data_MVA,
#' env_data = LJ_daily_temperatures,
#' method = "cor",
#' fixed_width = 30,
#' row_names_subset = TRUE,
#' previous_year = TRUE,
#' aggregate_function = "quantile",
#' quantile_prob = 0.95
#' )
#'
#' # Example using two previous years plus the current year
#' example_two_previous_years <- daily_response(
#' response = data_MVA,
#' env_data = LJ_daily_temperatures,
#' method = "cor",
#' fixed_width = 60,
#' row_names_subset = TRUE,
#' previous_year = TRUE,
#' number_previous_years = 2,
#' day_interval = c(-1, 250),
#' skip_window_length = 50,
#' skip_window_position = 50
#' )
#'
#' # Example with previous-previous year effects
#' example_basic <- daily_response(response = data_MVA,
#'                           env_data = LJ_daily_temperatures,
#'                           row_names_subset = TRUE,
#'                           reference_window = "end",
#'                           lower_limit = 35, upper_limit = 75,
#'                           previous_year = TRUE,
#'                           remove_insignificant = TRUE,
#'                           number_previous_years = 3)
#' plot_heatmap(example_basic)
#' plot_extreme(example_basic)
#' summary(example_basic)
#' }

daily_response <- function(response, env_data, method = "cor",
                           metric = "r.squared", cor_method = "pearson",
                           lower_limit = 30, upper_limit = 90, fixed_width = 0,
                           previous_year = FALSE, number_previous_years = NULL,
                           neurons = 1,
                           brnn_smooth = TRUE, remove_insignificant = FALSE,
                           alpha = .05, row_names_subset = FALSE,
                           aggregate_function = 'mean',
                           quantile_prob = 0.5,
                           temporal_stability_check = "sequential", k = 2,
                           k_running_window = 30, cross_validation_type = "blocked",
                           subset_years = NULL,
                           ylimits = NULL, seed = NULL, tidy_env_data = FALSE,
                           reference_window = 'start',  boot = FALSE, boot_n = 1000,
                           boot_ci_type = "norm", boot_conf_int = 0.95,
                           day_interval = NULL,
                           dc_method = NULL,
                           cor_na_use = "everything",
                           skip_window_length = 1,
                           skip_window_position = 1
) {

  ##############################################################################
  # 1 day interval is organized

  days_per_year <- 366L

  # previous_year is the master switch for previous-year analyses.
  # If previous_year = FALSE, number_previous_years is ignored.
  if (!is.logical(previous_year) ||
      length(previous_year) != 1 ||
      is.na(previous_year)) {
    stop("previous_year must be either TRUE or FALSE.")
  }

  if (isFALSE(previous_year)) {

    if (!is.null(number_previous_years) &&
        is.numeric(number_previous_years) &&
        length(number_previous_years) == 1 &&
        !is.na(number_previous_years) &&
        number_previous_years > 0) {
      warning(paste0("number_previous_years is ignored because ",
                     "previous_year = FALSE."))
    }

    number_previous_years <- 0L

  } else {

    # Backward compatibility:
    # previous_year = TRUE still means one previous year unless
    # number_previous_years is explicitly supplied.
    if (is.null(number_previous_years)) {
      number_previous_years <- 1L
    }

    if (!is.numeric(number_previous_years) ||
        length(number_previous_years) != 1 ||
        is.na(number_previous_years) ||
        number_previous_years %% 1 != 0 ||
        number_previous_years < 1 ||
        number_previous_years > 5) {
      stop(paste0("When previous_year = TRUE, number_previous_years must be ",
                  "a single integer between 1 and 5."))
    }

    number_previous_years <- as.integer(number_previous_years)
  }

  year_block_width <- days_per_year * (number_previous_years + 1L)

  # Default interval:
  # current year only: c(1, 366)
  # previous-year analysis: c(-1, 366), i.e. from DOY 1 of the earliest
  # included previous year to DOY 366 of the current year.
  if (is.null(day_interval)) {
    day_interval <- if (isTRUE(previous_year)) c(-1, days_per_year) else c(1, days_per_year)
  }

  if (!is.numeric(day_interval) ||
      length(day_interval) != 2 ||
      any(is.na(day_interval))) {
    stop("day_interval must be a numeric vector of length 2.")
  }

  # Negative values in day_interval imply previous-year climate data.
  # If previous_year = FALSE, previous-year analyses are explicitly switched off,
  # so day_interval is ignored and replaced by the full current-year interval.
  if (isFALSE(previous_year) && any(day_interval < 0)) {

    day_interval <- c(1, days_per_year)

    warning(paste0("Negative values were supplied in day_interval, but ",
                   "previous_year = FALSE. The day_interval argument is ignored ",
                   "and the analysis is performed for the current year only ",
                   "using day_interval = c(1, 366)."))
  }

  offset_start <- day_interval[1]
  offset_end <- day_interval[2]

  if (offset_start == 0 || offset_end == 0) {
    stop("day_interval cannot contain 0. Use negative values for previous-year days and positive values for current-year days.")
  }

  # If both limits are positive while previous_year = TRUE, previous years are
  # not actually included in the selected interval. In that case, keep the
  # analysis current-year only.
  if (offset_start > 0 && offset_end > 0 && isTRUE(previous_year)) {

    number_previous_years <- 0L
    previous_year <- FALSE
    year_block_width <- days_per_year

    warning(paste0("Previous years are not included in selected day_interval. ",
                   "The analysis is performed for the current year only."))
  }

  # Convert the user-facing day_interval to column positions in the lagged matrix.
  # For number_previous_years = 2, the internal matrix is:
  # columns 1:366     = year t - 2
  # columns 367:732   = year t - 1
  # columns 733:1098  = year t
  if (offset_start < 0 && offset_end < 0) {

    offset_start <- abs(offset_start)
    offset_end <- abs(offset_end)

  } else if (offset_start < 0 && offset_end > 0) {

    offset_start <- abs(offset_start)
    offset_end <- offset_end + days_per_year * number_previous_years

  } else if (offset_start > 0 && offset_end > 0) {

    # Current-year-only interval; already in current-year coordinates.
    offset_start <- offset_start
    offset_end <- offset_end

  } else {

    stop("day_interval must not run from current-year days back to previous-year days.")

  }

  if (offset_start > offset_end) {
    stop("day_interval is invalid after conversion. The start of the interval is after the end.")
  }

  if (offset_start < 1 || offset_end > year_block_width) {
    stop(paste0("day_interval is outside the available climate sequence. ",
                "For number_previous_years = ", number_previous_years,
                ", the available width is ", year_block_width, " days."))
  }

  # Calculate the max_window allowed
  max_window <- offset_end - offset_start + 1

  # If max_window is smaller than upper_limit, upper_limit must be reduced
  if (upper_limit > max_window) {

    upper_limit <- max_window

    if (fixed_width == 0) {
      warning(paste0("The upper_limit is outside your day_interval and",
                     " therefore reduced to the maximum allowed: ", max_window, "."))
    }
  }

  if (lower_limit > max_window) {

    lower_limit <- max_window

    if (fixed_width == 0) {
      warning(paste0("The lower_limit is outside your day_interval and",
                     " therefore reduced to the minimum allowed: ", max_window, "."))
    }
  }

  # Also correction for fixed-window approach
  if (fixed_width > max_window) {

    stop(paste0("The selected fixed_width is outside your day_interval. ",
                "Decrease the fixed_width argument to at least: ", max_window, "."))
  }

  # offset_end is converted to the number of days that must be skipped at the
  # end of the full environmental matrix.
  offset_end <- year_block_width - offset_end

  ##############################################################################

  if (fixed_width != 0){
    lower_limit = 30
    upper_limit = 200
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Defining global variables
  median <- NULL
  proxy <- NULL
  optimized_return <- NULL
  transfer_f <- NULL
  journal_theme <- NULL
  CV <- NULL
  Period <- NULL
  Years <- NULL
  yearABC <- NULL
  RMSE <- NULL
  RE <- NULL
  CE <- NULL
  DE <- NULL
  d <- NULL

  temporal_matrix_lower <- NULL
  temporal_matrix_upper <- NULL

  # Check selected aggregation function
  allowed_aggregate_functions <- c("mean", "median", "sum", "min", "max", "quantile")

  if (!(aggregate_function %in% allowed_aggregate_functions)) {
    stop(paste0(
      "aggregate_function is '", aggregate_function,
      "'. Instead it should be one of: ",
      paste(allowed_aggregate_functions, collapse = ", "), "."
    ))
  }

  # Check quantile probability
  if (!is.numeric(quantile_prob) ||
      length(quantile_prob) != 1 ||
      is.na(quantile_prob) ||
      quantile_prob < 0 ||
      quantile_prob > 1) {
    stop("quantile_prob must be a single numeric value between 0 and 1.")
  }

  # Internal helper for aggregating daily data across rows
  aggregate_daily_window <- function(x) {

    x <- data.frame(x)

    if (aggregate_function == "mean") {
      return(rowMeans(x, na.rm = TRUE))
    }

    if (aggregate_function == "median") {
      return(apply(x, 1, median, na.rm = TRUE))
    }

    if (aggregate_function == "sum") {
      return(apply(x, 1, sum, na.rm = TRUE))
    }

    if (aggregate_function == "min") {
      return(apply(x, 1, min, na.rm = TRUE))
    }

    if (aggregate_function == "max") {
      return(apply(x, 1, max, na.rm = TRUE))
    }

    if (aggregate_function == "quantile") {
      return(apply(
        x, 1, quantile,
        probs = quantile_prob,
        na.rm = TRUE,
        names = FALSE,
        type = 7
      ))
    }
  }


  # Internal helper for robust BRNN model fitting.
  # BRNN is occasionally numerically unstable for weak or noisy relationships.
  # In such cases, the calculation should return NA rather than stopping the
  # entire moving-window analysis.
  safe_brnn_metrics <- function(temporal_df,
                                formula,
                                observed,
                                neurons,
                                tol = 1e-6) {

    out <- list(
      r.squared = NA_real_,
      adj.r.squared = NA_real_,
      model = NULL,
      predictions = NULL
    )

    temporal_df <- data.frame(temporal_df)
    observed <- as.numeric(observed)

    if (nrow(temporal_df) != length(observed) ||
        length(observed) < 3 ||
        all(is.na(observed))) {
      return(out)
    }

    fit <- try({
      capture.output(
        tmp_model <- brnn(formula,
                          data = temporal_df,
                          neurons = neurons,
                          tol = tol)
      )
      tmp_model
    }, silent = TRUE)

    if (inherits(fit, "try-error") || is.null(fit)) {
      return(out)
    }

    predictions <- try(as.numeric(predict.brnn(fit, temporal_df)),
                       silent = TRUE)

    if (inherits(predictions, "try-error") ||
        length(predictions) != length(observed) ||
        any(!is.finite(predictions))) {
      return(out)
    }

    ss_total <- sum((observed - mean(observed, na.rm = TRUE)) ^ 2,
                    na.rm = TRUE)
    ss_res <- sum((observed - predictions) ^ 2, na.rm = TRUE)

    if (!is.finite(ss_total) ||
        !is.finite(ss_res) ||
        ss_total <= 0) {
      return(out)
    }

    r_squared <- 1 - ss_res / ss_total

    if (!is.finite(r_squared)) {
      r_squared <- NA_real_
    }

    n_obs <- length(observed)
    n_predictors <- ncol(temporal_df) - 1L
    denominator <- n_obs - n_predictors - 1L

    adj_r_squared <- if (is.finite(r_squared) && denominator > 0) {
      1 - ((1 - r_squared) * (n_obs - 1)) / denominator
    } else {
      NA_real_
    }

    if (!is.finite(adj_r_squared)) {
      adj_r_squared <- NA_real_
    }

    out$r.squared <- r_squared
    out$adj.r.squared <- adj_r_squared
    out$model <- fit
    out$predictions <- predictions

    out
  }

  # Internal helper for returning two-row NA cross-validation metrics.
  # calculate_metrics() normally returns one Calibration row and one Validation row.
  # If a BRNN fit fails in one fold, we must still return two rows so that the
  # cross-validation table can be combined with its two corresponding period labels.
  empty_cv_metrics <- function(template = NULL) {

    if (!is.null(template)) {

      template <- as.data.frame(template)
      out <- template[rep(NA_integer_, 2), , drop = FALSE]
      row.names(out) <- NULL

      for (tmp_name in names(out)) {

        if (is.factor(out[[tmp_name]])) {
          out[[tmp_name]] <- factor(rep(NA_character_, 2),
                                    levels = levels(out[[tmp_name]]))
        } else if (is.character(out[[tmp_name]])) {
          out[[tmp_name]] <- rep(NA_character_, 2)
        } else {
          out[[tmp_name]] <- rep(NA_real_, 2)
        }
      }

      return(out)
    }

    data.frame(
      cor = c(NA_real_, NA_real_),
      RMSE = c(NA_real_, NA_real_),
      RRSE = c(NA_real_, NA_real_),
      d = c(NA_real_, NA_real_),
      RE = c(NA_real_, NA_real_),
      CE = c(NA_real_, NA_real_),
      DE = c(NA_real_, NA_real_),
      bias = c(NA_real_, NA_real_),
      check.names = FALSE
    )
  }

  normalize_cv_metric_list <- function(metric_list) {

    template <- NULL

    for (tmp_item in metric_list) {

      tmp_df <- try(as.data.frame(tmp_item), silent = TRUE)

      if (!inherits(tmp_df, "try-error") &&
          nrow(tmp_df) == 2 &&
          ncol(tmp_df) > 1) {
        template <- tmp_df
        break
      }
    }

    if (is.null(template)) {
      template <- empty_cv_metrics()
    }

    for (tmp_i in seq_along(metric_list)) {

      tmp_df <- try(as.data.frame(metric_list[[tmp_i]]), silent = TRUE)

      if (inherits(tmp_df, "try-error") ||
          nrow(tmp_df) != 2 ||
          ncol(tmp_df) != ncol(template) ||
          !identical(names(tmp_df), names(template))) {

        metric_list[[tmp_i]] <- empty_cv_metrics(template)

      } else {

        metric_list[[tmp_i]] <- tmp_df
      }
    }

    metric_list
  }

  # Internal helper for constructing a multi-year climate matrix.
  # Rows are current response years. Columns are ordered from the oldest previous
  # year to the current year.
  build_lagged_env_data <- function(x, n_previous_years) {

    x <- data.frame(x, check.names = FALSE)

    if (n_previous_years == 0L) {
      return(x)
    }

    env_years <- suppressWarnings(as.integer(row.names(x)))

    if (any(is.na(env_years))) {
      stop("For previous-year analyses, row.names of env_data must be calendar years.")
    }

    if (any(duplicated(env_years))) {
      stop("Duplicated years are present in row.names(env_data).")
    }

    row.names(x) <- as.character(env_years)
    x <- x[order(env_years), , drop = FALSE]
    env_years <- as.integer(row.names(x))

    current_years <- env_years[
      vapply(env_years, function(y) {
        all((y - n_previous_years):y %in% env_years)
      }, logical(1))
    ]

    if (length(current_years) == 0L) {
      stop(paste0("No years in env_data have the required ",
                  n_previous_years, " previous year(s)."))
    }

    lag_blocks <- lapply(seq(n_previous_years, 0L), function(lag_i) {

      block <- x[as.character(current_years - lag_i), , drop = FALSE]
      row.names(block) <- as.character(current_years)
      colnames(block) <- paste0("Y", -lag_i, "_", colnames(x))

      block
    })

    out <- do.call(cbind, lag_blocks)
    row.names(out) <- as.character(current_years)

    data.frame(out, check.names = FALSE)
  }

  # If there is a column name samp.depth in response data frame, warning is given
  if ("samp.depth" %in% colnames(response)){

    samp.depth_index <- grep("samp.depth", colnames(response))
    response <- response[, -samp.depth_index,F]

    warning("Removed the samp.depth from response data frame")
  }

  # If there is more than 2 columns in response data frame, give a warning
  if (ncol(response) > 1){
    warning(paste0("Your response data frame has more than 1 column! Are you doing a multiproxy research?",
                   " If so, OK. If not, check your response data frame!"))
  }


  # If env_data is given in tidy version, transformation is needed
  if (tidy_env_data == TRUE){

    n_col_tidy_DF <- ncol(env_data)
    colnames_tidy_DF <- colnames(env_data)

    if (ncol(env_data) != 3){
      stop(paste("env_data was inserted in tidy version (tidy_env_data is set to TRUE).",
                 "env_data should have 3 columns, but it has", n_col_tidy_DF, "instead!"))
    }

    if (colnames_tidy_DF[1] != "Year"){
      stop(paste("env_data was inserted in tidy version (tidy_env_data is set to TRUE).",
                 "The first column name of the env_data should be 'Year', but it is",
                 colnames_tidy_DF[1], "instead!"))
    }

    if (colnames_tidy_DF[2] != "DOY"){
      stop(paste("env_data was inserted in tidy version (tidy_env_data is set to TRUE).",
                 "The second column name of the env_data should be 'DOY', but it is",
                 colnames_tidy_DF[2], "instead!"))
    }

    value_variable = colnames(env_data)[3]
    env_data <- dcast(env_data, Year~DOY, value.var = value_variable)
    env_data <- years_to_rownames(env_data, "Year")



  }

  # PART 1 - general data arrangements, warnings and stops
  # Both bojects (response and env_data) are converted to data frames
  response <- data.frame(response)
  env_data <- data.frame(env_data)

  # Here we save the original response data that will be used later
  response_original <- response

  # Previous-year analyses require alignment by calendar year names. This avoids
  # accidental row-position mismatches after adding lagged climate years.
  if (number_previous_years > 0L && row_names_subset == FALSE) {
    row_names_subset <- TRUE
    warning(paste0("Previous-year analyses require alignment by year names. ",
                   "row_names_subset is set to TRUE."))
  }

  # Build current-year or multi-year environmental matrix.
  # For number_previous_years = 0 this returns env_data unchanged.
  env_data <- build_lagged_env_data(env_data, number_previous_years)

  # This object is used later for optimized_return_all. It should represent the
  # full lagged climate matrix before subset_years is applied.
  env_data_original <- env_data

  # For metric calculations, both objects need to have the same length,
  # with the exception, when row_names_subset is set to TRUE
  # Stop message in case both data frames do not have the same length
  if (nrow(response) != nrow(env_data) & row_names_subset == FALSE)
    stop("Length of env_data and response records differ")

  # Stop in case of method == "cor" and ncol(proxies) > 1
  # Correlations could be calculated only for one variable
  if (method == "cor" & ncol(response) > 1)
    stop(paste("More than 1 variable in response data frame not suitable ",
               "for 'cor' method. Use 'lm' or 'brnn'"))

  #######################################################
  # Rules for selected window limits

  if (fixed_width < 0 | fixed_width > year_block_width)
    stop(paste0("fixed_width should be between 0 and ", year_block_width))

  if (lower_limit > upper_limit)
    stop("lower_limit can not be higher than upper_limit!")

  if (lower_limit > year_block_width | lower_limit < 1)
    stop(paste0("lower_limit out of bounds! It should be between 1 and ",
                year_block_width))

  if (upper_limit > year_block_width | upper_limit < 1)
    stop(paste0("upper_limit out of bounds! It should be between 1 and ",
                year_block_width))

  # Make sure the selected method is appropriate
  if (!is.null(dc_method)){

    if (!(dc_method %in% c("SLD"))){

      stop(paste0('dc_method should be SLD but instead it is:',dc_method))

    }
  }



  # Warn users in case of missing values. The original threshold was 270
  # missing days for one 366-day climate year, so it is scaled when multiple
  # previous years are included.
  missing_day_threshold <- 270 * (number_previous_years + 1L)
  env_temp <- env_data[row.names(env_data) %in% row.names(response),]

  # Subset of years
  if (!is.null(subset_years)){
    lower_subset <- subset_years[1]
    upper_subset <- subset_years[2]

    subset_seq <- seq(lower_subset, upper_subset)
    env_temp <- subset(env_temp, row.names(env_temp) %in% subset_seq)
  }

  na_problem <- data.frame(na_sum = rowSums(is.na(env_temp)))
  na_problem <- na_problem[na_problem$na_sum > missing_day_threshold, , F]
  problematic_years <- paste0(row.names(na_problem), sep = "", collapse=", ")

  if (nrow(na_problem) > 0){

    warning(paste0("Problematic years with missing values are present: ", problematic_years))

  }

  # Previous-year climate data have already been added by
  # build_lagged_env_data().

  # If row_names_subset == TRUE, data is subseted and ordered based on matching
  # row.names. Additionally, number of characters in row.names is checked.
  # There should be at least three characters (assuming years before 100 will
  # never be analysed, there is no such environmental data available)
  if (row_names_subset == TRUE & nchar(row.names(env_data)[1]) >= 3){

    ncol_response <- ncol(response)

    colnames_response <- colnames(response)

    env_data$yearABC <- row.names(env_data)
    response$yearABC <- row.names(response)

    temporal_data <- merge(response, env_data, by = "yearABC")

    response <- data.frame(temporal_data[, c(2:(1 + ncol_response))],
                           row.names = temporal_data$yearABC)
    colnames(response) <- colnames_response

    env_data <- data.frame(temporal_data[, c((1 + ncol_response + 1):
                                               ncol(temporal_data))],
                           row.names = temporal_data$yearABC)
  }

  # if row.names of env_data and the response data frames are not equal,
  # warning is given.
  if (sum(row.names(env_data) == row.names(response)) != nrow(env_data)) {
    warning("row.names between env_data and response do not match!")
  }

  # If row_names_subset == TRUE, but row.names does not appear to be years,
  # error is given.
  if (row_names_subset == TRUE & nchar(row.names(env_data)[1]) < 3){
    stop(paste("row.names does not appear to be years!",
               "At least three characters needed!"))
  }


  # Subset of years
  if (!is.null(subset_years)){
    lower_subset <- subset_years[1]
    upper_subset <- subset_years[2]

    if (lower_subset > upper_subset){
      stop("Change the order of elements in the subset_years argument! First element should be lower than the second!")
    }

    subset_seq <- seq(lower_subset, upper_subset)

    if (any(!(subset_seq %in% row.names(response)))){

      stop(paste0("Undefined columns selected. Subset years don't exist",
                  " in the response data frame. Change the subset_years argument"))
    }

    if (any(!(subset_seq %in% row.names(env_data)))){

      stop(paste0("Undefined columns selected. Subset years don't exist",
                  " in the env_data data frame. Change the subset_years argument"))
    }

    response <- subset(response, row.names(response) %in% subset_seq)
    env_data <- subset(env_data, row.names(env_data) %in% subset_seq)
  }

  # NA values are not allowed and must be removed from response data.frame
  # exception if cor_na_us accounts for missing values
  if (sum(is.na(response)) > 0 & !(cor_na_use %in% c("complete.obs", "na.or.complete", "pairwise.complete.obs"))){

    prob_year <- row.names(response[is.na(response), , drop = F])

    stop(paste0("NA is not allowed in response data frame. ",
                "Problematic year is ", prob_year))

  }


  # PART 2 - Based on the selected function arguments, different chunks of code
  # will be used. For demonstration:
  # A) Chunks are used if fixed.withd != 0
  # A.1 method == "cor"
  # A.2 method == "lm"
  # A.3 method == "brnn"

  # B) Chunks are used if fixed.withd == 0
  # B.1 method == "cor"
  # B.2 method == "lm"
  # B.3 method == "brnn"

  # this is a list for climate and and holder for saving mm
  list_climate <- list()
  mm <- 1


  # A.1 method = "cor"
  if (fixed_width != 0 & method == "cor") {

    # This is an empty matrix, currently filled with NA's
    # Latter, calculations will be stored in this matrix
    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = (ncol(env_data) - fixed_width) + 1)
    } else if (reference_window == 'end') {
      temporal_matrix <- matrix(NA, nrow = 1, ncol = (ncol(env_data)))
    } else if (reference_window == 'middle') {
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = round2((ncol(env_data) - fixed_width +
                                                 1 + fixed_width/2 ),0))
    }

    # Here I create two additional temporal matrices, which will be used to store
    # lower and upper limits of bootstrap estimates
    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    if(interactive()){

      if (fixed_width != max_window){
        pb <- txtProgressBar(min = 0, max = ceiling((ncol(env_data) - fixed_width - offset_end - offset_start + 1)/(skip_window_position)),
                             style = 3)
      }

    }

    b = 0

    # An iterating loop. In each itteration x is calculated and represents
    # response (dependent) variable. X is a moving average. Window width of
    # a moving window is fixed_width. Next, statistical metric is calculated
    # based on a selected method (cor, lm or brnn). Calculation is stored in
    # temporal matrix.

    for (j in (seq((0 + offset_start -1), (ncol(env_data) - max((fixed_width + offset_end), offset_end)), by = skip_window_position))) {

      b = b + 1

      x <- aggregate_daily_window(
        env_data[1:nrow(env_data), (1 + j):(j + fixed_width), drop = FALSE]
      )

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          tmp_model <- lm(x ~ seq(1:length(x)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- x - tmp_pred

          x <- data.frame(x = tmp_res/sd(tmp_res, na.rm = TRUE))

        }

      } else {

        x <- matrix(x, nrow = nrow(env_data), ncol = 1)

      }

      x_list <- x
      colnames(x_list) <- paste0(j + 1, "_" ,j + fixed_width)
      row.names(x_list) <- row.names(env_data)
      list_climate[[mm]] <- x_list
      mm = mm + 1

      if (boot == FALSE){

        temporal_correlation <- cor(response[, 1], x[, 1], method = cor_method, use = cor_na_use)
        temporal_lower <- NA
        temporal_upper <- NA

      } else if (boot == TRUE){

        temp_df_boot <- cbind(response[, 1], x[, 1])
        calc <- boot(temp_df_boot, boot_f_cor, cor.type = cor_method, R = boot_n)

        temporal_correlation <- colMeans(calc$t)[1]

        ci_int <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type), silent = TRUE)

        if (class(ci_int)[[1]] == "try-error"){

          temporal_lower <- NA
          temporal_upper <- NA

        } else {

          if (boot_ci_type == "norm"){

            temporal_lower <- ci_int$norm[2]
            temporal_upper <- ci_int$norm[3]

          } else if (boot_ci_type == "perc"){

            temporal_lower <- ci_int$perc[4]
            temporal_upper <- ci_int$perc[5]

          } else if (boot_ci_type == "stud") {

            temporal_lower <- ci_int$student[4]
            temporal_upper <- ci_int$student[5]

          } else if (boot_ci_type == "basic") {

            temporal_lower <- ci_int$basic[4]
            temporal_upper <- ci_int$basic[5]

          } else if (boot_ci_type == "bca") {

            temporal_lower <- ci_int$bca[4]
            temporal_upper <- ci_int$bca[5]

          } else {

            stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

          }
        }
      } else {
        print(paste0("boot should be TRUE or FALSE, instead it is ", boot))
      }



      # Each calculation is printed. Reason: usually it takes several minutes
      # to go through all loops and therefore, users might think that R is
      # not responding. But if each calculation is printed, user could be
      # confident, that R is responding.
      if (reference_window == 'start'){
        temporal_matrix[1, j + 1] <- temporal_correlation
        temporal_matrix_lower[1, j + 1] <- temporal_lower
        temporal_matrix_upper[1, j + 1] <- temporal_upper
      } else if (reference_window == 'end'){
        temporal_matrix[1, j + fixed_width] <- temporal_correlation
        temporal_matrix_lower[1, j + fixed_width] <- temporal_lower
        temporal_matrix_upper[1, j + fixed_width] <- temporal_upper
      } else if (reference_window == 'middle'){
        temporal_matrix[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_correlation
        temporal_matrix_lower[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_lower
        temporal_matrix_upper[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_upper
      }

      if(interactive()){
        if (fixed_width != max_window){setTxtProgressBar(pb, b)}
      }
    }


    if(interactive()){
      if (fixed_width != max_window){close(pb)}
    }

    # temporal_matrix is given rownames and colnames. Rownames represent a
    # window width used fot calculations. Colnames represent the position of
    # moving window in a original env_data data frame.
    row.names(temporal_matrix) <- fixed_width
    row.names(temporal_matrix_lower) <- fixed_width
    row.names(temporal_matrix_upper) <- fixed_width

    temporal_colnames <- as.vector(seq(from = 1,
                                       to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames
  }

  # A.2 method == "lm"
  # For a description see A.1
  if (fixed_width != 0 & method == "lm") {

    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = (ncol(env_data) - fixed_width) + 1)
    } else if (reference_window == 'end') {
      temporal_matrix <- matrix(NA, nrow = 1, ncol = (ncol(env_data)))
    } else if (reference_window == 'middle') {
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = round2((ncol(env_data) - fixed_width +
                                                 1 + fixed_width/2 ),0))
    }

    # Here I create two additional temporal matrices, which will be used to store
    # lower and upper limits of bootstrap estimates
    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    if (fixed_width != max_window){

      if(interactive()){

        pb <- txtProgressBar(min = 0, max = ceiling((ncol(env_data) - fixed_width - offset_end - offset_start + 1)/(skip_window_position)),
                             style = 3)}
    }

    b = 0

    for (j in (seq((0 + offset_start -1), (ncol(env_data) - max((fixed_width + offset_end), offset_end)), by = skip_window_position))) {



      b = b + 1

      x <- aggregate_daily_window(
        env_data[1:nrow(env_data), (1 + j):(j + fixed_width), drop = FALSE]
      )

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          tmp_model <- lm(x ~ seq(1:length(x)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- x - tmp_pred

          x <- data.frame(x = tmp_res/sd(tmp_res, na.rm = TRUE))

        }

      } else {

        x <- matrix(x, nrow = nrow(env_data), ncol = 1)

      }

      x_list <- x
      colnames(x_list) <- paste0(j + 1, "_" ,j + fixed_width)
      row.names(x_list) <- row.names(env_data)
      list_climate[[mm]] <- x_list
      mm = mm + 1


      if (boot == FALSE){

        temporal_df <- data.frame(cbind(x, response))
        temporal_model <- lm(x ~ ., data = temporal_df)
        temporal_summary <- summary(temporal_model)
        temporal_r_squared <- temporal_summary$r.squared
        temporal_adj_r_squared <- temporal_summary$adj.r.squared

        temporal_r_squared_lower<- NA
        temporal_r_squared_upper<- NA
        temporal_adj_r_squared_lower <- NA
        temporal_adj_r_squared_upper <- NA

      } else if (boot == TRUE){

        temporal_df <- data.frame(cbind(x, response))
        calc <- boot(data = temporal_df, statistic = boot_f_lm, R = boot_n, lm.formula = "x ~ .")

        temporal_r_squared <- colMeans(calc$t)[1]
        temporal_adj_r_squared <- colMeans(calc$t)[2]

        ci_int_r_squared <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 1), silent = TRUE)
        ci_int_adj_r_squared <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 2), silent = TRUE)


        if (class(ci_int_r_squared)[[1]] == "try-error"){

          temporal_r_squared_lower<- NA
          temporal_r_squared_upper<- NA
          temporal_adj_r_squared_lower <- NA
          temporal_adj_r_squared_upper <- NA

        } else {

          if (boot_ci_type == "norm"){

            temporal_r_squared_lower <- ci_int_r_squared$norm[2]
            temporal_r_squared_upper <- ci_int_r_squared$norm[3]
            temporal_adj_r_squared_lower <- ci_int_adj_r_squared$norm[2]
            temporal_adj_r_squared_upper <- ci_int_adj_r_squared$norm[3]

          } else if (boot_ci_type == "perc"){

            temporal_r_squared_lower <- ci_int_r_squared$perc[4]
            temporal_r_squared_upper <- ci_int_r_squared$perc[5]
            temporal_adj_r_squared_lower <- ci_int_adj_r_squared$perc[4]
            temporal_adj_r_squared_upper <- ci_int_adj_r_squared$perc[5]

          } else if (boot_ci_type == "stud") {

            temporal_r_squared_lower <- ci_int_r_squared$studen[4]
            temporal_r_squared_upper <- ci_int_r_squared$studen[5]
            temporal_adj_r_squared_lower <- ci_int_adj_r_squared$studen[4]
            temporal_adj_r_squared_upper <- ci_int_adj_r_squared$studen[5]

          } else if (boot_ci_type == "basic") {

            temporal_r_squared_lower <- ci_int_r_squared$basic[4]
            temporal_r_squared_upper <- ci_int_r_squared$basic[5]
            temporal_adj_r_squared_lower <- ci_int_adj_r_squared$basic[4]
            temporal_adj_r_squared_upper <- ci_int_adj_r_squared$basic[5]

          } else if (boot_ci_type == "bca") {

            temporal_r_squared_lower <- ci_int_r_squared$bca[4]
            temporal_r_squared_upper <- ci_int_r_squared$bca[5]
            temporal_adj_r_squared_lower <- ci_int_adj_r_squared$bca[4]
            temporal_adj_r_squared_upper <- ci_int_adj_r_squared$bca[5]

          } else {

            stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

          }

        }

      } else {
        stop(paste0("boot should be TRUE or FALSE, instead it is ", boot))
      }

      if (metric == "r.squared"){

        if (reference_window == 'start'){

          temporal_matrix[1, j + 1]  <- temporal_r_squared
          temporal_matrix_lower[1, j + 1]  <- temporal_r_squared_lower
          temporal_matrix_upper[1, j + 1]  <- temporal_r_squared_upper

        } else if (reference_window == 'end') {

          temporal_matrix[1, j + fixed_width] <- temporal_r_squared
          temporal_matrix_lower[1, j + fixed_width] <- temporal_r_squared_lower
          temporal_matrix_upper[1, j + fixed_width] <- temporal_r_squared_upper

        } else if (reference_window == 'middle'){

          temporal_matrix[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_r_squared
          temporal_matrix_lower[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_r_squared_lower
          temporal_matrix_upper[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_r_squared_upper

        }
      }

      if (metric == "adj.r.squared"){
        if (reference_window == 'start'){

          temporal_matrix[1, j + 1]  <- temporal_adj_r_squared
          temporal_matrix_lower[1, j + 1]  <- temporal_adj_r_squared_lower
          temporal_matrix_upper[1, j + 1]  <- temporal_adj_r_squared_upper

        } else if (reference_window == 'end'){

          temporal_matrix[1, j + fixed_width] <- temporal_adj_r_squared
          temporal_matrix_lower[1, j + fixed_width] <- temporal_adj_r_squared_lower
          temporal_matrix_upper[1, j + fixed_width] <- temporal_adj_r_squared_upper

        } else if (reference_window == 'middle'){

          temporal_matrix[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_adj_r_squared
          temporal_matrix_lower[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_adj_r_squared_lower
          temporal_matrix_upper[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_adj_r_squared_upper
        }
      }

      if(interactive()){
        if (fixed_width != max_window){setTxtProgressBar(pb, b)}
      }
    }

    if(interactive()){
      if (fixed_width != max_window){close(pb)}
    }


    row.names(temporal_matrix) <- fixed_width
    row.names(temporal_matrix_lower) <- fixed_width
    row.names(temporal_matrix_upper) <- fixed_width

    temporal_colnames <- as.vector(seq(from = 1,
                                       to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames
  }

  # A.3 method == "brnn"
  # For a description see A.1
  if (fixed_width != 0 & method == "brnn") {

    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = (ncol(env_data) - fixed_width) + 1)
    } else if (reference_window == 'end') {
      temporal_matrix <- matrix(NA, nrow = 1, ncol = (ncol(env_data)))
    } else if (reference_window == 'middle') {
      temporal_matrix <- matrix(NA, nrow = 1,
                                ncol = round2((ncol(env_data) - fixed_width +
                                                 1 + fixed_width/2 ),0))
    }

    # Here I create two additional temporal matrices, which will be used to store
    # lower and upper limits of bootstrap estimates
    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    if (fixed_width != max_window){

      if(interactive()){

        pb <- txtProgressBar(min = 0, max = ceiling((ncol(env_data) - fixed_width - offset_end - offset_start + 1)/(skip_window_position)),
                             style = 3)}
    }
    b = 0

    for (j in (seq((0 + offset_start -1), (ncol(env_data) - max((fixed_width + offset_end), offset_end)), by = skip_window_position))) {

      b = b + 1

      x <- aggregate_daily_window(
        env_data[1:nrow(env_data), (1 + j):(j + fixed_width), drop = FALSE]
      )

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          tmp_model <- lm(x ~ seq(1:length(x)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- x - tmp_pred

          x <- data.frame(x = tmp_res/sd(tmp_res, na.rm = TRUE))

        }

      } else {

        x <- matrix(x, nrow = nrow(env_data), ncol = 1)

      }

      x_list <- x
      colnames(x_list) <- paste0(j + 1, "_" ,j + fixed_width)
      row.names(x_list) <- row.names(env_data)
      list_climate[[mm]] <- x_list
      mm = mm + 1

      if (boot == FALSE){

        temporal_df <- data.frame(cbind(x, response))

        brnn_result <- safe_brnn_metrics(
          temporal_df = temporal_df,
          formula = x ~ .,
          observed = x[, 1],
          neurons = neurons
        )

        temporal_r_squared <- brnn_result$r.squared
        temporal_adj_r_squared <- brnn_result$adj.r.squared

        temporal_r_squared_lower <- NA
        temporal_r_squared_upper <- NA

        temporal_adj_r_squared_lower <- NA
        temporal_adj_r_squared_upper <- NA

      } else if (boot == TRUE){

        temporal_df <- data.frame(cbind(x, response))

        calc <- try(
          boot(data = temporal_df,
               statistic = boot_f_brnn,
               R = boot_n,
               brnn.formula = "x ~ .",
               neurons = neurons),
          silent = TRUE
        )

        if (inherits(calc, "try-error") ||
            is.null(calc$t) ||
            ncol(calc$t) < 2 ||
            all(!is.finite(calc$t[, 1])) ||
            all(!is.finite(calc$t[, 2]))) {

          temporal_r_squared <- NA
          temporal_adj_r_squared <- NA
          temporal_r_squared_lower <- NA
          temporal_r_squared_upper <- NA
          temporal_adj_r_squared_lower <- NA
          temporal_adj_r_squared_upper <- NA

        } else {

          temporal_r_squared <- suppressWarnings(mean(calc$t[, 1], na.rm = TRUE))
          temporal_adj_r_squared <- suppressWarnings(mean(calc$t[, 2], na.rm = TRUE))

          if (!is.finite(temporal_r_squared)) temporal_r_squared <- NA
          if (!is.finite(temporal_adj_r_squared)) temporal_adj_r_squared <- NA

          ci_int_r_squared <- try(
            boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 1),
            silent = TRUE
          )

          ci_int_adj_r_squared <- try(
            boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 2),
            silent = TRUE
          )

          temporal_r_squared_lower <- NA
          temporal_r_squared_upper <- NA
          temporal_adj_r_squared_lower <- NA
          temporal_adj_r_squared_upper <- NA

          if (!inherits(ci_int_r_squared, "try-error") &&
              !inherits(ci_int_adj_r_squared, "try-error")) {

            if (boot_ci_type == "norm"){

              temporal_r_squared_lower <- ci_int_r_squared$norm[2]
              temporal_r_squared_upper <- ci_int_r_squared$norm[3]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$norm[2]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$norm[3]

            } else if (boot_ci_type == "perc"){

              temporal_r_squared_lower <- ci_int_r_squared$perc[4]
              temporal_r_squared_upper <- ci_int_r_squared$perc[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$perc[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$perc[5]

            } else if (boot_ci_type == "stud") {

              temporal_r_squared_lower <- ci_int_r_squared$student[4]
              temporal_r_squared_upper <- ci_int_r_squared$student[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$student[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$student[5]

            } else if (boot_ci_type == "basic") {

              temporal_r_squared_lower <- ci_int_r_squared$basic[4]
              temporal_r_squared_upper <- ci_int_r_squared$basic[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$basic[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$basic[5]

            } else if (boot_ci_type == "bca") {

              temporal_r_squared_lower <- ci_int_r_squared$bca[4]
              temporal_r_squared_upper <- ci_int_r_squared$bca[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$bca[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$bca[5]

            } else {

              stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

            }
          }
        }

      } else {

        stop(paste0("boot should be TRUE or FALSE, instead it is ", boot))

      }



      if (metric == "r.squared"){

        if (reference_window == 'start'){

          temporal_matrix[1, j + 1]  <- temporal_r_squared
          temporal_matrix_lower[1, j + 1]  <- temporal_r_squared_lower
          temporal_matrix_upper[1, j + 1]  <- temporal_r_squared_upper

        } else if (reference_window == 'end') {

          temporal_matrix[1, j + fixed_width] <- temporal_r_squared
          temporal_matrix_lower[1, j + fixed_width] <- temporal_r_squared_lower
          temporal_matrix_upper[1, j + fixed_width] <- temporal_r_squared_upper

        } else if (reference_window == 'middle'){

          temporal_matrix[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_r_squared
          temporal_matrix_lower[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_r_squared_lower
          temporal_matrix_upper[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_r_squared_upper

        }
      }

      if (metric == "adj.r.squared"){
        if (reference_window == 'start'){

          temporal_matrix[1, j + 1]  <- temporal_adj_r_squared
          temporal_matrix_lower[1, j + 1]  <- temporal_adj_r_squared_lower
          temporal_matrix_upper[1, j + 1]  <- temporal_adj_r_squared_upper

        } else if (reference_window == 'end'){

          temporal_matrix[1, j + fixed_width] <- temporal_adj_r_squared
          temporal_matrix_lower[1, j + fixed_width] <- temporal_adj_r_squared_lower
          temporal_matrix_upper[1, j + fixed_width] <- temporal_adj_r_squared_upper

        } else if (reference_window == 'middle'){

          temporal_matrix[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_adj_r_squared
          temporal_matrix_lower[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_adj_r_squared_lower
          temporal_matrix_upper[1, round2(j + 1 + fixed_width/2, 0)] <- temporal_adj_r_squared_upper
        }
      }
      if(interactive()){
        if (fixed_width != max_window){setTxtProgressBar(pb, b)}
      }
    }

    if(interactive()){
      if (fixed_width != max_window){close(pb)}
    }

    row.names(temporal_matrix) <- fixed_width
    row.names(temporal_matrix_lower) <- fixed_width
    row.names(temporal_matrix_upper) <- fixed_width

    temporal_colnames <- as.vector(seq(from = 1,
                                       to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames
  }

  # B fixed_width == 0, in this case, lower_limit and upper_limit arguments
  # will be used to define window width of a moving window.
  # B.1 method == "cor"

  if (fixed_width == 0 & method == "cor") {

    # This is an empty matrix, currently filled with NA's
    # Latter, calculations will be stored in this matrix

    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data) - lower_limit) + 1)
    } else if (reference_window == 'end'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data)))
    } else if (reference_window == 'middle'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = round2((ncol(env_data) - lower_limit +
                                                 1 + lower_limit/2 ),0))
    }

    # Here I create two additional temporal matrices, which will be used to store
    # lower and upper limits of bootstrap estimates
    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    # An iterating double loop: 1 outer loop) iterating from lower_limit :
    # upper_limit defines windo.width used for a moving window. 2) inner loop
    # defines the starting position of a moving window.
    # In each itteration, x is calculated and represents a response (dependent)
    # variable. x is a moving average, based on rowMeans/apply function.
    # Next, statistical metric is calculated based on a selected method (cor,
    # lm or brnn). Calculation is stored in temporal matrix in a proper place.
    # The position of stored calculation is informative later used for
    # indiciating optimal values.

    if (upper_limit != lower_limit){

      if(interactive()){

        pb <- txtProgressBar(min = 0, max = ceiling((upper_limit - lower_limit)/(skip_window_length*skip_window_position)), style = 3)

      }
    }

    b = 0



    for (K in seq(lower_limit, upper_limit, by = skip_window_length)) {

      b = b + 1

      for (j in seq((0 + offset_start -1), (ncol(env_data) - max((K + offset_end), offset_end)), by = skip_window_position)) {

        x <- aggregate_daily_window(
          env_data[1:nrow(env_data), (1 + j):(j + K), drop = FALSE]
        )

        if (!is.null(dc_method)){

          if (dc_method == "SLD"){

            tmp_model <- lm(x ~ seq(1:length(x)))
            tmp_pred <- predict(tmp_model)
            tmp_res <- x - tmp_pred

            x <- data.frame(x = tmp_res/sd(tmp_res, na.rm = TRUE))

          }

        } else {

          x <- matrix(x, nrow = nrow(env_data), ncol = 1)

        }

        # }
        x_list <- x
        colnames(x_list) <- paste0(j + 1, "_" ,j + K)
        row.names(x_list) <- row.names(env_data)
        list_climate[[mm]] <- x_list
        mm = mm + 1

        if (boot == FALSE){

          temporal_correlation <- cor(response[, 1], x[, 1], method = cor_method, use = cor_na_use)
          temporal_lower <- NA
          temporal_upper <- NA

        } else if (boot == TRUE){

          temp_df_boot <- cbind(response[, 1], x[, 1])
          calc <- boot(temp_df_boot, boot_f_cor, cor.type = cor_method, R = boot_n)

          temporal_correlation <- colMeans(calc$t)[1]

          ci_int <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type), silent = TRUE)

          if (class(ci_int)[[1]] == "try-error"){

            temporal_lower <- NA
            temporal_upper <- NA

          } else {

            if (boot_ci_type == "norm"){

              temporal_lower <- ci_int$norm[2]
              temporal_upper <- ci_int$norm[3]

            } else if (boot_ci_type == "perc"){

              temporal_lower <- ci_int$perc[4]
              temporal_upper <- ci_int$perc[5]

            } else if (boot_ci_type == "stud") {

              temporal_lower <- ci_int$student[4]
              temporal_upper <- ci_int$student[5]

            } else if (boot_ci_type == "basic") {

              temporal_lower <- ci_int$basic[4]
              temporal_upper <- ci_int$basic[5]

            } else if (boot_ci_type == "bca") {

              temporal_lower <- ci_int$bca[4]
              temporal_upper <- ci_int$bca[5]

            } else {

              stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

            }
          }
        } else {
          print(paste0("boot should be TRUE or FALSE, instead it is ", boot))
        }

        if (reference_window == 'start'){
          temporal_matrix[(K - lower_limit) + 1, j + 1] <- temporal_correlation
          temporal_matrix_lower[(K - lower_limit) + 1, j + 1] <- temporal_lower
          temporal_matrix_upper[(K - lower_limit) + 1, j + 1] <- temporal_upper
        } else if (reference_window == 'end'){
          temporal_matrix[(K - lower_limit) + 1, j + K] <- temporal_correlation
          temporal_matrix_lower[(K - lower_limit) + 1, j + K] <- temporal_lower
          temporal_matrix_upper[(K - lower_limit) + 1, j + K] <- temporal_upper
        } else if (reference_window == 'middle'){
          temporal_matrix[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_correlation
          temporal_matrix_lower[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_lower
          temporal_matrix_upper[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_upper
        }


      }

      if(interactive()){
        if (upper_limit != lower_limit){setTxtProgressBar(pb, b)}
      }
    }

    if(interactive()){
      if (upper_limit != lower_limit){close(pb)}
    }

    # temporal_matrix is given rownames and colnames. Rownames represent a
    # window width used fot calculations. Colnames represent the position of
    # moving window in a original env_data data frame.
    temporal_rownames <- as.vector(seq(from = lower_limit, to = upper_limit,
                                       by = 1))
    row.names(temporal_matrix) <- temporal_rownames
    row.names(temporal_matrix_lower) <- temporal_rownames
    row.names(temporal_matrix_upper) <- temporal_rownames


    temporal_colnames <- as.vector(seq(from = 1,
                                       to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames
  }

  # B.2 method == "lm"
  # For a description see B.1
  if (fixed_width == 0 & method == "lm") {

    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data) - lower_limit) + 1)
    } else if (reference_window == 'end'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data)))
    } else if (reference_window == 'middle'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = round2((ncol(env_data) - lower_limit +
                                                 1 + lower_limit/2 ),0))
    }

    # Here I create two additional temporal matrices, which will be used to store
    # lower and upper limits of bootstrap estimates
    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    if (upper_limit != lower_limit){
      if(interactive()){
        pb <- txtProgressBar(min = 0, max = ceiling((upper_limit - lower_limit)/(skip_window_length*skip_window_position)), style = 3)
      }
    }
    b = 0

    for (K in seq(lower_limit, upper_limit, by = skip_window_length)) {

      b = b + 1

      for (j in seq((0 + offset_start -1), (ncol(env_data) - max((K + offset_end), offset_end)), by = skip_window_position)) {


        x <- aggregate_daily_window(
          env_data[1:nrow(env_data), (1 + j):(j + K), drop = FALSE]
        )

        if (!is.null(dc_method)){

          if (dc_method == "SLD"){

            tmp_model <- lm(x ~ seq(1:length(x)))
            tmp_pred <- predict(tmp_model)
            tmp_res <- x - tmp_pred

            x <- data.frame(x = tmp_res/sd(tmp_res, na.rm = TRUE))

          }

        } else {

          x <- matrix(x, nrow = nrow(env_data), ncol = 1)

        }

        x_list <- x
        colnames(x_list) <- paste0(j + 1, "_" ,j + K)
        row.names(x_list) <- row.names(env_data)
        list_climate[[mm]] <- x_list
        mm = mm + 1

        if (boot == FALSE){

          temporal_df <- data.frame(cbind(x, response))
          temporal_model <- lm(x ~ ., data = temporal_df)
          temporal_summary <- summary(temporal_model)
          temporal_r_squared <- temporal_summary$r.squared
          temporal_adj_r_squared <- temporal_summary$adj.r.squared

          temporal_r_squared_lower<- NA
          temporal_r_squared_upper<- NA
          temporal_adj_r_squared_lower <- NA
          temporal_adj_r_squared_upper <- NA

        } else if (boot == TRUE){

          temporal_df <- data.frame(cbind(x, response))
          calc <- boot(data = temporal_df, statistic = boot_f_lm, R = boot_n, lm.formula = "x ~ .")

          temporal_r_squared <- colMeans(calc$t)[1]
          temporal_adj_r_squared <- colMeans(calc$t)[2]

          ci_int_r_squared <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 1), silent = TRUE)
          ci_int_adj_r_squared <- try(boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 2), silent = TRUE)


          if (class(ci_int_r_squared)[[1]] == "try-error"){

            temporal_r_squared_lower<- NA
            temporal_r_squared_upper<- NA
            temporal_adj_r_squared_lower <- NA
            temporal_adj_r_squared_upper <- NA

          } else {

            if (boot_ci_type == "norm"){

              temporal_r_squared_lower <- ci_int_r_squared$norm[2]
              temporal_r_squared_upper <- ci_int_r_squared$norm[3]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$norm[2]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$norm[3]

            } else if (boot_ci_type == "perc"){

              temporal_r_squared_lower <- ci_int_r_squared$perc[4]
              temporal_r_squared_upper <- ci_int_r_squared$perc[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$perc[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$perc[5]

            } else if (boot_ci_type == "stud") {

              temporal_r_squared_lower <- ci_int_r_squared$studen[4]
              temporal_r_squared_upper <- ci_int_r_squared$studen[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$studen[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$studen[5]

            } else if (boot_ci_type == "basic") {

              temporal_r_squared_lower <- ci_int_r_squared$basic[4]
              temporal_r_squared_upper <- ci_int_r_squared$basic[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$basic[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$basic[5]

            } else if (boot_ci_type == "bca") {

              temporal_r_squared_lower <- ci_int_r_squared$bca[4]
              temporal_r_squared_upper <- ci_int_r_squared$bca[5]
              temporal_adj_r_squared_lower <- ci_int_adj_r_squared$bca[4]
              temporal_adj_r_squared_upper <- ci_int_adj_r_squared$bca[5]

            } else {

              stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

            }

          }

        } else {
          stop(paste0("boot should be TRUE or FALSE, instead it is ", boot))
        }

        if (metric == "r.squared"){

          if (reference_window == 'start'){
            temporal_matrix[(K - lower_limit) + 1, j + 1]  <-  temporal_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + 1]  <-  temporal_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + 1]  <-  temporal_r_squared_upper

          } else if (reference_window == 'end') {

            temporal_matrix[(K - lower_limit) + 1, j + K] <- temporal_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + K] <- temporal_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + K] <- temporal_r_squared_upper

          } else if (reference_window == 'middle'){

            temporal_matrix[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_r_squared_upper
          }

        }

        if (metric == "adj.r.squared"){


          if (reference_window == 'start'){
            temporal_matrix[(K - lower_limit) + 1, j + 1]  <-  temporal_adj_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + 1]  <-  temporal_adj_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + 1]  <-  temporal_adj_r_squared_upper

          } else if (reference_window == 'end') {

            temporal_matrix[(K - lower_limit) + 1, j + K] <- temporal_adj_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + K] <- temporal_adj_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + K] <- temporal_adj_r_squared_upper

          } else if (reference_window == 'middle'){

            temporal_matrix[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_adj_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_adj_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_adj_r_squared_upper
          }

        }
      }
      if(interactive()){
        if (upper_limit != lower_limit){setTxtProgressBar(pb, b)}
      }
    }

    if(interactive()){
      if (upper_limit != lower_limit){close(pb)}
    }

    temporal_rownames <- as.vector(seq(from = lower_limit, to = upper_limit, by = 1))
    row.names(temporal_matrix) <- temporal_rownames
    row.names(temporal_matrix_lower) <- temporal_rownames
    row.names(temporal_matrix_upper) <- temporal_rownames

    temporal_colnames <- as.vector(seq(from = 1, to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames

  }

  # B.3 method == "brnn"
  # For a description see B.1
  if (fixed_width == 0 & method == "brnn") {

    if (reference_window == 'start'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data) - lower_limit) + 1)
    } else if (reference_window == 'end'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = (ncol(env_data)))
    } else if (reference_window == 'middle'){
      temporal_matrix <- matrix(NA, nrow = (upper_limit - lower_limit + 1),
                                ncol = round2((ncol(env_data) - lower_limit +
                                                 1 + lower_limit/2 ),0))
    }
    # Here I create two additional temporal matrices, which will be used to store
    # lower and upper limits of bootstrap estimates
    temporal_matrix_lower <- temporal_matrix
    temporal_matrix_upper <- temporal_matrix

    if (upper_limit != lower_limit){
      if(interactive()){
        pb <- txtProgressBar(min = 0, max = ceiling((upper_limit - lower_limit)/(skip_window_length*skip_window_position)), style = 3)
      }
    }

    b = 0

    for (K in seq(lower_limit, upper_limit, by = skip_window_length)) {

      b = b + 1

      for (j in seq((0 + offset_start -1), (ncol(env_data) - max((K + offset_end), offset_end)), by = skip_window_position)) {

        x <- aggregate_daily_window(
          env_data[1:nrow(env_data), (1 + j):(j + K), drop = FALSE]
        )

        if (!is.null(dc_method)){

          if (dc_method == "SLD"){

            tmp_model <- lm(x ~ seq(1:length(x)))
            tmp_pred <- predict(tmp_model)
            tmp_res <- x - tmp_pred

            x <- data.frame(x = tmp_res/sd(tmp_res, na.rm = TRUE))

          }

        } else {

          x <- matrix(x, nrow = nrow(env_data), ncol = 1)

        }

        x_list <- x
        colnames(x_list) <- paste0(j + 1, "_" ,j + K)
        row.names(x_list) <- row.names(env_data)
        list_climate[[mm]] <- x_list
        mm = mm + 1

        if (boot == FALSE){

          temporal_df <- data.frame(cbind(x, response))

          brnn_result <- safe_brnn_metrics(
            temporal_df = temporal_df,
            formula = x ~ .,
            observed = x[, 1],
            neurons = neurons
          )

          temporal_r_squared <- brnn_result$r.squared
          temporal_adj_r_squared <- brnn_result$adj.r.squared

          temporal_r_squared_lower <- NA
          temporal_r_squared_upper <- NA

          temporal_adj_r_squared_lower <- NA
          temporal_adj_r_squared_upper <- NA

        } else if (boot == TRUE){

          temporal_df <- data.frame(cbind(x, response))

          calc <- try(
            boot(data = temporal_df,
                 statistic = boot_f_brnn,
                 R = boot_n,
                 brnn.formula = "x ~ .",
                 neurons = neurons),
            silent = TRUE
          )

          if (inherits(calc, "try-error") ||
              is.null(calc$t) ||
              ncol(calc$t) < 2 ||
              all(!is.finite(calc$t[, 1])) ||
              all(!is.finite(calc$t[, 2]))) {

            temporal_r_squared <- NA
            temporal_adj_r_squared <- NA
            temporal_r_squared_lower <- NA
            temporal_r_squared_upper <- NA
            temporal_adj_r_squared_lower <- NA
            temporal_adj_r_squared_upper <- NA

          } else {

            temporal_r_squared <- suppressWarnings(mean(calc$t[, 1], na.rm = TRUE))
            temporal_adj_r_squared <- suppressWarnings(mean(calc$t[, 2], na.rm = TRUE))

            if (!is.finite(temporal_r_squared)) temporal_r_squared <- NA
            if (!is.finite(temporal_adj_r_squared)) temporal_adj_r_squared <- NA

            ci_int_r_squared <- try(
              boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 1),
              silent = TRUE
            )

            ci_int_adj_r_squared <- try(
              boot.ci(calc, conf = boot_conf_int, type = boot_ci_type, index = 2),
              silent = TRUE
            )

            temporal_r_squared_lower <- NA
            temporal_r_squared_upper <- NA
            temporal_adj_r_squared_lower <- NA
            temporal_adj_r_squared_upper <- NA

            if (!inherits(ci_int_r_squared, "try-error") &&
                !inherits(ci_int_adj_r_squared, "try-error")) {

              if (boot_ci_type == "norm"){

                temporal_r_squared_lower <- ci_int_r_squared$norm[2]
                temporal_r_squared_upper <- ci_int_r_squared$norm[3]
                temporal_adj_r_squared_lower <- ci_int_adj_r_squared$norm[2]
                temporal_adj_r_squared_upper <- ci_int_adj_r_squared$norm[3]

              } else if (boot_ci_type == "perc"){

                temporal_r_squared_lower <- ci_int_r_squared$perc[4]
                temporal_r_squared_upper <- ci_int_r_squared$perc[5]
                temporal_adj_r_squared_lower <- ci_int_adj_r_squared$perc[4]
                temporal_adj_r_squared_upper <- ci_int_adj_r_squared$perc[5]

              } else if (boot_ci_type == "stud") {

                temporal_r_squared_lower <- ci_int_r_squared$student[4]
                temporal_r_squared_upper <- ci_int_r_squared$student[5]
                temporal_adj_r_squared_lower <- ci_int_adj_r_squared$student[4]
                temporal_adj_r_squared_upper <- ci_int_adj_r_squared$student[5]

              } else if (boot_ci_type == "basic") {

                temporal_r_squared_lower <- ci_int_r_squared$basic[4]
                temporal_r_squared_upper <- ci_int_r_squared$basic[5]
                temporal_adj_r_squared_lower <- ci_int_adj_r_squared$basic[4]
                temporal_adj_r_squared_upper <- ci_int_adj_r_squared$basic[5]

              } else if (boot_ci_type == "bca") {

                temporal_r_squared_lower <- ci_int_r_squared$bca[4]
                temporal_r_squared_upper <- ci_int_r_squared$bca[5]
                temporal_adj_r_squared_lower <- ci_int_adj_r_squared$bca[4]
                temporal_adj_r_squared_upper <- ci_int_adj_r_squared$bca[5]

              } else {

                stop("boot_ci_type should be 'norm', 'perc', 'stud', 'basic' or 'bca'")

              }
            }
          }

        } else {

          stop(paste0("boot should be TRUE or FALSE, instead it is ", boot))

        }

        if (metric == "r.squared"){

          if (reference_window == 'start'){
            temporal_matrix[(K - lower_limit) + 1, j + 1]  <-  temporal_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + 1]  <-  temporal_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + 1]  <-  temporal_r_squared_upper

          } else if (reference_window == 'end') {

            temporal_matrix[(K - lower_limit) + 1, j + K] <- temporal_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + K] <- temporal_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + K] <- temporal_r_squared_upper

          } else if (reference_window == 'middle'){

            temporal_matrix[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_r_squared_upper
          }

        }

        if (metric == "adj.r.squared"){


          if (reference_window == 'start'){
            temporal_matrix[(K - lower_limit) + 1, j + 1]  <-  temporal_adj_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + 1]  <-  temporal_adj_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + 1]  <-  temporal_adj_r_squared_upper

          } else if (reference_window == 'end') {

            temporal_matrix[(K - lower_limit) + 1, j + K] <- temporal_adj_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, j + K] <- temporal_adj_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, j + K] <- temporal_adj_r_squared_upper

          } else if (reference_window == 'middle'){

            temporal_matrix[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_adj_r_squared
            temporal_matrix_lower[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_adj_r_squared_lower
            temporal_matrix_upper[(K - lower_limit) + 1, round2(j + 1 + K/2, 0)] <- temporal_adj_r_squared_upper
          }

        }

      }
      if(interactive()){
        if (upper_limit != lower_limit){setTxtProgressBar(pb, b)}
      }
    }

    if(interactive()){
      if (upper_limit != lower_limit){close(pb)}
    }

    temporal_rownames <- as.vector(seq(from = lower_limit, to = upper_limit, by = 1))
    row.names(temporal_matrix) <- temporal_rownames
    row.names(temporal_matrix_lower) <- temporal_rownames
    row.names(temporal_matrix_upper) <- temporal_rownames

    temporal_colnames <- as.vector(seq(from = 1, to = ncol(temporal_matrix), by = 1))
    colnames(temporal_matrix) <- temporal_colnames
    colnames(temporal_matrix_lower) <- temporal_colnames
    colnames(temporal_matrix_upper) <- temporal_colnames

  }

  # PART 3: smoothing function, if brnn method is used. It turnes out, that
  # brnn sometimes (1 - 3 % of calculations) fails to construct a realistic
  # result. In those cases, much lower r.squared (or adj.r.squared) are
  # calculated. smooth_matrix function removes unrealistic calculations and
  # replace them with mean of values in a window 3 x 3. Maximum value
  # is not affected by any means.
  # [i - 1, j - 1], [i - 1, j], [i - 1, j + 1]
  # [    i, j - 1], [    i, j], [    i, j + 1]
  # [i + 1, j - 1], [i + 1, j], [i + 1, j + 1]
  if (method == "brnn" & brnn_smooth == TRUE){
    temporal_matrix <- smooth_matrix(temporal_matrix, factor_drop = 0.7,
                                     repeats = 2)
  }

  # To enhance the visualisation, insignificant values
  # are removed if remove_insignificant == TRUE
  if (remove_insignificant == TRUE){

    critical_threshold_cor <- critical_r(nrow(response), alpha = alpha)
    critical_threshold_cor2 <- critical_threshold_cor ^ 2

    # 1 Method is correlation
    if (method == "cor") {

      temporal_matrix[abs(temporal_matrix) < abs(critical_threshold_cor)] <- NA

      # 2 lm and brnn method
    } else if (method == "lm" | method == "brnn") {
      temporal_matrix[abs(temporal_matrix) < abs(critical_threshold_cor2)] <- NA
    }
  }




  if(is.finite(mean(temporal_matrix, na.rm = TRUE)) == FALSE){

    warning("All calculations are insignificant! Please change the alpha argument.")

    final_list <- list(calculations = temporal_matrix,
                       method = method,
                       metric = cor_method,
                       analysed_period = NA,
                       optimized_return = NA,
                       optimized_return_all = NA,
                       transfer_function = NA, temporal_stability = NA,
                       cross_validation = NA,
                       plot_heatmap = NA,
                       plot_extreme = NA,
                       type = "daily",
                       reference_window = reference_window,
                       boot_lower = temporal_matrix_lower,
                       boot_upper = temporal_matrix_upper,
                       aggregated_climate = NA,
                       previous_year = previous_year,
                       number_previous_years = number_previous_years)

    class(final_list) <- 'dmrs'

    return(final_list)


  } else {

    ########################################################################
    # PART 4: Final list is being created and returned as a function output#
    ########################################################################

    # The first three elements of the final list are already created: calculated
    # values, method and metric used.
    # Here we create the fourth element: the optimal sequence of days that
    # returns the best selected statistical metric. We name it optimal_return.

    # In case of negative correlations, different strategy is applied.
    # For more detailed description see plot_extreme()

    if(is.finite(mean(temporal_matrix, na.rm = TRUE)) == FALSE){
      stop("All calculations are insignificant! Change the alpha argument!")
    }

    overall_max <- max(temporal_matrix, na.rm = TRUE)
    overall_min <- min(temporal_matrix, na.rm = TRUE)

    # absolute vales of overall_maximum and overall_minimum are compared and
    # one of the following two if functions is used
    # There are unimportant warnings produced:
    # no non-missing arguments to max; returning -Inf

    if ((abs(overall_max) >= abs(overall_min)) == TRUE) {

      # maximum value is located. Row indeces are needed to query information
      # about the window width used to calculate the maximum. Column name is
      # needed to query the starting day.
      max_result <- suppressWarnings(which.max(apply(temporal_matrix,
                                                     MARGIN = 2, max,
                                                     na.rm = TRUE)))
      plot_column <- max_result
      max_index <- which.max(temporal_matrix[, names(max_result)])
      row_index <- row.names(temporal_matrix)[max_index]
    }

    if ((abs(overall_max) < abs(overall_min)) == TRUE) {

      min_result <- suppressWarnings(which.min(apply(temporal_matrix,
                                                     MARGIN = 2, min,
                                                     na.rm = TRUE)))
      plot_column <- min_result
      min_index <- which.min(temporal_matrix[, names(min_result)])
      row_index <- row.names(temporal_matrix)[min_index]
    }

    # The fourth return element is being created: rowMeans/ apply of optimal sequence:
    # So, here we consider more options, based on the reference_winow
    # option 1: reference window = "start"
    if (reference_window == 'start'){

      dataf <- data.frame(aggregate_daily_window(
        env_data[, as.numeric(plot_column):
                   (as.numeric(plot_column) + as.numeric(row_index) - 1),
                 drop = FALSE]
      ))

      # if detrending was applied, should also be applied here
      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          dataf <- as.numeric(dataf[, 1])
          tmp_model <- lm(dataf ~ seq(1:length(dataf)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- dataf - tmp_pred

          dataf <- data.frame(tmp_res / sd(tmp_res, na.rm = TRUE))

        }

      }

      dataf_full <- cbind(response, dataf)
      colnames(dataf_full)[ncol(dataf_full)] <- "Optimized_return"
      colnames(dataf) <- "Optimized.rowNames"

      ## Once again, the same procedure, to get the optimal sequence, but this time for whole data, not only
      # for the analysed period.

      dataf_original <- data.frame(aggregate_daily_window(
        env_data_original[, as.numeric(plot_column):
                            (as.numeric(plot_column) + as.numeric(row_index) - 1),
                          drop = FALSE]
      ))

      dataf_full_original <- dataf_original

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          dataf_full_original <- as.numeric(dataf_full_original[, 1])
          tmp_model <- lm(dataf_full_original ~ seq(1:length(dataf_full_original)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- dataf_full_original - tmp_pred

          dataf_full_original <- data.frame(tmp_res / sd(tmp_res, na.rm = TRUE))

        }

      }

      colnames(dataf_full_original) <- "Optimized_return"
      colnames(dataf) <- "Optimized.rowNames"

      # Additional check: (we should get the same metric as before in the loop)
      if (method == "lm" & metric == "r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))
        temporal_model <- lm(Optimized.rowNames ~ ., data = temporal_df)
        temporal_summary <- summary(temporal_model)
        optimized_result <- temporal_summary$r.squared
      }

      if (method == "lm" & metric == "adj.r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))
        temporal_model <- lm(Optimized.rowNames ~ ., data = temporal_df)
        temporal_summary <- summary(temporal_model)
        optimized_result <- temporal_summary$adj.r.squared
      }

      if (method == "brnn" & metric == "r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))

        brnn_result <- safe_brnn_metrics(
          temporal_df = temporal_df,
          formula = Optimized.rowNames ~ .,
          observed = temporal_df[, 1],
          neurons = neurons
        )

        optimized_result <- brnn_result$r.squared
      }

      if (method == "brnn" & metric == "adj.r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))

        brnn_result <- safe_brnn_metrics(
          temporal_df = temporal_df,
          formula = Optimized.rowNames ~ .,
          observed = temporal_df[, 1],
          neurons = neurons
        )

        optimized_result <- brnn_result$adj.r.squared
      }

      if (method == "cor"){
        optimized_result <- cor(dataf, response, method = cor_method, use = cor_na_use)
      }

      # Just give a nicer colname
      colnames(dataf) <- "Optimized return"

    }


    # Option 2: reference window = "end"
    if (reference_window == 'end'){

      dataf <- data.frame(aggregate_daily_window(
        env_data[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                   as.numeric(plot_column),
                 drop = FALSE]
      ))

      # if detrending was applied, should also be applied here
      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          dataf <- as.numeric(dataf[, 1])
          tmp_model <- lm(dataf ~ seq(1:length(dataf)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- dataf - tmp_pred

          dataf <- data.frame(tmp_res / sd(tmp_res, na.rm = TRUE))

        }

      }

      dataf_full <- cbind(response, dataf)
      colnames(dataf_full)[ncol(dataf_full)] <- "Optimized_return"
      colnames(dataf) <- "Optimized.rowNames"

      ## Once again, the same procedure, to get the optimal sequence, but this time for whole data, not only
      # for the analysed period.

      dataf_original <- data.frame(aggregate_daily_window(
        env_data_original[, (as.numeric(plot_column) - as.numeric(row_index) + 1):
                            as.numeric(plot_column),
                          drop = FALSE]
      ))

      dataf_full_original <- dataf_original

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          dataf_full_original <- as.numeric(dataf_full_original[, 1])
          tmp_model <- lm(dataf_full_original ~ seq(1:length(dataf_full_original)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- dataf_full_original - tmp_pred

          dataf_full_original <- data.frame(tmp_res / sd(tmp_res, na.rm = TRUE))

        }

      }

      colnames(dataf_full_original) <- "Optimized_return"
      colnames(dataf) <- "Optimized.rowNames"

      # Additional check: (we should get the same metric as before in the loop)
      if (method == "lm" & metric == "r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))
        temporal_model <- lm(Optimized.rowNames ~ ., data = temporal_df)
        temporal_summary <- summary(temporal_model)
        optimized_result <- temporal_summary$r.squared
      }

      if (method == "lm" & metric == "adj.r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))
        temporal_model <- lm(Optimized.rowNames ~ ., data = temporal_df)
        temporal_summary <- summary(temporal_model)
        optimized_result <- temporal_summary$adj.r.squared
      }

      if (method == "brnn" & metric == "r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))

        brnn_result <- safe_brnn_metrics(
          temporal_df = temporal_df,
          formula = Optimized.rowNames ~ .,
          observed = temporal_df[, 1],
          neurons = neurons
        )

        optimized_result <- brnn_result$r.squared
      }

      if (method == "brnn" & metric == "adj.r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))

        brnn_result <- safe_brnn_metrics(
          temporal_df = temporal_df,
          formula = Optimized.rowNames ~ .,
          observed = temporal_df[, 1],
          neurons = neurons
        )

        optimized_result <- brnn_result$adj.r.squared
      }

      if (method == "cor"){
        optimized_result <- cor(dataf, response, method = cor_method, use = cor_na_use)
      }

      # Just give a nicer colname
      colnames(dataf) <- "Optimized return"

    }


    # Option 3: reference window = "middle"
    if (reference_window == 'middle'){

      if (as.numeric(row_index) %% 2 == 0){
        adjustment_1 <- 0
        adjustment_2 <- 1
      } else {
        adjustment_1 <- 1
        adjustment_2 <- 2
      }

      dataf <- data.frame(aggregate_daily_window(
        env_data[, (round2((as.numeric(plot_column) - as.numeric(row_index) / 2)) - adjustment_1):
                   (round2((as.numeric(plot_column) + as.numeric(row_index) / 2)) - adjustment_2),
                 drop = FALSE]
      ))

      # if detrending was applied, should also be applied here
      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          dataf <- as.numeric(dataf[, 1])
          tmp_model <- lm(dataf ~ seq(1:length(dataf)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- dataf - tmp_pred

          dataf <- data.frame(tmp_res / sd(tmp_res, na.rm = TRUE))

        }

      }

      dataf_full <- cbind(response, dataf)
      colnames(dataf_full)[ncol(dataf_full)] <- "Optimized_return"
      colnames(dataf) <- "Optimized.rowNames"

      ## Once again, the same procedure, to get the optimal sequence, but this time for whole data, not only
      # for the analysed period.

      dataf_original <- data.frame(aggregate_daily_window(
        env_data_original[, (round2((as.numeric(plot_column) - as.numeric(row_index) / 2)) - adjustment_1):
                            (round2((as.numeric(plot_column) + as.numeric(row_index) / 2)) - adjustment_2),
                          drop = FALSE]
      ))

      dataf_full_original <- dataf_original

      if (!is.null(dc_method)){

        if (dc_method == "SLD"){

          dataf_full_original <- as.numeric(dataf_full_original[, 1])
          tmp_model <- lm(dataf_full_original ~ seq(1:length(dataf_full_original)))
          tmp_pred <- predict(tmp_model)
          tmp_res <- dataf_full_original - tmp_pred

          dataf_full_original <- data.frame(tmp_res / sd(tmp_res, na.rm = TRUE))

        }

      }

      colnames(dataf_full_original) <- "Optimized_return"
      colnames(dataf) <- "Optimized.rowNames"

      # Additional check: (we should get the same metric as before in the loop)
      if (method == "lm" & metric == "r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))
        temporal_model <- lm(Optimized.rowNames ~ ., data = temporal_df)
        temporal_summary <- summary(temporal_model)
        optimized_result <- temporal_summary$r.squared
      }

      if (method == "lm" & metric == "adj.r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))
        temporal_model <- lm(Optimized.rowNames ~ ., data = temporal_df)
        temporal_summary <- summary(temporal_model)
        optimized_result <- temporal_summary$adj.r.squared
      }

      if (method == "brnn" & metric == "r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))

        brnn_result <- safe_brnn_metrics(
          temporal_df = temporal_df,
          formula = Optimized.rowNames ~ .,
          observed = temporal_df[, 1],
          neurons = neurons
        )

        optimized_result <- brnn_result$r.squared
      }

      if (method == "brnn" & metric == "adj.r.squared"){
        temporal_df <- data.frame(cbind(dataf, response))

        brnn_result <- safe_brnn_metrics(
          temporal_df = temporal_df,
          formula = Optimized.rowNames ~ .,
          observed = temporal_df[, 1],
          neurons = neurons
        )

        optimized_result <- brnn_result$adj.r.squared
      }

      if (method == "cor"){
        optimized_result <- cor(dataf, response, method = cor_method, use = cor_na_use)
      }

      # Just give a nicer colname
      colnames(dataf) <- "Optimized return"

    }
    ##############################################################################
    # If detrending was used, it also needs to be applied on optimized return

    # Element 5
    # Here we create the fifth element of the final list: Analysed period in the
    # form of min(year) - max(year), e.g. 1950 - 2015
    min_env_data <- min(as.numeric(row.names(env_data)))
    min_response <- min(as.numeric(row.names(response)))

    max_env_data <- max(as.numeric(row.names(env_data)))
    max_response <- max(as.numeric(row.names(response)))

    min_together <- min(min_env_data, min_response)
    max_together <- min(max_env_data, max_response)


    analysed_period <- paste(as.character(min_together),
                             as.character(max_together),
                             sep = " - ")
    if (nchar(analysed_period) < 9) {
      analysed_period <- NA
    }

    # Here, the transfer function is being created
    transfer_data = data.frame(proxy = response[,1], optimized_return = dataf[,1])
    lm_model = lm(optimized_return ~ proxy, data = transfer_data)

    brnn_model <- NULL
    if (method == "brnn") {
      brnn_model <- try({
        capture.output(
          tmp_model <- brnn(optimized_return ~ proxy,
                            data = transfer_data,
                            neurons = neurons,
                            tol = 1e-6)
        )
        tmp_model
      }, silent = TRUE)
    }

    full_range = data.frame(proxy = seq(from = min(response[,1], na.rm = TRUE),
                                        to = max(response[,1], na.rm = TRUE),
                                        length.out = 100))

    if (method == "lm" | method == "cor"){
      full_range$transfer_f = predict(lm_model, full_range)
    }

    if (method == "brnn"){
      transfer_predictions <- try(predict.brnn(brnn_model, full_range),
                                  silent = TRUE)

      if (inherits(brnn_model, "try-error") ||
          inherits(transfer_predictions, "try-error") ||
          length(transfer_predictions) != nrow(full_range)) {
        full_range$transfer_f <- NA_real_
      } else {
        full_range$transfer_f <- as.numeric(transfer_predictions)
      }
    }

    # String for titles
    if (method == "cor"){
      title_string <- "Correlation Coefficients"
    } else if (method == "lm"){
      title_string <- "Linear Regression"
    } else if (method == "brnn"){
      title_string <- "ANN With Bayesian Regularization"
    } else (print("The selection of method is not correct"))

    # The definition of theme
    journal_theme <- theme_bw() +
      theme(axis.text = element_text(size = 16, face = "bold"),
            axis.title = element_text(size = 18), text = element_text(size = 18),
            plot.title = element_text(size = 16,  face = "bold"))

    p1 <- ggplot(transfer_data, aes(proxy, optimized_return)) +
      geom_point() +
      geom_line(aes(proxy, transfer_f), full_range) +
      journal_theme +
      ggtitle(paste("Analysed Period:", analysed_period, "\nMethod:", title_string))

    # If there is more than one independent variable in the model,
    # transfer function is not given, since we should return a 3d model
    if (ncol(response) > 1){
      p1 <- "No transfer function is created for two or more response variables"
    }


    #######################################################################
    ############## The temporal stability of optimized_return #############
    #######################################################################

    dataset = data.frame(optimized_return = dataf[,1], proxy = response)

    empty_list = list()
    empty_list_period = list()
    empty_list_significance = list()

    temporal_stability <- data.frame()

    # 1. Progressive stability check
    if (temporal_stability_check == "progressive"){
      foldi <- seq(1:k)
      #foldi <- paste("fold_", foldi)
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      for (m in 1:k){
        #Segement your data by fold using the which() function
        trainIndexes <- which(folds <= m, arr.ind = TRUE)
        dataset_temp <- dataset[trainIndexes, ]
        MAKS <- max(as.numeric(row.names(dataset_temp)))
        MIN <- min(as.numeric(row.names(dataset_temp)))
        empty_list_period[[m]] <- paste(MIN, "-", MAKS)

        if (method == "cor"){
          calculation <- cor(dataset_temp[,1], dataset_temp[,2], method = cor_method, use = cor_na_use)
          sig <- cor.test(dataset_temp[,1], dataset_temp[,2], method = cor_method, exact=F)$p.value
          empty_list_significance[[m]] <- sig
          empty_list[[m]] <- calculation
          colname = "correlation"
        } else if (method == "lm" & metric == "r.squared"){
          MLR <- lm(optimized_return ~ ., data = dataset_temp)
          colname = "r.squared"
          empty_list[[m]] <- summary(MLR)$r.squared
          empty_list_significance[[m]] <- NA
        } else if (method == "lm" & metric == "adj.r.squared"){
          MLR <- lm(optimized_return ~ ., data = dataset_temp)
          empty_list[[m]] <- summary(MLR)$adj.r.squared
          empty_list_significance[[m]] <- NA
          colname = "adj.r.squared"        } else if (method == "brnn" & metric == "r.squared"){

            brnn_result <- safe_brnn_metrics(
              temporal_df = dataset_temp,
              formula = optimized_return ~ .,
              observed = dataset_temp[, 1],
              neurons = neurons
            )

            empty_list[[m]] <- brnn_result$r.squared
            empty_list_significance[[m]] <- NA
            colname = "r.squared"

          } else if (method == "brnn" & metric == "adj.r.squared"){

            brnn_result <- safe_brnn_metrics(
              temporal_df = dataset_temp,
              formula = optimized_return ~ .,
              observed = dataset_temp[, 1],
              neurons = neurons
            )

            empty_list[[m]] <- brnn_result$adj.r.squared
            empty_list_significance[[m]] <- NA
            colname = "adj.r.squared"
          }
      }
      m1 <- do.call(rbind, empty_list)
      m2 <- do.call(rbind, empty_list_period)
      m3 <- do.call(rbind, empty_list_significance)

      temporal_stability <- data.frame(cbind(m2, format(round(m1, 3), nsmall = 3), format(round(m3, 4), nsmall = 3)))
      colnames(temporal_stability) <-c("Period", colname, "p value")
      temporal_stability
    }

    # 2. Sequential stability check
    if (temporal_stability_check == "sequential"){
      foldi <- seq(1:k)
      #foldi <- paste("fold_", foldi)
      folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)

      for (m in 1:k){
        #Segement your data by fold using the which() function
        trainIndexes <- which(folds == m, arr.ind = TRUE)
        dataset_temp <- dataset[trainIndexes, ]

        MAKS <- max(as.numeric(row.names(dataset_temp)))
        MIN <- min(as.numeric(row.names(dataset_temp)))
        empty_list_period[[m]] <- paste(MIN, "-", MAKS)

        if (method == "cor"){
          calculation <- cor(dataset_temp[,1], dataset_temp[,2], method = cor_method, use = cor_na_use)
          sig <- cor.test(dataset_temp[,1], dataset_temp[,2], method = cor_method, exact=F)$p.value
          empty_list[[m]] <- calculation
          empty_list_significance[[m]] <- sig
          colname = "correlation"

        } else if (method == "lm" & metric == "r.squared"){
          MLR <- lm(optimized_return ~ ., data = dataset_temp)
          colname = "r.squared"
          empty_list[[m]] <- summary(MLR)$r.squared
          empty_list_significance[[m]] <- NA
        } else if (method == "lm" & metric == "adj.r.squared"){
          MLR <- lm(optimized_return ~ ., data = dataset_temp)
          empty_list[[m]] <- summary(MLR)$adj.r.squared
          empty_list_significance[[m]] <- NA
          colname = "adj.r.squared"        } else if (method == "brnn" & metric == "r.squared"){

            brnn_result <- safe_brnn_metrics(
              temporal_df = dataset_temp,
              formula = optimized_return ~ .,
              observed = dataset_temp[, 1],
              neurons = neurons
            )

            empty_list[[m]] <- brnn_result$r.squared
            empty_list_significance[[m]] <- NA
            colname = "r.squared"

          } else if (method == "brnn" & metric == "adj.r.squared"){

            brnn_result <- safe_brnn_metrics(
              temporal_df = dataset_temp,
              formula = optimized_return ~ .,
              observed = dataset_temp[, 1],
              neurons = neurons
            )

            empty_list[[m]] <- brnn_result$adj.r.squared
            empty_list_significance[[m]] <- NA
            colname = "adj.r.squared"
          }
      }
      m1 <- do.call(rbind, empty_list)
      m2 <- do.call(rbind, empty_list_period)
      m3 <- do.call(rbind, empty_list_significance)

      temporal_stability <- data.frame(cbind(m2, format(round(m1, 3), nsmall = 3), format(round(m3, 4), nsmall = 3)))
      colnames(temporal_stability) <-c("Period", colname, "p value")
      temporal_stability

    }


    # 3. running_window
    if (temporal_stability_check == "running_window"){

      k_end <- nrow(dataset)
      place_list <- 1
      empty_list_datasets <- list()
      empty_list_period <- list()

      empty_list <- list()
      empty_list_significance <- list()

      place_list = 1

      if (k_end < k_running_window){
        stop("k_running_window is less than the number of analysed years. Reduce the argument k_running_window")
      }

      for (w in 0:(k_end - k_running_window)){

        dataset_temp <- dataset[(1+w):(k_running_window + w),]
        empty_list_datasets[[place_list]] <- dataset_temp

        MAKS <- max(as.numeric(row.names(dataset_temp)))
        MIN <- min(as.numeric(row.names(dataset_temp)))
        empty_list_period[[place_list]] <- paste(MIN, "-", MAKS)

        place_list <- place_list + 1


      }

      for (m in 1:length(empty_list_datasets)){

        dataset_temp <- empty_list_datasets[[m]]

        if (method == "cor"){
          calculation <- cor(dataset_temp[,1], dataset_temp[,2], method = cor_method, use = cor_na_use)
          sig <- cor.test(dataset_temp[,1], dataset_temp[,2], method = cor_method, exact=F)$p.value
          empty_list[[m]] <- calculation
          empty_list_significance[[m]] <- sig
          colname = "correlation"

        } else if (method == "lm" & metric == "r.squared"){
          MLR <- lm(optimized_return ~ ., data = dataset_temp)
          colname = "r.squared"
          empty_list[[m]] <- summary(MLR)$r.squared
          empty_list_significance[[m]] <- NA
        } else if (method == "lm" & metric == "adj.r.squared"){
          MLR <- lm(optimized_return ~ ., data = dataset_temp)
          empty_list[[m]] <- summary(MLR)$adj.r.squared
          empty_list_significance[[m]] <- NA
          colname = "adj.r.squared"        } else if (method == "brnn" & metric == "r.squared"){

            brnn_result <- safe_brnn_metrics(
              temporal_df = dataset_temp,
              formula = optimized_return ~ .,
              observed = dataset_temp[, 1],
              neurons = neurons
            )

            empty_list[[m]] <- brnn_result$r.squared
            empty_list_significance[[m]] <- NA
            colname = "r.squared"

          } else if (method == "brnn" & metric == "adj.r.squared"){

            brnn_result <- safe_brnn_metrics(
              temporal_df = dataset_temp,
              formula = optimized_return ~ .,
              observed = dataset_temp[, 1],
              neurons = neurons
            )

            empty_list[[m]] <- brnn_result$adj.r.squared
            empty_list_significance[[m]] <- NA
            colname = "adj.r.squared"
          }
      }
      m1 <- do.call(rbind, empty_list)
      m2 <- do.call(rbind, empty_list_period)
      m3 <- do.call(rbind, empty_list_significance)

      temporal_stability <- data.frame(cbind(m2, format(round(m1, 3), nsmall = 3), format(round(as.numeric(m3), digits = 3), nsmall = 3)))
      colnames(temporal_stability) <-c("Period", colname, "p value")
      temporal_stability
    }

    #########################################################################
    ################## Out of sample estimates ##############################
    #########################################################################

    dataset = data.frame(optimized_return =dataf[,1], proxy = response)

    empty_list = list()
    empty_list_period = list()

    if (cross_validation_type == "blocked"){
      dataset <- dataset
    } else if (cross_validation_type == "randomized"){
      dataset <- dataset[sample(nrow(dataset)), ]
    } else (stop(paste("The cross_validation_type is not selected correctly! It is ", cross_validation_type,
                       ". It should be 'blocked' or 'randomized'!", sep = "")))

    foldi <- seq(1:k)
    #foldi <- paste("fold_", foldi)
    folds <- cut(seq(1, nrow(dataset)), breaks = k, labels = FALSE)
    bl = 1

    for (m in 1:k){
      #Segement your data by fold using the which() function
      testIndexes <- which(folds == m, arr.ind = TRUE)
      test <- dataset[testIndexes, ]
      train <- dataset[-testIndexes, ]

      MAKS <- max(as.numeric(row.names(train)))
      MIN <- min(as.numeric(row.names(train)))
      empty_list_period[[bl]] <- paste(MIN, "-", MAKS)
      bl <- bl + 1

      MAKS <- max(as.numeric(row.names(test)))
      MIN <- min(as.numeric(row.names(test)))
      empty_list_period[[bl]] <- paste(MIN, "-", MAKS)
      bl <- bl + 1

      if (method == "lm" | method == "cor"){
        MLR <- lm(optimized_return ~ ., data = train)
        train_predicted <- predict(MLR, train)
        test_predicted <- predict(MLR, test)
        train_observed <- train[, 1]
        test_observed <- test[, 1]
        calculations <- calculate_metrics(train_predicted, test_predicted,
                                          train_observed, test_observed, test = test,
                                          formula = optimized_return ~ ., digits = 15)

        empty_list[[m]] <- calculations
      }

      if (method == "brnn"){

        # Default to two NA rows, because calculate_metrics() normally returns
        # one Calibration row and one Validation row. This prevents failures such
        # as: "arguments imply differing number of rows: 4, 3" when one fold fails.
        calculations <- empty_cv_metrics()

        BRNN <- try({
          capture.output(
            tmp_model <- brnn(optimized_return ~ .,
                              data = train,
                              neurons = neurons,
                              tol = 1e-6)
          )
          tmp_model
        }, silent = TRUE)

        if (!inherits(BRNN, "try-error") && !is.null(BRNN)){

          train_predicted <- try(as.numeric(predict.brnn(BRNN, train)),
                                 silent = TRUE)
          test_predicted <- try(as.numeric(predict.brnn(BRNN, test)),
                                silent = TRUE)

          train_observed <- train[, 1]
          test_observed <- test[, 1]

          if (!inherits(train_predicted, "try-error") &&
              !inherits(test_predicted, "try-error") &&
              length(train_predicted) == length(train_observed) &&
              length(test_predicted) == length(test_observed) &&
              all(is.finite(train_predicted)) &&
              all(is.finite(test_predicted))) {

            calculations_tmp <- try(
              calculate_metrics(train_predicted, test_predicted,
                                train_observed, test_observed, digits = 15,
                                test = test,
                                formula = optimized_return ~ .),
              silent = TRUE
            )

            if (!inherits(calculations_tmp, "try-error") &&
                !is.null(calculations_tmp) &&
                nrow(as.data.frame(calculations_tmp)) == 2) {
              calculations <- as.data.frame(calculations_tmp)
            }
          }
        }

        empty_list[[m]] <- calculations
      }

    }

    empty_list <- normalize_cv_metric_list(empty_list)

    m1 <- do.call(rbind, empty_list)
    # m1 <- m1[, -c(3, 4, 7)]
    m2 <- do.call(rbind, empty_list_period)

    if (nrow(as.data.frame(m1)) != nrow(as.data.frame(m2))) {
      stop(paste0("Internal cross-validation error: metric table has ",
                  nrow(as.data.frame(m1)), " rows but period table has ",
                  nrow(as.data.frame(m2)), " rows."))
    }

    cross_validation <- cbind(Years = m2, m1)
    cross_validation$Period <- c("Calibration", "Validation")
    cross_validation$CV <- rep(1:k, each = 2)
    row.names(cross_validation) <- NULL

    if (cross_validation_type == "blocked"){
      cross_validation <- dplyr::select(cross_validation, CV, Period, Years, cor, RMSE, RRSE, d, RE, CE, DE)
    }
    if (cross_validation_type == "randomized"){
      cross_validation <- dplyr::select(cross_validation, CV, Period, cor, RMSE, RRSE, d, RE, CE, DE)
    }



    ################################################################
    #### Here the final list is being filled with six elements #####
    ################################################################




    # When metohod == "cor", different final_list is created
    if (method == "lm" | method == "brnn") {
      final_list <- list(calculations = temporal_matrix, method = method,
                         metric = metric, analysed_period = analysed_period,
                         optimized_return = dataf_full,
                         optimized_return_all = dataf_full_original,
                         transfer_function = p1, temporal_stability = temporal_stability,
                         cross_validation = cross_validation,
                         aggregated_climate = do.call(cbind, list_climate),
                         previous_year = previous_year,
                         number_previous_years = number_previous_years)
    }

    if (method == "cor"){
      final_list <- list(calculations = temporal_matrix, method = method,
                         metric = cor_method, analysed_period = analysed_period,
                         optimized_return = dataf_full,
                         optimized_return_all = dataf_full_original,
                         transfer_function = p1, temporal_stability = temporal_stability,
                         cross_validation = cross_validation,
                         aggregated_climate = do.call(cbind, list_climate),
                         previous_year = previous_year,
                         number_previous_years = number_previous_years)
    }


    plot_heatmapA <- plot_heatmap(final_list, reference_window = reference_window, type = "daily")
    plot_extremeA <- plot_extreme(final_list, ylimits = ylimits, reference_window = reference_window, type = "daily")

    # Here, for the sake of simplicity, we create final list again
    if (method == "lm" | method == "brnn") {
      final_list <- list(calculations = temporal_matrix, method = method,
                         metric = metric, analysed_period = analysed_period,
                         optimized_return = dataf_full,
                         optimized_return_all = dataf_full_original,
                         transfer_function = p1, temporal_stability = temporal_stability,
                         cross_validation = cross_validation,
                         plot_heatmap = plot_heatmapA,
                         plot_extreme = plot_extremeA,
                         type = "daily",
                         reference_window = reference_window,
                         boot_lower = temporal_matrix_lower,
                         boot_upper = temporal_matrix_upper,
                         aggregated_climate = do.call(cbind, list_climate),
                         previous_year = previous_year,
                         number_previous_years = number_previous_years)
    }

    if (method == "cor"){

      final_list <- list(calculations = temporal_matrix, method = method,
                         metric = cor_method, analysed_period = analysed_period,
                         optimized_return = dataf_full,
                         optimized_return_all = dataf_full_original,
                         transfer_function = p1, temporal_stability = temporal_stability,
                         cross_validation = cross_validation,
                         plot_heatmap = plot_heatmapA,
                         plot_extreme = plot_extremeA,
                         type = "daily",
                         reference_window = reference_window,
                         boot_lower = temporal_matrix_lower,
                         boot_upper = temporal_matrix_upper,
                         aggregated_climate = do.call(cbind, list_climate),
                         previous_year = previous_year,
                         number_previous_years = number_previous_years)
    }

    class(final_list) <- 'dmrs'

    return(final_list)
  }

}

library(dendroTools)
library(testthat)
library(dplyr)

data(LJ_daily_temperatures)
data(example_proxies_1)

MVA_parameter <- select(example_proxies_1, MVA)
TRW_parameter <- select(example_proxies_1, TRW)

# daily_response function should return a list with matrix and two characters
test3 <- daily_response(response = example_proxies_1,
  env_data = LJ_daily_temperatures, method = "lm",
  metric = "adj.r.squared", lower = 250, upper = 251, previous_year = FALSE,
  row_names_subset = TRUE)

expect_is(test3, "dmrs")
expect_is(test3[[1]], "matrix")
expect_is(test3[[2]], "character")
expect_is(test3[[2]], "character")

# stop functions were included to prevent wrong results
expect_error(daily_response(response = TRW_parameter,
                            env_data = LJ_daily_temperatures,
                            lower = 200, upper = 270, fixed_width = -368))

expect_error(daily_response(response = example_proxies_1,
  env_data = LJ_daily_temperatures, method = "cor", lower = 250,
  upper = 270, previous_year = FALSE))

expect_error(daily_response(response = example_proxies_1,
  env_data = LJ_daily_temperatures, method = "cor", lower = 280,
  upper = 270, previous_year = FALSE))

# If row.names of env_data and response do not match, error should be given
example_proxies_1_temporal <- example_proxies_1
row.names(example_proxies_1_temporal)[1] <- "1520" # random year is assigned

# to row.name of the firest row
expect_error(daily_response(response = example_proxies_1_temporal,
                            env_data = LJ_daily_temperatures,
                            method = "lm", lower = 260, upper = 270,
                            previous_year = FALSE, remove_insignificant = FALSE))

# The order of data is unimportant, if row_names_subset is set to TRUE and
# row.names are years. In this case, data is merged based on matching
# row names.
# will be ordered data
data(example_proxies_1)
MVA_parameter <- dplyr::select(example_proxies_1, MVA)

#' Mean vessel area example proxy from 2012 - 1940
#'
#' A dataset with MVA proxy records from a lowland forest Mlače in Slovenia.
#' The first row represents a value of a year in 2012. Row names represent
#' years.
#'
#' @format A data frame with 73 rows and 1 variable:
#' \describe{
#'   \item{MVA}{Mean vessel area [mm^2] indices from 2012 - 1940}
#' }
#' @source Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"data_MVA"

#' plot_heatmap
#'
#' Graphs a heatmap of values stored in a matrix, such as produced
#' by \code{\link{daily_response}} or \code{\link{monthly_response}} functions.
#'
#' @param result_daily_response a list with objects as produced by
#' \code{\link{daily_response}}, \code{\link{monthly_response}},
#' \code{\link{daily_response_seascorr}} or
#' \code{\link{monthly_response_seascorr}}.
#' @param reference_window character string, the reference_window argument describes
#' how each calculation is referred. Options are 'start', 'end' and 'middle'.
#' If available, the value stored in result_daily_response$reference_window is used.
#' @param type the character string describing type of analysis: daily or monthly.
#' @param show_title logical. If TRUE, the plot title with analysed period, method
#' and reference-window information is shown. The default is FALSE.
#' @param show_year_separators logical. If TRUE, dashed vertical lines are
#' added between relative-year blocks in plots with previous-year data.
#' The default is TRUE.
#'
#' @return A ggplot2 object containing the heatmap display
#'
#' @keywords internal

plot_heatmap <- function(result_daily_response,
                         reference_window = "start",
                         type = "daily",
                         show_title = FALSE,
                         show_year_separators = TRUE) {

  variable <- NULL
  temp_row_names <- NULL
  Value <- NULL

  # Prefer metadata stored in the result object
  if (!is.null(result_daily_response$reference_window)) {
    reference_window <- result_daily_response$reference_window
  }

  reference_position_label <- if (reference_window == "start") {
    "start of window"
  } else if (reference_window == "end") {
    "end of window"
  } else if (reference_window == "middle") {
    "middle of window"
  } else {
    reference_window
  }

  # Extract matrix and keep original column names
  result_daily_element1 <- data.frame(result_daily_response[[1]],
                                      check.names = FALSE)

  n_matrix_cols <- ncol(result_daily_element1)

  a <- max(result_daily_element1, na.rm = TRUE)
  b <- min(result_daily_element1, na.rm = TRUE)

  if ((a > 0 & b > 0) | (a < 0 & b < 0)) {
    paleta <- "viridis"
  } else {
    paleta <- "no_viridis"
  }

  # Strings for optional title
  period_string <- paste0("\nAnalysed Period: ", result_daily_response[[4]])

  if (result_daily_response[[2]] == "cor") {
    method_string <- paste0("\nMethod: Correlation Coefficient (",
                            result_daily_response[[3]], ")")
  } else if (result_daily_response[[2]] == "pcor") {
    method_string <- paste0("\nMethod: Partial Correlation Coefficient (",
                            result_daily_response[[3]], ")")
  } else if (result_daily_response[[2]] == "lm") {
    method_string <- paste0("\nMethod: Linear Regression")
  } else if (result_daily_response[[2]] == "brnn") {
    method_string <- paste0("\nMethod: ANN with Bayesian Regularization")
  } else {
    method_string <- paste0("\nMethod: ", result_daily_response[[2]])
  }

  if (reference_window == "start") {
    reference_string <- "\nDOY Reference of Each Calculation is the Beginning of the Window"
  } else if (reference_window == "end") {
    reference_string <- "\nDOY Reference of Each Calculation is the End Day of the Window"
  } else if (reference_window == "middle") {
    reference_string <- "\nDOY Reference of Each Calculation is the Middle Day of the Window"
  } else {
    reference_string <- ""
  }

  # Legend label
  if (result_daily_response[[2]] == "cor") {
    temp_string <- "Correlation Coefficient"
  } else if (result_daily_response[[2]] == "pcor") {
    temp_string <- "Partial Correlation Coefficient"
  } else if (result_daily_response[[3]] == "r.squared") {
    temp_string <- "Explained Variance"
  } else if (result_daily_response[[3]] == "adj.r.squared") {
    temp_string <- "Adjusted Explained Variance"
  } else {
    temp_string <- "Value"
  }

  # Prepare data for ggplot
  result_daily_element1$temp_row_names <- row.names(result_daily_element1)

  result_daily_element1_melted <- melt(result_daily_element1,
                                       id.vars = "temp_row_names")

  colnames(result_daily_element1_melted)[3] <- "Value"

  # Ensure x positions are numeric
  result_daily_element1_melted$variable <- as.numeric(gsub(
    "^X", "", as.character(result_daily_element1_melted$variable)
  ))

  result_daily_element1_melted <- result_daily_element1_melted[
    !is.na(result_daily_element1_melted$variable), , drop = FALSE
  ]

  # Colour scaling
  min_limit <- min(result_daily_element1_melted$Value, na.rm = TRUE)
  max_limit <- max(result_daily_element1_melted$Value, na.rm = TRUE)

  bounds <- quantile(result_daily_element1_melted$Value,
                     probs = seq(0, 1, 0.01),
                     na.rm = TRUE)

  bound1 <- bounds[1]
  bound2 <- bounds[20]
  bound3 <- bounds[50]
  bound4 <- bounds[100]

  if (nrow(result_daily_response[[1]]) * n_matrix_cols < 500) {

    bounds <- quantile(result_daily_element1_melted$Value,
                       probs = seq(0, 1, 0.1),
                       na.rm = TRUE)

    bound1 <- bounds[1]
    bound2 <- bounds[2]
    bound3 <- bounds[5]
    bound4 <- bounds[11]
  }

  journal_theme <- theme_bw() +
    theme(axis.text = element_text(size = 16, face = "bold"),
          axis.title = element_text(size = 18),
          text = element_text(size = 18),
          plot.title = element_text(size = 16, face = "bold"),
          legend.position = "bottom",
          legend.key.width = unit(3, "line"))

  if (type == "daily") {

    days_per_year <- 366L

    if (!is.null(result_daily_response$number_previous_years)) {
      number_previous_years <- result_daily_response$number_previous_years
    } else {
      number_previous_years <- ceiling(n_matrix_cols / days_per_year) - 1L
    }

    if (is.na(number_previous_years) || number_previous_years < 0L) {
      number_previous_years <- 0L
    }

    number_previous_years <- as.integer(number_previous_years)

    year_labels <- if (number_previous_years == 0L) {
      "Y"
    } else {
      c(paste0("Y-", number_previous_years:1), "Y")
    }

    year_start <- seq(1,
                      by = days_per_year,
                      length.out = length(year_labels))

    # Breaks: first day of each year block gets only Y-label,
    # then 100, 200 and 300 are repeated within each block.
    x_breaks <- c()
    x_labels <- c()

    for (i in seq_along(year_start)) {

      start_i <- year_start[i]

      candidate_breaks <- c(start_i,
                            start_i + 99,
                            start_i + 199,
                            start_i + 299)

      candidate_breaks <- candidate_breaks[
        candidate_breaks <= n_matrix_cols
      ]

      candidate_labels <- c(year_labels[i], "100", "200", "300")
      candidate_labels <- candidate_labels[seq_along(candidate_breaks)]

      x_breaks <- c(x_breaks, candidate_breaks)
      x_labels <- c(x_labels, candidate_labels)
    }

    year_separator_positions <- year_start[-1] - 0.5
    year_separator_positions <- year_separator_positions[
      year_separator_positions <= n_matrix_cols
    ]

    x_axis_label <- if (number_previous_years > 0L) {
      paste0("Relative year and DOY (", reference_position_label, ")")
    } else {
      paste0("DOY (", reference_position_label, ")")
    }

    final_plot <- suppressWarnings(
      ggplot(result_daily_element1_melted,
             aes(x = variable,
                 y = as.numeric(temp_row_names),
                 fill = Value)) +
        geom_tile() +
        xlab(x_axis_label) +
        ylab("Window Width") +
        scale_x_continuous(
          expand = c(0, 0),
          breaks = x_breaks,
          labels = x_labels
        ) +
        journal_theme +
        theme(axis.text.x.bottom = element_text(size = 13))
    )

    if (show_year_separators && number_previous_years > 0L) {
      final_plot <- final_plot +
        geom_vline(xintercept = year_separator_positions,
                   linewidth = 0.4,
                   linetype = "dashed",
                   colour = "grey40")
    }

    if (paleta == "viridis") {

      final_plot <- final_plot +
        scale_fill_viridis(temp_string,
                           na.value = "white",
                           direction = -1)

    } else {

      final_plot <- final_plot +
        scale_fill_gradientn(temp_string,
                             colours = c("red4", "orange", "cyan", "blue4"),
                             values = rescale(c(bound1, bound2, bound3, bound4)),
                             guide = "colorbar",
                             limits = c(min_limit, max_limit),
                             na.value = "white")
    }

    if (nrow(result_daily_response[[1]]) < 5) {

      final_plot <- final_plot +
        scale_y_continuous(expand = c(0, 0),
                           breaks = pretty_breaks(
                             n = nrow(result_daily_response[[1]])
                           ))

    } else {

      final_plot <- final_plot +
        scale_y_continuous(expand = c(0, 0),
                           breaks = pretty_breaks())
    }

    if (show_title) {
      final_plot <- final_plot +
        ggtitle(paste0(period_string, method_string, reference_string))
    }
  }

  if (type == "monthly") {

    months_per_year <- 12L

    if (!is.null(result_daily_response$number_previous_years)) {
      number_previous_years <- result_daily_response$number_previous_years
    } else {
      number_previous_years <- ceiling(n_matrix_cols / months_per_year) - 1L
    }

    if (is.na(number_previous_years) || number_previous_years < 0L) {
      number_previous_years <- 0L
    }

    number_previous_years <- as.integer(number_previous_years)

    year_labels <- if (number_previous_years == 0L) {
      "Y"
    } else {
      c(paste0("Y-", number_previous_years:1), "Y")
    }

    year_start <- seq(1,
                      by = months_per_year,
                      length.out = length(year_labels))

    month_labels <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
                      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")

    x_breaks <- c()
    x_labels <- c()

    for (i in seq_along(year_start)) {

      start_i <- year_start[i]

      candidate_breaks <- start_i:(start_i + months_per_year - 1)
      candidate_breaks <- candidate_breaks[
        candidate_breaks <= n_matrix_cols
      ]

      candidate_labels <- month_labels[seq_along(candidate_breaks)]

      if (number_previous_years > 0L && length(candidate_labels) > 0) {
        candidate_labels[1] <- year_labels[i]
      }

      x_breaks <- c(x_breaks, candidate_breaks)
      x_labels <- c(x_labels, candidate_labels)
    }

    year_separator_positions <- year_start[-1] - 0.5
    year_separator_positions <- year_separator_positions[
      year_separator_positions <= n_matrix_cols
    ]

    x_axis_label <- if (number_previous_years > 0L) {
      paste0("Relative year and month (", reference_position_label, ")")
    } else {
      paste0("Month (", reference_position_label, ")")
    }

    final_plot <- suppressWarnings(
      ggplot(result_daily_element1_melted,
             aes(x = variable,
                 y = as.numeric(temp_row_names),
                 fill = Value)) +
        geom_tile() +
        ylab("Number of Consecutive Months") +
        xlab(x_axis_label) +
        scale_x_continuous(expand = c(0, 0),
                           breaks = x_breaks,
                           labels = x_labels) +
        scale_y_continuous(expand = c(0, 0),
                           breaks = pretty_breaks()) +
        journal_theme +
        theme(axis.text.x.bottom = element_text(size = 13))
    )

    if (show_year_separators && number_previous_years > 0L) {
      final_plot <- final_plot +
        geom_vline(xintercept = year_separator_positions,
                   linewidth = 0.4,
                   linetype = "dashed",
                   colour = "grey40")
    }

    if (show_title) {
      final_plot <- final_plot +
        ggtitle(paste0(period_string, method_string))
    }

    if (paleta == "viridis") {

      final_plot <- final_plot +
        scale_fill_viridis(temp_string,
                           na.value = "white",
                           direction = -1)

    } else {

      final_plot <- final_plot +
        scale_fill_gradientn(temp_string,
                             colours = c("red4", "orange", "cyan", "blue4"),
                             values = rescale(c(bound1, bound2, bound3, bound4)),
                             guide = "colorbar",
                             limits = c(min_limit, max_limit),
                             na.value = "white")
    }
  }

  final_plot
}

#' glimpse_daily_data
#'
#' Visual presentation of daily data to spot missing values.
#'
#' @param env_data a data frame of daily sequences of environmental data as
#' columns and years as row names. Each row represents a year and
#' each column represents a day of a year. Alternatively, env_data could be a
#' tidy data with three columns, i.e. Year, DOY and third column representing
#' values of mean temperatures, sum of precipitation etc. If tidy data is passed
#' to the function, set the argument tidy_env_data to TRUE.
#' @param tidy_env_data if set to TRUE, env_data should be inserted as a data frame with three
#' columns: "Year", "DOY", "Precipitation/Temperature/etc."
#' @param na.color color to use for missing values
#' @param low_color colours for low end of the gradient
#' @param high_color colours for high end of the gradient
#'
#' @examples
#' library(dendroTools)
#' data("LJ_daily_temperatures")
#' # glimpse_daily_data(env_data = LJ_daily_temperatures,
#' #tidy_env_data = FALSE, na.color = "white")
#'
#' data("LJ_daily_precipitation")
#' # glimpse_daily_data(env_data = LJ_daily_precipitation,
#' #   tidy_env_data = TRUE, na.color = "white")

glimpse_daily_data <- function(env_data, na.color = "red", low_color = "blue", high_color = "green",
                               tidy_env_data = FALSE){

  if (tidy_env_data == FALSE){

  a <- seq(from = 1, to = ncol(env_data), by = 1)
  colnames(env_data) <- a

  # Data manipulation. The goal of this part is to prepare data for ggplot
  env_data$temp_row_names <- row.names(env_data)
  env_data_melted <- melt(env_data, id.vars = "temp_row_names")
  colnames(env_data_melted)[3] <- "Legend"
  } else if (tidy_env_data == TRUE){
    colnames(env_data) <- c("temp_row_names", "variable", "Legend")
    env_data_melted <- env_data
  }

  min_year = min(as.numeric(row.names(env_data)))
  max_year = max(as.numeric(row.names(env_data)))

  b <- length(unique(env_data_melted$variable))

  journal_theme <- theme_bw() +
    theme(axis.text = element_text(size = 16, face = "bold"),
          axis.title = element_text(size = 18), text = element_text(size = 18),
          plot.title = element_text(size = 16,  face = "bold"))

  ggplot(env_data_melted, aes_(x = ~as.numeric(variable),
                               y = ~as.numeric(temp_row_names),
                               fill = ~Legend)) + geom_tile() +
    scale_fill_gradient(na.value = na.color, low = low_color, high = high_color) +
    scale_y_continuous(expand = c(0, 0), breaks = seq(from = min_year, to = max_year, by = 10)) +
    scale_x_continuous(expand = c(0, 0), breaks = seq(from = 50, to = b, by = 50)) +
    xlab("Day of Year") +
    ylab("Year") +
    journal_theme

}

#' data_transform
#'
#' Transforms daily data with two columns (date and variable) into data frame suitable for daily or
#' monthly analysis with dendroTools.
#'
#' @param input typical daily data format: Data frame with two columns, first column represents date,
#' second column represents variable, such as mean temperature, precipitation, etc. Date should be in
#' format Year-Month-Day (e.g. "2019-05-15")
#' @param format character string indicating the desired output format. Should be "daily" or "monthly".
#' Daily format returns a data frame with 366 columns (days), while monthly format returns data frame
#' with 12 columns (months). Years are indicated as row names.
#' @param monthly_aggregate_function character string indicating, how to aggregate daily into monthly
#' data. It can be "mean" or "sum". Third option is "auto" (default). In this case function will try
#' to guess whether input is temperature or precipitation data. For temperature, it will use "mean",
#' for precipitation "sum".
#' @param date_format Describe the format of date. It should be one of "ymd", "ydm", "myd", "mdy",
#' "dmy", "dym".
#'
#' @return env_data suitable for daily or monthly analysis with dendroTools.
#'
#' @examples
#' data(swit272_daily_temperatures)
#' proper_daily_data <- data_transform(swit272_daily_temperatures, format = "daily",
#'    date_format = "ymd")
#'
#' proper_monthly_data <- data_transform(swit272_daily_temperatures, format = "monthly",
#'    date_format = "ymd")
#'
#' data(swit272_daily_precipitation)
#' proper_daily_data <- data_transform(swit272_daily_precipitation, format = "daily",
#'    date_format = "ymd")
#'
#' proper_monthly_data <- data_transform(swit272_daily_precipitation, format = "monthly",
#'    date_format = "ymd")

data_transform <- function(input, format = "daily",
                           monthly_aggregate_function = "auto",
                            date_format = "ymd"){

  colnames(input) <- c("date", "variable")

  # This is needed to ensure the proper functionality
  input <- data.frame(input)

  if (date_format == "ymd"){
    input$date <- ymd(input[,"date"])
  } else if ( date_format == "ydm"){
    input$date <- ydm(input[,"date"])
  } else if (date_format == "mdy"){
    input$date <- mdy(input[,"date"])
  } else if ( date_format == "myd"){
    input$date <- myd(input[,"date"])
  } else if ( date_format == "dmy"){
    input$date <- dmy(input[,"date"])
  } else if ( date_format == "dym"){
    input$date <- dym(input[,"date"])
  } else {
    stop(paste("The argument date_format is not in one of ydm, ymd, mdy, myd, dym, dmy!"))
  }

    input$year <- year(input$date)

  if (monthly_aggregate_function == "auto"){
    share_na <- sum(input$variable == 0, na.rm = TRUE)/length(input$variable)

    if (share_na > 0.1){

      monthly_aggregate_function <- "sum"

    } else {

      monthly_aggregate_function <- "mean"
    }
  }

  if (format == "daily"){

    input$doy <- yday(input$date)
    input$date <- NULL
    daily_matrix <- dcast(year ~ doy, data = input, value.var = "variable", fill = NA)
    row.names(daily_matrix) <- daily_matrix$year
    daily_matrix$year <- NULL

  } else if (format == "monthly") {

    input$month <- month(input$date)
    input$date <- NULL
    colnames(input)[1] <- "value"


    if (monthly_aggregate_function == "mean"){
      daily_matrix <- dcast(year ~ month, data = input, value.var = "value", fun.aggregate = mean, na.rm = TRUE)

    } else if (monthly_aggregate_function == "sum") {

      # here we create user specified sum, since Primitive sum returns 0 when only NA values are present
      daily_matrix <- dcast(year ~ month, data = input, value.var = "value", fun.aggregate = function(x)

        if (length(x) < 1) NA_real_ else if (sum(!is.na(x)) < 1) NA_real_ else sum(x, na.rm = TRUE))

    } else {
      stop(paste0("monthly_aggregate_function argument should be 'mean' or 'sum'! It is ", monthly_aggregate_function))
    }

    row.names(daily_matrix) <- daily_matrix$year
    daily_matrix$year <- NULL

  } else {
    stop(paste0("format argument should be 'daily' or 'monthly'! It is ", format))
    }

  daily_matrix
}

#' Daily mean temperatures for Kredarica (Alps in Slovenia) from 2017 - 1955
#'
#' A dataset of daily mean temperatures in Kredarica (Slovenia). The first row
#' represents temperatures in 1955. The first column represents the first day
#' of a year, the second column represents the second day of a year, etc.
#' Row names represent years.
#'
#' @format A data frame with 63 rows and 366 variables:
#' \describe{
#'   \item{X1}{Temperatures on the day 1 of a year}
#'   \item{X2}{Temperatures on the day 2 of a year}
#'   \item{X3}{Temperatures on the day 3 of a year}
#'   \item{X4}{Temperatures on the day 4 of a year}
#'   \item{X5}{Temperatures on the day 5 of a year}
#'   \item{X6}{Temperatures on the day 6 of a year}
#'   \item{X7}{Temperatures on the day 7 of a year}
#'   \item{X8}{Temperatures on the day 8 of a year}
#'   \item{X9}{Temperatures on the day 9 of a year}
#'   \item{X10}{Temperatures on the day 10 of a year}
#'   \item{X11}{Temperatures on the day 11 of a year}
#'   \item{X12}{Temperatures on the day 12 of a year}
#'   \item{X13}{Temperatures on the day 13 of a year}
#'   \item{X14}{Temperatures on the day 14 of a year}
#'   \item{X15}{Temperatures on the day 15 of a year}
#'   \item{X16}{Temperatures on the day 16 of a year}
#'   \item{X17}{Temperatures on the day 17 of a year}
#'   \item{X18}{Temperatures on the day 18 of a year}
#'   \item{X19}{Temperatures on the day 19 of a year}
#'   \item{X20}{Temperatures on the day 20 of a year}
#'   \item{X21}{Temperatures on the day 21 of a year}
#'   \item{X22}{Temperatures on the day 22 of a year}
#'   \item{X23}{Temperatures on the day 23 of a year}
#'   \item{X24}{Temperatures on the day 24 of a year}
#'   \item{X25}{Temperatures on the day 25 of a year}
#'   \item{X26}{Temperatures on the day 26 of a year}
#'   \item{X27}{Temperatures on the day 27 of a year}
#'   \item{X28}{Temperatures on the day 28 of a year}
#'   \item{X29}{Temperatures on the day 29 of a year}
#'   \item{X30}{Temperatures on the day 30 of a year}
#'   \item{X31}{Temperatures on the day 31 of a year}
#'   \item{X32}{Temperatures on the day 32 of a year}
#'   \item{X33}{Temperatures on the day 33 of a year}
#'   \item{X34}{Temperatures on the day 34 of a year}
#'   \item{X35}{Temperatures on the day 35 of a year}
#'   \item{X36}{Temperatures on the day 36 of a year}
#'   \item{X37}{Temperatures on the day 37 of a year}
#'   \item{X38}{Temperatures on the day 38 of a year}
#'   \item{X39}{Temperatures on the day 39 of a year}
#'   \item{X40}{Temperatures on the day 40 of a year}
#'   \item{X41}{Temperatures on the day 41 of a year}
#'   \item{X42}{Temperatures on the day 42 of a year}
#'   \item{X43}{Temperatures on the day 43 of a year}
#'   \item{X44}{Temperatures on the day 44 of a year}
#'   \item{X45}{Temperatures on the day 45 of a year}
#'   \item{X46}{Temperatures on the day 46 of a year}
#'   \item{X47}{Temperatures on the day 47 of a year}
#'   \item{X48}{Temperatures on the day 48 of a year}
#'   \item{X49}{Temperatures on the day 49 of a year}
#'   \item{X50}{Temperatures on the day 50 of a year}
#'   \item{X51}{Temperatures on the day 51 of a year}
#'   \item{X52}{Temperatures on the day 52 of a year}
#'   \item{X53}{Temperatures on the day 53 of a year}
#'   \item{X54}{Temperatures on the day 54 of a year}
#'   \item{X55}{Temperatures on the day 55 of a year}
#'   \item{X56}{Temperatures on the day 56 of a year}
#'   \item{X57}{Temperatures on the day 57 of a year}
#'   \item{X58}{Temperatures on the day 58 of a year}
#'   \item{X59}{Temperatures on the day 59 of a year}
#'   \item{X60}{Temperatures on the day 60 of a year}
#'   \item{X61}{Temperatures on the day 61 of a year}
#'   \item{X62}{Temperatures on the day 62 of a year}
#'   \item{X63}{Temperatures on the day 63 of a year}
#'   \item{X64}{Temperatures on the day 64 of a year}
#'   \item{X65}{Temperatures on the day 65 of a year}
#'   \item{X66}{Temperatures on the day 66 of a year}
#'   \item{X67}{Temperatures on the day 67 of a year}
#'   \item{X68}{Temperatures on the day 68 of a year}
#'   \item{X69}{Temperatures on the day 69 of a year}
#'   \item{X70}{Temperatures on the day 70 of a year}
#'   \item{X71}{Temperatures on the day 71 of a year}
#'   \item{X72}{Temperatures on the day 72 of a year}
#'   \item{X73}{Temperatures on the day 73 of a year}
#'   \item{X74}{Temperatures on the day 74 of a year}
#'   \item{X75}{Temperatures on the day 75 of a year}
#'   \item{X76}{Temperatures on the day 76 of a year}
#'   \item{X77}{Temperatures on the day 77 of a year}
#'   \item{X78}{Temperatures on the day 78 of a year}
#'   \item{X79}{Temperatures on the day 79 of a year}
#'   \item{X80}{Temperatures on the day 80 of a year}
#'   \item{X81}{Temperatures on the day 81 of a year}
#'   \item{X82}{Temperatures on the day 82 of a year}
#'   \item{X83}{Temperatures on the day 83 of a year}
#'   \item{X84}{Temperatures on the day 84 of a year}
#'   \item{X85}{Temperatures on the day 85 of a year}
#'   \item{X86}{Temperatures on the day 86 of a year}
#'   \item{X87}{Temperatures on the day 87 of a year}
#'   \item{X88}{Temperatures on the day 88 of a year}
#'   \item{X89}{Temperatures on the day 89 of a year}
#'   \item{X90}{Temperatures on the day 90 of a year}
#'   \item{X91}{Temperatures on the day 91 of a year}
#'   \item{X92}{Temperatures on the day 92 of a year}
#'   \item{X93}{Temperatures on the day 93 of a year}
#'   \item{X94}{Temperatures on the day 94 of a year}
#'   \item{X95}{Temperatures on the day 95 of a year}
#'   \item{X96}{Temperatures on the day 96 of a year}
#'   \item{X97}{Temperatures on the day 97 of a year}
#'   \item{X98}{Temperatures on the day 98 of a year}
#'   \item{X99}{Temperatures on the day 99 of a year}
#'   \item{X100}{Temperatures on the day 100 of a year}
#'   \item{X101}{Temperatures on the day 101 of a year}
#'   \item{X102}{Temperatures on the day 102 of a year}
#'   \item{X103}{Temperatures on the day 103 of a year}
#'   \item{X104}{Temperatures on the day 104 of a year}
#'   \item{X105}{Temperatures on the day 105 of a year}
#'   \item{X106}{Temperatures on the day 106 of a year}
#'   \item{X107}{Temperatures on the day 107 of a year}
#'   \item{X108}{Temperatures on the day 108 of a year}
#'   \item{X109}{Temperatures on the day 109 of a year}
#'   \item{X110}{Temperatures on the day 110 of a year}
#'   \item{X111}{Temperatures on the day 111 of a year}
#'   \item{X112}{Temperatures on the day 112 of a year}
#'   \item{X113}{Temperatures on the day 113 of a year}
#'   \item{X114}{Temperatures on the day 114 of a year}
#'   \item{X115}{Temperatures on the day 115 of a year}
#'   \item{X116}{Temperatures on the day 116 of a year}
#'   \item{X117}{Temperatures on the day 117 of a year}
#'   \item{X118}{Temperatures on the day 118 of a year}
#'   \item{X119}{Temperatures on the day 119 of a year}
#'   \item{X120}{Temperatures on the day 120 of a year}
#'   \item{X121}{Temperatures on the day 121 of a year}
#'   \item{X122}{Temperatures on the day 122 of a year}
#'   \item{X123}{Temperatures on the day 123 of a year}
#'   \item{X124}{Temperatures on the day 124 of a year}
#'   \item{X125}{Temperatures on the day 125 of a year}
#'   \item{X126}{Temperatures on the day 126 of a year}
#'   \item{X127}{Temperatures on the day 127 of a year}
#'   \item{X128}{Temperatures on the day 128 of a year}
#'   \item{X129}{Temperatures on the day 129 of a year}
#'   \item{X130}{Temperatures on the day 130 of a year}
#'   \item{X131}{Temperatures on the day 131 of a year}
#'   \item{X132}{Temperatures on the day 132 of a year}
#'   \item{X133}{Temperatures on the day 133 of a year}
#'   \item{X134}{Temperatures on the day 134 of a year}
#'   \item{X135}{Temperatures on the day 135 of a year}
#'   \item{X136}{Temperatures on the day 136 of a year}
#'   \item{X137}{Temperatures on the day 137 of a year}
#'   \item{X138}{Temperatures on the day 138 of a year}
#'   \item{X139}{Temperatures on the day 139 of a year}
#'   \item{X140}{Temperatures on the day 140 of a year}
#'   \item{X141}{Temperatures on the day 141 of a year}
#'   \item{X142}{Temperatures on the day 142 of a year}
#'   \item{X143}{Temperatures on the day 143 of a year}
#'   \item{X144}{Temperatures on the day 144 of a year}
#'   \item{X145}{Temperatures on the day 145 of a year}
#'   \item{X146}{Temperatures on the day 146 of a year}
#'   \item{X147}{Temperatures on the day 147 of a year}
#'   \item{X148}{Temperatures on the day 148 of a year}
#'   \item{X149}{Temperatures on the day 149 of a year}
#'   \item{X150}{Temperatures on the day 150 of a year}
#'   \item{X151}{Temperatures on the day 151 of a year}
#'   \item{X152}{Temperatures on the day 152 of a year}
#'   \item{X153}{Temperatures on the day 153 of a year}
#'   \item{X154}{Temperatures on the day 154 of a year}
#'   \item{X155}{Temperatures on the day 155 of a year}
#'   \item{X156}{Temperatures on the day 156 of a year}
#'   \item{X157}{Temperatures on the day 157 of a year}
#'   \item{X158}{Temperatures on the day 158 of a year}
#'   \item{X159}{Temperatures on the day 159 of a year}
#'   \item{X160}{Temperatures on the day 160 of a year}
#'   \item{X161}{Temperatures on the day 161 of a year}
#'   \item{X162}{Temperatures on the day 162 of a year}
#'   \item{X163}{Temperatures on the day 163 of a year}
#'   \item{X164}{Temperatures on the day 164 of a year}
#'   \item{X165}{Temperatures on the day 165 of a year}
#'   \item{X166}{Temperatures on the day 166 of a year}
#'   \item{X167}{Temperatures on the day 167 of a year}
#'   \item{X168}{Temperatures on the day 168 of a year}
#'   \item{X169}{Temperatures on the day 169 of a year}
#'   \item{X170}{Temperatures on the day 170 of a year}
#'   \item{X171}{Temperatures on the day 171 of a year}
#'   \item{X172}{Temperatures on the day 172 of a year}
#'   \item{X173}{Temperatures on the day 173 of a year}
#'   \item{X174}{Temperatures on the day 174 of a year}
#'   \item{X175}{Temperatures on the day 175 of a year}
#'   \item{X176}{Temperatures on the day 176 of a year}
#'   \item{X177}{Temperatures on the day 177 of a year}
#'   \item{X178}{Temperatures on the day 178 of a year}
#'   \item{X179}{Temperatures on the day 179 of a year}
#'   \item{X180}{Temperatures on the day 180 of a year}
#'   \item{X181}{Temperatures on the day 181 of a year}
#'   \item{X182}{Temperatures on the day 182 of a year}
#'   \item{X183}{Temperatures on the day 183 of a year}
#'   \item{X184}{Temperatures on the day 184 of a year}
#'   \item{X185}{Temperatures on the day 185 of a year}
#'   \item{X186}{Temperatures on the day 186 of a year}
#'   \item{X187}{Temperatures on the day 187 of a year}
#'   \item{X188}{Temperatures on the day 188 of a year}
#'   \item{X189}{Temperatures on the day 189 of a year}
#'   \item{X190}{Temperatures on the day 190 of a year}
#'   \item{X191}{Temperatures on the day 191 of a year}
#'   \item{X192}{Temperatures on the day 192 of a year}
#'   \item{X193}{Temperatures on the day 193 of a year}
#'   \item{X194}{Temperatures on the day 194 of a year}
#'   \item{X195}{Temperatures on the day 195 of a year}
#'   \item{X196}{Temperatures on the day 196 of a year}
#'   \item{X197}{Temperatures on the day 197 of a year}
#'   \item{X198}{Temperatures on the day 198 of a year}
#'   \item{X199}{Temperatures on the day 199 of a year}
#'   \item{X200}{Temperatures on the day 200 of a year}
#'   \item{X201}{Temperatures on the day 201 of a year}
#'   \item{X202}{Temperatures on the day 202 of a year}
#'   \item{X203}{Temperatures on the day 203 of a year}
#'   \item{X204}{Temperatures on the day 204 of a year}
#'   \item{X205}{Temperatures on the day 205 of a year}
#'   \item{X206}{Temperatures on the day 206 of a year}
#'   \item{X207}{Temperatures on the day 207 of a year}
#'   \item{X208}{Temperatures on the day 208 of a year}
#'   \item{X209}{Temperatures on the day 209 of a year}
#'   \item{X210}{Temperatures on the day 210 of a year}
#'   \item{X211}{Temperatures on the day 211 of a year}
#'   \item{X212}{Temperatures on the day 212 of a year}
#'   \item{X213}{Temperatures on the day 213 of a year}
#'   \item{X214}{Temperatures on the day 214 of a year}
#'   \item{X215}{Temperatures on the day 215 of a year}
#'   \item{X216}{Temperatures on the day 216 of a year}
#'   \item{X217}{Temperatures on the day 217 of a year}
#'   \item{X218}{Temperatures on the day 218 of a year}
#'   \item{X219}{Temperatures on the day 219 of a year}
#'   \item{X220}{Temperatures on the day 220 of a year}
#'   \item{X221}{Temperatures on the day 221 of a year}
#'   \item{X222}{Temperatures on the day 222 of a year}
#'   \item{X223}{Temperatures on the day 223 of a year}
#'   \item{X224}{Temperatures on the day 224 of a year}
#'   \item{X225}{Temperatures on the day 225 of a year}
#'   \item{X226}{Temperatures on the day 226 of a year}
#'   \item{X227}{Temperatures on the day 227 of a year}
#'   \item{X228}{Temperatures on the day 228 of a year}
#'   \item{X229}{Temperatures on the day 229 of a year}
#'   \item{X230}{Temperatures on the day 230 of a year}
#'   \item{X231}{Temperatures on the day 231 of a year}
#'   \item{X232}{Temperatures on the day 232 of a year}
#'   \item{X233}{Temperatures on the day 233 of a year}
#'   \item{X234}{Temperatures on the day 234 of a year}
#'   \item{X235}{Temperatures on the day 235 of a year}
#'   \item{X236}{Temperatures on the day 236 of a year}
#'   \item{X237}{Temperatures on the day 237 of a year}
#'   \item{X238}{Temperatures on the day 238 of a year}
#'   \item{X239}{Temperatures on the day 239 of a year}
#'   \item{X240}{Temperatures on the day 240 of a year}
#'   \item{X241}{Temperatures on the day 241 of a year}
#'   \item{X242}{Temperatures on the day 242 of a year}
#'   \item{X243}{Temperatures on the day 243 of a year}
#'   \item{X244}{Temperatures on the day 244 of a year}
#'   \item{X245}{Temperatures on the day 245 of a year}
#'   \item{X246}{Temperatures on the day 246 of a year}
#'   \item{X247}{Temperatures on the day 247 of a year}
#'   \item{X248}{Temperatures on the day 248 of a year}
#'   \item{X249}{Temperatures on the day 249 of a year}
#'   \item{X250}{Temperatures on the day 250 of a year}
#'   \item{X251}{Temperatures on the day 251 of a year}
#'   \item{X252}{Temperatures on the day 252 of a year}
#'   \item{X253}{Temperatures on the day 253 of a year}
#'   \item{X254}{Temperatures on the day 254 of a year}
#'   \item{X255}{Temperatures on the day 255 of a year}
#'   \item{X256}{Temperatures on the day 256 of a year}
#'   \item{X257}{Temperatures on the day 257 of a year}
#'   \item{X258}{Temperatures on the day 258 of a year}
#'   \item{X259}{Temperatures on the day 259 of a year}
#'   \item{X260}{Temperatures on the day 260 of a year}
#'   \item{X261}{Temperatures on the day 261 of a year}
#'   \item{X262}{Temperatures on the day 262 of a year}
#'   \item{X263}{Temperatures on the day 263 of a year}
#'   \item{X264}{Temperatures on the day 264 of a year}
#'   \item{X265}{Temperatures on the day 265 of a year}
#'   \item{X266}{Temperatures on the day 266 of a year}
#'   \item{X267}{Temperatures on the day 267 of a year}
#'   \item{X268}{Temperatures on the day 268 of a year}
#'   \item{X269}{Temperatures on the day 269 of a year}
#'   \item{X270}{Temperatures on the day 270 of a year}
#'   \item{X271}{Temperatures on the day 271 of a year}
#'   \item{X272}{Temperatures on the day 272 of a year}
#'   \item{X273}{Temperatures on the day 273 of a year}
#'   \item{X274}{Temperatures on the day 274 of a year}
#'   \item{X275}{Temperatures on the day 275 of a year}
#'   \item{X276}{Temperatures on the day 276 of a year}
#'   \item{X277}{Temperatures on the day 277 of a year}
#'   \item{X278}{Temperatures on the day 278 of a year}
#'   \item{X279}{Temperatures on the day 279 of a year}
#'   \item{X280}{Temperatures on the day 280 of a year}
#'   \item{X281}{Temperatures on the day 281 of a year}
#'   \item{X282}{Temperatures on the day 282 of a year}
#'   \item{X283}{Temperatures on the day 283 of a year}
#'   \item{X284}{Temperatures on the day 284 of a year}
#'   \item{X285}{Temperatures on the day 285 of a year}
#'   \item{X286}{Temperatures on the day 286 of a year}
#'   \item{X287}{Temperatures on the day 287 of a year}
#'   \item{X288}{Temperatures on the day 288 of a year}
#'   \item{X289}{Temperatures on the day 289 of a year}
#'   \item{X290}{Temperatures on the day 290 of a year}
#'   \item{X291}{Temperatures on the day 291 of a year}
#'   \item{X292}{Temperatures on the day 292 of a year}
#'   \item{X293}{Temperatures on the day 293 of a year}
#'   \item{X294}{Temperatures on the day 294 of a year}
#'   \item{X295}{Temperatures on the day 295 of a year}
#'   \item{X296}{Temperatures on the day 296 of a year}
#'   \item{X297}{Temperatures on the day 297 of a year}
#'   \item{X298}{Temperatures on the day 298 of a year}
#'   \item{X299}{Temperatures on the day 299 of a year}
#'   \item{X300}{Temperatures on the day 300 of a year}
#'   \item{X301}{Temperatures on the day 301 of a year}
#'   \item{X302}{Temperatures on the day 302 of a year}
#'   \item{X303}{Temperatures on the day 303 of a year}
#'   \item{X304}{Temperatures on the day 304 of a year}
#'   \item{X305}{Temperatures on the day 305 of a year}
#'   \item{X306}{Temperatures on the day 306 of a year}
#'   \item{X307}{Temperatures on the day 307 of a year}
#'   \item{X308}{Temperatures on the day 308 of a year}
#'   \item{X309}{Temperatures on the day 309 of a year}
#'   \item{X310}{Temperatures on the day 310 of a year}
#'   \item{X311}{Temperatures on the day 311 of a year}
#'   \item{X312}{Temperatures on the day 312 of a year}
#'   \item{X313}{Temperatures on the day 313 of a year}
#'   \item{X314}{Temperatures on the day 314 of a year}
#'   \item{X315}{Temperatures on the day 315 of a year}
#'   \item{X316}{Temperatures on the day 316 of a year}
#'   \item{X317}{Temperatures on the day 317 of a year}
#'   \item{X318}{Temperatures on the day 318 of a year}
#'   \item{X319}{Temperatures on the day 319 of a year}
#'   \item{X320}{Temperatures on the day 320 of a year}
#'   \item{X321}{Temperatures on the day 321 of a year}
#'   \item{X322}{Temperatures on the day 322 of a year}
#'   \item{X323}{Temperatures on the day 323 of a year}
#'   \item{X324}{Temperatures on the day 324 of a year}
#'   \item{X325}{Temperatures on the day 325 of a year}
#'   \item{X326}{Temperatures on the day 326 of a year}
#'   \item{X327}{Temperatures on the day 327 of a year}
#'   \item{X328}{Temperatures on the day 328 of a year}
#'   \item{X329}{Temperatures on the day 329 of a year}
#'   \item{X330}{Temperatures on the day 330 of a year}
#'   \item{X331}{Temperatures on the day 331 of a year}
#'   \item{X332}{Temperatures on the day 332 of a year}
#'   \item{X333}{Temperatures on the day 333 of a year}
#'   \item{X334}{Temperatures on the day 334 of a year}
#'   \item{X335}{Temperatures on the day 335 of a year}
#'   \item{X336}{Temperatures on the day 336 of a year}
#'   \item{X337}{Temperatures on the day 337 of a year}
#'   \item{X338}{Temperatures on the day 338 of a year}
#'   \item{X339}{Temperatures on the day 339 of a year}
#'   \item{X340}{Temperatures on the day 340 of a year}
#'   \item{X341}{Temperatures on the day 341 of a year}
#'   \item{X342}{Temperatures on the day 342 of a year}
#'   \item{X343}{Temperatures on the day 343 of a year}
#'   \item{X344}{Temperatures on the day 344 of a year}
#'   \item{X345}{Temperatures on the day 345 of a year}
#'   \item{X346}{Temperatures on the day 346 of a year}
#'   \item{X347}{Temperatures on the day 347 of a year}
#'   \item{X348}{Temperatures on the day 348 of a year}
#'   \item{X349}{Temperatures on the day 349 of a year}
#'   \item{X350}{Temperatures on the day 350 of a year}
#'   \item{X351}{Temperatures on the day 351 of a year}
#'   \item{X352}{Temperatures on the day 352 of a year}
#'   \item{X353}{Temperatures on the day 353 of a year}
#'   \item{X354}{Temperatures on the day 354 of a year}
#'   \item{X355}{Temperatures on the day 355 of a year}
#'   \item{X356}{Temperatures on the day 356 of a year}
#'   \item{X357}{Temperatures on the day 357 of a year}
#'   \item{X358}{Temperatures on the day 358 of a year}
#'   \item{X359}{Temperatures on the day 359 of a year}
#'   \item{X360}{Temperatures on the day 360 of a year}
#'   \item{X361}{Temperatures on the day 361 of a year}
#'   \item{X362}{Temperatures on the day 362 of a year}
#'   \item{X363}{Temperatures on the day 363 of a year}
#'   \item{X364}{Temperatures on the day 364 of a year}
#'   \item{X365}{Temperatures on the day 365 of a year}
#'   \item{X366}{Temperatures on the day 366 of a year}
#' }
#' @source \url{https://meteo.arso.gov.si/met/sl/archive/}
#' @export
"KRE_daily_temperatures"

#' TRW and mean June -  July temperature from Albania
#'
#' A dataset with a tree-ring width (TRW) chronology of Pinus nigra from Albania and mean
#' June-July temperature. This TRW chronology has a span of 59 years (period 2009 - 1951)
#' and was already used to reconstruct summer temperatures by Levanič et al. (2015).
#' In this paper, all the details about sample replication, site description and correlation
#' statistics are described.
#'
#' @format A data frame with 59 rows and 2 variables:
#' \describe{
#'   \item{TRW}{Standardised tree-ring width chronology of Pinus nigra from Albania}
#'   \item{T_Jun_Jul}{Mean June - July temperature for Albania downloaded from KNMI Climate Explorer}
#' }
#' @source Levanič, T., Poljanšek, S., Toromani, E., 2015. Early summer temperatures reconstructed from
#' black pine (Pinus nigra Arnold) tree-ring widths from Albania. The Holocene 25, 469-481.
#'
#' @export
"dataset_TRW"

#' round2
#'
#' Rounding function, where numbers are rounded up, 0.5 = 1
#'
#' @param x a numeric vector
#' @param n integer indicating the number of decimal places
#'
#' @return a numeric vector with rounded numbers
#'
#' @references
#' http://alandgraf.blogspot.com/2012/06/rounding-in-r.html
#'
#' @keywords internal

# This is a small function to round numbers up
round2 = function(x, n = 0) {
  posneg = sign(x)
  z = abs(x)*10^n
  z = z + 0.5
  z = trunc(z)
  z = z/10^n
  z*posneg
}

#' Standardised tree-ring width chronology swit272, Larix decidua Mill.
#'
#' A TRW chronology swit272
#' Investigators: Bigler, C.; Claluna, A.
#' Site_Name:  Sils-Maria GR Blais dal Fo
#' Location: Switzerland
#' Northernmost_Latitude: 46.4333
#' Southernmost_Latitude: 46.4333
#' Easternmost_Longitude: 9.7833
#' Westernmost_Longitude: 9.7833
#' Elevation: 2100
#'
#' @format A data frame with 273 rows and 1 variable:
#' \describe{
#'   \item{TRWi}{Standardised tree-ring width chronology}
#' }
#' @source \url{https://www.ncei.noaa.gov/access/paleo-search/study/14108}
#' @export
"swit272"

#' Tree-ring width (TRW) example proxy from 1981 - 1757
#'
#' A dataset with TRW proxy records from a site in Slovenian Alps - Vrsic.
#' The first row represents a TRW value in a year 1757. Row names represent
#' years.
#'
#' @format A data frame with 225 rows and 1 variable:
#' \describe{
#'   \item{TRW}{residual TRW indices from 1981 - 1757}
#' }
#' @source
#' \itemize{
#'    \item Schweingruber, F.H., 1981. Vrsic Krajnska Gora - PCAB - ITRDB YUGO001.
#'    \item https://www.ncei.noaa.gov/access/paleo-search/study/4728}
#' @export
"data_TRW"

#' Tree-ring example proxies 1 from 2015 - 1961
#'
#' A dataset with three tree-ring proxy records from a site near Ljubljana
#' (Slovenia). The first row represents a value of a year in 1961. The three
#' proxy records are MVA (Mean vessel area [mm ^2]), O (stable oxygen isotope
#' ratios) and TRW (Tree-ring widths)
#'
#' @format A data frame with 55 rows and 3 variables:
#' \describe{
#'   \item{MVA}{Mean vessel area [mm^2] indices from 2015 - 1961}
#'   \item{O18}{Scaled Stable oxygen isotope ratios from 2015 - 1961}
#'   \item{TRW}{Tree-ring widths from 2015 - 1961}
#' }
#' @source Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
#' Ljubljana, Slovenia
#' @export
"example_proxies_1"

#' count_ones
#'
#' calculates share of integer 1 in a vector
#'
#' @param vector a vector of integers
#'
#' @return an integer of counted ones
#'
#'
#' @examples
#' vector_1 <- seq(1:10)
#' # count_ones(vector_1)
#' @keywords internal

count_ones <- function(vector){
  sum(vector == 1)
}

#' boot_f_pcor
#'
#' Generate R bootstrap replicates of a statistic applied to data.
#'
#' @param data A vector, matrix, or data frame containing the input data.
#' If a matrix or data frame is provided, each row is treated as a separate
#' multivariate observation.
#' @param indices A numeric or integer vector specifying the indices to be
#' utilized in the calculation. These indices determine which elements or rows
#' of the 'data' are to be included in the analysis
#' @param cor.type A string specifying the method of correlation to be applied
#' in the bootstrap procedure. Available options are "pearson" (default),
#' "kendall", or "spearman".
#'
#' @return A matrix with bootstrapped estimates of partial correlation coefficients
#'
#' @references https://www.datacamp.com/community/tutorials/bootstrap-r
#' @keywords internal
#'
boot_f_pcor <- function(data, indices, cor.type){

    partial.r(data=data[indices,], x=c("x","y"), y="z", use= "pairwise" , method = cor.type)[2]

  }

#' MVA and mean April temperature
#'
#' A dataset with a mean vessel area (MVA) chronology of Quercus robur from
#' a lowland oak forest in Eastern Slovenia and a mean April temperature.
#' This dataset includes years for the period 2012-1934. For a detailed
#' description about the MVA chronology development, sampling site and the
#' calculations of mean monthly correlations, see Jevšenak and Levanič (2015).
#'
#' @format A data frame with 79 rows and 2 variables:
#' \describe{
#'   \item{MVA}{Mean vessel area measurements from 2012 - 1934}
#'   \item{T_Apr}{Mean April temperature for the meteorological station Maribor from 2012 - 1934}
#' }
#' @source Jevšenak J., Levanič T. 2015. Dendrochronological and wood-anatomical features of
#' differently vital pedunculate oak (Quercus robur L.) stands and their response to climate.
#' Topola, 195/196: 85-96
#'
#' @export
"dataset_MVA"

#' @method summary dmrs
#' @export

summary.dmrs <- function(object, ...) {

  result_daily_response <- object
  result_matrix <- as.matrix(result_daily_response[[1]])

  if (!any(is.finite(result_matrix))) {

    return("All calculations are insignificant! No summary output available.")

  } else {

    # This needs to be set to provide results in English language
    Sys.setlocale("LC_TIME", "C")

    # -------------------------------------------------------------------------
    # Helper functions
    # -------------------------------------------------------------------------

    round2_local <- function(x, digits = 0) {
      if (exists("round2", mode = "function")) {
        round2(x, digits)
      } else {
        round(x, digits)
      }
    }

    make_daily_date_labels <- function() {

      doy <- seq(1:366)
      date <- seq(as.Date("2013-01-01"),
                  as.Date("2013-12-31"),
                  by = "+1 day")

      # Keep the original dendroTools convention with 366 daily columns.
      date[366] <- as.Date("2015-12-31")
      date <- format(date, "%b %d")

      data.frame(doy = doy, date = date, stringsAsFactors = FALSE)
    }

    get_daily_position_info <- function(position,
                                        number_previous_years,
                                        days_per_year = 366L) {

      block_index <- ceiling(position / days_per_year)
      doy <- ((position - 1L) %% days_per_year) + 1L

      if (number_previous_years == 0L) {
        relative_year_number <- 0L
        relative_year_label <- "Y"
      } else {
        relative_year_number <- block_index - (number_previous_years + 1L)

        relative_year_label <- if (relative_year_number == 0L) {
          "Y"
        } else {
          paste0("Y", relative_year_number)
        }
      }

      list(
        doy = doy,
        relative_year_number = relative_year_number,
        relative_year_label = relative_year_label
      )
    }

    get_monthly_position_info <- function(position,
                                          number_previous_years,
                                          months_per_year = 12L) {

      block_index <- ceiling(position / months_per_year)
      month_number <- ((position - 1L) %% months_per_year) + 1L

      if (number_previous_years == 0L) {
        relative_year_number <- 0L
        relative_year_label <- "Y"
      } else {
        relative_year_number <- block_index - (number_previous_years + 1L)

        relative_year_label <- if (relative_year_number == 0L) {
          "Y"
        } else {
          paste0("Y", relative_year_number)
        }
      }

      list(
        month_number = month_number,
        relative_year_number = relative_year_number,
        relative_year_label = relative_year_label
      )
    }

    # -------------------------------------------------------------------------
    # A) Extract basic information
    # -------------------------------------------------------------------------

    type <- as.character(result_daily_response$type)
    result_daily_element1 <- data.frame(result_daily_response[[1]],
                                        check.names = FALSE)

    reference_window <- result_daily_response$reference_window

    # -------------------------------------------------------------------------
    # B) Find the strongest absolute result
    # -------------------------------------------------------------------------

    overall_max <- max(result_matrix, na.rm = TRUE)
    overall_min <- min(result_matrix, na.rm = TRUE)

    if (abs(overall_max) >= abs(overall_min)) {

      calculated_value <- overall_max
      optimal_position <- which(result_matrix == overall_max, arr.ind = TRUE)[1, ]

    } else {

      calculated_value <- overall_min
      optimal_position <- which(result_matrix == overall_min, arr.ind = TRUE)[1, ]
    }

    row_position <- optimal_position[1]
    plot_column_source <- optimal_position[2]

    row_index <- row.names(result_daily_element1)[row_position]

    if (is.null(row_index) || is.na(row_index)) {
      row_index <- as.character(row_position)
    }

    optimal_window_width <- as.numeric(row_index)
    calculated_metric <- round(calculated_value, 3)

    if (!is.null(result_daily_response$boot_lower) &&
        !is.null(result_daily_response$boot_upper) &&
        row_position <= nrow(result_daily_response$boot_lower) &&
        plot_column_source <= ncol(result_daily_response$boot_lower)) {

      lower_bound <- result_daily_response$boot_lower[row_position,
                                                      plot_column_source]
      upper_bound <- result_daily_response$boot_upper[row_position,
                                                      plot_column_source]

    } else {

      lower_bound <- NA
      upper_bound <- NA
    }

    # -------------------------------------------------------------------------
    # C) Metric and method labels
    # -------------------------------------------------------------------------

    if (result_daily_response[[2]] == "cor") {

      metric_label <- "Correlation Coefficient"

    } else if (result_daily_response[[2]] == "pcor") {

      metric_label <- "Partial Correlation Coefficient"

    } else if (result_daily_response[[3]] == "r.squared") {

      metric_label <- "Explained Variance"

    } else if (result_daily_response[[3]] == "adj.r.squared") {

      metric_label <- "Adjusted Explained Variance"

    } else {

      metric_label <- "Calculated Metric"
    }

    if (result_daily_response[[2]] == "cor") {

      method_string <- paste0("Correlation Coefficient (",
                              result_daily_response[[3]], ")")

    } else if (result_daily_response[[2]] == "pcor") {

      method_string <- paste0("Partial Correlation Coefficient (",
                              result_daily_response[[3]], ")")

    } else if (result_daily_response[[2]] == "lm") {

      method_string <- "Linear Regression"

    } else if (result_daily_response[[2]] == "brnn") {

      method_string <- "ANN with Bayesian Regularization"

    } else {

      method_string <- as.character(result_daily_response[[2]])
    }

    # -------------------------------------------------------------------------
    # D) Window start and end positions
    # -------------------------------------------------------------------------

    if (optimal_window_width %% 2 == 0) {
      adjustment_1 <- 0
      adjustment_2 <- 1
    } else {
      adjustment_1 <- 1
      adjustment_2 <- 2
    }

    if (reference_window == "start") {

      optimal_start_position <- plot_column_source
      optimal_end_position <- plot_column_source + optimal_window_width - 1

    } else if (reference_window == "end") {

      optimal_start_position <- plot_column_source - optimal_window_width + 1
      optimal_end_position <- plot_column_source

    } else if (reference_window == "middle") {

      optimal_start_position <- round2_local(
        plot_column_source - optimal_window_width / 2, 0
      ) - adjustment_1

      optimal_end_position <- round2_local(
        plot_column_source + optimal_window_width / 2, 0
      ) - adjustment_2

    } else {

      stop("reference_window must be 'start', 'end' or 'middle'.")
    }

    # -------------------------------------------------------------------------
    # E) Daily output
    # -------------------------------------------------------------------------

    if (type == "daily") {

      days_per_year <- 366L

      if (!is.null(result_daily_response$number_previous_years)) {

        number_previous_years <- result_daily_response$number_previous_years

      } else {

        number_previous_years <- ceiling(ncol(result_daily_element1) /
                                           days_per_year) - 1L
      }

      if (is.na(number_previous_years) || number_previous_years < 0L) {
        number_previous_years <- 0L
      }

      number_previous_years <- as.integer(number_previous_years)

      full_sequence_width <- days_per_year * (number_previous_years + 1L)

      optimal_start_position <- max(1, optimal_start_position)
      optimal_end_position <- min(full_sequence_width, optimal_end_position)
      plot_column_source <- max(1, min(full_sequence_width,
                                       plot_column_source))

      date_codes <- make_daily_date_labels()

      daily_label <- function(position) {

        position_info <- get_daily_position_info(
          position = position,
          number_previous_years = number_previous_years,
          days_per_year = days_per_year
        )

        date_label <- date_codes$date[position_info$doy]

        if (number_previous_years > 0L) {
          paste0(position_info$relative_year_label, " ", date_label)
        } else {
          date_label
        }
      }

      reference_info <- get_daily_position_info(
        position = plot_column_source,
        number_previous_years = number_previous_years,
        days_per_year = days_per_year
      )

      if (reference_window == "start") {

        reference_string <- paste0("Starting Day of Optimal Window Width: Day ",
                                   reference_info$doy)

      } else if (reference_window == "end") {

        reference_string <- paste0("Ending Day of Optimal Window Width: Day ",
                                   reference_info$doy)

      } else if (reference_window == "middle") {

        reference_string <- paste0("Middle Day of Optimal Window Width: Day ",
                                   reference_info$doy)
      }

      if (number_previous_years > 0L) {
        reference_string <- paste0(reference_string,
                                   " of ",
                                   reference_info$relative_year_label)
      }

      if (optimal_window_width == 1) {

        Optimal_string <- daily_label(optimal_start_position)

      } else {

        Optimal_string <- paste0(daily_label(optimal_start_position),
                                 " - ",
                                 daily_label(optimal_end_position))
      }

      analysed_previous_year <- number_previous_years > 0L
      optimal_reference_year <- reference_info$relative_year_label
    }

    # -------------------------------------------------------------------------
    # F) Monthly output
    # -------------------------------------------------------------------------

    if (type == "monthly") {

      months_per_year <- 12L
      month_names <- c("Jan", "Feb", "Mar", "Apr", "May", "Jun",
                       "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")

      if (!is.null(result_daily_response$number_previous_years)) {

        number_previous_years <- result_daily_response$number_previous_years

      } else {

        number_previous_years <- ceiling(ncol(result_daily_element1) /
                                           months_per_year) - 1L
      }

      if (is.na(number_previous_years) || number_previous_years < 0L) {
        number_previous_years <- 0L
      }

      number_previous_years <- as.integer(number_previous_years)

      full_sequence_width <- months_per_year * (number_previous_years + 1L)

      optimal_start_position <- max(1, optimal_start_position)
      optimal_end_position <- min(full_sequence_width, optimal_end_position)
      plot_column_source <- max(1, min(full_sequence_width,
                                       plot_column_source))

      monthly_label <- function(position) {

        position_info <- get_monthly_position_info(
          position = position,
          number_previous_years = number_previous_years,
          months_per_year = months_per_year
        )

        month_label <- month_names[position_info$month_number]

        if (number_previous_years > 0L) {
          paste0(position_info$relative_year_label, " ", month_label)
        } else {
          month_label
        }
      }

      reference_info <- get_monthly_position_info(
        position = plot_column_source,
        number_previous_years = number_previous_years,
        months_per_year = months_per_year
      )

      if (reference_window == "start") {

        reference_string <- paste0("Starting Month of Optimal Window Width: Month ",
                                   reference_info$month_number)

      } else if (reference_window == "end") {

        reference_string <- paste0("Ending Month of Optimal Window Width: Month ",
                                   reference_info$month_number)

      } else if (reference_window == "middle") {

        reference_string <- paste0("Middle Month of Optimal Window Width: Month ",
                                   reference_info$month_number)
      }

      if (number_previous_years > 0L) {
        reference_string <- paste0(reference_string,
                                   " of ",
                                   reference_info$relative_year_label)
      }

      if (optimal_window_width == 1) {

        Optimal_string <- monthly_label(optimal_start_position)

      } else {

        Optimal_string <- paste0(monthly_label(optimal_start_position),
                                 " - ",
                                 monthly_label(optimal_end_position))
      }

      analysed_previous_year <- number_previous_years > 0L
      optimal_reference_year <- reference_info$relative_year_label
    }

    # -------------------------------------------------------------------------
    # G) Final output
    # -------------------------------------------------------------------------

    output_df <- data.frame(
      Variable = c("approach",
                   "method",
                   "metric",
                   "analysed_years",
                   "maximal_calculated_metric",
                   "lower_ci",
                   "upper_ci",
                   "reference_window",
                   "analysed_previous_year",
                   "number_previous_years",
                   "optimal_reference_year",
                   "optimal_time_window",
                   "optimal_time_window_length"),

      Value = c(result_daily_response$type,
                method_string,
                metric_label,
                result_daily_response[[4]],
                calculated_metric,
                round(lower_bound, 3),
                round(upper_bound, 3),
                reference_string,
                analysed_previous_year,
                number_previous_years,
                optimal_reference_year,
                Optimal_string,
                optimal_window_width),
      stringsAsFactors = FALSE
    )

    return(output_df)
  }
}

#' boot_f_lm
#'
#' Generate R bootstrap replicates of a lm function applied to data.
#'
#' @param data data frame with variables for model fitting
#' @param index indices to be used for calculation
#' @param lm.formula object of class formula to be passed into lm function
#'
#' @return A matrix with bootstrapped estimates of r squared and adjusted r squared
#'
#' @references https://www.datacamp.com/community/tutorials/bootstrap-r
#' @keywords internal
#'

boot_f_lm <- function(data, index, lm.formula) {
  bsFit <- lm(lm.formula, data=data[index,])
  c(
    summary(bsFit)$r.squared,
    summary(bsFit)$adj.r.squared
  )
}

---
title: "Examples with daily climate data"
author: "Jernej Jevšenak"
date: "`r Sys.Date()`"
output: rmarkdown::html_document
fig_caption: yes
        
vignette: >
  %\VignetteIndexEntry{Examples_with_daily_climate_data}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
body {
text-align: justify; font-size: 16px}
</style>


# 1 Introduction

The dendroTools R package was developed in 2018 and is still being updated regularly and will continue to do so in the future. It provides dendroclimatological methods for analysing statistical relationships between tree rings and daily climate data. The most commonly used method is the Pearson correlation coefficient, but users can also use non-parametric correlations such as Kendall or Spearman correlations, and in the case of a multiproxy approach, linear regression can also be applied. Artificial neural networks (brnn) are also available for nonlinear analyses.

In this document I describe the basic principles behind the dendroTools R package and give some basic examples. All data included in the examples below is already included in the dendroTools R package. Please note that the examples presented here are less made computationally less intensive to accommodate the policy of CRAN. You are welcome to explore the full potential of my package by using the wide range of possible window widths.

# 2 Transformation and quick preview of daily data

One of the crucial step before using a dendroTools is to prepare climate data into a proper format. For *daily_response()* climate data must be in a format of 366 columns and n number of rows, which represent years, which are given as row names. A common format of daily data provided by many online sources is a table with two columns, where one column represents the date and the second is the value of the climate variable. To quickly transform such a format into a data frame with dimensions of 366 x n, dendroTools now offers the function *data_transform()*. The date can be in different formats, but it must be correctly speciﬁed with the argument date_format. For example, if the date is in the format "1988-01-30′′ ("year-month-day"), the argument date_format must be "ymd".

When your data is in the proper format, the *glimpse_daily_data()* can be used for visual inspection of daily climate data. The main purpose here is to spot missing values and to evaluate the suitability of using specific daily data.


```{r, fig.align='center', fig.width=8, fig.height=8, fig.cap=paste("Figure 1: Glimpse of daily temperautres for swit272 site."), message=FALSE, warning=FALSE, include=TRUE}
# Load the dendroTools and ggplot2 R packages
library(dendroTools)
library(ggplot2)

# 1 Load an example data (source: E-OBS)
data("swit272_daily_temperatures")
data("swit272_daily_precipitation")

# 2 Transform data into wide format
swit272_daily_temperatures <- data_transform(swit272_daily_temperatures, format = 'daily', date_format = 'ymd')
swit272_daily_precipitation <- data_transform(swit272_daily_precipitation, format = 'daily', date_format = 'ymd')

# 3 Glimpse daily data
glimpse_daily_data(env_data = swit272_daily_temperatures, na.color = "red") + 
  theme(legend.position = "bottom")
```

```{r, fig.align='center', fig.width=8, fig.height=8, fig.cap=paste("Figure 2: Glimpse of daily precipitation for swit272 site."), message=FALSE, warning=FALSE, include=TRUE}

glimpse_daily_data(env_data = swit272_daily_precipitation, na.color = "red") + 
  theme(legend.position = "bottom")

```

# 3 The *daily_response()* in action

The *daily_response()* is the most commonly used function in the R package dendroTools, and works by sliding a moving window through the daily environmental (climate) data and computing statistical metrics using a tree-ring proxy. In dendroclimatology, such an analysis typically involves a site chronology that has been previously detrended, but a multiproxy approach involving multiple individual chronologies can also be used (see examples below). Possible metrics for the single-proxy approach include correlation coefficients, coefficient of determination (r-squared), and adjusted coefficient of determination (adjusted r-squared). In addition to linear regression, it is possible to use a nonlinear artificial neural network with a Bayesian regularisation training algorithm (brnn).
In general, the user can use a fixed window or a progressive window to calculate the moving averages. To use a fixed window, choose its width by assigning an integer to the *fixed_width* argument. To use a so-called variable window, which includes many different windows, define the arguments *lower_limit* and *upper_limit*. In this case, all window widths between the lower and upper limits are taken into account. The window width is defined here as the number of days between the start and end days of a calculation. Thus, the window width represents a season of interest used in the calculations. 
All calculated statistical metrics (correlation coefficients, r-squared or adjusted r-squared) are stored in a matrix that is later used to interpret the results. Such interpretation usually involves identifying the optimal season in relation to tree-ring chronology or analysing temporal patterns of correlations between climate and growth. The so-called optimal season (also called optimal window or time window with the highest correlation value) is later extracted and used to evaluate the temporal stability correlations.

## 3.1 An example of analysing the relationship between Mean Vessel Area (MVA) tree-ring parameter and daily temperature data with fixed window approach

In this example, I analyse the relationship between the tree ring parameter Mean Early Wood Vessel Area (MVA) and daily temperature data (meteorological station Ljubljana, Slovenia) for a chosen window width of 60 days (argument *fixed_width = 60*). Note that the *fixed_width* argument overrides the *upper_limit* and *lower_limit* arguments when used.

Here I also demonstrate the usability of the *row_names_subset* argument, which I highly recommend using. In most real cases, tree-ring chronologies and climate data do not completely overlap - the chronologies are usually longer than the available climate data. So if you use the argument *row_names_subset = TRUE *, your tree ring chronology (response) and your climate data (env_data) will be automatically subdivided, keeping only the overlapping years. Therefore, it is very important that the years are correctly specified in the row names of both data inputs.

Another option I use here is to remove non-significant correlations. All non-significant correlations are removed by setting the argument *remove_insignificant = TRUE * while controlling the threshold for significance with the argument *alpha*.

The results are interpreted with generic *summary()* and visualized with the generic *plot()* functions. There are two types of plots available, 1) highlighted results for a window with the highest calculated value (type = 1), and 2) heatmap of all calculated values (type = 2).

```{r, results = 'hide'}
# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_MVA)
data(LJ_daily_temperatures)

# Example with fixed width
example_fixed_width <- daily_response(response = data_MVA, env_data = LJ_daily_temperatures,
                                   method = "cor", fixed_width = 60,
                                   row_names_subset = TRUE, remove_insignificant = TRUE,
                                   alpha = 0.05)

```
```{r, fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 3: The MVA parameter contains the optimal temperature signal from March 14 (DOY 73) to May 12 (DOY 132).")}

summary(example_fixed_width)

plot(example_fixed_width, type = 1)

```

## 3.2 Analysis of climate-growth correlations only for a subset of selected time interval

If users wish to restrict the time interval of interest for calculating climate-growth correlations, the *day_interval* argument can be used for daily data and *month_interval* for monthly data in *monthly_response()*. For example, if you use *daily_response()* and want to calculate correlations only from the beginning of previous October to the current end of September, use the argument *daily_interval =c(-274, 273)*. Note that a negative sign indicates previous_year doy, while a positive sign indicates the current year doy. In normal (non-leap ) years, October 1 represents doy 274, while September 30 represents doy 273. If you use *monthly_response()* and want to calculate correlations only from the beginning of the previous October to the current end of September, use the argument *monthly_interval =c(-10, 9)*. Note that a negative sign indicates the month of the previous year, while a positive sign indicates the month of the current year.

```{r, results = 'hide'}
# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_TRW_1)
data(LJ_daily_temperatures)

# Example negative correlations
data(data_TRW_1)
example_restricted_interval <- daily_response(response = data_TRW_1, env_data = LJ_daily_temperatures,
                                   method = "cor", lower_limit = 55, upper_limit = 65,
                                   previous_year = FALSE, row_names_subset = TRUE, remove_insignificant = TRUE,
                                   alpha = 0.05, day_interval = c(150, 210))
```

```{r, fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 6: Climate-growth correlations calculated only for a subset of time window from DOY 150 to DOY 210"), message=FALSE, warning=FALSE, include=TRUE}

plot(example_restricted_interval, type = 2)

```

# 4 An example with multiple tree-ring proxies

As mentioned in the introduction, *daily_response()* allows multiproxy analysis where the *response* data frame consists of more than one tree-ring proxy. In such cases, linear or non-linear (brnn) models are fitted at each step and r-squared or adjusted r-squared is computed. However, users should choose multiple proxies judiciously and with caution, as there is nothing to prevent from including colinear variables. Using multiple proxies will result in higher explained variance, but at the expense of degrees of freedom. In these cases, you should also check the stability of the relationship over time and the result of cross-validation. If the metrics on the validation data are much lower than on the calibration data, there is a problem of overfitting and you should exclude some proxies and repeat the analysis. It is highly recommended that you use the adjusted r-squared metric when using the multiproxy approach.

```{r }
# Load the dendroTools and brnn R package
library(dendroTools)

# Example of multiproxy analysis
data(example_proxies_1)
data(LJ_daily_temperatures)

# Summary of the example_proxies_1 data frame
summary(example_proxies_1)
```

There are three proxy variables: Tree-ring width (TRWi), stable oxygen isotopes (O18) and Mean Vessel Area (MVA).

```{r }
cor(example_proxies_1)
```

The correlation matrix shows only low correlations among the three proxies.

```{r, results = 'hide'}
example_multiproxy <- daily_response(response = example_proxies_1, 
                                     env_data = LJ_daily_temperatures, 
                                     method = "lm", metric = "adj.r.squared", 
                                     lower_limit = 63, upper_limit = 67,
                                     skip_window_position = 10,
                                     row_names_subset = TRUE, previous_year = FALSE, 
                                     remove_insignificant = TRUE, alpha = 0.05)
```

```{r, fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 7: The temporal pattern of adjusted r-squared for the multiproxy example. The temporal pattern shows similar result than for the example with MVA only. Therefore, the highest percentage of (adjusted) explained variance is most likely related to MVA variable."), message=FALSE, warning=FALSE, include=TRUE}
plot(example_multiproxy, type = 2)
```



# 5 An example of climate reconstruction

Reconstructions of past climatic conditions include reconstructions of past temperatures, precipitation, vegetation, water flows, sea surface temperatures, and other climatic or climate-related conditions. To reconstruct climate with the dendroTools, we first use *daily_response()* to find the optimal sequence of consecutive days that yields the highest metric, e.g. correlation coefficient. In this example, we use data_TRW and the daily temperatures of Kredarica ('KRE_daily_temperatures'). The temporal stability of the statistical relationship is afterwards analysed with the "progressive" method using 3 splits (*k = 3*). Progressive testing involves splitting the data into *k* splits, calculating the metric for the first split, and then incrementally adding 1 split at a time and calculating the selected metric. The *cross_validation_type* argument is set to "randomised" so that the years are reshuffled before the cross-validation test. 

```{r, results = 'hide'}
# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_TRW)
data(KRE_daily_temperatures)

example_reconstruction_lin <- daily_response(response = data_TRW, 
                                             env_data = KRE_daily_temperatures, 
                                             method = "lm", metric = "r.squared", 
                                             lower_limit = 42, upper_limit = 42,
                                             row_names_subset = TRUE, 
                                             temporal_stability_check = "progressive",
                                             cross_validation_type = "randomized", k = 3)
```
```{r, fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 9: The highest r squared was calculated for the period from May 15 to June 27. The aggregated (averaged) daily data for this period is saved in a data frame $optimized_return."), message=FALSE, warning=FALSE, include=TRUE}
plot(example_reconstruction_lin)
```


Before reconstructing the May 15 to June 27 mean temperature, we should check temporal stability, cross validation and transfer function.
```{r }
example_reconstruction_lin$temporal_stability
example_reconstruction_lin$cross_validation
```

```{r, fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 10: Linear transfer function"), message=FALSE, warning=FALSE, include=TRUE}
example_reconstruction_lin$transfer_function
```

A sample code for climate reconstruction with the lm method:
```{r }
linear_model <- lm(Optimized_return ~ TRW, data = example_reconstruction_lin$optimized_return)
reconstruction <- data.frame(predictions = predict(linear_model, newdata = data_TRW))
```

```{r, fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 11: The reconstructed average temperature May 15 - June 27 with linear model"), message=FALSE, warning=FALSE, include=TRUE}
plot(row.names(data_TRW), reconstruction$predictions, type = "l", xlab = "Year", ylab = "Mean temperature May 15 - Jun 27 [ºC]")
```

To reconstruct climate using the nonlinear brnn model, use the argument method = "brnn" (see the examples in published paper in Dendrochronologia journal).

# 6 The calculation of partial correlations with *daily_response_seascorr()* 

A partial correlation coefﬁcient describes the strength of the linear relationship between two variables, holding constant a number of other variables. It is often used in dendroclimatological investigations to analyse the effect of temperature on a tree-ring parameter while at the same time controlling for the precipitation effect, or vice versa. Partial correlations in dendroTools can be calculated using *daily_response_seascorr()* and *monthly_response_seascorr()*. To analyse partial correlations, three data frames are needed: 1) a tree-ring proxy, 2) primary climate data and 3) secondary climate data for control. The tree-ring proxy must be organized as a data frame with one column representing proxy values, while years are indicated as row names. Primary climate data is assigned to the *env_data_primary* argument, while secondary climate data is assigned to *env_data_control*.

```{r, results = 'hide'}

# Example with precipitation and temperatures
partial_cor <- daily_response_seascorr(response = swit272,
                    env_data_primary = swit272_daily_temperatures,
                    env_data_control = swit272_daily_precipitation,
                    row_names_subset = TRUE, fixed_width = 45,
                    remove_insignificant = TRUE, alpha = 0.05, 
                    aggregate_function_env_data_primary = 'mean',
                    aggregate_function_env_data_control = 'sum',
                    pcor_method = "spearman",
                    skip_window_position = 10,
                    boot = FALSE, reference_window = "end")

```

```{r, fig.align='center', warning=FALSE, fig.width=10, fig.height=8, fig.cap=paste("Figure 12: Temporal partern of partial climate-growth correlations calculated with daily_response_seascorr()"), message=FALSE, warning=FALSE, include=TRUE}
summary(partial_cor)
plot(partial_cor, type = 2)

```

---
title: "Compare different regression methods"
author: "Jernej Jevsenak"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Compare_different_regression_methods}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
body {
text-align: justify; font-size: 16px}
</style>

## 1. The *compare_methods()* 

This is a short vignette about the *compare_methods()* function from the dendroTools R package. The *compare_methods()* uses k-fold cross-validation to compare different regression methods. In addition, methods could be compared by 1) using a holdout data, where data is excluded from the hyperparameter optimization and cross-validation and methods are later additionally evaluated for those points, or 2) similarly by using edge data, where extreme (edge) points are estimated as a validation data. Currently, there are five regression methods implemented: artificial neural networks with the Bayesian regularization training algorithm (BRNN), (ensemble of) model trees (MT), random forests of regression trees (RF) and (multiple) linear regression (MLR). The calculated performance metrics are the correlation coefficient (r), the root mean squared error (RMSE), the root relative squared error (RRSE), the index of agreement (d), the reduction of error (RE), the coefficient of efficiency (CE), the detrended efficiency (DE) and mean bias, calculated as the difference between observed and estimated mean response for the validation and calibration data. The output of the *compare_methods()* function is a list with 18 elements, which could be retrieved by calling the “$” operator and the element name. Please note, the examples presented here are made computationally less intensive to satisfy the CRAN policy. 
```{r, echo = FALSE, warning = FALSE}
library(knitr)
dt <- data.frame(Element = c("$mean_std", "$std_ranks", "$edge_results", "$holdout_results", "$bias_cal", "$bias_val", "$transfer_functions", "$transfer_functions_together", "$parameter_values", "$PCA_output", "$reconstructions", "$reconstructions_together", "$normal_QQ_cal", "$normal_QQ_holdout", "$normal_QQ_edge", "$residuals_vs_fitted_cal", "$residuals_vs_fitted_holdout", "$residuals_vs_fitted_edge"), 
                 Element_description = c("data frame with calculated metrics for the selected regression methods. For each regression method and each calculated metric, mean and standard deviation are given", "data frame with ranks of calculated metrics: mean rank and  share of rank_1 are given", "data frame with calculated performance metrics for the central-edge test. The central part of the data represents the calibration data, while the edge data, i.e. extreme values, represent the validation data. Different regression models are calibrated using the central data and validated for the edge (extreme) data. This test is particularly important to assess the performance of models for the prediction of the extreme data. The share of the edge (extreme) data is defined with the edge_share argument", "calculated metrics for the holdout data", "ggplot object of mean bias for calibration data", "ggplot object of mean bias for validation data", "ggplot or plotly object with transfer functions of different methods, facet is used to separate methods", "ggplot or plotly object with transfer functions of methods plotted together", "a data frame with specifications of parameters used for different regression methods", "princomp object: the result output of the PCA analysis", "ggplot object: reconstructed dependent variable based on the dataset_complete argument, facet is used to split plots by methods", "ggplot object: reconstructed dependent variable based on the dataset_complete argument, all reconstructions are on the same plot", "normal q-q plot for calibration data", "normal q-q plot for holdout data", "normal q-q plot for edge data", "residuals vs fitted values plot for calibration data", "residuals vs fitted values plot for holdout data", "residuals vs fitted values plot for edge data"))
kable(dt, "html")

```
## 2. Basic example

For the basic example, we will use the dataset with the Mean Vessel Area (MVA) chronology and the mean April temperature, the dataset is saved as dataset_MVA. All five regression methods will be compared with 10-fold cross-validation repeated 1 times. The *optimize* argument is set to TRUE, therefore all tuning parameters will be defined in a preliminary optimization phase. After the comparison, the output elements are retrieved with the "$" operator. 

```{r, results = 'hide', warning=FALSE}
# Load the dendroTools R package
library(dendroTools)

# Load the data
data(dataset_MVA)

# Basic example
basic_example <- compare_methods(formula = T_Apr ~ MVA, dataset = dataset_MVA, k = 10, repeats = 1, optimize = TRUE, MT_committees_vector = c(1), RF_maxnodes_vector = c(5), RF_nodesize_vector = c(10))
```

```{r }
# The data frame with mean and standard deviation of performance metrics for the calibration and the validation data
kable(basic_example$mean_std)
```

```{r }
# The data frame with non-parametric estimation of different methods: average rank and share of rank one
kable(basic_example$rank)
```

```{r, fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("Histogram for the validation data for the basic_example")}
# See the histogram of mean bias for the validation data
basic_example$bias_val
```

```{r, fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("Histogram for the calibration data for the basic_example")}
# See the histogram of mean bias for the calibration data
basic_example$bias_cal
```

```{r, fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The transfer functions of different methods, facet is used to separate plots by method.")}
# See the transfer functions, separated by facets. This is a ggplot object and could be easily customized. 
library(ggplot2)
basic_example$transfer_functions +   
  xlab(expression(paste('MVA [',mm^2,']'))) +
  ylab("April Mean Temperature [°C]")
```

```{r, fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The transfer functions of different methods, all functions are on the same plot, therefore it is easy to see the differences among different methods.")}
# See the transfer functions, plotted together. This is a ggplot object and could be easily customized. 
basic_example$transfer_functions_together +   
  xlab(expression(paste('MVA [',mm^2,']'))) +
  ylab("April Mean Temperature [°C]")
```

```{r }
# The data frame of optimized tuning parameters for different methods
kable(basic_example$parameter_values)
```

```{r rb, fig.align='center', warning=FALSE, fig.width=8, fig.height=10, fig.cap=paste("Residual diagnostic plots for calibration data: Normal Q-Q plot")}
# For calibration data, there are residual diagnostic plots available. Similar plots are available also for holdout and edge data. 

basic_example$normal_QQ_cal
```

```{r r, fig.align='center', warning=FALSE, fig.width=8, fig.height=10, fig.cap=paste("Residual diagnostic plots for calibration data: residuals vs fitted plot")}
# For calibration data, there are residual diagnostic plots available. Similar plots are available also for holdout and edge data. 

basic_example$residuals_vs_fitted_cal
```


## 3. Principal component analysis in combination with the *compare_methods()*

Principal Component Analysis (PCA) is commonly used with tree-ring data to reduce the full set of original tree-ring chronologies to a more manageable set of transformed variables. These transformed variables, the set of principal component scores, are then used as predictors in climate reconstruction models. The PCA also acts to strengthen the common regional-scale climate response within a group of tree-ring chronologies by concentrating the common signal in the components with the largest eigenvalues. 

To use PCA regression within the *compare_methods()*, set the argument *PCA_transformation* to TRUE. All independent variables in the *dataset* data frame will be transformed using the PCA transformation. If the parameter *log_preprocess* is set to TRUE, variables will be transformed with logarithmic transformation before used in PCA. With the argument *components_selection*, we specify how to select PC scores that will be used as predictors. There are three options: "automatic", "manual" and "plot_selection". If argument is set to "automatic", all PC scores with eigenvalues above 1 will be selected. This threshold could be changed by changing the *eigenvalues_threshold* argument. If argument is set to "manual", user should set the number of components with *N_components* argument. If *components_selection* is set to "plot_selection", A scree plot will be shown, and a user must manually enter the number of components to be used as predictors. The latter seems to be the most reasonable choice. 

For the example with PCA, we use dataset dataset_MVA_individual, which consist of 10 individual Mean Vessel Area (MVA) chronologies and mean April temperature for the Ljubljana region, Slovenia. The dataset has 56 observations. The selection of components is set to "manual", *N_components* to be used in the later analysis is set to 2. In this example, the 5-fold cross-validation with 1 repeat will be used to compare MT, MLR and BRNN. The subset of methods could be set with the *methods* argument. The argument *optimize* is set to TRUE, therefore all tuning parameters will be set automatically. 

```{r, results = 'hide', warning=FALSE}
# Load the dendroTools R package
library(dendroTools)

# Load data
data(dataset_MVA_individual)

# Example PCA
example_PCA <- compare_methods(formula = T_Apr ~ ., dataset = dataset_MVA_individual, k = 5, repeats = 1, optimize = TRUE, methods = c("MLR", "MT", "BRNN"), PCA_transformation = TRUE, components_selection = "manual", N_components = 2, seed_factor = 5, MT_committees_vector = c(1), RF_maxnodes_vector = c(5), RF_nodesize_vector = c(10))
```
```{r }
# Get the summary statistics for the PCA
summary(example_PCA$PCA_output)
```

```{r }
# The mean and standard deviation data frame 
kable(example_PCA$mean_std)
```


## 4. Example of multiproxy analysis

The *compare_methods()* enables the comparison of methods for regression problems with two or more independent variables. However, users should select multiple proxies reasonably and with caution, since there is nothing to prevent from including colinear variables. To perform the comparison of methods with multiproxy variables, simply include dataset with more than one independent variable and specify the relationship with the *formula* argument. If metrics on validation data are much lower than on calibration data, there is a problem of overfitting and users should exclude some independent variables and repeat the analysis.

For the *multiproxy_example*, we will use *example_dataset_1*, which consist of the Mean Vessel Area (MVA) chronology and two temperature variables, the mean April temperature (T_APR) and the mean temperature from August to September (T_aug_sep) from the previous growing season. To compare methods with multiproxy approach, specify formula with two independent variables, such as *formula = MVA ~ T_APR + T_aug_sep*. Here, we will compare MT, BRNN and RF with 10-fold cross-validation and 1 repeat. 

```{r, results = 'hide'}
# Load the dendroTools R package
library(dendroTools)

# Load data
data(example_dataset_1)

# Example multiproxy
example_multiproxy <- compare_methods(formula = MVA ~ T_APR + T_aug_sep, dataset = example_dataset_1, k = 10, repeats = 1, optimize = FALSE, methods = c("MT", "BRNN", "RF"))
```

```{r }
# The mean and standard deviation data frame 
kable(example_multiproxy$mean_std)
```

## 5. Example of climate reconstruction

Reconstructions of past climate conditions include reconstructions of past temperature, precipitation, vegetation, stream flow, sea surface temperature, and other climatic or climate-dependent conditions. With the *compare_methods()* it is possible to directly reconstruct the dependent variable specified with the *formula* argument. To do so, supply additional complete dataset with tree-ring chronology that goes beyond the observations of instrumental records. 

For the *example_reconstruction*, we use *data_TRW* dataset, which includes a tree-ring width (TRW) chronology of *Pinus nigra* from Albania and mean June-July temperature from Albania. The complete TRW chronology is supplied with the *dataset_TRW_complete*. In this example, we will compare RF and MLR models with 3-fold cross-validation and 1 repeat.

```{r, results = 'hide'}
# Load the dendroTools R package
library(dendroTools)

# Load the data
data(dataset_TRW)
data(dataset_TRW_complete)

# Example reconstruction
example_reconstruction <- compare_methods(formula = T_Jun_Jul ~ TRW, dataset = dataset_TRW, k = 3, optimize = FALSE, methods = c("MLR", "BRNN", "MT", "RF"), dataset_complete = dataset_TRW_complete)
```

```{r, fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The reconstructed June-July temperatures based on the dataset_complete argument, facet is used to split plots by methods.")}
example_reconstruction$reconstructions
```

```{r, fig.align='center', warning=FALSE, fig.width=7, fig.height=5, fig.cap=paste("The reconstructed June-July temperatures based on the dataset_complete argument, all reconstructions are on the same plot. The RF model reconstructed temperatures with much lower variance than the MLR model.")}
example_reconstruction$reconstructions_together
```


The comparison among different methods usually shows, that different regression techniques perform relatively similar for the central-calibration data, while for the extreme values, predictions differ. Therefore, we implemented additional central-edge test, where the central part of the dataset represents the calibration data, while the edge data, i.e. extreme values, represent the edge data. Different regression models are calibrated using the central data and validated using the edge (extreme) data. This test is particularly important to assess the performance of different methods for the prediction of the extreme and out of calibration data. The share of the edge (extreme) data is defined with the *edge_share* argument. To get the results for the central-edge test, use example_reconstruction$edge_results. 

```{r }
# The central-edge test
kable(example_reconstruction$edge_results)
```

In this example, MLR and MT had worse validation results than RF and BRNN. This indicates that linear interpolation for extreme values is not the best choice. 

## 6. Tuning the machine learning parameters

Machine learning methods have several tuning parameters that need to be set, e.g. the number of neurons for the BRNN (parameter *BRNN_neurons = 2*). By default the *optimize* argument is set to *TRUE*, therefore all parameters will be automatically optimized in a preliminary cross-validation phase, where different combinations of parameters are tested and the best combination for each method is later used for the final model comparison. Each parameter has a pre-defined vector of possible values, however, this vector of possible values could be extended and therefore a wider space of tuned values could be explored. To change the vector of possible values for the *BRNN_neurons* parameter, use e.g.  *BRNN_neurons_vector = c(1, 2, 3, 4, 5)*. Bellow, see the table of all tuning parameters together with the vectors of possible values.

```{r, echo = FALSE, warning = FALSE}
library(knitr)
dt <- data.frame(Method = c("BRNN", "MT", "MT", "MT", "MT","MT" , "MT", "RF", "RF", "RF", "RF"),
                 Parameter = c("BRNN_neurons", "MT_committees", "MT_neighbors", "MT_rules", "MT_unbiased",
                "MT_extrapolation", "MT_sample", "RF_mtry", "RF_maxnodes", "RF_ntree", "RF_nodesize"), Vector_for_optimization = c("BRNN_neurons_vector", "MT_committees_vector", "MT_neighbors_vector", "MT_rules_vector", "MT_unbiased_vector",
                "MT_extrapolation_vector", "MT_sample_vector", "RF_mtry_vector", "RF_maxnodes_vector", "RF_ntree_vector", "RF_nodesize_vector"))
kable(dt, "html")

```
To set the tuning parameters manually, set the parameter *optimize* to *FALSE* and supply the selected value of each tuning parameter with the corresponding argument (if not, the default value will be used). Here is a simple example, where tuning parameters are set manually.

```{r, results = 'hide'}
# Load the dendroTools R package
library(dendroTools)

# Load the data
data(example_dataset_1)

example_optimize <- compare_methods(formula = MVA ~  T_APR, dataset = example_dataset_1, k = 5, repeats = 2, optimize = FALSE, BRNN_neurons = 1, MT_committees = 1, MT_neighbors = 0, MT_rules = 100, MT_unbiased = FALSE, MT_extrapolation = 100, MT_sample = 0, RF_mtry = 1, RF_ntree = 100, RF_maxnodes = 20, seed_factor = 5)
```
```{r }
# The data frame of tuning parameters, as defined by the user
kable(example_optimize$parameter_values)
```

---
title: "Modify plots using ggplot"
author: "Jernej Jevšenak"
date: "`r Sys.Date()`"
output: rmarkdown::html_document
fig_caption: yes

vignette: >
  %\VignetteIndexEntry{Modify_plots_with_ggplot}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

<style>
body {
text-align: justify; font-size: 16px}
</style>


# 1 Introduction

Many functions in the **dendroTools** R package provide built-in plotting methods for quick inspection and interpretation of results. These plots are created with **ggplot2**, therefore the returned plot is a **ggplot** object. This is very convenient because users can directly modify the default dendroTools plot by adding any ggplot2 layers (themes, scales, labels, annotations, etc.) with the `+` operator.

In this vignette I demonstrate a basic workflow:
1) calculate a `daily_response()` example,  
2) create a default plot with `plot()`,  
3) polish the plot using ggplot2,  
4) build a similar heatmap from scratch by extracting calculated values from the returned object.


# 2 A basic daily_response example

All data used below is included in the dendroTools package.

```{r, results='hide', message=FALSE, warning=FALSE}
# Load packages
library(dendroTools)
library(ggplot2)

# Load example data
data(data_MVA)
data(LJ_daily_temperatures)

# Run daily_response()
example_basic <- daily_response(response = data_MVA,
                                env_data = LJ_daily_temperatures, 
                                row_names_subset = TRUE,
                                lower_limit = 35, upper_limit = 45, 
                                remove_insignificant = FALSE,
                                previous_year = FALSE, 
                                reference_window = "end")
```


# 3 Default plot and ggplot-based modifications

The simplest way to visualize the results is to use the generic `plot()` method.

```{r, fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 1: Default dendroTools plot for daily_response() output."), message=FALSE, warning=FALSE, include=TRUE}
plot(example_basic)
```

## 3.1 Polish the default plot with ggplot2

Because `plot(example_basic)` returns a **ggplot** object, it can be modified directly. In this example I:
- set a diverging colour scale and fix the limits to `-1` and `1`,  
- apply a minimal theme,  
- move the legend to the bottom.

```{r, fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 2: The same plot modified with ggplot2 layers (scale + theme)."), message=FALSE, warning=FALSE, include=TRUE}
plot(example_basic) +
  scale_fill_gradient2(
    name = "cor",
    low = "blue",
    mid = "white",
    high = "red",
    na.value = "white",
    limits = c(-1, 1) # select min-max here
  ) +
  theme_minimal() +
  theme(panel.background = element_blank(),
        plot.background = element_blank(),
        plot.title = element_blank(),
        legend.position = "bottom"
        )
```

## 3.2 Additional common modifications

Here is another example with renamed axis labels and rotated x-axis labels.

```{r, fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 3: Example with modified labels and rotated x-axis text."), message=FALSE, warning=FALSE, include=TRUE}
plot(example_basic) +
  scale_fill_gradient2(
    name = "Correlation",
    low = "blue",
    mid = "white",
    high = "red",
    na.value = "white",
    limits = c(-1, 1)
  ) +
  labs(x = "Season end (DOY)",
       y = "Season length (days)") +
  theme_bw() +
  theme(legend.position = "bottom",
        axis.text.x = element_text(angle = 45, hjust = 1))
```


# 4 Create a similar plot from scratch

Sometimes you may want complete control over the plot (e.g., different geometries, custom annotations, combining multiple plots, etc.). In such cases you can extract the computed values from the returned dendroTools object and create your own plot.

For `daily_response()` outputs, the calculated values are stored in `object$calculations`. The code below converts this matrix-like object to a long format suitable for `geom_tile()`.

```{r, results='hide', message=FALSE, warning=FALSE}
# Extract calculations (correlation table) from the dmrs object
cor_mat <- example_basic$calculations

# Convert matrix-like object to long format using base R
melted <- as.data.frame(as.table(as.matrix(cor_mat)))
colnames(melted) <- c("season_length", "season_end", "value")

# Convert labels such as "X35" into numeric values (if present)
melted$season_end  <- as.numeric(gsub("X", "", melted$season_end))
melted$season_length <- as.numeric(gsub("X", "", melted$season_length))

# Remove NA values (if any)
melted <- melted[!is.na(melted$value), ]

summary(melted)
```

```{r, fig.align='center', fig.width=10, fig.height=8, fig.cap=paste("Figure 4: Heatmap created from scratch using ggplot2 and extracted calculations."), message=FALSE, warning=FALSE, include=TRUE}
ggplot(melted, aes(x = season_end, y = season_length, fill = value)) + 
  geom_tile() +
  scale_y_continuous(expand = c(0, 0)) +
  scale_x_continuous(expand = c(0, 0)) +
  scale_fill_gradient2(
    name = "cor",
    low = "blue",
    mid = "white",
    high = "red",
    na.value = "white",
    limits = c(-1, 1)
  ) +
  xlab("Season end") +
  ylab("Season Length") +
  theme_bw() +
  theme(legend.position = "bottom")
```


# 5 Summary

- dendroTools plotting methods return **ggplot2** objects, so plots can be modified directly using `+`.  
- A typical workflow is to produce a default plot with `plot(object)` and then polish it with ggplot2 scales and themes.  
- For full flexibility, the computed values can be extracted from `object$calculations` and plotted from scratch using `ggplot()` and `geom_tile()`.
- The same principles apply to other dendroTools functions, namely `monthly_response()`, `monthly_response_seascor()`,  `daily_response_seascor()`.   

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_proxies_1.R
\docType{data}
\name{example_proxies_1}
\alias{example_proxies_1}
\title{Tree-ring example proxies 1 from 2015 - 1961}
\format{
A data frame with 55 rows and 3 variables:
\describe{
  \item{MVA}{Mean vessel area [mm^2] indices from 2015 - 1961}
  \item{O18}{Scaled Stable oxygen isotope ratios from 2015 - 1961}
  \item{TRW}{Tree-ring widths from 2015 - 1961}
}
}
\source{
Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
Ljubljana, Slovenia
}
\usage{
example_proxies_1
}
\description{
A dataset with three tree-ring proxy records from a site near Ljubljana
(Slovenia). The first row represents a value of a year in 1961. The three
proxy records are MVA (Mean vessel area [mm ^2]), O (stable oxygen isotope
ratios) and TRW (Tree-ring widths)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot_f_cor.R
\name{boot_f_cor}
\alias{boot_f_cor}
\title{boot_f_cor}
\usage{
boot_f_cor(data, indices, cor.type)
}
\arguments{
\item{data}{A vector, matrix, or data frame containing the input data.
If a matrix or data frame is provided, each row is treated as a separate
multivariate observation.}

\item{indices}{A numeric or integer vector specifying the indices to be
utilized in the calculation. These indices determine which elements or rows
of the 'data' are to be included in the analysis}

\item{cor.type}{A string specifying the method of correlation to be applied
in the bootstrap procedure. Available options are "pearson" (default),
"kendall", or "spearman".}
}
\value{
A matrix with bootstrapped estimates of correlation coefficients
}
\description{
Generate R bootstrap replicates of a statistic applied to data.
}
\references{
https://www.datacamp.com/community/tutorials/bootstrap-r
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/swit272.R
\docType{data}
\name{swit272}
\alias{swit272}
\title{Standardised tree-ring width chronology swit272, Larix decidua Mill.}
\format{
A data frame with 273 rows and 1 variable:
\describe{
  \item{TRWi}{Standardised tree-ring width chronology}
}
}
\source{
\url{https://www.ncei.noaa.gov/access/paleo-search/study/14108}
}
\usage{
swit272
}
\description{
A TRW chronology swit272
Investigators: Bigler, C.; Claluna, A.
Site_Name:  Sils-Maria GR Blais dal Fo
Location: Switzerland
Northernmost_Latitude: 46.4333
Southernmost_Latitude: 46.4333
Easternmost_Longitude: 9.7833
Westernmost_Longitude: 9.7833
Elevation: 2100
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_MVA.R
\docType{data}
\name{data_MVA}
\alias{data_MVA}
\title{Mean vessel area example proxy from 2012 - 1940}
\format{
A data frame with 73 rows and 1 variable:
\describe{
  \item{MVA}{Mean vessel area [mm^2] indices from 2012 - 1940}
}
}
\source{
Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
Ljubljana, Slovenia
}
\usage{
data_MVA
}
\description{
A dataset with MVA proxy records from a lowland forest Mlače in Slovenia.
The first row represents a value of a year in 2012. Row names represent
years.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smooth_matrix.R
\name{smooth_matrix}
\alias{smooth_matrix}
\title{smooth_matrix}
\usage{
smooth_matrix(mat, factor_drop = 0.7, repeats = 3)
}
\arguments{
\item{mat}{a matrix or data.frame}

\item{factor_drop}{a number that specifies by how many % should a value drop,
comparing to two the closest values in a row (i +1 and i -1), to be
considered as a unrealistic value.}

\item{repeats}{an integer that specifies number of repeats of smoothing.
Important when there are more unrealistic values one by another.}
}
\value{
a matrix with replaced unrealistic values
}
\description{
Removes unrealistic values in a matrix and replaces them with mean of
values in a window 3 x 3 around the unrealistic value. Unrealistic value is
determined by a factor_drop.
}
\examples{
\donttest{
library(dendroTools)
data(LJ_daily_temperatures)
data(example_proxies_1)
Example1 <- daily_response(response = example_proxies_1,
env_data = LJ_daily_temperatures, method = "brnn",
metric = "r.squared", lower = 250, upper = 251,
previous_year = FALSE, brnn_smooth = TRUE, alpha = 0.1,
row_names_subset = TRUE)
smoothed <- smooth_matrix(mat = Example1[[1]])

mat_1 <-  matrix(seq(1.01, 2, by = 0.01)  , ncol = 10, byrow = TRUE)
mat_1[5 ,5] <- -1
mat_2 <- smooth_matrix(mat_1)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LJ_daily_temperatures.R
\docType{data}
\name{LJ_daily_temperatures}
\alias{LJ_daily_temperatures}
\title{Daily mean temperatures for Ljubljana from 2016 - 1930}
\format{
A data frame with 87 rows and 366 variables:
\describe{
  \item{X1}{Temperatures on the day 1 of a year}
  \item{X2}{Temperatures on the day 2 of a year}
  \item{X3}{Temperatures on the day 3 of a year}
  \item{X4}{Temperatures on the day 4 of a year}
  \item{X5}{Temperatures on the day 5 of a year}
  \item{X6}{Temperatures on the day 6 of a year}
  \item{X7}{Temperatures on the day 7 of a year}
  \item{X8}{Temperatures on the day 8 of a year}
  \item{X9}{Temperatures on the day 9 of a year}
  \item{X10}{Temperatures on the day 10 of a year}
  \item{X11}{Temperatures on the day 11 of a year}
  \item{X12}{Temperatures on the day 12 of a year}
  \item{X13}{Temperatures on the day 13 of a year}
  \item{X14}{Temperatures on the day 14 of a year}
  \item{X15}{Temperatures on the day 15 of a year}
  \item{X16}{Temperatures on the day 16 of a year}
  \item{X17}{Temperatures on the day 17 of a year}
  \item{X18}{Temperatures on the day 18 of a year}
  \item{X19}{Temperatures on the day 19 of a year}
  \item{X20}{Temperatures on the day 20 of a year}
  \item{X21}{Temperatures on the day 21 of a year}
  \item{X22}{Temperatures on the day 22 of a year}
  \item{X23}{Temperatures on the day 23 of a year}
  \item{X24}{Temperatures on the day 24 of a year}
  \item{X25}{Temperatures on the day 25 of a year}
  \item{X26}{Temperatures on the day 26 of a year}
  \item{X27}{Temperatures on the day 27 of a year}
  \item{X28}{Temperatures on the day 28 of a year}
  \item{X29}{Temperatures on the day 29 of a year}
  \item{X30}{Temperatures on the day 30 of a year}
  \item{X31}{Temperatures on the day 31 of a year}
  \item{X32}{Temperatures on the day 32 of a year}
  \item{X33}{Temperatures on the day 33 of a year}
  \item{X34}{Temperatures on the day 34 of a year}
  \item{X35}{Temperatures on the day 35 of a year}
  \item{X36}{Temperatures on the day 36 of a year}
  \item{X37}{Temperatures on the day 37 of a year}
  \item{X38}{Temperatures on the day 38 of a year}
  \item{X39}{Temperatures on the day 39 of a year}
  \item{X40}{Temperatures on the day 40 of a year}
  \item{X41}{Temperatures on the day 41 of a year}
  \item{X42}{Temperatures on the day 42 of a year}
  \item{X43}{Temperatures on the day 43 of a year}
  \item{X44}{Temperatures on the day 44 of a year}
  \item{X45}{Temperatures on the day 45 of a year}
  \item{X46}{Temperatures on the day 46 of a year}
  \item{X47}{Temperatures on the day 47 of a year}
  \item{X48}{Temperatures on the day 48 of a year}
  \item{X49}{Temperatures on the day 49 of a year}
  \item{X50}{Temperatures on the day 50 of a year}
  \item{X51}{Temperatures on the day 51 of a year}
  \item{X52}{Temperatures on the day 52 of a year}
  \item{X53}{Temperatures on the day 53 of a year}
  \item{X54}{Temperatures on the day 54 of a year}
  \item{X55}{Temperatures on the day 55 of a year}
  \item{X56}{Temperatures on the day 56 of a year}
  \item{X57}{Temperatures on the day 57 of a year}
  \item{X58}{Temperatures on the day 58 of a year}
  \item{X59}{Temperatures on the day 59 of a year}
  \item{X60}{Temperatures on the day 60 of a year}
  \item{X61}{Temperatures on the day 61 of a year}
  \item{X62}{Temperatures on the day 62 of a year}
  \item{X63}{Temperatures on the day 63 of a year}
  \item{X64}{Temperatures on the day 64 of a year}
  \item{X65}{Temperatures on the day 65 of a year}
  \item{X66}{Temperatures on the day 66 of a year}
  \item{X67}{Temperatures on the day 67 of a year}
  \item{X68}{Temperatures on the day 68 of a year}
  \item{X69}{Temperatures on the day 69 of a year}
  \item{X70}{Temperatures on the day 70 of a year}
  \item{X71}{Temperatures on the day 71 of a year}
  \item{X72}{Temperatures on the day 72 of a year}
  \item{X73}{Temperatures on the day 73 of a year}
  \item{X74}{Temperatures on the day 74 of a year}
  \item{X75}{Temperatures on the day 75 of a year}
  \item{X76}{Temperatures on the day 76 of a year}
  \item{X77}{Temperatures on the day 77 of a year}
  \item{X78}{Temperatures on the day 78 of a year}
  \item{X79}{Temperatures on the day 79 of a year}
  \item{X80}{Temperatures on the day 80 of a year}
  \item{X81}{Temperatures on the day 81 of a year}
  \item{X82}{Temperatures on the day 82 of a year}
  \item{X83}{Temperatures on the day 83 of a year}
  \item{X84}{Temperatures on the day 84 of a year}
  \item{X85}{Temperatures on the day 85 of a year}
  \item{X86}{Temperatures on the day 86 of a year}
  \item{X87}{Temperatures on the day 87 of a year}
  \item{X88}{Temperatures on the day 88 of a year}
  \item{X89}{Temperatures on the day 89 of a year}
  \item{X90}{Temperatures on the day 90 of a year}
  \item{X91}{Temperatures on the day 91 of a year}
  \item{X92}{Temperatures on the day 92 of a year}
  \item{X93}{Temperatures on the day 93 of a year}
  \item{X94}{Temperatures on the day 94 of a year}
  \item{X95}{Temperatures on the day 95 of a year}
  \item{X96}{Temperatures on the day 96 of a year}
  \item{X97}{Temperatures on the day 97 of a year}
  \item{X98}{Temperatures on the day 98 of a year}
  \item{X99}{Temperatures on the day 99 of a year}
  \item{X100}{Temperatures on the day 100 of a year}
  \item{X101}{Temperatures on the day 101 of a year}
  \item{X102}{Temperatures on the day 102 of a year}
  \item{X103}{Temperatures on the day 103 of a year}
  \item{X104}{Temperatures on the day 104 of a year}
  \item{X105}{Temperatures on the day 105 of a year}
  \item{X106}{Temperatures on the day 106 of a year}
  \item{X107}{Temperatures on the day 107 of a year}
  \item{X108}{Temperatures on the day 108 of a year}
  \item{X109}{Temperatures on the day 109 of a year}
  \item{X110}{Temperatures on the day 110 of a year}
  \item{X111}{Temperatures on the day 111 of a year}
  \item{X112}{Temperatures on the day 112 of a year}
  \item{X113}{Temperatures on the day 113 of a year}
  \item{X114}{Temperatures on the day 114 of a year}
  \item{X115}{Temperatures on the day 115 of a year}
  \item{X116}{Temperatures on the day 116 of a year}
  \item{X117}{Temperatures on the day 117 of a year}
  \item{X118}{Temperatures on the day 118 of a year}
  \item{X119}{Temperatures on the day 119 of a year}
  \item{X120}{Temperatures on the day 120 of a year}
  \item{X121}{Temperatures on the day 121 of a year}
  \item{X122}{Temperatures on the day 122 of a year}
  \item{X123}{Temperatures on the day 123 of a year}
  \item{X124}{Temperatures on the day 124 of a year}
  \item{X125}{Temperatures on the day 125 of a year}
  \item{X126}{Temperatures on the day 126 of a year}
  \item{X127}{Temperatures on the day 127 of a year}
  \item{X128}{Temperatures on the day 128 of a year}
  \item{X129}{Temperatures on the day 129 of a year}
  \item{X130}{Temperatures on the day 130 of a year}
  \item{X131}{Temperatures on the day 131 of a year}
  \item{X132}{Temperatures on the day 132 of a year}
  \item{X133}{Temperatures on the day 133 of a year}
  \item{X134}{Temperatures on the day 134 of a year}
  \item{X135}{Temperatures on the day 135 of a year}
  \item{X136}{Temperatures on the day 136 of a year}
  \item{X137}{Temperatures on the day 137 of a year}
  \item{X138}{Temperatures on the day 138 of a year}
  \item{X139}{Temperatures on the day 139 of a year}
  \item{X140}{Temperatures on the day 140 of a year}
  \item{X141}{Temperatures on the day 141 of a year}
  \item{X142}{Temperatures on the day 142 of a year}
  \item{X143}{Temperatures on the day 143 of a year}
  \item{X144}{Temperatures on the day 144 of a year}
  \item{X145}{Temperatures on the day 145 of a year}
  \item{X146}{Temperatures on the day 146 of a year}
  \item{X147}{Temperatures on the day 147 of a year}
  \item{X148}{Temperatures on the day 148 of a year}
  \item{X149}{Temperatures on the day 149 of a year}
  \item{X150}{Temperatures on the day 150 of a year}
  \item{X151}{Temperatures on the day 151 of a year}
  \item{X152}{Temperatures on the day 152 of a year}
  \item{X153}{Temperatures on the day 153 of a year}
  \item{X154}{Temperatures on the day 154 of a year}
  \item{X155}{Temperatures on the day 155 of a year}
  \item{X156}{Temperatures on the day 156 of a year}
  \item{X157}{Temperatures on the day 157 of a year}
  \item{X158}{Temperatures on the day 158 of a year}
  \item{X159}{Temperatures on the day 159 of a year}
  \item{X160}{Temperatures on the day 160 of a year}
  \item{X161}{Temperatures on the day 161 of a year}
  \item{X162}{Temperatures on the day 162 of a year}
  \item{X163}{Temperatures on the day 163 of a year}
  \item{X164}{Temperatures on the day 164 of a year}
  \item{X165}{Temperatures on the day 165 of a year}
  \item{X166}{Temperatures on the day 166 of a year}
  \item{X167}{Temperatures on the day 167 of a year}
  \item{X168}{Temperatures on the day 168 of a year}
  \item{X169}{Temperatures on the day 169 of a year}
  \item{X170}{Temperatures on the day 170 of a year}
  \item{X171}{Temperatures on the day 171 of a year}
  \item{X172}{Temperatures on the day 172 of a year}
  \item{X173}{Temperatures on the day 173 of a year}
  \item{X174}{Temperatures on the day 174 of a year}
  \item{X175}{Temperatures on the day 175 of a year}
  \item{X176}{Temperatures on the day 176 of a year}
  \item{X177}{Temperatures on the day 177 of a year}
  \item{X178}{Temperatures on the day 178 of a year}
  \item{X179}{Temperatures on the day 179 of a year}
  \item{X180}{Temperatures on the day 180 of a year}
  \item{X181}{Temperatures on the day 181 of a year}
  \item{X182}{Temperatures on the day 182 of a year}
  \item{X183}{Temperatures on the day 183 of a year}
  \item{X184}{Temperatures on the day 184 of a year}
  \item{X185}{Temperatures on the day 185 of a year}
  \item{X186}{Temperatures on the day 186 of a year}
  \item{X187}{Temperatures on the day 187 of a year}
  \item{X188}{Temperatures on the day 188 of a year}
  \item{X189}{Temperatures on the day 189 of a year}
  \item{X190}{Temperatures on the day 190 of a year}
  \item{X191}{Temperatures on the day 191 of a year}
  \item{X192}{Temperatures on the day 192 of a year}
  \item{X193}{Temperatures on the day 193 of a year}
  \item{X194}{Temperatures on the day 194 of a year}
  \item{X195}{Temperatures on the day 195 of a year}
  \item{X196}{Temperatures on the day 196 of a year}
  \item{X197}{Temperatures on the day 197 of a year}
  \item{X198}{Temperatures on the day 198 of a year}
  \item{X199}{Temperatures on the day 199 of a year}
  \item{X200}{Temperatures on the day 200 of a year}
  \item{X201}{Temperatures on the day 201 of a year}
  \item{X202}{Temperatures on the day 202 of a year}
  \item{X203}{Temperatures on the day 203 of a year}
  \item{X204}{Temperatures on the day 204 of a year}
  \item{X205}{Temperatures on the day 205 of a year}
  \item{X206}{Temperatures on the day 206 of a year}
  \item{X207}{Temperatures on the day 207 of a year}
  \item{X208}{Temperatures on the day 208 of a year}
  \item{X209}{Temperatures on the day 209 of a year}
  \item{X210}{Temperatures on the day 210 of a year}
  \item{X211}{Temperatures on the day 211 of a year}
  \item{X212}{Temperatures on the day 212 of a year}
  \item{X213}{Temperatures on the day 213 of a year}
  \item{X214}{Temperatures on the day 214 of a year}
  \item{X215}{Temperatures on the day 215 of a year}
  \item{X216}{Temperatures on the day 216 of a year}
  \item{X217}{Temperatures on the day 217 of a year}
  \item{X218}{Temperatures on the day 218 of a year}
  \item{X219}{Temperatures on the day 219 of a year}
  \item{X220}{Temperatures on the day 220 of a year}
  \item{X221}{Temperatures on the day 221 of a year}
  \item{X222}{Temperatures on the day 222 of a year}
  \item{X223}{Temperatures on the day 223 of a year}
  \item{X224}{Temperatures on the day 224 of a year}
  \item{X225}{Temperatures on the day 225 of a year}
  \item{X226}{Temperatures on the day 226 of a year}
  \item{X227}{Temperatures on the day 227 of a year}
  \item{X228}{Temperatures on the day 228 of a year}
  \item{X229}{Temperatures on the day 229 of a year}
  \item{X230}{Temperatures on the day 230 of a year}
  \item{X231}{Temperatures on the day 231 of a year}
  \item{X232}{Temperatures on the day 232 of a year}
  \item{X233}{Temperatures on the day 233 of a year}
  \item{X234}{Temperatures on the day 234 of a year}
  \item{X235}{Temperatures on the day 235 of a year}
  \item{X236}{Temperatures on the day 236 of a year}
  \item{X237}{Temperatures on the day 237 of a year}
  \item{X238}{Temperatures on the day 238 of a year}
  \item{X239}{Temperatures on the day 239 of a year}
  \item{X240}{Temperatures on the day 240 of a year}
  \item{X241}{Temperatures on the day 241 of a year}
  \item{X242}{Temperatures on the day 242 of a year}
  \item{X243}{Temperatures on the day 243 of a year}
  \item{X244}{Temperatures on the day 244 of a year}
  \item{X245}{Temperatures on the day 245 of a year}
  \item{X246}{Temperatures on the day 246 of a year}
  \item{X247}{Temperatures on the day 247 of a year}
  \item{X248}{Temperatures on the day 248 of a year}
  \item{X249}{Temperatures on the day 249 of a year}
  \item{X250}{Temperatures on the day 250 of a year}
  \item{X251}{Temperatures on the day 251 of a year}
  \item{X252}{Temperatures on the day 252 of a year}
  \item{X253}{Temperatures on the day 253 of a year}
  \item{X254}{Temperatures on the day 254 of a year}
  \item{X255}{Temperatures on the day 255 of a year}
  \item{X256}{Temperatures on the day 256 of a year}
  \item{X257}{Temperatures on the day 257 of a year}
  \item{X258}{Temperatures on the day 258 of a year}
  \item{X259}{Temperatures on the day 259 of a year}
  \item{X260}{Temperatures on the day 260 of a year}
  \item{X261}{Temperatures on the day 261 of a year}
  \item{X262}{Temperatures on the day 262 of a year}
  \item{X263}{Temperatures on the day 263 of a year}
  \item{X264}{Temperatures on the day 264 of a year}
  \item{X265}{Temperatures on the day 265 of a year}
  \item{X266}{Temperatures on the day 266 of a year}
  \item{X267}{Temperatures on the day 267 of a year}
  \item{X268}{Temperatures on the day 268 of a year}
  \item{X269}{Temperatures on the day 269 of a year}
  \item{X270}{Temperatures on the day 270 of a year}
  \item{X271}{Temperatures on the day 271 of a year}
  \item{X272}{Temperatures on the day 272 of a year}
  \item{X273}{Temperatures on the day 273 of a year}
  \item{X274}{Temperatures on the day 274 of a year}
  \item{X275}{Temperatures on the day 275 of a year}
  \item{X276}{Temperatures on the day 276 of a year}
  \item{X277}{Temperatures on the day 277 of a year}
  \item{X278}{Temperatures on the day 278 of a year}
  \item{X279}{Temperatures on the day 279 of a year}
  \item{X280}{Temperatures on the day 280 of a year}
  \item{X281}{Temperatures on the day 281 of a year}
  \item{X282}{Temperatures on the day 282 of a year}
  \item{X283}{Temperatures on the day 283 of a year}
  \item{X284}{Temperatures on the day 284 of a year}
  \item{X285}{Temperatures on the day 285 of a year}
  \item{X286}{Temperatures on the day 286 of a year}
  \item{X287}{Temperatures on the day 287 of a year}
  \item{X288}{Temperatures on the day 288 of a year}
  \item{X289}{Temperatures on the day 289 of a year}
  \item{X290}{Temperatures on the day 290 of a year}
  \item{X291}{Temperatures on the day 291 of a year}
  \item{X292}{Temperatures on the day 292 of a year}
  \item{X293}{Temperatures on the day 293 of a year}
  \item{X294}{Temperatures on the day 294 of a year}
  \item{X295}{Temperatures on the day 295 of a year}
  \item{X296}{Temperatures on the day 296 of a year}
  \item{X297}{Temperatures on the day 297 of a year}
  \item{X298}{Temperatures on the day 298 of a year}
  \item{X299}{Temperatures on the day 299 of a year}
  \item{X300}{Temperatures on the day 300 of a year}
  \item{X301}{Temperatures on the day 301 of a year}
  \item{X302}{Temperatures on the day 302 of a year}
  \item{X303}{Temperatures on the day 303 of a year}
  \item{X304}{Temperatures on the day 304 of a year}
  \item{X305}{Temperatures on the day 305 of a year}
  \item{X306}{Temperatures on the day 306 of a year}
  \item{X307}{Temperatures on the day 307 of a year}
  \item{X308}{Temperatures on the day 308 of a year}
  \item{X309}{Temperatures on the day 309 of a year}
  \item{X310}{Temperatures on the day 310 of a year}
  \item{X311}{Temperatures on the day 311 of a year}
  \item{X312}{Temperatures on the day 312 of a year}
  \item{X313}{Temperatures on the day 313 of a year}
  \item{X314}{Temperatures on the day 314 of a year}
  \item{X315}{Temperatures on the day 315 of a year}
  \item{X316}{Temperatures on the day 316 of a year}
  \item{X317}{Temperatures on the day 317 of a year}
  \item{X318}{Temperatures on the day 318 of a year}
  \item{X319}{Temperatures on the day 319 of a year}
  \item{X320}{Temperatures on the day 320 of a year}
  \item{X321}{Temperatures on the day 321 of a year}
  \item{X322}{Temperatures on the day 322 of a year}
  \item{X323}{Temperatures on the day 323 of a year}
  \item{X324}{Temperatures on the day 324 of a year}
  \item{X325}{Temperatures on the day 325 of a year}
  \item{X326}{Temperatures on the day 326 of a year}
  \item{X327}{Temperatures on the day 327 of a year}
  \item{X328}{Temperatures on the day 328 of a year}
  \item{X329}{Temperatures on the day 329 of a year}
  \item{X330}{Temperatures on the day 330 of a year}
  \item{X331}{Temperatures on the day 331 of a year}
  \item{X332}{Temperatures on the day 332 of a year}
  \item{X333}{Temperatures on the day 333 of a year}
  \item{X334}{Temperatures on the day 334 of a year}
  \item{X335}{Temperatures on the day 335 of a year}
  \item{X336}{Temperatures on the day 336 of a year}
  \item{X337}{Temperatures on the day 337 of a year}
  \item{X338}{Temperatures on the day 338 of a year}
  \item{X339}{Temperatures on the day 339 of a year}
  \item{X340}{Temperatures on the day 340 of a year}
  \item{X341}{Temperatures on the day 341 of a year}
  \item{X342}{Temperatures on the day 342 of a year}
  \item{X343}{Temperatures on the day 343 of a year}
  \item{X344}{Temperatures on the day 344 of a year}
  \item{X345}{Temperatures on the day 345 of a year}
  \item{X346}{Temperatures on the day 346 of a year}
  \item{X347}{Temperatures on the day 347 of a year}
  \item{X348}{Temperatures on the day 348 of a year}
  \item{X349}{Temperatures on the day 349 of a year}
  \item{X350}{Temperatures on the day 350 of a year}
  \item{X351}{Temperatures on the day 351 of a year}
  \item{X352}{Temperatures on the day 352 of a year}
  \item{X353}{Temperatures on the day 353 of a year}
  \item{X354}{Temperatures on the day 354 of a year}
  \item{X355}{Temperatures on the day 355 of a year}
  \item{X356}{Temperatures on the day 356 of a year}
  \item{X357}{Temperatures on the day 357 of a year}
  \item{X358}{Temperatures on the day 358 of a year}
  \item{X359}{Temperatures on the day 359 of a year}
  \item{X360}{Temperatures on the day 360 of a year}
  \item{X361}{Temperatures on the day 361 of a year}
  \item{X362}{Temperatures on the day 362 of a year}
  \item{X363}{Temperatures on the day 363 of a year}
  \item{X364}{Temperatures on the day 364 of a year}
  \item{X365}{Temperatures on the day 365 of a year}
  \item{X366}{Temperatures on the day 366 of a year}
}
}
\source{
\url{http://climexp.knmi.nl/start.cgi}
}
\usage{
LJ_daily_temperatures
}
\description{
A dataset of daily mean temperatures in Ljubljana (Slovenia). The first row
represents temperatures in 1930. The first column represents the first day
of a year, the second column represents the second day of a year, etc.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/years_to_rownames.R
\name{years_to_rownames}
\alias{years_to_rownames}
\title{Function returns a data frame with row names as years}
\usage{
years_to_rownames(data, column_year)
}
\arguments{
\item{data}{a data frame to be manipulated}

\item{column_year}{string specifying a column with years}
}
\value{
a data frame with years as row names
}
\description{
Function returns a data frame with row names as years
}
\examples{
data <- data.frame(years = seq(1950, 2015), observations = rnorm(66))
new_data <- years_to_rownames(data = data, column_year = "years")

data <- data.frame(observations1 = rnorm(66), years = seq(1950, 2015),
observations2 = rnorm(66), observations3 = rnorm(66))
new_data <- years_to_rownames(data = data, column_year = "years")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_proxies_individual.R
\docType{data}
\name{example_proxies_individual}
\alias{example_proxies_individual}
\title{Example of dataset with individual chronologies of MVA.}
\format{
A data frame with 56 rows and 54 columns :
\describe{
  \item{MVA_1}{Mean vessel area chronology for tree 1}
  \item{MVA_2}{Mean vessel area chronology for tree 2}
  \item{MVA_3}{Mean vessel area chronology for tree 3}
  \item{MVA_4}{Mean vessel area chronology for tree 4}
  \item{MVA_5}{Mean vessel area chronology for tree 5}
  \item{MVA_6}{Mean vessel area chronology for tree 6}
  \item{MVA_7}{Mean vessel area chronology for tree 7}
  \item{MVA_8}{Mean vessel area chronology for tree 8}
  \item{MVA_9}{Mean vessel area chronology for tree 9}
  \item{MVA_10}{Mean vessel area chronology for tree 10}
}
}
\source{
Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
Ljubljana, Slovenia
}
\usage{
example_proxies_individual
}
\description{
A dataset of individual tree-ring chronologies from a lowland
forest in Slovenia. The first row represents a value of a year in 2015.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_methods.R
\name{compare_methods}
\alias{compare_methods}
\title{compare_methods}
\usage{
compare_methods(
  formula,
  dataset,
  k = 10,
  repeats = 2,
  optimize = TRUE,
  dataset_complete = NULL,
  BRNN_neurons = 1,
  MT_committees = 1,
  MT_neighbors = 5,
  MT_rules = 200,
  MT_unbiased = TRUE,
  MT_extrapolation = 100,
  MT_sample = 0,
  RF_ntree = 500,
  RF_maxnodes = 5,
  RF_mtry = 1,
  RF_nodesize = 1,
  seed_factor = 5,
  digits = 3,
  blocked_CV = FALSE,
  PCA_transformation = FALSE,
  log_preprocess = TRUE,
  components_selection = "automatic",
  eigenvalues_threshold = 1,
  N_components = 2,
  round_bias_cal = 15,
  round_bias_val = 4,
  n_bins = 30,
  edge_share = 0.1,
  MLR_stepwise = FALSE,
  stepwise_direction = "backward",
  methods = c("MLR", "BRNN", "MT", "RF"),
  tuning_metric = "RMSE",
  BRNN_neurons_vector = c(1, 2, 3),
  MT_committees_vector = c(1, 5, 10),
  MT_neighbors_vector = c(0, 5),
  MT_rules_vector = c(100, 200),
  MT_unbiased_vector = c(TRUE, FALSE),
  MT_extrapolation_vector = c(100),
  MT_sample_vector = c(0),
  RF_ntree_vector = c(100, 250, 500),
  RF_maxnodes_vector = c(5, 10, 20, 25),
  RF_mtry_vector = c(1),
  RF_nodesize_vector = c(1, 5, 10),
  holdout = NULL,
  holdout_share = 0.1,
  holdout_manual = NULL,
  total_reproducibility = FALSE
)
}
\arguments{
\item{formula}{an object of class "formula" (or one that can be coerced
to that class): a symbolic description of the model to be fitted.}

\item{dataset}{a data frame with dependent and independent variables as
columns and (optional) years as row names.}

\item{k}{number of folds for cross-validation}

\item{repeats}{number of cross-validation repeats. Should be equal or more
than 1}

\item{optimize}{if set to TRUE (default), the optimal values for the tuning
parameters will be selected in a preliminary cross-validation procedure}

\item{dataset_complete}{optional, a data frame with the full length of tree-ring
parameter, which will be used to reconstruct the climate variable specified
with the formula argument}

\item{BRNN_neurons}{number of neurons to be used for the brnn method}

\item{MT_committees}{an integer:  how many committee models (e.g.  boosting
iterations) should be used?}

\item{MT_neighbors}{how many, if any, neighbors should be used to correct the
model predictions}

\item{MT_rules}{an integer (or NA): define an explicit limit to the number of
rules used (NA let’s Cubist decide).}

\item{MT_unbiased}{a logical: should unbiased rules be used?}

\item{MT_extrapolation}{a number between 0 and 100: since Cubist uses linear models,
predictions can be outside of the outside of the range seen the training set. This
parameter controls how much rule predictions are adjusted to be consistent with the
training set.}

\item{MT_sample}{a number between 0 and 99.9:  this is the percentage of the dataset
to be randomly selected for model building (not for out-of-bag type evaluation)}

\item{RF_ntree}{number of trees to grow. This should not be set to too small
a number, to ensure that every input row gets predicted at least a few times}

\item{RF_maxnodes}{maximum number of terminal nodes trees in the forest can
have}

\item{RF_mtry}{number of variables randomly sampled as candidates at each
split}

\item{RF_nodesize}{minimum size of terminal nodes. Setting this number larger
causes smaller trees to be grown (and thus take less time).}

\item{seed_factor}{an integer that will be used to change the seed options
for different repeats.}

\item{digits}{integer of number of digits to be displayed in the final
result tables}

\item{blocked_CV}{default is FALSE, if changed to TRUE, blocked cross-validation
will be used to compare regression methods.}

\item{PCA_transformation}{if set to TRUE, all independent variables will be
transformed using PCA transformation.}

\item{log_preprocess}{if set to TRUE, variables will be transformed with
logarithmic transformation before used in PCA}

\item{components_selection}{character string specifying how to select the Principal
Components used as predictors.
There are three options: "automatic", "manual" and "plot_selection". If
parameter is set to automatic, all scores with eigenvalues above 1 will be
selected. This threshold could be changed by changing the
eigenvalues_threshold argument. If parameter is set to "manual", user should
set the number of components with N_components argument. If component
selection is se to "plot_selection", Scree plot will be shown and user must
manually enter the number of components used as predictors.}

\item{eigenvalues_threshold}{threshold for automatic selection of Principal Components}

\item{N_components}{number of Principal Components used as predictors}

\item{round_bias_cal}{number of digits for bias in calibration period. Effects
the outlook of the final ggplot  of mean bias for calibration data (element 3 of
the output list)}

\item{round_bias_val}{number of digits for bias in validation period. Effects
the outlook of the final ggplot of mean bias for validation data (element 4 of
the output list)}

\item{n_bins}{number of bins used for the histograms of mean bias}

\item{edge_share}{the share of the data to be considered as the edge (extreme) data.
This argument could be between 0.10 and 0.50. If the argument is set to 0.10, then
the 5 % of the maximal extreme values and 5 % of the minimal extreme values are
considered to be the edge data.}

\item{MLR_stepwise}{if set to TRUE, stepwise selection of predictors will be used
for the MLR method}

\item{stepwise_direction}{the mode of stepwise search, can be one of "both",
"backward", or "forward", with a default of "backward".}

\item{methods}{a vector of strings related to methods that will be compared. A full
method vector is methods = c("MLR", "BRNN", "MT", "RF").
To use only a subset of methods, pass a vector of methods that you would like to compare.}

\item{tuning_metric}{a string that specifies what summary metric will be used to select
the optimal value of tuning parameters. By default, the argument is set to "RMSE". It is
also possible to use "RSquared".}

\item{BRNN_neurons_vector}{a vector of possible values for BRNN_neurons argument optimization}

\item{MT_committees_vector}{a vector of possible values for MT_committees argument optimization}

\item{MT_neighbors_vector}{a vector of possible values for MT_neighbors argument optimization}

\item{MT_rules_vector}{a vector of possible values for MT_rules argument optimization}

\item{MT_unbiased_vector}{a vector of possible values for MT_unbiased argument optimization}

\item{MT_extrapolation_vector}{a vector of possible values for MT_extrapolation argument optimization}

\item{MT_sample_vector}{a vector of possible values for MT_sample argument optimization}

\item{RF_ntree_vector}{a vector of possible values for RF_ntree argument optimization}

\item{RF_maxnodes_vector}{a vector of possible values for RF_maxnodes argument optimization}

\item{RF_mtry_vector}{a vector of possible values for RF_mtry argument optimization}

\item{RF_nodesize_vector}{a vector of possible values for RF_nodesize argument optimization}

\item{holdout}{this argument is used to define observations, which are excluded
from the cross-validation and hyperparameters optimization. The holdout argument must be
a character with one of the following inputs: “early”, “late” or “manual”. If
"early" or "late" characters are specified, then the early or late years will be
used as a holdout data. How many of the "early" or "late" years are used as a holdout
is specified with the argument holdout_share. If the argument holdout is set to “manual”,
then supply a vector of years (or row names) to the argument holdout_manual. Defined
years will be used as a holdout. For the holdout data, the same statistical measures are
calculated as for the cross-validation. The results for holdout metrics are given in the
output element $holdout_results.}

\item{holdout_share}{the share of the whole dataset to be used as a holdout.
Default is 0.10.}

\item{holdout_manual}{a vector of years (or row names) which will be used as a holdout.
calculated as for the cross-validation.}

\item{total_reproducibility}{logical, default is FALSE. This argument ensures total
reproducibility despite the inclusion/exclusion of different methods. By default, the
optimization is done only for the methods, that are included in the methods vector. If
one method is absent or added, the optimization phase is different, and this affects
all the final cross-validation results. By setting the total_reproducibility = TRUE,
all methods will be optimized, even though they are not included in the methods vector
and the final results will be subset based on the methods vector. Setting the
total_reproducibility to TRUE will result in longer optimization phase as well.}
}
\value{
a list with 19 elements:
\enumerate{
 \item $mean_std - data frame with calculated metrics for the selected \\ regression methods. For each regression method and each calculated metric, mean and standard deviation are given
 \item $ranks - data frame with ranks of calculated metrics: mean rank and  share of rank_1 are given
 \item $edge_results - data frame with calculated performance metrics for the central-edge test. The central part of the data represents the calibration data, while the edge data, i.e. extreme values, represent the test/validation data. Different regression models are calibrated using the central data and validated for the edge (extreme) data. This test is particularly important to assess the performance of models for the predictions of the extreme data. The share of the edge (extreme) data is defined with the edge_share argument
 \item $holdout_results - calculated metrics for the holdout data
 \item $bias_cal - ggplot object of mean bias for calibration data
 \item $bias_val - ggplot object of mean bias for validation data
 \item $transfer_functions - ggplot or plotly object with transfer functions of methods
 \item $transfer_functions_together - ggplot or plotly object with transfer functions of methods plotted together
 \item $parameter_values - a data frame with specifications of parameters used for different regression methods
 \item $PCA_output - princomp object: the result output of the PCA analysis
 \item $reconstructions - ggplot object: reconstructed dependent variable based on the dataset_complete argument, facet is used to split plots by methods
 \item $reconstructions_together - ggplot object: reconstructed dependent variable based on the dataset_complete argument, all reconstructions are on the same plot
 \item $normal_QQ_cal - normal q-q plot for calibration data
 \item $normal_QQ_holdout - normal q-q plot for holdout data
 \item $normal_QQ_edge- normal q-q plot for edge data
 \item $residuals_vs_fitted_cal - residuals vs fitted values plot for calibration data
 \item $residuals_vs_fitted_holdout - residuals vs fitted values plot for holdout data
 \item $residuals_vs_fitted_edge - residuals vs fitted values plot for edge data
 \item $reconstructions_data - raw data that is used for creating reconstruction plots
}
}
\description{
Calculates performance metrics for calibration (train) and validation (test)
data of different regression methods: multiple linear regression (MLR),
artificial neural networks with Bayesian regularization training
algorithm (BRNN), (ensemble of) model trees (MT) and random forest of regression
trees (RF). With the subset argument, specific methods of interest could be
specified. Calculated performance metrics are the correlation coefficient (r),
the root mean squared error (RMSE), the root relative squared error (RRSE),
the index of agreement (d), the reduction of error (RE), the coefficient of
efficiency (CE), the detrended efficiency (DE) and mean bias. For each of the
considered methods, there are also residual diagnostic plots available,
separately for calibration, holdout and edge data, if applicable.
}
\examples{
\donttest{

# The examples below are enclosed within donttest{} to minimize the execution
# time during R package checks. #'

# An example with default settings of machine learning algorithms
library(dendroTools)
library(ggplot2)

data(example_dataset_1)
data(dataset_TRW)

example_1 <- compare_methods(formula = MVA ~  T_APR,
dataset = example_dataset_1, k = 5, repeats = 1, BRNN_neurons = 1,
RF_ntree = 100, RF_mtry = 2, RF_maxnodes = 35, seed_factor = 5)

# example_1$mean_std
# example_1$ranks
# example_1$bias_cal
# example_1$transfer_functions
# example_1$transfer_functions_together
# example_1$PCA_output
# example_1$parameter_values

example_2 <- compare_methods(formula = MVA ~ .,
dataset = example_dataset_1, k = 2, repeats = 2,
methods = c("MLR", "BRNN", "MT"),
optimize = TRUE, MLR_stepwise = TRUE)
# example_2$mean_std
# example_2$ranks
# example_2$bias_val
# example_2$transfer_functions
# example_2$transfer_functions_together
# example_2$parameter_values

comparison_TRW <- compare_methods(formula = T_Jun_Jul ~ TRW, dataset = dataset_TRW,
k = 3, repeats = 5, optimize = FALSE, methods = c("MLR", "BRNN", "RF", "MT"),
seed_factor = 5, dataset_complete = dataset_TRW_complete, MLR_stepwise = TRUE,
stepwise_direction = "backward")

# comparison_TRW$mean_std
# comparison_TRW$bias_val
# comparison_TRW$transfer_functions
# comparison_TRW$reconstructions
# comparison_TRW$reconstructions_together
# comparison_TRW$edge_results
# comparison_TRW$reconstructions_data

}
}
\references{
Bishop, C.M., 1995. Neural Networks for Pattern Recognition. Oxford
University Press, Inc. 482 pp.

Breiman, L., 1996. Bagging predictors. Machine Learning 24, 123-140.

Breiman, L., 2001. Random forests. Machine Learning 45, 5-32.

Burden, F., Winkler, D., 2008. Bayesian Regularization of Neural Networks,
in: Livingstone, D.J. (ed.), Artificial Neural Networks: Methods and
Applications, vol. 458. Humana Press, Totowa, NJ, pp. 23-42.

Hastie, T., Tibshirani, R., Friedman, J.H., 2009. The Elements of
Statistical Learning : Data Mining, Inference, and Prediction, 2nd ed.
Springer, New York xxii, 745 p. pp.

Ho, T.K., 1995. Random decision forests, Proceedings of the Third
International Conference on Document Analysis and Recognition Volume 1.
IEEE Computer Society, pp. 278-282.

Hornik, K., Buchta, C., Zeileis, A., 2009. Open-source machine learning: R
meets Weka. Comput. Stat. 24, 225-232.

Perez-Rodriguez, P., Gianola, D., 2016. Brnn: Brnn (Bayesian Regularization
for Feed-forward Neural Networks). R package version 0.6.

Quinlan, J.R., 1992. Learning with Continuous Classes, Proceedings of the
5th Australian Joint Conference on Artificial Intelligence (AI '92). World
Scientific, Hobart, pp. 343-348.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count_ones.R
\name{count_ones}
\alias{count_ones}
\title{count_ones}
\usage{
count_ones(vector)
}
\arguments{
\item{vector}{a vector of integers}
}
\value{
an integer of counted ones
}
\description{
calculates share of integer 1 in a vector
}
\examples{
vector_1 <- seq(1:10)
# count_ones(vector_1)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LJ_daily_precipitation.R
\docType{data}
\name{LJ_daily_precipitation}
\alias{LJ_daily_precipitation}
\title{Daily precipitation for Ljubljana from 2017 - 1900}
\format{
A data frame with 43067 rows and 3 variables:
\describe{
  \item{Year}{year}
  \item{DOY}{day of year}
  \item{Precipitation}{Sum of precipitation in mm}
}
}
\source{
\url{http://climexp.knmi.nl/start.cgi}
}
\usage{
LJ_daily_precipitation
}
\description{
A dataset of daily sum of precipitation [mm] in Ljubljana (Slovenia).
The first row represents precipitation in 1900 on DOY 1.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_MVA.R
\docType{data}
\name{dataset_MVA}
\alias{dataset_MVA}
\title{MVA and mean April temperature}
\format{
A data frame with 79 rows and 2 variables:
\describe{
  \item{MVA}{Mean vessel area measurements from 2012 - 1934}
  \item{T_Apr}{Mean April temperature for the meteorological station Maribor from 2012 - 1934}
}
}
\source{
Jevšenak J., Levanič T. 2015. Dendrochronological and wood-anatomical features of
differently vital pedunculate oak (Quercus robur L.) stands and their response to climate.
Topola, 195/196: 85-96
}
\usage{
dataset_MVA
}
\description{
A dataset with a mean vessel area (MVA) chronology of Quercus robur from
a lowland oak forest in Eastern Slovenia and a mean April temperature.
This dataset includes years for the period 2012-1934. For a detailed
description about the MVA chronology development, sampling site and the
calculations of mean monthly correlations, see Jevšenak and Levanič (2015).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_metrics.R
\name{calculate_metrics}
\alias{calculate_metrics}
\title{calculate_metrics}
\usage{
calculate_metrics(
  train_predicted,
  test_predicted,
  train_observed,
  test_observed,
  digits = 4,
  formula,
  test
)
}
\arguments{
\item{train_predicted}{a vector indicating predicted data for training set}

\item{test_predicted}{a vector indicating predicted data for testing set}

\item{train_observed}{a vector indicating observed data for training set}

\item{test_observed}{a vector indicating observed data for training set}

\item{digits}{integer of number of digits to be displayed}

\item{formula}{an object of class "formula" (or one that can be coerced
to that class): a symbolic description of the model to be fitted. This
additional argument is needed to calculate DE metrics.}

\item{test}{data frame with test data.}
}
\value{
a data frame of calculated test and train metrics
}
\description{
Calculates performance metrics for train and test data. Calculated
performance metrics are correlation coefficient (r), root mean squared
error (RMSE), root relative squared error (RRSE), index of agreement
(d), reduction of error (RE), coefficient of efficiency (CE), detrended
efficiency (DE) and bias.
}
\examples{
\donttest{
data(example_dataset_1)
test_data <- example_dataset_1[1:30, ]
train_data <- example_dataset_1[31:55, ]
lin_mod <- lm(MVA ~., data = train_data)
train_predicted <- predict(lin_mod, train_data)
test_predicted <- predict(lin_mod, test_data)
train_observed <- train_data[, 1]
test_observed <- test_data[, 1]
calculate_metrics(train_predicted, test_predicted, train_observed,
test_observed, test = test_data, formula = MVA ~.)

test_data <- example_dataset_1[1:20, ]
train_data <- example_dataset_1[21:55, ]
library(brnn)
lin_mod <- brnn(MVA ~., data = train_data)
train_predicted <- predict(lin_mod, train_data)
test_predicted <- predict(lin_mod, test_data)
train_observed <- train_data[, 1]
test_observed <- test_data[, 1]
calculate_metrics(train_predicted, test_predicted, train_observed,
test_observed, test = test_data, formula = MVA ~.)
}
}
\references{
Briffa, K.R., Jones, P.D., Pilcher, J.R., Hughes, M.K., 1988. Reconstructing
summer temperatures in northern Fennoscandinavia back to A.D.1700 using tree
ring data from Scots Pine. Arct. Alp. Res. 20, 385-394.

Fritts, H.C., 1976. Tree Rings and Climate. Academic Press, London 567 pp.

Lorenz, E.N., 1956. Empirical Orthogonal Functions and Statistical Weather
Prediction. Massachusetts Institute of Technology, Department of Meteorology.

Willmott, C.J., 1981. On the validation of models. Phys. Geogr. 2, 184-194.

Witten, I.H., Frank, E., Hall, M.A., 2011. Data Mining: Practical Machine
Learning Tools and Techniques, 3rd ed. Morgan Kaufmann Publishers, Burlington
629 pp.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot_f_brnn.R
\name{boot_f_brnn}
\alias{boot_f_brnn}
\title{boot_f_brnn}
\usage{
boot_f_brnn(data, index, brnn.formula, neurons = 3)
}
\arguments{
\item{data}{data frame with variables for model fitting}

\item{index}{indices to be used for calculation}

\item{brnn.formula}{object of class formula to be passed into brnn function}
}
\value{
A matrix with bootstrapped estimates of r squared and adjusted r squared
}
\description{
Generate R bootstrap replicates of a brnn function applied to data. Function is based on boot() from
boot R package
}
\references{
https://www.datacamp.com/community/tutorials/bootstrap-r
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_TRW_complete.R
\docType{data}
\name{dataset_TRW_complete}
\alias{dataset_TRW_complete}
\title{The complete dataset of standardized tree-ring chronology from Albania}
\format{
A data frame with 551 rows and 1 variable:
\describe{
  \item{TRW}{Standardised tree-ring width chronology of Pinus nigra from Albania}
}
}
\source{
Levanič, T., Poljanšek, S., Toromani, E., 2015. Early summer temperatures reconstructed from
black pine (Pinus nigra Arnold) tree-ring widths from Albania. The Holocene 25, 469-481.
}
\usage{
dataset_TRW_complete
}
\description{
A dataset with a tree-ring width (TRW) chronology of Pinus nigra from Albania This TRW
chronology has a span of 551 years (period 2009 - 1459) and was already used to reconstruct
summer temperatures by Levanič et al. (2015). In this paper, all the details about sample
replication, site description and correlation statistics are described.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_TRW_1.R
\docType{data}
\name{data_TRW_1}
\alias{data_TRW_1}
\title{Tree-ring width (TRW) data from 2012 - 1961}
\format{
A data frame with 52 rows and 1 variable:
\describe{
  \item{TRW}{Standardized tree-ring width indices from 2012 - 1961}
}
}
\source{
Tom Levanič, Slovenian Forestry Institute, Večna pot 2,
Ljubljana, Slovenia
}
\usage{
data_TRW_1
}
\description{
A dataset of tree-ring widths (TRW) from a site in Krakovo forest (Slovenia).
The first row represents a value of a year in 1961.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_transform.R
\name{data_transform}
\alias{data_transform}
\title{data_transform}
\usage{
data_transform(
  input,
  format = "daily",
  monthly_aggregate_function = "auto",
  date_format = "ymd"
)
}
\arguments{
\item{input}{typical daily data format: Data frame with two columns, first column represents date,
second column represents variable, such as mean temperature, precipitation, etc. Date should be in
format Year-Month-Day (e.g. "2019-05-15")}

\item{format}{character string indicating the desired output format. Should be "daily" or "monthly".
Daily format returns a data frame with 366 columns (days), while monthly format returns data frame
with 12 columns (months). Years are indicated as row names.}

\item{monthly_aggregate_function}{character string indicating, how to aggregate daily into monthly
data. It can be "mean" or "sum". Third option is "auto" (default). In this case function will try
to guess whether input is temperature or precipitation data. For temperature, it will use "mean",
for precipitation "sum".}

\item{date_format}{Describe the format of date. It should be one of "ymd", "ydm", "myd", "mdy",
"dmy", "dym".}
}
\value{
env_data suitable for daily or monthly analysis with dendroTools.
}
\description{
Transforms daily data with two columns (date and variable) into data frame suitable for daily or
monthly analysis with dendroTools.
}
\examples{
data(swit272_daily_temperatures)
proper_daily_data <- data_transform(swit272_daily_temperatures, format = "daily",
   date_format = "ymd")

proper_monthly_data <- data_transform(swit272_daily_temperatures, format = "monthly",
   date_format = "ymd")

data(swit272_daily_precipitation)
proper_daily_data <- data_transform(swit272_daily_precipitation, format = "daily",
   date_format = "ymd")

proper_monthly_data <- data_transform(swit272_daily_precipitation, format = "monthly",
   date_format = "ymd")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_extreme.R
\name{plot_extreme}
\alias{plot_extreme}
\title{plot_extreme}
\usage{
plot_extreme(
  result_daily_response,
  title = TRUE,
  ylimits = NULL,
  reference_window = "start",
  type = "daily",
  show_year_separators = TRUE
)
}
\arguments{
\item{result_daily_response}{a list with objects as produced by
daily_response or monthly_response function}

\item{title}{logical, if set to FALSE, no plot title is displayed}

\item{ylimits}{limit of the y axes. It should be given as ylimits = c(0,1)}

\item{reference_window}{character string, the reference_window argument describes,
how each calculation is referred. There are three different options: 'start'
(default), 'end' and 'middle'. If available, the value stored in
result_daily_response$reference_window is used.}

\item{type}{the character string describing type of analysis: daily or monthly}

\item{show_year_separators}{logical. If TRUE, dashed vertical lines are
added between relative-year blocks. The default is TRUE.}
}
\value{
A ggplot2 object containing the plot display
}
\description{
Graphs a line or bar plot of a row with the highest metric in a matrix,
produced by \code{\link{daily_response}} or \code{\link{monthly_response}}
functions. Bar plot is drawn for monthly_response(), while for daily_response,
line plot is produced.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_MVA_individual.R
\docType{data}
\name{dataset_MVA_individual}
\alias{dataset_MVA_individual}
\title{Example of dataset with individual chronologies of MVA and mean April temperature}
\format{
A data frame with 56 rows and 54 columns :
\describe{
  \item{T_Apr}{mean April temperature for Ljubljana}
  \item{MVA_1}{Mean vessel area chronology for tree 1}
  \item{MVA_2}{Mean vessel area chronology for tree 2 [mm^2]}
  \item{MVA_3}{Mean vessel area chronology for tree 3 [mm^2]}
  \item{MVA_4}{Mean vessel area chronology for tree 4 [mm^2]}
  \item{MVA_5}{Mean vessel area chronology for tree 5 [mm^2]}
  \item{MVA_6}{Mean vessel area chronology for tree 6 [mm^2]}
  \item{MVA_7}{Mean vessel area chronology for tree 7 [mm^2]}
  \item{MVA_8}{Mean vessel area chronology for tree 8 [mm^2]}
  \item{MVA_9}{Mean vessel area chronology for tree 9 [mm^2]}
  \item{MVA_10}{Mean vessel area chronology for tree 10 [mm^2]}
}
}
\source{
Slovenian Forestry Institute, Večna pot 2,
Ljubljana, Slovenia
}
\usage{
dataset_MVA_individual
}
\description{
A dataset of individual tree-ring chronologies from a lowland
forest in Slovenia. The first row represents a value of a year in 2015.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_heatmap.R
\name{plot_heatmap}
\alias{plot_heatmap}
\title{plot_heatmap}
\usage{
plot_heatmap(
  result_daily_response,
  reference_window = "start",
  type = "daily",
  show_title = FALSE,
  show_year_separators = TRUE
)
}
\arguments{
\item{result_daily_response}{a list with objects as produced by
\code{\link{daily_response}}, \code{\link{monthly_response}},
\code{\link{daily_response_seascorr}} or
\code{\link{monthly_response_seascorr}}.}

\item{reference_window}{character string, the reference_window argument describes
how each calculation is referred. Options are 'start', 'end' and 'middle'.
If available, the value stored in result_daily_response$reference_window is used.}

\item{type}{the character string describing type of analysis: daily or monthly.}

\item{show_title}{logical. If TRUE, the plot title with analysed period, method
and reference-window information is shown. The default is FALSE.}

\item{show_year_separators}{logical. If TRUE, dashed vertical lines are
added between relative-year blocks in plots with previous-year data.
The default is TRUE.}
}
\value{
A ggplot2 object containing the heatmap display
}
\description{
Graphs a heatmap of values stored in a matrix, such as produced
by \code{\link{daily_response}} or \code{\link{monthly_response}} functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataset_TRW.R
\docType{data}
\name{dataset_TRW}
\alias{dataset_TRW}
\title{TRW and mean June -  July temperature from Albania}
\format{
A data frame with 59 rows and 2 variables:
\describe{
  \item{TRW}{Standardised tree-ring width chronology of Pinus nigra from Albania}
  \item{T_Jun_Jul}{Mean June - July temperature for Albania downloaded from KNMI Climate Explorer}
}
}
\source{
Levanič, T., Poljanšek, S., Toromani, E., 2015. Early summer temperatures reconstructed from
black pine (Pinus nigra Arnold) tree-ring widths from Albania. The Holocene 25, 469-481.
}
\usage{
dataset_TRW
}
\description{
A dataset with a tree-ring width (TRW) chronology of Pinus nigra from Albania and mean
June-July temperature. This TRW chronology has a span of 59 years (period 2009 - 1951)
and was already used to reconstruct summer temperatures by Levanič et al. (2015).
In this paper, all the details about sample replication, site description and correlation
statistics are described.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/example_dataset_1.R
\docType{data}
\name{example_dataset_1}
\alias{example_dataset_1}
\title{Example of dataset as required for compare_methods()}
\format{
A data frame with 58 rows and 3 columns :
\describe{
  \item{MVA}{Mean Vessel Area measurements from 2012 - 1955}
  \item{T_APR}{Mean April temperatures from 2012 - 1955}
  \item{T_aug_sep}{Mean August-September temperatures from preceding
  growing season from 2012 - 1955}
}
}
\source{
Jernej Jevšenak, Slovenian Forestry Institute, Večna pot 2,
Ljubljana, Slovenia
}
\usage{
example_dataset_1
}
\description{
A dataset of Mean Vessel Area (MVA) tree-ring parameter from a lowland
forest in Slovenia. The first row represents a value of a year in 2012.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/round_df.R
\name{round_df}
\alias{round_df}
\title{round_df}
\usage{
round_df(df, digits = 3)
}
\arguments{
\item{df}{a data frame}

\item{digits}{number of digits for the round function}
}
\value{
data frame with rounded values
}
\description{
Round all numeric columns in a data frame.
}
\examples{
ID <- c("a", "b", "c", "d", "e")
Value1 <- as.numeric(c("3.4", "6.4", "8.7", "1.1", "0.1"))
Value2 <- as.numeric(c("8.2", "1.7", "6.4", "1.9", "10.3"))
df <- data.frame(ID, Value1, Value2, stringsAsFactors = FALSE)
# round_df(df, digits = 0)

}
\references{
https://stackoverflow.com/questions/9063889/how-to-round-a-data-frame-in-r-that-contains-some-character-variables
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daily_response.R
\name{daily_response}
\alias{daily_response}
\title{daily_response}
\usage{
daily_response(
  response,
  env_data,
  method = "cor",
  metric = "r.squared",
  cor_method = "pearson",
  lower_limit = 30,
  upper_limit = 90,
  fixed_width = 0,
  previous_year = FALSE,
  number_previous_years = NULL,
  neurons = 1,
  brnn_smooth = TRUE,
  remove_insignificant = FALSE,
  alpha = 0.05,
  row_names_subset = FALSE,
  aggregate_function = "mean",
  quantile_prob = 0.5,
  temporal_stability_check = "sequential",
  k = 2,
  k_running_window = 30,
  cross_validation_type = "blocked",
  subset_years = NULL,
  ylimits = NULL,
  seed = NULL,
  tidy_env_data = FALSE,
  reference_window = "start",
  boot = FALSE,
  boot_n = 1000,
  boot_ci_type = "norm",
  boot_conf_int = 0.95,
  day_interval = NULL,
  dc_method = NULL,
  cor_na_use = "everything",
  skip_window_length = 1,
  skip_window_position = 1
)
}
\arguments{
\item{response}{a data frame with tree-ring proxy variables as columns and
(optional) years as row names. Row.names should be matched with those from a
env_data data frame. If not, set row_names_subset = TRUE.}

\item{env_data}{a data frame of daily sequences of environmental data as
columns and years as row names. Each row represents a year and
each column represents a day of a year. Row.names should be matched with
those from a response data frame. If not, set row_names_subset = TRUE.
Alternatively, env_data could be a tidy data with three columns,
i.e. Year, DOY and third column representing values of mean temperatures,
sum of precipitation etc. If tidy data is passed to the function, set the argument
tidy_env_data to TRUE.}

\item{method}{a character string specifying which method to use. Current
possibilities are "cor" (default), "lm" and "brnn".}

\item{metric}{a character string specifying which metric to use. Current
possibilities are "r.squared" and "adj.r.squared". If method = "cor",
metric is not relevant.}

\item{cor_method}{a character string indicating which correlation
coefficient is to be computed. One of "pearson" (default), "kendall", or
"spearman".}

\item{lower_limit}{lower limit of window width}

\item{upper_limit}{upper limit of window width}

\item{fixed_width}{fixed width used for calculation. If fixed_width is
assigned a value, upper_limit and lower_limit will be ignored}

\item{previous_year}{logical. If TRUE, previous-year climate data are included
in the analysis. If FALSE, no previous-year climate data are included and
number_previous_years is ignored.}

\item{number_previous_years}{integer between 1 and 5 specifying how many
previous years should be included in the environmental matrix when
previous_year = TRUE. For example, number_previous_years = 2 uses climate
data from years t - 2, t - 1 and t for response year t. If NULL and
previous_year = TRUE, one previous year is included.}

\item{neurons}{positive integer that indicates the number of neurons used
for brnn method}

\item{brnn_smooth}{if set to TRUE, a smoothing algorithm is applied that
removes unrealistic calculations which are a result of neural net failure.}

\item{remove_insignificant}{if set to TRUE, removes all correlations bellow
the significant threshold level, based on a selected alpha. For "lm" and
"brnn" method, squared correlation is used as a threshold}

\item{alpha}{significance level used to remove insignificant calculations.}

\item{row_names_subset}{if set to TRUE, row.names are used to subset
env_data and response data frames. Only years from both data frames are
kept.}

\item{aggregate_function}{character string specifying how the daily data
should be aggregated. The default is 'mean', the other options are 'median',
'sum', 'min', 'max' and 'quantile'.}

\item{quantile_prob}{numeric value between 0 and 1 specifying the quantile
probability used when aggregate_function = 'quantile'. For example,
quantile_prob = 0.95 calculates the 95th percentile. The default is 0.5.}

\item{temporal_stability_check}{character string, specifying, how temporal stability
between the optimal selection and response variable(s) will be analysed. Current
possibilities are "sequential", "progressive" and "running_window". Sequential check
will split data into k splits and calculate selected metric for each split. Progressive
check will split data into k splits, calculate metric for the first split and then
progressively add 1 split at a time and calculate selected metric. For running window,
select the length of running window with the k_running_window argument.}

\item{k}{integer, number of breaks (splits) for temporal stability and cross validation
analysis.}

\item{k_running_window}{the length of running window for temporal stability check.
Applicable only if temporal_stability argument is set to running window.}

\item{cross_validation_type}{character string, specifying, how to perform cross validation
between the optimal selection and response variables. If the argument is set to "blocked",
years will not be shuffled. If the argument is set to "randomized", years will be shuffled.}

\item{subset_years}{a subset of years to be analyzed. Should be given in the form of
subset_years = c(1980, 2005)}

\item{ylimits}{limit of the y axes for plot_extreme. It should be given in
the form of: ylimits = c(0,1)}

\item{seed}{optional seed argument for reproducible results}

\item{tidy_env_data}{if set to TRUE, env_data should be inserted as a data frame with three
columns: "Year", "DOY", "Precipitation/Temperature/etc."}

\item{reference_window}{character string, the reference_window argument describes,
how each calculation is referred. There are three different options: 'start'
(default), 'end' and 'middle'. If the reference_window argument is set to 'start',
then each calculation is related to the starting day of window. If the
reference_window argument is set to 'middle', each calculation is related to the
middle day of window calculation. If the reference_window argument is set to
'end', then each calculation is related to the ending day of window calculation.
For example, if we consider correlations with window from DOY 15 to DOY 35. If
reference window is set to  'start', then this calculation will be related to the
DOY 15. If the reference window is set to 'end', then this calculation will be
related to the DOY 35. If the reference_window is set to 'middle', then this
calculation is related to DOY 25.
The optimal selection, which describes the optimal consecutive days that returns
the highest calculated metric and is obtained by the $plot_extreme output, is the
same for all three reference windows.}

\item{boot}{logical, if TRUE, bootstrap procedure will be used to calculate
estimates correlation coefficients, R squared or adjusted R squared metrices}

\item{boot_n}{The number of bootstrap replicates}

\item{boot_ci_type}{A character string representing the type of bootstrap intervals
required. The value should be any subset of the values c("norm","basic", "stud",
"perc", "bca").}

\item{boot_conf_int}{A scalar or vector containing the confidence level(s) of
the required interval(s)}

\item{day_interval}{a vector of two values defining the interval of days used
for calculations. Positive values indicate days in the current year. Negative
values indicate days in the previous-year block and are used only when
previous_year = TRUE. If previous_year = FALSE and negative values are
supplied, day_interval is ignored, a warning is issued, and the analysis is
performed for the current year only using day_interval = c(1, 366). If
number_previous_years > 1, the previous-year block starts with the earliest
included previous year. For example, previous_year = TRUE,
number_previous_years = 2 and day_interval = c(-1, 366) analyses the full
sequence from DOY 1 of year t - 2 to DOY 366 of year t.}

\item{dc_method}{a character string to determine the method to detrend climate
data.  Possible values are "none" (default) and "SLD" which refers to Simple
Linear Detrending}

\item{cor_na_use}{an optional character string giving a method for computing
covariances in the presence of missing values for correlation coefficients.
This must be (an abbreviation of) one of the strings "everything" (default),
"all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs". See
also the documentation for the base cor() function.}

\item{skip_window_length}{an integer specifying the frequency of window
selection for the calculations of climate-growth relationships. The default
value is 1, indicating that every window is included in the calculations.
When set to a value greater than 1, the function selectively processes
windows at regular intervals defined by this parameter. For instance, if
skip_window_length = 2, the function processes every second window.
Similarly, if skip_window_length = 3, every third window is processed,
skipping two windows in between each selected one. This parameter allows for
controlling the granularity of the analysis and can help in reducing
computation time by focusing on a subset of the data.}

\item{skip_window_position}{an integer specifying the frequency of window
positions used in the calculations of climate-growth relationships. The
default value is 1, indicating that every window position is included in the
calculations. When set to a value greater than 1, the function selectively
processes window positions at regular intervals defined by this parameter.
For instance, if skip_window_position = 2, the function processes every
second window position. Similarly, if skip_window_position = 3, every third
window position is processed, skipping two positions in between each selected
one. This parameter allows for controlling the granularity of the analysis
and can help in reducing computation time by focusing on a subset of the data.}
}
\value{
a list with 19 elements:
\enumerate{
 \item $calculations - a matrix with calculated metrics
 \item $method - the character string of a method
 \item $metric - the character string indicating the metric used for calculations
 \item $analysed_period - the character string specifying the analysed period based on the information from row names. If there are no row names, this argument is given as NA
 \item $optimized_return - data frame with two columns, response variable and aggregated (averaged) daily data that return the optimal results. This data.frame could be directly used to calibrate a model for climate reconstruction
 \item $optimized_return_all - a data frame with aggregated daily data, that returned the optimal result for the entire env_data (and not only subset of analysed years)
 \item $transfer_function - a ggplot object: scatter plot of optimized return and a transfer line of the selected method
 \item $temporal_stability -  a data frame with calculations of selected metric for different temporal subsets
 \item $cross_validation - a data frame with cross validation results
 \item $plot_heatmap - ggplot2 object: a heatmap of calculated metrics
 \item $plot_extreme - ggplot2 object: line plot of a row with the highest value in a matrix of calculated metrics
 \item $type - the character string describing type of analysis: daily or monthly
 \item $reference_window - character string, which reference window was used for calculations
 \item $boot_lower - matrix with lower limit of confidence intervals of bootstrap calculations
 \item $boot_upper - matrix with upper limit of confidence intervals of bootstrap calculations
 \item $aggregated_climate - matrix with all aggregated climate series
 \item $previous_year - logical indicating whether previous-year climate data were used
 \item $number_previous_years - integer indicating how many previous years were used
}
}
\description{
Function calculates all possible values of a selected statistical metric
between one or more response variables and daily sequences of environmental
data. Calculations are based on moving window which is defined with two
arguments: window width and a location in a matrix of daily sequences of
environmental data. Window width could be fixed (use fixed_width) or
variable width (use lower_limit and upper_limit arguments). In this case,
all window widths between lower and upper limit will be used. All calculated
metrics are stored in a matrix. The location of stored calculated metric
in the matrix is indicating a window width (row names) and a location in a
matrix of daily sequences of environmental data (column names).
}
\examples{
\donttest{

# The examples below are enclosed within donttest{} to minimize the execution
# time during R package checks. Additionally, all examples include the
# parameters `skip_window_length` and `skip_window_position`, which limit the
# number of combinations evaluated in climate-growth correlation calculations.
# To explore all possible combinations, users should set both parameters to 1.

# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_MVA)
data(data_TRW)
data(data_TRW_1)
data(example_proxies_individual)
data(example_proxies_1)
data(LJ_daily_temperatures)

example_basic <- daily_response(response = data_MVA,
                          env_data = LJ_daily_temperatures,
                          row_names_subset = TRUE,
                          fixed_width = 25,
                          lower_limit = 35, upper_limit = 45,
                          remove_insignificant = FALSE,
                          aggregate_function = 'median',
                          alpha = 0.05, cor_method = "spearman",
                          previous_year = FALSE, boot = TRUE,
                          boot_n = 10,
                          skip_window_length = 50,
                          skip_window_position = 50,
                          reference_window = "end", k = 5,
                          dc_method = "SLD",
                          day_interval = c(1, 250))

# 1 Example with fixed width. Lower and upper limits are ignored.
example_daily_response <- daily_response(response = data_MVA,
    env_data = LJ_daily_temperatures,
    method = "cor", fixed_width = 40, cor_method = "spearman",
    row_names_subset = TRUE, previous_year = TRUE,
    remove_insignificant = TRUE, boot = TRUE,
    alpha = 0.005, aggregate_function = 'mean',
    day_interval = c(-100, 250), skip_window_length = 100,
    reference_window = "start", skip_window_position = 100)

# summary(example_daily_response)
# plot(example_daily_response, type = 1)
# plot(example_daily_response, type = 2)

# 2 Example for past and present. Use subset_years argument.
example_MVA_early <- daily_response(response = data_MVA,
    env_data = LJ_daily_temperatures, cor_method = "kendall",
    method = "lm", lower_limit = 21, upper_limit = 91,
    row_names_subset = TRUE, previous_year = TRUE,
    remove_insignificant = TRUE, alpha = 0.05,
    subset_years = c(1940, 1980),
    fixed_width = 45,
    aggregate_function = 'sum',
    skip_window_length = 50,
    skip_window_position = 50)

example_MVA_late <- daily_response(response = data_MVA,
    env_data = LJ_daily_temperatures,
    method = "cor", lower_limit = 21, upper_limit = 60,
    row_names_subset = TRUE, previous_year = TRUE,
    remove_insignificant = TRUE, alpha = 0.05,
    subset_years = c(1981, 2010),
    skip_window_length = 50,
    skip_window_position = 50)

# plot(example_MVA_early, type = 1)
# plot(example_MVA_late, type = 1)
# plot(example_MVA_early, type = 2)
# plot(example_MVA_late, type = 2)

# 3 Example with negative correlations
example_neg_cor <- daily_response(response = data_TRW_1,
    env_data = LJ_daily_temperatures, previous_year = TRUE,
    method = "cor", lower_limit = 21, upper_limit = 90,
    row_names_subset = TRUE, remove_insignificant = TRUE,
    alpha = 0.05, skip_window_length = 50,
    skip_window_position = 50)

# summary(example_neg_cor)
# plot(example_neg_cor, type = 1)
# plot(example_neg_cor, type = 2)

# 4 Example of multiproxy analysis
# summary(example_proxies_1)
# cor(example_proxies_1)

example_multiproxy <- daily_response(response = example_proxies_1,
   env_data = LJ_daily_temperatures,
   method = "lm", metric = "adj.r.squared",
   lower_limit = 21, upper_limit = 180,
   row_names_subset = TRUE, previous_year = FALSE,
   remove_insignificant = TRUE, alpha = 0.05,
   skip_window_length = 50,
   skip_window_position = 50)

# plot(example_multiproxy, type = 1)

# 5 Example to test the temporal stability
example_MVA_ts <- daily_response(response = data_MVA,
   env_data = LJ_daily_temperatures, method = "brnn",
   lower_limit = 100, metric = "adj.r.squared", upper_limit = 180,
   row_names_subset = TRUE, remove_insignificant = TRUE, alpha = 0.05,
   temporal_stability_check = "running_window", k_running_window = 10,
   skip_window_length = 50, skip_window_position = 50)

# Check the results for temporal stability
# example_MVA_ts$temporal_stability

# 6 Example with nonlinear brnn estimation
example_brnn <- daily_response(response = data_MVA,
   env_data = LJ_daily_temperatures, method = "brnn", boot = FALSE,
   lower_limit = 100, metric = "adj.r.squared", upper_limit = 101,
   row_names_subset = TRUE, remove_insignificant = TRUE, boot_n = 10,
   skip_window_length = 50, skip_window_position = 50)

# summary(example_brnn)

# Example using quantiles for the aggregation
example_q95 <- daily_response(
response = data_MVA,
env_data = LJ_daily_temperatures,
method = "cor",
fixed_width = 30,
row_names_subset = TRUE,
previous_year = TRUE,
aggregate_function = "quantile",
quantile_prob = 0.95
)

# Example using two previous years plus the current year
example_two_previous_years <- daily_response(
response = data_MVA,
env_data = LJ_daily_temperatures,
method = "cor",
fixed_width = 60,
row_names_subset = TRUE,
previous_year = TRUE,
number_previous_years = 2,
day_interval = c(-1, 250),
skip_window_length = 50,
skip_window_position = 50
)

# Example with previous-previous year effects
example_basic <- daily_response(response = data_MVA,
                          env_data = LJ_daily_temperatures,
                          row_names_subset = TRUE,
                          reference_window = "end",
                          lower_limit = 35, upper_limit = 75,
                          previous_year = TRUE,
                          remove_insignificant = TRUE,
                          number_previous_years = 3)
plot_heatmap(example_basic)
plot_extreme(example_basic)
summary(example_basic)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_TRW.R
\docType{data}
\name{data_TRW}
\alias{data_TRW}
\title{Tree-ring width (TRW) example proxy from 1981 - 1757}
\format{
A data frame with 225 rows and 1 variable:
\describe{
  \item{TRW}{residual TRW indices from 1981 - 1757}
}
}
\source{
\itemize{
   \item Schweingruber, F.H., 1981. Vrsic Krajnska Gora - PCAB - ITRDB YUGO001.
   \item https://www.ncei.noaa.gov/access/paleo-search/study/4728}
}
\usage{
data_TRW
}
\description{
A dataset with TRW proxy records from a site in Slovenian Alps - Vrsic.
The first row represents a TRW value in a year 1757. Row names represent
years.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LJ_monthly_precipitation.R
\docType{data}
\name{LJ_monthly_precipitation}
\alias{LJ_monthly_precipitation}
\title{Monthly sums of precipitation for Ljubljana from 2018 - 1900. Tidy format.}
\format{
A data frame with 1417 rows and 3 variables:
\describe{
  \item{Year}{year}
  \item{Month}{Month}
  \item{Precipitation}{Sum of precipitation}
}
}
\source{
\url{http://climexp.knmi.nl/start.cgi}
}
\usage{
LJ_monthly_precipitation
}
\description{
A dataset of monthly sums of precipitations in Ljubljana (Slovenia). The first row
represents precipitation sum for January 1900.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/monthly_response_seascorr.R
\name{monthly_response_seascorr}
\alias{monthly_response_seascorr}
\title{monthly_response_seascorr}
\usage{
monthly_response_seascorr(
  response,
  env_data_primary,
  env_data_control,
  previous_year = FALSE,
  number_previous_years = NULL,
  pcor_method = "pearson",
  remove_insignificant = TRUE,
  lower_limit = 1,
  upper_limit = 12,
  fixed_width = 0,
  alpha = 0.05,
  row_names_subset = FALSE,
  reference_window = "start",
  aggregate_function_env_data_primary = "mean",
  quantile_prob_env_data_primary = 0.5,
  aggregate_function_env_data_control = "mean",
  quantile_prob_env_data_control = 0.5,
  temporal_stability_check = "sequential",
  k = 2,
  k_running_window = 30,
  subset_years = NULL,
  ylimits = NULL,
  seed = NULL,
  tidy_env_data_primary = FALSE,
  tidy_env_data_control = FALSE,
  boot = FALSE,
  boot_n = 1000,
  boot_ci_type = "norm",
  boot_conf_int = 0.95,
  month_interval = NULL,
  dc_method = NULL,
  pcor_na_use = "pairwise.complete"
)
}
\arguments{
\item{response}{a data frame with tree-ring proxy variable and (optional)
years as row names. Row.names should be matched with those from env_data_primary
and env_data_control data frame. If not, set the row_names_subset argument to
TRUE.}

\item{env_data_primary}{primary data frame of monthly sequences of environmental
data as columns and years as row names. Each row represents a year and
each column represents a day of a year. Row.names should be matched with
those from the response data frame. If not, set the argument row_names_subset
to TRUE. Alternatively, env_data_primary could be a tidy data with three columns,
i.e. Year, Month and third column representing values of mean temperatures,
sum of precipitation etc. If tidy data is passed to the function, set the argument
tidy_env_data_primary to TRUE.}

\item{env_data_control}{a data frame of monthly sequences of environmental data as
columns and years as row names. This data is used as control for calculations of
partial correlation coefficients. Each row represents a year and each column
represents a day of a year. Row.names should be matched with those from the
response data frame. If not, set the row_names_subset argument to TRUE.
Alternatively, env_data_control could be a tidy data with three columns,
i.e. Year, Month and third column representing values of mean temperatures, sum
of precipitation etc. If tidy data is passed to the function, set the argument
tidy_env_data_control to TRUE.}

\item{previous_year}{logical. If TRUE, previous-year climate data are included
in the analysis. If FALSE, no previous-year climate data are included and
number_previous_years is ignored.}

\item{number_previous_years}{integer between 1 and 5 specifying how many
previous years should be included in the environmental matrices when
previous_year = TRUE. For example, number_previous_years = 2 uses climate
data from years t - 2, t - 1 and t for response year t. If NULL and
previous_year = TRUE, one previous year is included.}

\item{pcor_method}{a character string indicating which partial correlation
coefficient is to be computed. One of "pearson" (default), "kendall", or
"spearman", can be abbreviated.}

\item{remove_insignificant}{if set to TRUE, removes all correlations bellow
the significant threshold level, based on a selected alpha.}

\item{lower_limit}{lower limit of window width (i.e. number of consecutive months
to be used for calculations)}

\item{upper_limit}{upper limit of window width (i.e. number of consecutive months
to be used for calculations)}

\item{fixed_width}{fixed width used for calculations (i.e. number of consecutive
months to be used for calculations)}

\item{alpha}{significance level used to remove insignificant calculations.}

\item{row_names_subset}{if set to TRUE, row.names are used to subset
env_data_primary, env_data_control and response data frames. Only years from
all three data frames are kept.}

\item{reference_window}{character string, the reference_window argument describes,
how each calculation is referred. There are two different options: 'start'
(default) and 'end'. If the reference_window argument is set to 'start',
then each calculation is related to the starting month of window. If the
reference_window argument is set to 'end', then each calculation is related
to the ending day of window calculation.}

\item{aggregate_function_env_data_primary}{character string specifying how the
monthly data from env_data_primary should be aggregated. The default is 'mean',
the other options are 'median', 'sum' and 'quantile'.}

\item{quantile_prob_env_data_primary}{numeric value between 0 and 1 specifying
the quantile probability used when aggregate_function_env_data_primary =
'quantile'. For example, 0.95 calculates the 95th percentile. The default is 0.5.}

\item{aggregate_function_env_data_control}{character string specifying how the
monthly data from env_data_control should be aggregated. The default is 'mean',
the other options are 'median', 'sum' and 'quantile'.}

\item{quantile_prob_env_data_control}{numeric value between 0 and 1 specifying
the quantile probability used when aggregate_function_env_data_control =
'quantile'. For example, 0.95 calculates the 95th percentile. The default is 0.5.}

\item{temporal_stability_check}{character string, specifying, how temporal stability
between the optimal selection and response variable(s) will be analysed. Current
possibilities are "sequential", "progressive" and "running_window". Sequential check
will split data into k splits and calculate selected metric for each split. Progressive
check will split data into k splits, calculate metric for the first split and then
progressively add 1 split at a time and calculate selected metric. For running window,
select the length of running window with the k_running_window argument.}

\item{k}{integer, number of breaks (splits) for temporal stability}

\item{k_running_window}{the length of running window for temporal stability check.
Applicable only if temporal_stability argument is set to running window.}

\item{subset_years}{a subset of years to be analyzed. Should be given in the form of
subset_years = c(1980, 2005)}

\item{ylimits}{limit of the y axes for plot_extreme. It should be given in
the form of: ylimits = c(0,1)}

\item{seed}{optional seed argument for reproducible results}

\item{tidy_env_data_primary}{if set to TRUE, env_data_primary should be inserted as a
data frame with three columns: "Year", "Month", "Precipitation/Temperature/etc."}

\item{tidy_env_data_control}{if set to TRUE, env_data_control should be inserted as a
data frame with three columns: "Year", "Month", "Precipitation/Temperature/etc."}

\item{boot}{logical, if TRUE, bootstrap procedure will be used to calculate
partial correlation coefficients}

\item{boot_n}{The number of bootstrap replicates}

\item{boot_ci_type}{A character string representing the type of bootstrap intervals
required. The value should be any subset of the values c("norm","basic", "stud",
"perc", "bca").}

\item{boot_conf_int}{A scalar or vector containing the confidence level(s) of
the required interval(s)}

\item{month_interval}{a vector of two values defining the interval of months
used for calculations. Positive values indicate months in the current year.
Negative values indicate months in the previous-year block and are used only
when previous_year = TRUE. If previous_year = FALSE and negative values are
supplied, month_interval is ignored and the analysis is performed for the
current year only using month_interval = c(1, 12). If number_previous_years > 1,
the previous-year block starts with the earliest included previous year. For
example, previous_year = TRUE, number_previous_years = 2 and
month_interval = c(-1, 12) analyses the full sequence from January of year
t - 2 to December of year t.}

\item{dc_method}{a character string to determine the method to detrend climate
data.  Possible values are "none" (default) and "SLD" which refers to Simple
Linear Detrending}

\item{pcor_na_use}{an optional character string giving a method for computing
covariances in the presence of missing values for partial correlation
coefficients. This must be (an abbreviation of) one of the strings "all.obs",
"everything", "complete.obs", "na.or.complete", or "pairwise.complete.obs"
(default). See also the documentation for the base partial.r in psych R package}
}
\value{
a list with 17 elements:
\enumerate{
 \item $calculations - a matrix with calculated metrics
 \item $method - the character string of a method
 \item $metric - the character string indicating the metric used for calculations
 \item $analysed_period - the character string specifying the analysed period based on the information from row names. If there are no row names, this argument is given as NA
 \item $optimized_return - data frame with two columns, response variable and aggregated (averaged) monthly data that return the optimal results. This data.frame could be directly used to calibrate a model for climate reconstruction
 \item $optimized_return_all - a data frame with aggregated monthly data, that returned the optimal result for the entire env_data_primary (and not only subset of analysed years)
 \item $transfer_function - a ggplot object: scatter plot of optimized return and a transfer line of the selected method
 \item $temporal_stability - a data frame with calculations of selected metric for different temporal subsets
 \item $cross_validation - not available for partial correlation method
 \item $plot_heatmap - ggplot2 object: a heatmap of calculated metrics
 \item $plot_extreme - ggplot2 object: line plot of a row with the highest value in a matrix of calculated metrics
 \item $type - the character string describing type of analysis: monthly or monthly
 \item $reference_window - character string, which reference window was used for calculations
 \item $aggregated_climate_primary - matrix with all aggregated climate series of primary data
 \item $aggregated_climate_control - matrix with all aggregated climate series of control data
 \item $previous_year - logical indicating whether previous-year climate data were used
 \item $number_previous_years - integer indicating how many previous years were used
}
}
\description{
Function calculates all possible partial correlation coefficients between
tree-ring chronology and monthly environmental (usually climate) data.
All calculated (partial) correlation coefficients are stored in a matrix.
The location of stored correlation in the matrix is indicating a window
width (row names) and a location in a matrix of monthly sequences of
environmental data (column names).
}
\examples{
\donttest{
# Load the dendroTools R package
library(dendroTools)

# Load data
data(data_MVA)
data(data_TRW)
data(data_TRW_1)
data(example_proxies_individual)
data(example_proxies_1)
data(LJ_monthly_temperatures)
data(LJ_monthly_precipitation)

# 1 Basic example
example_basic <- monthly_response_seascorr(response = data_MVA,
   fixed_width = 11,
   env_data_primary = LJ_monthly_temperatures,
   env_data_control = LJ_monthly_precipitation,
   row_names_subset = TRUE,
   remove_insignificant = TRUE,
   reference_window = "start",
   aggregate_function_env_data_primary = 'median',
   aggregate_function_env_data_control = 'median',
   alpha = 0.05, pcor_method = "spearman",
   tidy_env_data_primary = FALSE,
   tidy_env_data_control = TRUE,
   previous_year = TRUE)

# summary(example_basic)
# plot(example_basic, type = 1)
# plot(example_basic, type = 2)
# example_basic$optimized_return
# example_basic$optimized_return_all
# example_basic$temporal_stability

# 2 Extended example
example_extended <- monthly_response_seascorr(response = data_MVA,
   env_data_primary = LJ_monthly_temperatures,
   env_data_control = LJ_monthly_precipitation,
   row_names_subset = TRUE, dc_method = "SLD",
   remove_insignificant = FALSE,
   aggregate_function_env_data_primary = 'mean',
   aggregate_function_env_data_control = 'mean',
   alpha = 0.05, pcor_na_use = "pairwise.complete",
   reference_window = "end",
   tidy_env_data_primary = FALSE,
   tidy_env_data_control = TRUE)

# summary(example_extended)
# plot(example_extended, type = 1)
# plot(example_extended, type = 2)
# example_extended$optimized_return
# example_extended$optimized_return_all

# 3 Example using quantiles for primary and control variables
example_quantile <- monthly_response_seascorr(
   response = data_MVA,
   env_data_primary = LJ_monthly_temperatures,
   env_data_control = LJ_monthly_precipitation,
   row_names_subset = TRUE,
   fixed_width = 3,
   previous_year = TRUE,
   aggregate_function_env_data_primary = "quantile",
   quantile_prob_env_data_primary = 0.50,
   aggregate_function_env_data_control = "quantile",
   quantile_prob_env_data_control = 0.50,
   tidy_env_data_primary = FALSE,
   tidy_env_data_control = TRUE)

 # summary(example_quantile)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/critical.r
\name{critical_r}
\alias{critical_r}
\title{critical_r}
\usage{
critical_r(n, alpha = 0.05)
}
\arguments{
\item{n}{number of observations}

\item{alpha}{significance level}
}
\value{
calculated critical value of Pearson correlation coefficient
}
\description{
Calculates critical value of Pearson correlation coefficient
for a selected alpha.
}
\examples{
threshold_1 <- critical_r(n = 55, alpha = 0.01)
threshold_2 <- critical_r(n = 55, alpha = 0.05)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boot_f_lm.R
\name{boot_f_lm}
\alias{boot_f_lm}
\title{boot_f_lm}
\usage{
boot_f_lm(data, index, lm.formula)
}
\arguments{
\item{data}{data frame with variables for model fitting}

\item{index}{indices to be used for calculation}

\item{lm.formula}{object of class formula to be passed into lm function}
}
\value{
A matrix with bootstrapped estimates of r squared and adjusted r squared
}
\description{
Generate R bootstrap replicates of a lm function applied to data.
}
\references{
https://www.datacamp.com/community/tutorials/bootstrap-r
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LJ_monthly_temperatures.R
\docType{data}
\name{LJ_monthly_temperatures}
\alias{LJ_monthly_temperatures}
\title{Monthly mean air temperatures for Ljubljana from 2015 - 1900}
\format{
A data frame with 116 rows and 12 variables:
\describe{
  \item{Jan}{Mean monthly air temperature for January from 1900 to 2015}
  \item{Feb}{Mean monthly air temperature for February from 1900 to 2015}
  \item{Mar}{Mean monthly air temperature for March from 1900 to 2015}
  \item{Apr}{Mean monthly air temperature for April from 1900 to 2015}
  \item{May}{Mean monthly air temperature for May from 1900 to 2015}
  \item{Jun}{Mean monthly air temperature for June from 1900 to 2015}
  \item{Jul}{Mean monthly air temperature for July from 1900 to 2015}
  \item{Aug}{Mean monthly air temperature for August from 1900 to 2015}
  \item{Sep}{Mean monthly air temperature for September from 1900 to 2015}
  \item{Oct}{Mean monthly air temperature for October from 1900 to 2015}
  \item{Nov}{Mean monthly air temperature for November from 1900 to 2015}
  \item{Dec}{Mean monthly air temperature for December from 1900 to 2015}
}
}
\source{
\url{https://meteo.arso.gov.si/met/sl/archive/}
}
\usage{
LJ_monthly_temperatures
}
\description{
A dataset of monthly mean air temperatures in Ljubljana (Slovenia). The
first row represents temperatures in 2015. The first column represents
mean January temperature, the second column represents mean February
temperature. etc. Row names represent year.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/swit272_daily_temperatures.R
\docType{data}
\name{swit272_daily_temperatures}
\alias{swit272_daily_temperatures}
\title{Daily temperatures for swit272 chronology}
\format{
A data frame with 25414 rows and 2 variables:
\describe{
  \item{date}{character string describing date}
  \item{t_avg}{mean temperature}
}
}
\source{
\url{https://www.ecad.eu/download/ensembles/download.php}
}
\usage{
swit272_daily_temperatures
}
\description{
Mean daily temperature in Celsius for the period 1950 - 2019.
This gridded E-OBS data on 0.1° regular grid, version 20e.
Extracted data is for the grid point with lon = 9.75 and lat = 46.45.
}
\details{
We acknowledge the E-OBS dataset from the EU-FP6 project UERRA (http://www.uerra.eu) and the Copernicus Climate Change Service, and the data providers in the ECA&D project (https://www.ecad.eu). Cornes, R., G. van der Schrier, E.J.M. van den Besselaar, and P.D. Jones. 2018: An Ensemble Version of the E-OBS Temperature and Precipitation Datasets, J. Geophys. Res. Atmos., 123. doi:10.1029/2017JD028200
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/glimpse_daily_data.R
\name{glimpse_daily_data}
\alias{glimpse_daily_data}
\title{glimpse_daily_data}
\usage{
glimpse_daily_data(
  env_data,
  na.color = "red",
  low_color = "blue",
  high_color = "green",
  tidy_env_data = FALSE
)
}
\arguments{
\item{env_data}{a data frame of daily sequences of environmental data as
columns and years as row names. Each row represents a year and
each column represents a day of a year. Alternatively, env_data could be a
tidy data with three columns, i.e. Year, DOY and third column representing
values of mean temperatures, sum of precipitation etc. If tidy data is passed
to the function, set the argument tidy_env_data to TRUE.}

\item{na.color}{color to use for missing values}

\item{low_color}{colours for low end of the gradient}

\item{high_color}{colours for high end of the gradient}

\item{tidy_env_data}{if set to TRUE, env_data should be inserted as a data frame with three
columns: "Year", "DOY", "Precipitation/Temperature/etc."}
}
\description{
Visual presentation of daily data to spot missing values.
}
\examples{
library(dendroTools)
data("LJ_daily_temperatures")
# glimpse_daily_data(env_data = LJ_daily_temperatures,
#tidy_env_data = FALSE, na.color = "white")

data("LJ_daily_precipitation")
# glimpse_daily_data(env_data = LJ_daily_precipitation,
#   tidy_env_data = TRUE, na.color = "white")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/round2.R
\name{round2}
\alias{round2}
\title{round2}
\usage{
round2(x, n = 0)
}
\arguments{
\item{x}{a numeric vector}

\item{n}{integer indicating the number of decimal places}
}
\value{
a numeric vector with rounded numbers
}
\description{
Rounding function, where numbers are rounded up, 0.5 = 1
}
\references{
http://alandgraf.blogspot.com/2012/06/rounding-in-r.html
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/monthly_response.R
\name{monthly_response}
\alias{monthly_response}
\title{monthly_response}
\usage{
monthly_response(
  response,
  env_data,
  method = "cor",
  metric = "r.squared",
  cor_method = "pearson",
  previous_year = FALSE,
  number_previous_years = NULL,
  neurons = 1,
  lower_limit = 1,
  upper_limit = 12,
  fixed_width = 0,
  brnn_smooth = TRUE,
  remove_insignificant = TRUE,
  alpha = 0.05,
  row_names_subset = FALSE,
  reference_window = "start",
  aggregate_function = "mean",
  quantile_prob = 0.5,
  temporal_stability_check = "sequential",
  k = 2,
  k_running_window = 30,
  cross_validation_type = "blocked",
  subset_years = NULL,
  ylimits = NULL,
  seed = NULL,
  tidy_env_data = FALSE,
  boot = FALSE,
  boot_n = 1000,
  boot_ci_type = "norm",
  boot_conf_int = 0.95,
  month_interval = NULL,
  dc_method = NULL,
  cor_na_use = "everything"
)
}
\arguments{
\item{response}{a data frame with tree-ring proxy variables as columns and
(optional) years as row names. Row.names should be matched with those from a
env_data data frame. If not, set row_names_subset = TRUE.}

\item{env_data}{a data frame of monthly sequences of environmental
data as columns and years as row names. Each row represents a year and
each column represents a day of a year (or month). Row.names should be matched with
those from a response data frame. If not, set row_names_subset = TRUE.
Alternatively, env_data could be a tidy data with three columns,
i.e. Year, DOY (Month) and third column representing values of mean temperatures,
sum of precipitation etc. If tidy data is passed to the function, set the argument
tidy_env_data to TRUE.}

\item{method}{a character string specifying which method to use. Current
possibilities are "cor" (default), "lm" and "brnn".}

\item{metric}{a character string specifying which metric to use. Current
possibilities are "r.squared" and "adj.r.squared". If method = "cor",
metric is not relevant.}

\item{cor_method}{a character string indicating which correlation
coefficient is to be computed. One of "pearson" (default), "kendall", or
"spearman".}

\item{previous_year}{logical. If TRUE, previous-year climate data are included
in the analysis. If FALSE, no previous-year climate data are included and
number_previous_years is ignored.}

\item{number_previous_years}{integer between 1 and 5 specifying how many
previous years should be included in the environmental matrix when
previous_year = TRUE. For example, number_previous_years = 2 uses monthly
climate data from years t - 2, t - 1 and t for response year t. If NULL and
previous_year = TRUE, one previous year is included.}

\item{neurons}{positive integer that indicates the number of neurons used
for brnn method}

\item{lower_limit}{lower limit of window width (i.e. number of consecutive months
to be used for calculations)}

\item{upper_limit}{upper limit of window width (i.e. number of consecutive months
to be used for calculations)}

\item{fixed_width}{fixed width used for calculations (i.e. number of consecutive
months to be used for calculations)}

\item{brnn_smooth}{if set to TRUE, a smoothing algorithm is applied that
removes unrealistic calculations which are a result of neural net failure.}

\item{remove_insignificant}{if set to TRUE, removes all correlations bellow
the significant threshold level, based on a selected alpha. For "lm" and
"brnn" method, squared threshold is used, which corresponds to R squared
statistics.}

\item{alpha}{significance level used to remove insignificant calculations.}

\item{row_names_subset}{if set to TRUE, row.names are used to subset
env_data and response data frames. Only years from both data frames are
kept.}

\item{reference_window}{character string, the reference_window argument describes,
how each calculation is referred. There are two different options: 'start'
(default) and 'end'. If the reference_window argument is set to 'start',
then each calculation is related to the starting month of window. If the
reference_window argument is set to 'end', then each calculation is related
to the ending day of window calculation.}

\item{aggregate_function}{character string specifying how the monthly data should be
aggregated. The default is 'mean', the other options are 'median', 'sum'
and 'quantile'.}

\item{quantile_prob}{numeric value between 0 and 1 specifying the quantile
probability used when aggregate_function = 'quantile'. For example,
quantile_prob = 0.95 calculates the 95th percentile. The default is 0.5.}

\item{temporal_stability_check}{character string, specifying, how temporal stability
between the optimal selection and response variable(s) will be analysed. Current
possibilities are "sequential", "progressive" and "running_window". Sequential check
will split data into k splits and calculate selected metric for each split. Progressive
check will split data into k splits, calculate metric for the first split and then
progressively add 1 split at a time and calculate selected metric. For running window,
select the length of running window with the k_running_window argument.}

\item{k}{integer, number of breaks (splits) for temporal stability and cross validation
analysis.}

\item{k_running_window}{the length of running window for temporal stability check.
Applicable only if temporal_stability argument is set to running window.}

\item{cross_validation_type}{character string, specifying, how to perform cross validation
between the optimal selection and response variables. If the argument is set to "blocked",
years will not be shuffled. If the argument is set to "randomized", years will be shuffled.}

\item{subset_years}{a subset of years to be analyzed. Should be given in the form of
subset_years = c(1980, 2005)}

\item{ylimits}{limit of the y axes for plot_extreme. It should be given in
the form of: ylimits = c(0,1)}

\item{seed}{optional seed argument for reproducible results}

\item{tidy_env_data}{if set to TRUE, env_data should be inserted as a data frame with three
columns: "Year", "Month", "Precipitation/Temperature/etc."}

\item{boot}{logical, if TRUE, bootstrap procedure will be used to calculate
estimates correlation coefficients, R squared or adjusted R squared metrices}

\item{boot_n}{The number of bootstrap replicates}

\item{boot_ci_type}{A character string representing the type of bootstrap intervals
required. The value should be any subset of the values c("norm","basic", "stud",
"perc", "bca").}

\item{boot_conf_int}{A scalar or vector containing the confidence level(s) of
the required interval(s)}

\item{month_interval}{a vector of two values defining the interval of months
used for calculations. Positive values indicate months in the current year.
Negative values indicate months in the previous-year block and are only used
when previous_year = TRUE. If previous_year = FALSE and negative values are
supplied, month_interval is ignored and the analysis is performed for the
current year only using month_interval = c(1, 12). If number_previous_years > 1,
the previous-year block starts with the earliest included previous year. For
example, previous_year = TRUE, number_previous_years = 2 and
month_interval = c(-1, 12) analyses the full sequence from January of year
t - 2 to December of year t.}

\item{dc_method}{a character string to determine the method to detrend climate
data.  Possible values are "none" (default) and "SLD" which refers to Simple
Linear Detrending}

\item{cor_na_use}{an optional character string giving a method for computing
covariances in the presence of missing values for correlation coefficients.
This must be (an abbreviation of) one of the strings "everything" (default),
"all.obs", "complete.obs", "na.or.complete", or "pairwise.complete.obs". See
also the documentation for the base cor() function.}
}
\value{
a list with 19 elements:
\enumerate{
 \item $calculations - a matrix with calculated metrics
 \item $method - the character string of a method
 \item $metric - the character string indicating the metric used for calculations
 \item $analysed_period - the character string specifying the analysed period based on the information from row names. If there are no row names, this argument is given as NA
 \item $optimized_return - data frame with two columns, response variable and aggregated (averaged) monthly data that return the optimal results. This data.frame could be directly used to calibrate a model for climate reconstruction
 \item $optimized_return_all - a data frame with aggregated monthly data, that returned the optimal result for the entire env_data (and not only subset of analysed years)
 \item $transfer_function - a ggplot object: scatter plot of optimized return and a transfer line of the selected method
 \item $temporal_stability - a data frame with calculations of selected metric for different temporal subsets
 \item $cross_validation - a data frame with cross validation results
 \item $plot_heatmap - ggplot2 object: a heatmap of calculated metrics
 \item $plot_extreme - ggplot2 object: line or bar plot of a row with the highest value in a matrix of calculated metrics
 \item $type - the character string describing type of analysis: daily or monthly
 \item $reference_window - character string, which reference window was used for calculations
 \item $boot_lower - matrix with lower limit of confidence intervals of bootstrap calculations
 \item $boot_upper - matrix with upper limit of confidence intervals of bootstrap calculations
 \item $aggregated_climate - matrix with all aggregated climate series
 \item $previous_year - logical indicating whether previous-year climate data were used
 \item $number_previous_years - integer indicating how many previous years were used
}
}
\description{
Function calculates all possible values of a selected statistical metric
between one or more response variables and monthly sequences of environmental
data. Calculations are based on moving window which slides through monthly
environmental data. All calculated metrics are stored in a matrix. The
location of stored calculated metric in the matrix is indicating a window
width (row names) and a location in a matrix of monthly sequences of
environmental data (column names).
}
\examples{
\donttest{

# The examples below are enclosed within donttest{} to minimize the execution
# time during R package checks.

# Load the dendroTools R package
library(dendroTools)

# Load data used for examples
data(data_MVA)
data(data_TRW)
data(data_TRW_1)
data(example_proxies_individual)
data(example_proxies_1)
data(LJ_monthly_temperatures)
data(LJ_monthly_precipitation)

# 1 Example with tidy precipitation data
example_tidy_data <- monthly_response(response = data_MVA,
    lower_limit = 1, upper = 24, dc_method = "SLD",
    env_data = LJ_monthly_precipitation, fixed_width = 0,
    method = "cor", row_names_subset = TRUE,
    remove_insignificant = FALSE, previous_year = FALSE,
    reference_window = "end",
    alpha = 0.05, aggregate_function = 'sum', boot = FALSE,
    tidy_env_data = TRUE, boot_n = 100, month_interval = c(-5, 10))

# summary(example_tidy_data)
# plot(example_tidy_data, type = 1)
# plot(example_tidy_data, type = 2)

# 2 Example with split data for early and late
example_MVA_early <- monthly_response(response = data_MVA,
    env_data = LJ_monthly_temperatures,
    method = "cor", row_names_subset = TRUE, previous_year = TRUE,
    remove_insignificant = TRUE, alpha = 0.05,
    subset_years = c(1940, 1980), aggregate_function = 'mean')

example_MVA_late <- monthly_response(response = data_MVA,
    env_data = LJ_monthly_temperatures,
    method = "cor", row_names_subset = TRUE, alpha = 0.05,
    previous_year = TRUE, remove_insignificant = TRUE,
    subset_years = c(1981, 2010), aggregate_function = 'mean')

# summary(example_MVA_late)
# plot(example_MVA_early, type = 1)
# plot(example_MVA_late, type = 1)
# plot(example_MVA_early, type = 2)
# plot(example_MVA_late, type = 2)

# 3 Example negative correlations
example_neg_cor <- monthly_response(response = data_TRW_1, alpha = 0.05,
   env_data = LJ_monthly_temperatures,
   method = "cor", row_names_subset = TRUE,
   remove_insignificant = TRUE, boot = FALSE)

# summary(example_neg_cor)
# plot(example_neg_cor, type = 1)
# plot(example_neg_cor, type = 2)
# example_neg_cor$temporal_stability

# 4 Example of multiproxy analysis
# summary(example_proxies_1)
# cor(example_proxies_1)

example_multiproxy <- monthly_response(response = example_proxies_1,
   env_data = LJ_monthly_temperatures,
   method = "lm", metric = "adj.r.squared",
   row_names_subset = TRUE, previous_year = FALSE,
   remove_insignificant = TRUE, alpha = 0.05)

# summary(example_multiproxy)
# plot(example_multiproxy, type = 1)

# 5 Example to test the temporal stability
example_MVA_ts <- monthly_response(response = data_MVA,
   env_data = LJ_monthly_temperatures,
   method = "lm", metric = "adj.r.squared", row_names_subset = TRUE,
   remove_insignificant = TRUE, alpha = 0.05,
   temporal_stability_check = "running_window", k_running_window = 10)

# summary(example_MVA_ts)
# example_MVA_ts$temporal_stability

# 6 Example using quantiles for the aggregation
example_q95 <- monthly_response(
  response = data_MVA,
  env_data = LJ_monthly_temperatures,
  method = "cor",
  fixed_width = 3,
  row_names_subset = TRUE,
  previous_year = TRUE,
  aggregate_function = "quantile",
  quantile_prob = 0.95
)

# summary(example_q95)
# example_q95$temporal_stability

# Example using two previous years plus the current year
example_two_previous_years <- monthly_response(
  response = data_MVA,
  env_data = LJ_monthly_temperatures,
  method = "cor",
  fixed_width = 3,
  row_names_subset = TRUE,
  previous_year = TRUE,
  number_previous_years = 2,
  month_interval = c(-1, 10),
  aggregate_function = "mean"
)

}
}
